@@ -50,6 +50,7 @@ class GSymbol:
 class GWindow:
   __metaclass__ = metaIDWrapper
   methods = {
+    'GetRect': _GemRB.Window_GetRect,
     'SetSize': _GemRB.Window_SetSize,
     'SetFrame': _GemRB.Window_SetFrame,
     'SetPicture': _GemRB.Window_SetPicture,
@@ -57,11 +58,11 @@ class GWindow:
     'HasControl': _GemRB.Window_HasControl,
     'DeleteControl': _GemRB.Window_DeleteControl,
     'SetupEquipmentIcons': _GemRB.Window_SetupEquipmentIcons,
-    'SetupSpellIcons': _GemRB.Window_SetupSpellIcons,
     'SetupControls': _GemRB.Window_SetupControls,
     'SetVisible': _GemRB.Window_SetVisible,
     'ShowModal': _GemRB.Window_ShowModal,
-    'Invalidate': _GemRB.Window_Invalidate
+    'Invalidate': _GemRB.Window_Invalidate,
+    'UpdateVarAssoc': _GemRB.Window_UpdateVarAssoc
   }
   def __nonzero__(self):
     return self.ID != -1
@@ -94,6 +95,8 @@ class GWindow:
 class GControl:
   __metaclass__ = metaControl
   methods = {
+    'GetRect': _GemRB.Control_GetRect,
+    'HasAnimation': _GemRB.Control_HasAnimation,
     'SetVarAssoc': _GemRB.Control_SetVarAssoc,
     'SetPos': _GemRB.Control_SetPos,
     'SetSize': _GemRB.Control_SetSize,
@@ -113,6 +116,7 @@ class GControl:
 class GLabel(GControl):
   __metaclass__ = metaControl
   methods = {
+    'SetFont': _GemRB.Label_SetFont,
     'SetTextColor': _GemRB.Label_SetTextColor,
     'SetUseRGB': _GemRB.Label_SetUseRGB
   }
@@ -137,7 +141,8 @@ class GTextArea(GControl):
 class GTextEdit(GControl):
   __metaclass__ = metaControl
   methods = {
-    'SetBufferLength': _GemRB.TextEdit_SetBufferLength
+    'SetBufferLength': _GemRB.TextEdit_SetBufferLength,
+    'SetBackground': _GemRB.TextEdit_SetBackground
   }
   def ConvertEdit(self, ScrollBarID):
     newID = _GemRB.TextEdit_ConvertEdit(self.WinID, self.ID, ScrollBarID)
@@ -158,6 +163,8 @@ class GButton(GControl):
     'SetBorder': _GemRB.Button_SetBorder,
     'EnableBorder': _GemRB.Button_EnableBorder,
     'SetFont': _GemRB.Button_SetFont,
+    'SetAnchor': _GemRB.Button_SetAnchor,
+    'SetPushOffset': _GemRB.Button_SetPushOffset,
     'SetTextColor': _GemRB.Button_SetTextColor,
     'SetFlags': _GemRB.Button_SetFlags,
     'SetState': _GemRB.Button_SetState,
