@@ -22,11 +22,12 @@
 import GemRB
 import GUIClasses
 import CommonTables
-from ie_restype import RES_CHU, RES_2DA, RES_WMP, RES_ARE
-from ie_spells import LS_MEMO, LSR_KNOWN, LSR_LEVEL, LSR_STAT
+from ie_restype import RES_CHU, RES_WMP, RES_ARE
+from ie_spells import LS_MEMO
 from GUIDefines import *
 from ie_stats import *
 from ie_slots import SLOT_ALL
+from ie_feats import FEAT_STRONG_BACK
 
 OtherWindowFn = None
 NextWindowFn = None
@@ -87,21 +88,17 @@ def GetWindowPack():
 
 def LocationPressed ():
 	AreaInfo = GemRB.GetAreaInfo()
-	print( "%s [%d.%d]\n"%(AreaInfo["CurrentArea"], AreaInfo["PositionX"], AreaInfo["PositionY"]) );
-	return
-
-def RestPress ():
-	GemRB.RestParty(0,0,0)
+	print( "%s [%d.%d]\n"%(AreaInfo["CurrentArea"], AreaInfo["PositionX"], AreaInfo["PositionY"]) )
 	return
 
 def SelectFormation ():
 	GemRB.GameSetFormation (GemRB.GetVar ("Formation"))
 	return
 
-def OpenFloatMenuWindow ():
+def OpenFloatMenuWindow (x, y):
 	if GameIsPST():
 		import FloatMenuWindow
-		FloatMenuWindow.OpenFloatMenuWindow()
+		FloatMenuWindow.OpenFloatMenuWindow(x, y)
 	else:
 		GemRB.GameControlSetTargetMode (TARGET_MODE_NONE)
 
@@ -122,104 +119,9 @@ def GearsClicked ():
 	#GemRB.SetPlayerStat(GemRB.GameGetFirstSelectedPC (),44,249990)
 	GemRB.GamePause (2, 0)
 
-def GetMageSpells (Kit, Alignment, Level):
-	MageSpells = []
-	SpellType = 99
-	Table = GemRB.LoadTable ("aligns")
-	v = Table.FindValue (3, Alignment)
-	Usability = Kit | Table.GetValue(v, 5)
-
-	SpellsTable = GemRB.LoadTable ("spells")
-	for i in range(SpellsTable.GetValue ("MAGE", str(Level), 1) ):
-		SpellName = "SPWI%d%02d"%(Level,i+1)
-		ms = GemRB.GetSpell (SpellName, 1)
-		if ms == None:
-			continue
-
-		if Usability & ms['SpellExclusion']:
-			SpellType = 0
-		else:
-			SpellType = 1
-			if Kit & (1 << ms['SpellSchool']+5): # of matching specialist school
-				SpellType = 2
-			# Wild mage spells are of normal schools, so we have to find them
-			# separately. Generalists can learn any spell but the wild ones, so
-			# we check if the mage is wild and if a generalist wouldn't be able
-			# to learn the spell.
-			if Kit == 0x8000 and (0x4000 & ms['SpellExclusion']):
-				SpellType = 2
-		MageSpells.append ([SpellName, SpellType])
-
-	return MageSpells
-
-def GetLearnableMageSpells (Kit, Alignment, Level):
-	Learnable = []
-
-	for Spell in GetMageSpells (Kit, Alignment, Level):
-		if Spell[1]: 
-			Learnable.append (Spell[0])
-	return Learnable
-
-def GetLearnablePriestSpells (Class, Alignment, Level):
-	Learnable =[]
-
-	Table=GemRB.LoadTable("aligns")
-	v = Table.FindValue(3, Alignment)
-	#usability is the bitset we look for
-	Usability=Table.GetValue(v, 5)
-
-	SpellsTable = GemRB.LoadTable ("spells")
-	for i in range(SpellsTable.GetValue ("PRIEST", str (Level), 1) ):
-		SpellName = "SPPR%d%02d"%(Level,i+1)
-		ms = GemRB.GetSpell(SpellName, 1)
-		if ms == None:
-			continue
-		if Class & ms['SpellDivine']:
-			continue
-		if Usability & ms['SpellExclusion']:
-			continue
-		Learnable.append (SpellName)
-	return Learnable
-
-# there is no separate druid spell table in the originals
-#FIXME: try to do this in a non-hard way?
-def GetPriestSpellTable(tablename):
-	if not GemRB.HasResource (tablename, RES_2DA):
-		if tablename == "MXSPLDRU":
-			return "MXSPLPRS"
-	return tablename
-
-def SetupSpellLevels (pc, TableName, Type, Level):
-	#don't die on a missing reference
-	tmp = GetPriestSpellTable(TableName)
-	if tmp != TableName:
-		SetupSpellLevels (pc, tmp, Type, Level)
-		return
-
-	Table = GemRB.LoadTable (TableName)
-	for i in range(Table.GetColumnCount ()):
-		# do a string lookup since some tables don't have entries for all levels
-		value = Table.GetValue (str(Level), str(i+1), 1)
-		# specialist mages get an extra spell if they already know that level
-		# FIXME: get a general routine to find specialists
-		school = GemRB.GetVar("MAGESCHOOL")
-		if Type == IE_SPELL_TYPE_WIZARD and school != 0:
-			if value > 0:
-				value += 1
-		GemRB.SetMemorizableSpellsCount (pc, value, Type, i)
-	return
-
-def UnsetupSpellLevels (pc, TableName, Type, Level):
-	#don't die on a missing reference
-	tmp = GetPriestSpellTable(TableName)
-	if tmp != TableName:
-		UnsetupSpellLevels (pc, tmp, Type, Level)
-		return
-
-	Table = GemRB.LoadTable (TableName)
-	for i in range(Table.GetColumnCount ()):
-		GemRB.SetMemorizableSpellsCount (pc, 0, Type, i)
-	return
+def GetAbilityBonus (Actor, Stat):
+	Ability = GemRB.GetPlayerStat (Actor, Stat)
+	return Ability//2-5
 
 def SetColorStat (Actor, Stat, Value):
 	t = Value & 0xFF
@@ -278,8 +180,8 @@ def HasHOW ():
 def HasTOTL ():
 	return GemRB.HasResource ("ar9700", RES_ARE)
 
-def GetIWDSpellButtonCount ():
-	if HasHOW():
+def GetGUISpellButtonCount ():
+	if HasHOW() or GameIsBG2():
 		return 24
 	else:
 		return 20
@@ -288,23 +190,18 @@ def SetGamedaysAndHourToken ():
 	currentTime = GemRB.GetGameTime()
 	days = currentTime / 7200
 	hours = (currentTime % 7200) / 300
+	GemRB.SetToken ('GAMEDAY', str (days))
 	GemRB.SetToken ('GAMEDAYS', str (days))
 	GemRB.SetToken ('HOUR', str (hours))
 
-# Returns -1 if not found; otherwise, the index of the spell
-def HasSpell (Actor, SpellType, Level, Ref):
-	# loop through each spell in the spell level and check for a matching ref
-	for i in range (GemRB.GetKnownSpellsCount (Actor, SpellType, Level)):
-		Spell = GemRB.GetKnownSpell(Actor, SpellType, Level, i)
-		if Spell["SpellResRef"].upper() == Ref.upper(): # ensure case is the same
-			return i
-
-	# not found
-	return -1
+def Gain(infostr, ability):
+	GemRB.SetToken ('SPECIALABILITYNAME', GemRB.GetString(int(ability) ) )
+	GemRB.DisplayString (infostr)
 
 # Adds class/kit abilities
 def AddClassAbilities (pc, table, Level=1, LevelDiff=1, align=-1):
 	TmpTable = GemRB.LoadTable (table)
+	import Spellbook
 
 	# gotta stay positive
 	if Level-LevelDiff < 0:
@@ -335,25 +232,34 @@ def AddClassAbilities (pc, table, Level=1, LevelDiff=1, align=-1):
 					ab = "GA_" + ab
 
 				# apply spell (AP_) or gain spell (GA_)
-				if ab[:2] == "AP":
+				if ab[:3] == "AP_":
 					GemRB.ApplySpell (pc, ab[3:])
-				elif ab[:2] == "GA":
-					SpellIndex = HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, ab[3:])
+				elif ab[:3] == "GA_":
+					SpellIndex = Spellbook.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, ab[3:])
 					if SpellIndex == -1:
 						GemRB.LearnSpell (pc, ab[3:], LS_MEMO)
 					else:
-						# make room for one more memorization
-						max_mem_cnt = GemRB.GetMemorizableSpellsCount (pc, IE_SPELL_TYPE_INNATE, 0, 0)
-						GemRB.SetMemorizableSpellsCount (pc, max_mem_cnt+1, IE_SPELL_TYPE_INNATE, 0)
-						# memorize another spell instance
 						GemRB.MemorizeSpell (pc, IE_SPELL_TYPE_INNATE, 0, SpellIndex)
+				elif ab[:3] == "FS_":
+					Gain(26320, ab[3:])
+				elif ab[:3] == "FA_":
+					Gain(10514, ab[3:])
 				else:
 					print "ERROR, unknown class ability (type): ", ab
 
+def MakeSpellCount (pc, spell, count):
+	have = GemRB.CountSpells (pc, spell, 1)
+	if count<=have:
+		return
+	for i in range (count-have):
+		GemRB.LearnSpell (pc, spell, LS_MEMO)
+	return
+	
 # remove all class abilities up to the given level
 # for dual-classing mainly
 def RemoveClassAbilities (pc, table, Level):
 	TmpTable = GemRB.LoadTable (table)
+	import Spellbook
 
 	# gotta stay positive
 	if Level < 0:
@@ -369,57 +275,31 @@ def RemoveClassAbilities (pc, table, Level):
 			ab = TmpTable.GetValue (i, j, 0)
 			if ab and ab != "****":
 				# get the index
-				SpellIndex = HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, ab[3:])
+				SpellIndex = Spellbook.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, ab[3:])
 
 				# seems all SPINs act like GA_*
 				if ab[:4] == "SPIN":
 					ab = "GA_" + ab
 
 				# apply spell (AP_) or gain spell (GA_)?
-				if ab[:2] == "AP":
+				if ab[:3] == "AP_":
 					GemRB.RemoveEffects (pc, ab[3:])
-				elif ab[:2] == "GA":
+				elif ab[:3] == "GA_":
 					if SpellIndex >= 0:
 						# TODO: get the correct counts to avoid removing an innate ability
 						# given by more than one thing?
 						# RemoveSpell will unmemorize them all too
 						GemRB.RemoveSpell (pc, IE_SPELL_TYPE_INNATE, 0, SpellIndex)
-				else:
+				elif ab[:3] != "FA_" and ab[:3] != "FS_":
 					print "ERROR, unknown class ability (type): ", ab
 
-def CannotLearnSlotSpell ():
-	pc = GemRB.GameGetSelectedPCSingle ()
-
-	# disqualify sorcerors immediately
-	if GemRB.GetPlayerStat (pc, IE_CLASS) == 19:
-		return LSR_STAT
-
-	if GameIsPST():
-		import GUIINV
-		slot, slot_item = GUIINV.ItemHash[GemRB.GetVar ('ItemButton')]
-	else:
-		slot_item = GemRB.GetSlotItem (pc, GemRB.GetVar ("ItemButton"))
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
-	spell = GemRB.GetSpell (spell_ref)
-
-	# maybe she already knows this spell
-	if HasSpell (pc, IE_SPELL_TYPE_WIZARD, spell['SpellLevel']-1, spell_ref) != -1:
-		return LSR_KNOWN
-
-	# level check (needs enough intelligence for this level of spell)
-	dumbness = GemRB.GetPlayerStat (pc, IE_INT)
-	if spell['SpellLevel'] > GemRB.GetAbilityBonus (IE_INT, 1, dumbness):
-		return LSR_LEVEL
-
-	return 0
-
 def UpdateInventorySlot (pc, Button, Slot, Type, Equipped=False):
 	Button.SetFont ("NUMBER")
 	Button.SetBorder (0, 0,0,0,0, 128,128,255,64, 0,1)
 	Button.SetBorder (1, 2,2,2,2, 32,32,255,0, 0,0)
 	Button.SetBorder (2, 0,0,0,0, 255,128,128,64, 0,1)
-	Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT | IE_GUI_BUTTON_ALIGN_TOP | IE_GUI_BUTTON_PICTURE, OP_OR)
 	Button.SetText ("")
+	Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT | IE_GUI_BUTTON_ALIGN_BOTTOM | IE_GUI_BUTTON_PICTURE, OP_OR)
 
 	if Slot == None:
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
@@ -468,35 +348,13 @@ def UpdateInventorySlot (pc, Button, Slot, Type, Equipped=False):
 
 	return
 
-def LearnPriestSpells (pc, level, mask):
-	"""Learns all the priest spells through the given spell level.
-
-	Mask distinguishes clerical and druidic spells."""
-	if level > 7: # make sure we don't have too high a level
-		level = 7
-
-	# go through each level
-	alignment = GemRB.GetPlayerStat (pc, IE_ALIGNMENT)
-	for i in range (level):
-		learnable = GetLearnablePriestSpells (mask, alignment, i+1)
-
-		for spell in learnable:
-			# if the spell isn't learned, learn it
-			if HasSpell (pc, IE_SPELL_TYPE_PRIEST, i, spell) < 0:
-				GemRB.LearnSpell (pc, spell)
-	return
-
 # PST uses a button, IWD2 two types, the rest are the same with two labels
 def SetEncumbranceLabels (Window, ControlID, Control2ID, pc, invert_colors = False):
 	"""Displays the encumbrance as a ratio of current to maximum."""
 
-	# Getting the character's strength
-	sstr = GemRB.GetPlayerStat (pc, IE_STR)
-	ext_str = GemRB.GetPlayerStat (pc, IE_STREXTRA)
-
 	# encumbrance
-	max_encumb = CommonTables.StrMod.GetValue (sstr, 3) + CommonTables.StrModEx.GetValue (ext_str, 3)
 	encumbrance = GemRB.GetPlayerStat (pc, IE_ENCUMBRANCE)
+	max_encumb = GemRB.GetMaxEncumbrance (pc)
 
 	Control = Window.GetControl (ControlID)
 	if GameIsPST():
@@ -513,24 +371,33 @@ def SetEncumbranceLabels (Window, ControlID, Control2ID, pc, invert_colors = Fal
 		Control2.SetText (str (max_encumb) + ":")
 
 	ratio = (0.0 + encumbrance) / max_encumb
-	if ratio > 1.0:
-		if invert_colors:
-			Control.SetTextColor (255, 0, 0, True)
+	if GameIsIWD2 () or GameIsPST ():
+		if ratio > 1.0:
+			if invert_colors:
+				Control.SetTextColor (255, 0, 0, True)
+			else:
+				Control.SetTextColor (255, 0, 0)
+		elif ratio > 0.8:
+			if invert_colors:
+				Control.SetTextColor (255, 255, 0, True)
+			else:
+				Control.SetTextColor (255, 255, 0)
 		else:
-			Control.SetTextColor (255, 0, 0)
-	elif ratio > 0.8:
-		if invert_colors:
-			Control.SetTextColor (255, 255, 0, True)
-		else:
-			Control.SetTextColor (255, 255, 0)
-	else:
-		if invert_colors:
-			Control.SetTextColor (255, 255, 255, True)
-		else:
-			Control.SetTextColor (255, 255, 255)
+			if invert_colors:
+				Control.SetTextColor (255, 255, 255, True)
+			else:
+				Control.SetTextColor (255, 255, 255)
 
-	if Control2ID:
-		Control2.SetTextColor (255, 0, 0)
+		if Control2ID:
+			Control2.SetTextColor (255, 0, 0)
+
+	else:
+		if ratio > 1.0:
+			Control.SetFont ("NUMBER3");
+		elif ratio > 0.8:
+			Control.SetFont ("NUMBER2");
+		else:
+			Control.SetFont ("NUMBER");
 		
 	return
 
@@ -540,14 +407,13 @@ def GetActorClassTitle (actor):
 	ClassTitle = GemRB.GetPlayerStat (actor, IE_TITLE1)
 
 	if ClassTitle == 0:
-		Class = GemRB.GetPlayerStat (actor, IE_CLASS)
-		ClassIndex = CommonTables.Classes.FindValue ( 5, Class )
+		ClassName = GetClassRowName (actor)
 		KitIndex = GetKitIndex (actor)
-		Multi = CommonTables.Classes.GetValue (ClassIndex, 4)
+		Multi = HasMultiClassBits (actor)
 		Dual = IsDualClassed (actor, 1)
 
 		if Multi and Dual[0] == 0: # true multi class
-			ClassTitle = CommonTables.Classes.GetValue (ClassIndex, 2)
+			ClassTitle = CommonTables.Classes.GetValue (ClassName, "CAP_REF")
 			ClassTitle = GemRB.GetString (ClassTitle)
 		else:
 			if Dual[0]: # dual class
@@ -555,14 +421,14 @@ def GetActorClassTitle (actor):
 				if Dual[0] == 1:
 					ClassTitle = CommonTables.KitList.GetValue (Dual[1], 2)
 				elif Dual[0] == 2:
-					ClassTitle = CommonTables.Classes.GetValue (Dual[1], 2)
+					ClassTitle = CommonTables.Classes.GetValue (GetClassRowName(Dual[1], "index"), "CAP_REF")
 				ClassTitle = GemRB.GetString (ClassTitle) + " / "
-				ClassTitle += GemRB.GetString (CommonTables.Classes.GetValue (Dual[2], 2))
+				ClassTitle += GemRB.GetString (CommonTables.Classes.GetValue (GetClassRowName(Dual[2], "index"), "CAP_REF"))
 			else: # ordinary class or kit
 				if KitIndex:
 					ClassTitle = CommonTables.KitList.GetValue (KitIndex, 2)
 				else:
-					ClassTitle = CommonTables.Classes.GetValue (ClassIndex, 2)
+					ClassTitle = CommonTables.Classes.GetValue (ClassName, "CAP_REF")
 				if ClassTitle != "*":
 					ClassTitle = GemRB.GetString (ClassTitle)
 	else:
@@ -580,7 +446,6 @@ def GetKitIndex (actor):
 
 	Returns 0 if the class is not kitted."""
 
-	Class = GemRB.GetPlayerStat (actor, IE_CLASS)
 	Kit = GemRB.GetPlayerStat (actor, IE_KIT)
 	KitIndex = 0
 
@@ -596,6 +461,33 @@ def GetKitIndex (actor):
 
 	return KitIndex
 
+# fetches the rowname of the passed actor's (base) class from classes.2da
+# NOTE: only the "index" method is iwd2-ready, since you can have multiple classes and kits
+def GetClassRowName(value, which=-1):
+	if which == "index":
+		ClassIndex = value
+		# if barbarians cause problems, repeat the lookup again here
+	else:
+		if which == -1:
+			Class = GemRB.GetPlayerStat (value, IE_CLASS)
+		elif which == "class":
+			Class = value
+		else:
+			raise RuntimeError, "Bad type parameter for GetClassRowName: ", which
+		ClassIndex = CommonTables.Classes.FindValue ("ID", Class)
+	ClassRowName = CommonTables.Classes.GetRowName (ClassIndex)
+	return ClassRowName
+
+# checks the classes.2da table if the class is multiclass/dualclass capable (bits define the class combination)
+def HasMultiClassBits(actor):
+	MultiBits = CommonTables.Classes.GetValue (GetClassRowName(actor), "MULTI")
+
+	# we have no entries for npc creature classes, so treat them as single-classed
+	if MultiBits == "*":
+		MultiBits = 0
+
+	return MultiBits
+
 def IsDualClassed(actor, verbose):
 	"""Returns an array containing the dual class information.
 
@@ -607,18 +499,19 @@ def IsDualClassed(actor, verbose):
 	if GameIsIWD2():
 		return (0,-1,-1)
 
+	Multi = HasMultiClassBits (actor)
+
+	if Multi == 0:
+		return (0, -1, -1)
+
 	DualedFrom = GemRB.GetPlayerStat (actor, IE_MC_FLAGS) & MC_WAS_ANY_CLASS
 
 	if verbose:
-		Class = GemRB.GetPlayerStat (actor, IE_CLASS)
-		ClassIndex = CommonTables.Classes.FindValue (5, Class)
-		Multi = CommonTables.Classes.GetValue (ClassIndex, 4)
 		DualInfo = []
 		KitIndex = GetKitIndex (actor)
 
 		if DualedFrom > 0: # first (previous) class of the dual class
-			MCColumn = CommonTables.Classes.GetColumnIndex ("MC_WAS_ID")
-			FirstClassIndex = CommonTables.Classes.FindValue (MCColumn, DualedFrom)
+			FirstClassIndex = CommonTables.Classes.FindValue ("MC_WAS_ID", DualedFrom)
 			if KitIndex:
 				DualInfo.append (1)
 				DualInfo.append (KitIndex)
@@ -630,7 +523,7 @@ def IsDualClassed(actor, verbose):
 			Mask = 1
 			for i in range (1,16):
 				if Multi & Mask:
-					ClassIndex = CommonTables.Classes.FindValue (5, i)
+					ClassIndex = CommonTables.Classes.FindValue ("ID", i)
 					if ClassIndex == FirstClassIndex:
 						Mask = 1 << i
 						continue
@@ -639,7 +532,7 @@ def IsDualClassed(actor, verbose):
 				Mask = 1 << i
 			if len(DualInfo) != 3:
 				print "WARNING: Invalid dualclass combination, treating as a single class!"
-				print DualedFrom, Class, Multi, KitIndex, DualInfo
+				print DualedFrom, Multi, KitIndex, DualInfo
 				return (0,-1,-1)
 
 			return DualInfo
@@ -668,22 +561,18 @@ def IsDualSwap (actor):
 
 	# split the full class name into its individual parts
 	# i.e FIGHTER_MAGE becomes [FIGHTER, MAGE]
-	Class = GemRB.GetPlayerStat (actor, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, Class)
-	Class = CommonTables.Classes.GetRowName (Class)
-	Class = Class.split("_")
+	Class = GetClassRowName(actor).split("_")
 
 	# get our old class name
 	if Dual[0] == 2:
-		BaseClass = CommonTables.Classes.GetRowName (Dual[1])
+		BaseClass = GetClassRowName(Dual[1], "index")
 	else:
 		BaseClass = GetKitIndex (actor)
 		BaseClass = CommonTables.KitList.GetValue (BaseClass, 7)
 		if BaseClass == "*":
 			# mod boilerplate
 			return 0
-		BaseClass = CommonTables.Classes.FindValue (5, BaseClass)
-		BaseClass = CommonTables.Classes.GetRowName (BaseClass)
+		BaseClass = GetClassRowName(BaseClass, "index")
 
 	# if our old class is the first class, we need to swap
 	if Class[0] == BaseClass:
@@ -703,8 +592,7 @@ def IsMultiClassed (actor, verbose):
 		return (0,-1,-1,-1)
 
 	# get our base class
-	ClassIndex = CommonTables.Classes.FindValue (5, GemRB.GetPlayerStat (actor, IE_CLASS))
-	IsMulti = CommonTables.Classes.GetValue (ClassIndex, 4) # 0 if not multi'd
+	IsMulti = HasMultiClassBits (actor)
 	IsDual = IsDualClassed (actor, 0)
 
 	# dual-class char's look like multi-class chars
@@ -717,7 +605,7 @@ def IsMultiClassed (actor, verbose):
 	Classes = [0]*3
 	NumClasses = 0
 	Mask = 1 # we're looking at multiples of 2
-	ClassNames = CommonTables.Classes.GetRowName(ClassIndex).split("_")
+	ClassNames = GetClassRowName(actor).split("_")
 
 	# loop through each class and test it as a mask
 	ClassCount = CommonTables.Classes.GetRowCount()
@@ -725,7 +613,7 @@ def IsMultiClassed (actor, verbose):
 		if IsMulti&Mask: # it's part of this class
 			#we need to place the classes in the array based on their order in the name,
 			#NOT the order they are detected in
-			CurrentName = CommonTables.Classes.GetRowName (CommonTables.Classes.FindValue (5, i));
+			CurrentName = GetClassRowName (i, "class")
 			if CurrentName == "*":
 				# we read too far, as the upper range limit is greater than the number of "single" classes
 				break
@@ -743,78 +631,6 @@ def IsMultiClassed (actor, verbose):
 	# return the tuple
 	return (NumClasses, Classes[0], Classes[1], Classes[2])
 
-def RemoveKnownSpells (pc, type, level1=1, level2=1, noslots=0, kit=0):
-	"""Removes all known spells of a given type between two spell levels.
-
-	If noslots is true, all memorization counts are set to 0.
-	Kit is used to identify the priest spell mask of the spells to be removed;
-	this is only used when removing spells in a dualclass."""
-
-	# choose the correct limit based upon class type
-	if type == IE_SPELL_TYPE_WIZARD:
-		limit = 9
-	elif type == IE_SPELL_TYPE_PRIEST:
-		limit = 7
-
-		# make sure that we get the original kit, if we have one
-		if kit:
-			originalkit = GetKitIndex (pc)
-
-			if originalkit: # kitted; find the class value
-				originalkit = CommonTables.KitList.GetValue (originalkit, 7)
-			else: # just get the class value
-				originalkit = GemRB.GetPlayerStat (pc, IE_CLASS)
-
-			# this is is specifically for dual-classes and will not work to remove only one
-			# spell type from a ranger/cleric multi-class
-			if CommonTables.ClassSkills.GetValue (originalkit, 0, 0) != "*": # knows druid spells
-				originalkit = 0x8000
-			elif CommonTables.ClassSkills.GetValue (originalkit, 1, 0) != "*": # knows cleric spells
-				originalkit = 0x4000
-			else: # don't know any other spells
-				originalkit = 0
-
-			# don't know how this would happen, but better to be safe
-			if originalkit == kit:
-				originalkit = 0
-	elif type == IE_SPELL_TYPE_INNATE:
-		limit = 1
-	else: # can't do anything if an improper spell type is sent
-		return 0
-
-	# make sure we're within parameters
-	if level1 < 1 or level2 > limit or level1 > level2:
-		return 0
-
-	# remove all spells for each level
-	for level in range (level1-1, level2):
-		# we need the count because we remove each spell in reverse order
-		count = GemRB.GetKnownSpellsCount (pc, type, level)
-		mod = count-1
-
-		for spell in range (count):
-			# see if we need to check for kit
-			if type == IE_SPELL_TYPE_PRIEST and kit:
-				# get the spell's ref data
-				ref = GemRB.GetKnownSpell (pc, type, level, mod-spell)
-				ref = GemRB.GetSpell (ref['SpellResRef'], 1)
-
-				# we have to look at the originalkit as well specifically for ranger/cleric dual-classes
-				# we wouldn't want to remove all cleric spells and druid spells if we lost our cleric class
-				# only the cleric ones
-				if kit&ref['SpellDivine'] or (originalkit and not originalkit&ref['SpellDivine']):
-					continue
-
-			# remove the spell
-			GemRB.RemoveSpell (pc, type, level, mod-spell)
-
-		# remove memorization counts if desired
-		if noslots:
-			GemRB.SetMemorizableSpellsCount (pc, 0, type, level)
-
-	# return success
-	return 1
-
 def CanDualClass(actor):
 	# human
 	if GemRB.GetPlayerStat (actor, IE_RACE) != 1:
@@ -827,9 +643,7 @@ def CanDualClass(actor):
 
 	DualClassTable = GemRB.LoadTable ("dualclas")
 	CurrentStatTable = GemRB.LoadTable ("abdcscrq")
-	Class = GemRB.GetPlayerStat (actor, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
+	ClassName = GetClassRowName(actor)
 	KitIndex = GetKitIndex (actor)
 	if KitIndex == 0:
 		ClassTitle = ClassName
@@ -861,10 +675,12 @@ def CanDualClass(actor):
 		return 1
 
 	AlignmentTable = GemRB.LoadTable ("alignmnt")
-	AlignsTable = GemRB.LoadTable ("aligns")
 	Alignment = GemRB.GetPlayerStat (actor, IE_ALIGNMENT)
-	AlignmentColName = AlignsTable.FindValue (3, Alignment)
-	AlignmentColName = AlignsTable.GetValue (AlignmentColName, 4)
+	AlignmentColName = CommonTables.Aligns.FindValue (3, Alignment)
+	AlignmentColName = CommonTables.Aligns.GetValue (AlignmentColName, 4)
+	if AlignmentColName == -1:
+		print "CannotDualClass: extraordinary character alignment"
+		return 1
 	Sum = 0
 	for classy in matches:
 		Sum += AlignmentTable.GetValue (classy, AlignmentColName)
@@ -885,15 +701,16 @@ def CanDualClass(actor):
 
 	# check new class' stat limitations - make sure there are any good class choices
 	TargetStatTable = GemRB.LoadTable ("abdcdsrq")
+	bad = 0
 	for match in matches:
 		ClassStatIndex = TargetStatTable.GetRowIndex (match)
 		for stat in range (6):
 			minimum = TargetStatTable.GetValue (ClassStatIndex, stat)
 			name = TargetStatTable.GetColumnName (stat)
 			if GemRB.GetPlayerStat (actor, eval ("IE_" + name[4:])) < minimum:
-				matches.remove (match)
+				bad += 1
 				break
-	if len(matches) == 0:
+	if len(matches) == bad:
 		print "CannotDualClass: no good new class choices"
 		return 1
 
@@ -904,19 +721,16 @@ def CanDualClass(actor):
 	return 0
 
 def IsWarrior (actor):
-	Class = GemRB.GetPlayerStat (actor, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
-	IsWarrior = CommonTables.ClassSkills.GetValue (ClassName, "NO_PROF")
+	IsWarrior = CommonTables.ClassSkills.GetValue (GetClassRowName(actor), "NO_PROF")
 
 	# warriors get only a -2 penalty for wielding weapons they are not proficient with
+	# FIXME: make the check more robust, someone may change the value!
 	IsWarrior = (IsWarrior == -2)
 
 	Dual = IsDualClassed (actor, 0)
 	if Dual[0] > 0:
 		DualedFrom = GemRB.GetPlayerStat (actor, IE_MC_FLAGS) & MC_WAS_ANY_CLASS
-		MCColumn = CommonTables.Classes.GetColumnIndex ("MC_WAS_ID")
-		FirstClassIndex = CommonTables.Classes.FindValue (MCColumn, DualedFrom)
+		FirstClassIndex = CommonTables.Classes.FindValue ("MC_WAS_ID", DualedFrom)
 		FirstClassName = CommonTables.Classes.GetRowName (FirstClassIndex)
 		OldIsWarrior = CommonTables.ClassSkills.GetValue (FirstClassName, "NO_PROF")
 		# there are no warrior to warrior dualclasses, so if the previous class was one, the current one certainly isn't
@@ -926,7 +740,7 @@ def IsWarrior (actor):
 
 	return IsWarrior
 
-def SetupDamageInfo (pc, Button):
+def SetupDamageInfo (pc, Button, Window):
 	hp = GemRB.GetPlayerStat (pc, IE_HITPOINTS)
 	hp_max = GemRB.GetPlayerStat (pc, IE_MAXHITPOINTS)
 	state = GemRB.GetPlayerStat (pc, IE_STATE_ID)
@@ -938,12 +752,40 @@ def SetupDamageInfo (pc, Button):
 
 	if hp < 1 or (state & STATE_DEAD):
 		Button.SetOverlay (0, 64,64,64,200, 64,64,64,200)
+
+	if ratio == 1:
+		band = 0
+		color = (255, 255, 255)  # white
+	elif ratio >= 0.75:
+		band = 1
+		color = (0, 255, 0)  # green
+	elif ratio >= 0.50:
+		band = 2
+		color = (255, 255, 0)  # yellow
+	elif ratio >= 0.25:
+		band = 3
+		color = (255, 128, 0)  # orange
 	else:
-		Button.SetOverlay (ratio, 140,0,0,205, 128,0,0,200)
+		band = 4
+		color = (255, 0, 0)  # red
+
+	if GemRB.GetVar("Old Portrait Health") or not GameIsIWD2():
+		# draw the blood overlay
+		if hp >= 1 and not (state & STATE_DEAD):
+			Button.SetOverlay (ratio, 160,0,0,200, 60,0,0,190)
+	else:
+		# scale the hp bar under the portraits and recolor it
+		# GUIHITPT has 5 frames with different severity colors
+		# luckily their ids follow a nice pattern
+		hpBar = Window.GetControl (pc-1 + 50)
+		hpBar.SetBAM ("GUIHITPT", band, 0)
+		hpBar.SetPictureClipping (ratio)
+		hpBar.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
+
 	ratio_str = "\n%d/%d" %(hp, hp_max)
 	Button.SetTooltip (GemRB.GetPlayerName (pc, 1) + ratio_str)
 
-	return ratio_str
+	return ratio_str, color
 
 def SetCurrentDateTokens (stat):
 	# NOTE: currentTime is in seconds, joinTime is in seconds * 15
@@ -967,6 +809,16 @@ def SetCurrentDateTokens (stat):
 
 	return (days, hours)
 
+# gray out window or mark it as visible depending on the actor's state
+# Always greys it out for actors that are: dead, berserking
+# The third parameter is another check which must be 0 to maintain window visibility
+def AdjustWindowVisibility (Window, pc, additionalCheck):
+	if not additionalCheck and GemRB.ValidTarget (pc, GA_SELECT|GA_NO_DEAD):
+		Window.SetVisible (WINDOW_VISIBLE)
+	else:
+		Window.SetVisible (WINDOW_GRAYED)
+	return
+
 # return ceil(n/d)
 # 
 def ceildiv (n, d):
@@ -983,3 +835,22 @@ def ResolveKey():
 
 GameWindow = GUIClasses.GWindow(0)
 GameControl = GUIClasses.GControl(0,0)
+
+class _stdioWrapper(object):
+	def __init__(self, log_level):
+		self.log_level = log_level
+		self.buffer = ""
+	def write(self, message):
+		self.buffer += str(message)
+		if self.buffer.endswith("\n"):
+			out = self.buffer.rstrip("\n")
+			if out:
+				GemRB.Log(self.log_level, "Python", out)
+			self.buffer = ""
+
+def _wrapStdio():
+	import sys
+	sys.stdout = _stdioWrapper(LOG_MESSAGE)
+	sys.stderr = _stdioWrapper(LOG_ERROR)
+
+_wrapStdio()
