@@ -25,7 +25,7 @@ IE_GUI_BUTTON_NORMAL     = 0x00000004   #default button, doesn't stick
 IE_GUI_BUTTON_NO_IMAGE   = 0x00000001
 IE_GUI_BUTTON_PICTURE    = 0x00000002
 IE_GUI_BUTTON_SOUND      = 0x00000004
-IE_GUI_BUTTON_ALT_SOUND  = 0x00000008
+IE_GUI_BUTTON_CAPS       = 0x00000008   #capitalize all the text
 IE_GUI_BUTTON_CHECKBOX   = 0x00000010   #or radio button
 IE_GUI_BUTTON_RADIOBUTTON= 0x00000020   #sticks in a state
 IE_GUI_BUTTON_DEFAULT    = 0x00000040   #enter key triggers it
@@ -45,12 +45,12 @@ IE_GUI_BUTTON_DRAGGABLE    = 0x00008000
 IE_GUI_BUTTON_NO_TEXT    = 0x00010000   # don't draw button label
 IE_GUI_BUTTON_PLAYRANDOM = 0x00020000   # the button animation is random
 IE_GUI_BUTTON_PLAYONCE   = 0x00040000   # the button animation won't restart
+IE_GUI_BUTTON_PLAYALWAYS = 0x00080000   # animation will play when game is paused
 
-IE_GUI_BUTTON_CENTER_PICTURES = 0x00080000 # center the button's PictureList
-IE_GUI_BUTTON_BG1_PAPERDOLL   = 0x00100000 # BG1-style paperdoll
-IE_GUI_BUTTON_HORIZONTAL      = 0x00200000 # horizontal clipping of overlay
-IE_GUI_BUTTON_CANCEL          = 0x00400000 # escape key triggers it
-IE_GUI_BUTTON_CAPS            = 0x00800000 # capitalize all the text (default for bg2)
+IE_GUI_BUTTON_CENTER_PICTURES = 0x00100000 # center the button's PictureList
+IE_GUI_BUTTON_BG1_PAPERDOLL   = 0x00200000 # BG1-style paperdoll
+IE_GUI_BUTTON_HORIZONTAL      = 0x00400000 # horizontal clipping of overlay
+IE_GUI_BUTTON_CANCEL          = 0x00800000 # escape key triggers it
 
 #scrollbar flags
 IE_GUI_SCROLLBAR_DEFAULT = 0x00000040   # mousewheel triggers it (same value as default button)
@@ -91,7 +91,6 @@ IE_GUI_EDIT_ON_CHANGE       = 0x03000000
 IE_GUI_EDIT_ON_DONE         = 0x03000001
 IE_GUI_EDIT_ON_CANCEL       = 0x03000002
 IE_GUI_TEXTAREA_ON_CHANGE   = 0x05000000
-IE_GUI_TEXTAREA_OUT_OF_TEXT = 0x05000001
 IE_GUI_LABEL_ON_PRESS       = 0x06000000
 IE_GUI_SCROLLBAR_ON_CHANGE  = 0x07000000
 IE_GUI_WORLDMAP_ON_PRESS    = 0x08000000
@@ -112,9 +111,9 @@ IE_GUI_BUTTON_DISABLED   = 0x00000003
 # Like DISABLED, but processes MouseOver events and draws UNPRESSED bitmap
 IE_GUI_BUTTON_LOCKED   = 0x00000004
 # Draws DISABLED bitmap, but it isn't disabled
-IE_GUI_BUTTON_THIRD    = 0x00000005
+IE_GUI_BUTTON_FAKEDISABLED    = 0x00000005
 # Draws PRESSED bitmap, but it isn't shifted
-IE_GUI_BUTTON_SECOND   = 0x00000006
+IE_GUI_BUTTON_FAKEPRESSED   = 0x00000006
 
 #edit field states
 IE_GUI_EDIT_NUMBER    =  0x030000001
@@ -220,6 +219,19 @@ IE_SPELL_TYPE_PRIEST = 0
 IE_SPELL_TYPE_WIZARD = 1
 IE_SPELL_TYPE_INNATE = 2
 
+# IWD2 spell types
+IE_IWD2_SPELL_BARD = 0
+IE_IWD2_SPELL_CLERIC = 1
+IE_IWD2_SPELL_DRUID = 2
+IE_IWD2_SPELL_PALADIN = 3
+IE_IWD2_SPELL_RANGER = 4
+IE_IWD2_SPELL_SORCERER = 5
+IE_IWD2_SPELL_WIZARD = 6
+IE_IWD2_SPELL_DOMAIN = 7
+IE_IWD2_SPELL_INNATE = 8
+IE_IWD2_SPELL_SONG = 9
+IE_IWD2_SPELL_SHAPE = 10
+
 # Item Flags bits
 # !!! Keep these synchronized with Item.h !!!
 IE_ITEM_CRITICAL     = 0x00000001
@@ -283,6 +295,7 @@ TOOLTIP_DELAY_FACTOR = 250
 #game strings
 STR_LOADMOS  = 0
 STR_AREANAME = 1
+STR_TEXTSCREEN = 2
 
 #game integers
 SV_BPP = 0
@@ -290,3 +303,19 @@ SV_WIDTH = 1
 SV_HEIGHT = 2
 global GEMRB_VERSION
 GEMRB_VERSION = -1
+
+# GUIEnhancements bits
+GE_SCROLLBARS = 1
+GE_TRY_IDENTIFY_ON_TRANSFER = 2
+GE_ALWAYS_OPEN_CONTAINER_ITEMS = 4
+
+# Log Levels
+# !!! Keep this synchronized with System/Logging.h !!!
+# no need for LOG_INTERNAL here since its internal to the logger class
+LOG_NONE = -1 # here just for the scripts, not needed in core
+LOG_FATAL = 0
+LOG_ERROR = 1
+LOG_WARNING = 2
+LOG_MESSAGE = 3
+LOG_COMBAT = 4
+LOG_DEBUG = 5
