@@ -22,6 +22,7 @@
 ###################################################
 import GemRB
 import GUICommon
+import Spellbook
 import CommonTables
 import LUCommon
 import LevelUp
@@ -139,6 +140,13 @@ def OpenRecReformPartyWindow ():
 	GemRB.SetTimedEvent (GUIWORLD.OpenReformPartyWindow, 1)
 	return
 
+#don't allow exporting polymorphed or dead characters
+def Exportable(pc):
+	if not (GemRB.GetPlayerStat (pc, IE_MC_FLAGS)&MC_EXPORTABLE): return False
+	if GemRB.GetPlayerStat (pc, IE_POLYMORPHED): return False
+	if GemRB.GetPlayerStat (pc, IE_STATE_ID)&STATE_DEAD: return False
+	return True
+
 def UpdateRecordsWindow ():
 	global stats_overview, alignment_help
 
@@ -158,7 +166,7 @@ def UpdateRecordsWindow ():
 
 	# exportable
 	Button = Window.GetControl (36)
-	if GemRB.GetPlayerStat (pc, IE_MC_FLAGS)&MC_EXPORTABLE:
+	if Exportable (pc):
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -185,7 +193,7 @@ def UpdateRecordsWindow ():
 	Button = Window.GetControl (2)
 	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PICTURE, OP_SET)
 	Button.SetState (IE_GUI_BUTTON_LOCKED)
-	if GUICommon.GameIsBG2() or GUICommon.GameIsIWD1():
+	if (GUICommon.GameIsBG2() or GUICommon.GameIsIWD1()) and not GUICommon.GameIsBG2Demo():
 		Button.SetPicture (GemRB.GetPlayerPortrait (pc,0), "NOPORTMD")
 	else:
 		Button.SetPicture (GemRB.GetPlayerPortrait (pc,0), "NOPORTLG")
@@ -193,7 +201,6 @@ def UpdateRecordsWindow ():
 	# armorclass
 	Label = Window.GetControl (0x10000028)
 	ac = GemRB.GetPlayerStat (pc, IE_ARMORCLASS)
-	ac += GemRB.GetAbilityBonus (IE_DEX, 2, GemRB.GetPlayerStat (pc, IE_DEX) )
 	Label.SetText (str (ac))
 	Label.SetTooltip (17183)
 
@@ -262,12 +269,13 @@ def UpdateRecordsWindow ():
 	Label = Window.GetControl (0x1000000f)
 	Label.SetText (text)
 
-	Table = GemRB.LoadTable ("aligns")
-
-	text = Table.GetValue (Table.FindValue ( 3, GemRB.GetPlayerStat (pc, IE_ALIGNMENT) ), 0)
+	# alignment
+	text = CommonTables.Aligns.FindValue (3, GemRB.GetPlayerStat (pc, IE_ALIGNMENT))
+	text = CommonTables.Aligns.GetValue (text, 0)
 	Label = Window.GetControl (0x10000010)
 	Label.SetText (text)
 
+	# gender
 	Label = Window.GetControl (0x10000011)
 	if GemRB.GetPlayerStat (pc, IE_SEX) == 1:
 		Label.SetText (7198)
@@ -319,6 +327,9 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		str_None = GemRB.GetString (17093)
 
 	stats = []
+	cdet = GemRB.GetCombatDetails(pc, 0)
+	tohit = cdet["ToHitStats"]
+
 	# class levels
 	# 16480 <CLASS>: Level <LEVEL>
 	# Experience: <EXPERIENCE>
@@ -327,9 +338,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	# collecting tokens for stat overview
 	ClassTitle = GUICommon.GetActorClassTitle (pc)
 	GemRB.SetToken ("CLASS", ClassTitle)
-	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, Class)
-	Class = CommonTables.Classes.GetRowName (Class)
+	Class = GUICommon.GetClassRowName (pc)
 	Dual = GUICommon.IsDualClassed (pc, 1)
 	Multi = GUICommon.IsMultiClassed (pc, 1)
 	XP = GemRB.GetPlayerStat (pc, IE_XP)
@@ -346,10 +355,9 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		stats.append ( (19721,1,'c') )
 		stats.append (None)
 		for i in range (Multi[0]):
-			ClassIndex = CommonTables.Classes.FindValue (5, Multi[i+1])
-			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (ClassIndex, 2))
+			Class = GUICommon.GetClassRowName (Multi[i+1], "class")
+			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Class, "CAP_REF"))
 			GemRB.SetToken ("CLASS", ClassTitle)
-			Class = CommonTables.Classes.GetRowName (ClassIndex)
 			GemRB.SetToken ("LEVEL", str (Levels[i]+LevelDiff[i]-int(LevelDrain/Multi[0])) )
 			GemRB.SetToken ("EXPERIENCE", str (XP/Multi[0]) )
 			if LevelDrain:
@@ -375,10 +383,10 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 		Levels[0] += LevelDiff[0]
 
-		ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Dual[2], 2))
+		Class = CommonTables.Classes.GetRowName (Dual[2])
+		ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Class, "CAP_REF"))
 		GemRB.SetToken ("CLASS", ClassTitle)
 		GemRB.SetToken ("LEVEL", str (Levels[0]-LevelDrain))
-		Class = CommonTables.Classes.GetRowName (Dual[2])
 		XP2 = GemRB.GetPlayerStat (pc, IE_XP)
 		GemRB.SetToken ("EXPERIENCE", str (XP2) )
 		if LevelDrain:
@@ -393,7 +401,8 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		if Dual[0] == 1:
 			ClassTitle = GemRB.GetString (CommonTables.KitList.GetValue (Dual[1], 2))
 		elif Dual[0] == 2:
-			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (Dual[1], 2))
+			ClassTitle = GUICommon.GetClassRowName(Dual[1], "index")
+			ClassTitle = GemRB.GetString (CommonTables.Classes.GetValue (ClassTitle, "CAP_REF"))
 		GemRB.SetToken ("CLASS", ClassTitle)
 		GemRB.SetToken ("LEVEL", str (Levels[1]) )
 
@@ -403,8 +412,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		else:
 			BaseClass = GUICommon.GetKitIndex (pc)
 			BaseClass = CommonTables.KitList.GetValue (BaseClass, 7)
-			BaseClass = CommonTables.Classes.FindValue (5, BaseClass)
-			BaseClass = CommonTables.Classes.GetRowName (BaseClass)
+			BaseClass = GUICommon.GetClassRowName (BaseClass, "class")
 		# the first class' XP is discarded and set to the minimum level
 		# requirement, so if you don't dual class right after a levelup,
 		# the game would eat some of your XP
@@ -437,7 +445,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 		effects = GemRB.GetPlayerStates (pc)
 		if len (effects):
 			for c in effects:
-				tmp = StateTable.GetValue (ord(c)-66, 0)
+				tmp = StateTable.GetValue (str(ord(c)-66), "DESCRIPTION")
 				stats.append ( (tmp,c,'a') )
 			stats.append (None)
 
@@ -445,18 +453,20 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	stats.append ( (8442,1,'c') )
 
 	# look ma, I can use both hands
+	
 	if GUICommon.GameIsBG2():
-		stats.append ( (61932, GS (IE_TOHIT), '0') )
+		stats.append ( (61932, tohit["Base"], '0') )
 		if (GemRB.IsDualWielding(pc)):
-			stats.append ( (56911, GemRB.GetCombatDetails(pc, 0)["ToHit"], '0') )
-			stats.append ( (56910, GemRB.GetCombatDetails(pc, 1)["ToHit"], '0') )
+			stats.append ( (56911, tohit["Total"], '0') ) # main
+			stats.append ( (56910, GemRB.GetCombatDetails(pc, 1)["ToHitStats"]["Total"], '0') ) # offhand
 		else:
-			stats.append ( (9457, GemRB.GetCombatDetails(pc, 0)["ToHit"], '0') )
+			stats.append ( (9457, tohit["Total"], '0') )
 	else:
-		stats.append ( (9457, str(GS (IE_TOHIT))+" ("+str(GemRB.GetCombatDetails(pc, 0)["ToHit"])+")", '0') )
-	tmp = GS (IE_NUMBEROFATTACKS)
+		stats.append ( (9457, str(tohit["Base"])+" ("+str(tohit["Total"])+")", '0') )
+
+	tmp = cdet["APR"]
 	if (tmp&1):
-		tmp2 = str (tmp/2) + chr (188)
+		tmp2 = str (tmp/2) + chr (189) #must use one higher than the frame count
 	else:
 		tmp2 = str (tmp/2)
 	stats.append ( (9458, tmp2, '') )
@@ -489,7 +499,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 	#this hack only displays LOH if we know the spell
 	#TODO: the core should just not set LOH if the paladin can't learn it
-	if (GUICommon.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
+	if (Spellbook.HasSpell (pc, IE_SPELL_TYPE_INNATE, 0, "SPCL211") >= 0):
 		stats.append ( (12127, GS (IE_LAYONHANDSAMOUNT), '') )
 
 	#script
@@ -592,27 +602,47 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 
 	# only bg2 displayed all the resistances, but it is useful information
 	# Resistances
-	if GUICommon.GameIsIWD1():
+	if GUICommon.GameIsBG2():
+		stats.append(32204)
+	elif not GUICommon.GameIsBG1():
 		stats.append (15544)
-	else:
-		stats.append (32204)
 	# 32213 Normal Fire
-	stats.append ((32213, GS (IE_RESISTFIRE), '%'))
 	# 32222 Magic Fire
-	stats.append ((32222, GS (IE_RESISTMAGICFIRE), '%'))
 	# 32214 Normal Cold
-	stats.append ((32214, GS (IE_RESISTCOLD), '%'))
 	# 32223 Magic Cold
-	stats.append ((32223, GS (IE_RESISTMAGICCOLD), '%'))
+	if GUICommon.GameIsBG2():
+		stats.append ((32213, GS (IE_RESISTFIRE), '%'))
+		stats.append ((32222, GS (IE_RESISTMAGICFIRE), '%'))
+		stats.append ((32214, GS (IE_RESISTCOLD), '%'))
+		stats.append ((32223, GS (IE_RESISTMAGICCOLD), '%'))
+	elif GUICommon.GameIsBG1():
+		stats.append ((14012, GS (IE_RESISTFIRE), '%'))
+		stats.append ((14077, GS (IE_RESISTMAGICFIRE), '%'))
+		stats.append ((14014, GS (IE_RESISTCOLD), '%'))
+		stats.append ((14078, GS (IE_RESISTMAGICCOLD), '%'))
+	else:
+		stats.append ((15545, GS (IE_RESISTFIRE), '%'))
+		stats.append ((15579, GS (IE_RESISTMAGICFIRE), '%'))
+		stats.append ((15546, GS (IE_RESISTCOLD), '%'))
+		stats.append ((15580, GS (IE_RESISTMAGICCOLD), '%'))
 	# 32220 Electricity
-	stats.append ((32220, GS (IE_RESISTELECTRICITY), '%'))
+	if GUICommon.GameIsBG1() or GUICommon.GameIsIWD1():
+		stats.append ((14013, GS (IE_RESISTELECTRICITY), '%'))
+	else:
+		stats.append ((32220, GS (IE_RESISTELECTRICITY), '%'))
 	# 32221 Acid
-	stats.append ((32221, GS (IE_RESISTACID), '%'))
+	if GUICommon.GameIsBG1():
+		stats.append ((14015, GS (IE_RESISTACID), '%'))
+	else:
+		stats.append ((32221, GS (IE_RESISTACID), '%'))
 	if GUICommon.GameIsBG2():
 		# Magic (others show it higher up)
 		stats.append ((62146, GS (IE_RESISTMAGIC), '%'))
 	# Magic Damage
-	stats.append ((32233, GS (IE_MAGICDAMAGERESISTANCE), '%'))
+	if GUICommon.GameIsIWD2():
+		stats.append ((40319, GS (IE_MAGICDAMAGERESISTANCE), '%'))
+	else: # bg2
+		stats.append ((32233, GS (IE_MAGICDAMAGERESISTANCE), '%'))
 	# Missile
 	stats.append ((11767, GS (IE_RESISTMISSILE), '%'))
 	# Slashing
@@ -628,7 +658,7 @@ def GetStatOverview (pc, LevelDiff=[0,0,0]):
 	if GUICommon.GameIsBG2():
 		# Weapon Style bonuses
 		stats.append (32131)
-		wstyle = GemRB.GetCombatDetails (pc, 0)["Style"] # equipped weapon style + 1000 * proficiency level
+		wstyle = cdet["Style"] # equipped weapon style + 1000 * proficiency level
 		profcount = wstyle / 1000
 		if profcount:
 			wstyletables = { IE_PROFICIENCY2WEAPON:"wstwowpn", IE_PROFICIENCY2HANDED:"wstwohnd", IE_PROFICIENCYSINGLEWEAPON:"wssingle", IE_PROFICIENCYSWORDANDSHIELD:"wsshield" }
@@ -719,6 +749,7 @@ def OpenInformationWindow ():
 	Button = Window.GetControl (24)
 	Button.SetText (11973)
 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseInformationWindow)
+	Button.SetFlags (IE_GUI_BUTTON_CANCEL, OP_OR)
 
 	TotalPartyExp = 0
 	ChapterPartyExp = 0
@@ -838,13 +869,12 @@ def OpenKitInfoWindow ():
 	TextArea = KitInfoWindow.GetControl (0)
 
 	pc = GemRB.GameGetSelectedPCSingle ()
-	Class = GemRB.GetPlayerStat (pc, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	Multi = CommonTables.Classes.GetValue (ClassIndex, 4)
+	ClassName = GUICommon.GetClassRowName (pc)
+	Multi = GUICommon.HasMultiClassBits (pc)
 	Dual = GUICommon.IsDualClassed (pc, 1)
 
 	if Multi and Dual[0] == 0: # true multi class
-		text = CommonTables.Classes.GetValue (ClassIndex, 1)
+		text = CommonTables.Classes.GetValue (ClassName, "DESC_REF")
 		TextArea.SetText (text)
 		KitInfoWindow.ShowModal (MODAL_SHADOW_GRAY)
 		return
@@ -856,17 +886,19 @@ def OpenKitInfoWindow ():
 		if Dual[0] == 1:
 			text = CommonTables.KitList.GetValue (Dual[1], 3)
 		elif Dual[0] == 2:
-			text = CommonTables.Classes.GetValue (Dual[1], 1)
+			text = GUICommon.GetClassRowName(Dual[1], "index")
+			text = CommonTables.Classes.GetValue (text, "DESC_REF")
 
 		TextArea.SetText (text)
 		TextArea.Append ("\n\n")
-		text = CommonTables.Classes.GetValue (Dual[2], 1)
+		text = GUICommon.GetClassRowName(Dual[2], "index")
+		text = CommonTables.Classes.GetValue (text, "DESC_REF")
 
 	else: # ordinary class or kit
 		if KitIndex:
 			text = CommonTables.KitList.GetValue (KitIndex, 3)
 		else:
-			text = CommonTables.Classes.GetValue (ClassIndex, 1)
+			text = CommonTables.Classes.GetValue (ClassName, "DESC_REF")
 
 	TextArea.Append (text)
 
