@@ -23,6 +23,7 @@ import GemRB
 import GUICommon
 import CommonTables
 from ie_stats import *
+from ie_feats import *
 
 def GetNextLevelExp (Level, Class):
 	"""Returns the amount of XP required to gain the next level."""
@@ -30,15 +31,15 @@ def GetNextLevelExp (Level, Class):
 	if Level < CommonTables.NextLevel.GetColumnCount (Row):
 		return str (CommonTables.NextLevel.GetValue (Row, Level) )
 
-	return 0
+	# we could display the current level's max, but likely nobody cares
+	# if you change it, check that all callers can handle it
+	return "0"
 
 def CanLevelUp(actor):
 	"""Returns true if the actor can level up."""
 
 	# get our class and placements for Multi'd and Dual'd characters
-	Class = GemRB.GetPlayerStat (actor, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, Class)
-	Class = CommonTables.Classes.GetRowName (Class)
+	Class = GUICommon.GetClassRowName (actor)
 	Multi = GUICommon.IsMultiClassed (actor, 1)
 	Dual = GUICommon.IsDualClassed (actor, 1)
 
@@ -47,7 +48,6 @@ def CanLevelUp(actor):
 	Levels = [GemRB.GetPlayerStat (actor, IE_LEVEL), GemRB.GetPlayerStat (actor, IE_LEVEL2),\
 		GemRB.GetPlayerStat (actor, IE_LEVEL3)]
 
-	#TODO: double-check this
 	if GemRB.GetPlayerStat(actor, IE_LEVELDRAIN)>0:
 		return 0
 
@@ -55,8 +55,8 @@ def CanLevelUp(actor):
 		xp = xp/Multi[0] # divide the xp evenly between the classes
 		for i in range (Multi[0]):
 			# if any class can level, return 1
-			ClassIndex = CommonTables.Classes.FindValue (5, Multi[i+1])
-			tmpNext = int(GetNextLevelExp (Levels[i], CommonTables.Classes.GetRowName (ClassIndex) ) )
+			TmpClassName = GUICommon.GetClassRowName (Multi[i+1], "class")
+			tmpNext = int(GetNextLevelExp (Levels[i], TmpClassName))
 			if tmpNext != 0 and tmpNext <= xp:
 				return 1
 
@@ -72,27 +72,23 @@ def CanLevelUp(actor):
 	tmpNext = int(GetNextLevelExp (Levels[0], Class) )
 	return (tmpNext != 0 and tmpNext <= xp)
 
-def SetupSavingThrows (pc, Level=None):
-	"""Updates an actors saving throws based upon level.
-
-	Level should contain the actors current level.
-	If Level is None, it is filled with the actors current level."""
-
+# internal shared function for the various Setup* stat updaters
+def _SetupLevels (pc, Level, offset=0, noclass=0):
 	#storing levels as an array makes them easier to deal with
 	if not Level:
-		Levels = [GemRB.GetPlayerStat (pc, IE_LEVEL)-1, \
-			GemRB.GetPlayerStat (pc, IE_LEVEL2)-1, \
-			GemRB.GetPlayerStat (pc, IE_LEVEL3)-1]
+		Levels = [GemRB.GetPlayerStat (pc, IE_LEVEL)+offset, \
+			GemRB.GetPlayerStat (pc, IE_LEVEL2)+offset, \
+			GemRB.GetPlayerStat (pc, IE_LEVEL3)+offset]
 	else:
 		Levels = []
 		for level in Level:
-			Levels.append (level-1)
+			Levels.append (level+offset)
 
-	#get some basic values
+	if noclass:
+		return Levels, -1, -1
+
+	# adjust the class for multi/dual chars
 	Class = [GemRB.GetPlayerStat (pc, IE_CLASS)]
-	Race = GemRB.GetPlayerStat (pc, IE_RACE)
-
-	#adjust the class for multi/dual chars
 	Multi = GUICommon.IsMultiClassed (pc, 1)
 	Dual = GUICommon.IsDualClassed (pc, 1)
 	NumClasses = 1
@@ -100,13 +96,27 @@ def SetupSavingThrows (pc, Level=None):
 		NumClasses = Multi[0]
 		Class = [Multi[1], Multi[2], Multi[3]]
 	elif Dual[0]: #only worry about the newer class
-		Class = [CommonTables.Classes.GetValue (Dual[2], 5)]
+		Class = GUICommon.GetClassRowName(Dual[2], "index")
+		Class = [CommonTables.Classes.GetValue (Class, "ID")]
 		#assume Level is correct if passed
 		if GUICommon.IsDualSwap(pc) and not Level:
 			Levels = [Levels[1], Levels[0], Levels[2]]
-	if NumClasses>len(Levels):
+
+	return Levels, NumClasses, Class
+
+def SetupSavingThrows (pc, Level=None):
+	"""Updates an actors saving throws based upon level.
+
+	Level should contain the actors current level.
+	If Level is None, it is filled with the actors current level."""
+
+	Levels, NumClasses, Class = _SetupLevels (pc, Level, -1)
+	if NumClasses > len(Levels):
 		return
 
+	#get some basic values
+	Race = GemRB.GetPlayerStat (pc, IE_RACE)
+
 	#see if we can add racial bonuses to saves
 	Race = CommonTables.Races.GetRowName (CommonTables.Races.FindValue (3, Race) )
 	RaceSaveTableName = CommonTables.Races.GetValue (Race, "SAVE", 0)
@@ -121,8 +131,7 @@ def SetupSavingThrows (pc, Level=None):
 	SaveTables = []
 	ClassBonus = 0
 	for i in range (NumClasses):
-		Row = CommonTables.Classes.FindValue (5, Class[i])
-		RowName = CommonTables.Classes.GetRowName (Row)
+		RowName = GUICommon.GetClassRowName (Class[i], "class")
 		SaveName = CommonTables.Classes.GetValue (RowName, "SAVE", 0)
 		SaveTables.append (GemRB.LoadTable (SaveName) )
 		#use numeric value
@@ -159,14 +168,14 @@ def SetupSavingThrows (pc, Level=None):
 def GetNextLevelFromExp (XP, Class):
 	"""Gets the next level based on current experience."""
 
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
+	ClassName = GUICommon.GetClassRowName (Class, "class")
 	Row = CommonTables.NextLevel.GetRowIndex (ClassName)
-	for i in range(1, CommonTables.NextLevel.GetColumnCount()-1):
+	NLNumCols = CommonTables.NextLevel.GetColumnCount()
+	for i in range(1, NLNumCols):
 		if XP < CommonTables.NextLevel.GetValue (Row, i):
 			return i
 	# fix hacked characters that have more xp than the xp cap
-	return 40
+	return NLNumCols
 
 def SetupThaco (pc, Level=None):
 	"""Updates an actors THAC0 based upon level.
@@ -174,35 +183,13 @@ def SetupThaco (pc, Level=None):
 	Level should contain the actors current level.
 	If Level is None it is filled with the actors current level."""
 
-	#storing levels as an array makes them easier to deal with
-	if not Level:
-		Levels = [GemRB.GetPlayerStat (pc, IE_LEVEL)-1, \
-			GemRB.GetPlayerStat (pc, IE_LEVEL2)-1, \
-			GemRB.GetPlayerStat (pc, IE_LEVEL3)-1]
-	else:
-		Levels = []
-		for level in Level:
-			Levels.append (level-1)
+	Levels, NumClasses, Class = _SetupLevels (pc, Level, -1)
+	if NumClasses > len(Levels):
+		return
 
 	#get some basic values
-	Class = [GemRB.GetPlayerStat (pc, IE_CLASS)]
 	ThacoTable = GemRB.LoadTable ("THAC0")
 
-	#adjust the class for multi/dual chars
-	Multi = GUICommon.IsMultiClassed (pc, 1)
-	Dual = GUICommon.IsDualClassed (pc, 1)
-	NumClasses = 1
-	if Multi[0]>1: #get each of the multi-classes
-		NumClasses = Multi[0]
-		Class = [Multi[1], Multi[2], Multi[3]]
-	elif Dual[0]: #only worry about the newer class
-		Class = [CommonTables.Classes.GetValue (Dual[2], 5)]
-		#assume Level is correct if passed
-		if GUICommon.IsDualSwap(pc) and not Level:
-			Levels = [Levels[1], Levels[0], Levels[2]]
-	if NumClasses>len(Levels):
-		return
-
 	#make sure to limit the levels to the table allowable
 	MaxLevel = ThacoTable.GetColumnCount ()-1
 	for i in range (len(Levels)):
@@ -214,7 +201,7 @@ def SetupThaco (pc, Level=None):
 	for i in range (NumClasses):
 		#loop through each class and update the save value if we have
 		#a better thac0
-		ClassName = CommonTables.Classes.GetRowName (CommonTables.Classes.FindValue (5, Class[i]))
+		ClassName = GUICommon.GetClassRowName (Class[i], "class")
 		TmpThaco = ThacoTable.GetValue (ClassName, str(Levels[i]+1))
 		if TmpThaco < CurrentThaco:
 			NewThaco = 1
@@ -233,40 +220,18 @@ def SetupLore (pc, LevelDiff=None):
 	Level and LevelDiff must be of the same length.
 	If either are None, they are filled with the actors current level."""
 
-	#storing levels as an array makes them easier to deal with
-	if not LevelDiff:
-		LevelDiffs = [GemRB.GetPlayerStat (pc, IE_LEVEL), \
-			GemRB.GetPlayerStat (pc, IE_LEVEL2), \
-			GemRB.GetPlayerStat (pc, IE_LEVEL3)]
-	else:
-		LevelDiffs = []
-		for diff in LevelDiff:
-			LevelDiffs.append (diff)
+	LevelDiffs, NumClasses, Class = _SetupLevels (pc, LevelDiff)
+	if NumClasses > len(LevelDiffs):
+		return
 
 	#get some basic values
-	Class = [GemRB.GetPlayerStat (pc, IE_CLASS)]
 	LoreTable = GemRB.LoadTable ("lore")
 
-	#adjust the class for multi/dual chars
-	Multi = GUICommon.IsMultiClassed (pc, 1)
-	Dual = GUICommon.IsDualClassed (pc, 1)
-	NumClasses = 1
-	if Multi[0]>1: #get each of the multi-classes
-		NumClasses = Multi[0]
-		Class = [Multi[1], Multi[2], Multi[3]]
-	elif Dual[0]: #only worry about the newer class
-		Class = [CommonTables.Classes.GetValue (Dual[2], 5)]
-		#if LevelDiff is passed, we assume it is correct
-		if GUICommon.IsDualSwap(pc) and not LevelDiff:
-			LevelDiffs = [LevelDiffs[1], LevelDiffs[0], LevelDiffs[2]]
-	if NumClasses>len(LevelDiffs):
-		return
-
 	#loop through each class and update the lore value if we have
 	CurrentLore = GemRB.GetPlayerStat (pc, IE_LORE, 1)
 	for i in range (NumClasses):
 		#correct unlisted progressions
-		ClassName = CommonTables.Classes.GetRowName (CommonTables.Classes.FindValue (5, Class[i]) )
+		ClassName = GUICommon.GetClassRowName (Class[i], "class")
 		if ClassName == "SORCERER":
 			ClassName = "MAGE"
 		elif ClassName == "MONK": #monks have a rate of 1, so this is arbitrary
@@ -289,30 +254,13 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 	Level and LevelDiff must be of the same length.
 	If either are None, they are filled with the actors current level."""
 
-	#storing levels as an array makes them easier to deal with
-	if not Level:
-		Levels = [GemRB.GetPlayerStat (pc, IE_LEVEL), \
-			GemRB.GetPlayerStat (pc, IE_LEVEL2), \
-			GemRB.GetPlayerStat (pc, IE_LEVEL3)]
-	else:
-		Levels = []
-		for level in Level:
-			Levels.append (level)
-	if not LevelDiff:
-		LevelDiffs = [GemRB.GetPlayerStat (pc, IE_LEVEL), \
-			GemRB.GetPlayerStat (pc, IE_LEVEL2), \
-			GemRB.GetPlayerStat (pc, IE_LEVEL3)]
-	else:
-		LevelDiffs = []
-		for diff in LevelDiff:
-			LevelDiffs.append (diff)
+	Levels, NumClasses, Class = _SetupLevels (pc, Level, noclass=1)
+	LevelDiffs, NumClasses, Class = _SetupLevels (pc, LevelDiff, noclass=1)
 	if len (Levels) != len (LevelDiffs):
 		return
 
-	#get some basic values
-	Class = [GemRB.GetPlayerStat (pc, IE_CLASS)]
-		
 	#adjust the class for multi/dual chars
+	Class = [GemRB.GetPlayerStat (pc, IE_CLASS)]
 	Multi = GUICommon.IsMultiClassed (pc, 1)
 	Dual = GUICommon.IsDualClassed (pc, 1)
 	NumClasses = 1
@@ -323,7 +271,8 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 		#we only get the hp bonus if the old class is reactivated
 		if (Levels[0]<=Levels[1]):
 			return
-		Class = [CommonTables.Classes.GetValue (Dual[2], 5)]
+		Class = GUICommon.GetClassRowName(Dual[2], "index")
+		Class = [CommonTables.Classes.GetValue (Class, "ID")]
 		#if Level and LevelDiff are passed, we assume it is correct
 		if GUICommon.IsDualSwap(pc) and not Level and not LevelDiff:
 			LevelDiffs = [LevelDiffs[1], LevelDiffs[0], LevelDiffs[2]]
@@ -338,6 +287,10 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 		if CommonTables.Classes.GetRowIndex (KitName) >= 0:
 			ClassName = KitName
 
+	# determine the minimum hp roll
+	ConBonTable = GemRB.LoadTable ("hpconbon")
+	MinRoll = ConBonTable.GetValue (GemRB.GetPlayerStat (pc, IE_CON)-1, 2) # MIN_ROLL column
+
 	#loop through each class and update the hp
 	OldHP = GemRB.GetPlayerStat (pc, IE_MAXHITPOINTS, 1)
 	CurrentHP = 0
@@ -345,7 +298,7 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 	for i in range (NumClasses):
 		#check this classes hp table for any gain
 		if not ClassName or NumClasses > 1:
-			ClassName = CommonTables.Classes.GetRowName (CommonTables.Classes.FindValue (5, Class[i]) )
+			ClassName = GUICommon.GetClassRowName (Class[i], "class")
 		HPTable = CommonTables.Classes.GetValue (ClassName, "HP")
 		HPTable = GemRB.LoadTable (HPTable)
 
@@ -367,20 +320,142 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 		#add all the hp for the given level
 		#we use ceil to ensure each class gets hp
 		for level in range(LowLevel, HiLevel):
+			sides = HPTable.GetValue (level, 0)
 			rolls = HPTable.GetValue (level, 1)
 			bonus = HPTable.GetValue (level, 2)
 
-			# we only do a roll if core diff or higher, or uncheck max
+			# we only do a roll on core difficulty or higher
+			# and if maximum HP rolls (bg2 and later) are disabled
+			# and/or if it is bg1 chargen (I guess too many testers got annoyed)
+			# BUT when we do roll, constitution gives a kind of a luck bonus to the roll
 			if rolls:
-				if GemRB.GetVar ("Difficulty Level") >= 3 and not GemRB.GetVar ("Maximum HP"):
-					CurrentHP += int (GemRB.Roll (rolls, HPTable.GetValue (level, 0), bonus) / Divisor + 0.5)
+				if GemRB.GetVar ("Difficulty Level") >= 3 and not GemRB.GetVar ("Maximum HP") \
+				and not (GUICommon.GameIsBG1() and LowLevel == 0) and MinRoll < sides:
+					if MinRoll > 1:
+						roll = GemRB.Roll (rolls, sides, bonus)
+						if roll-bonus < MinRoll:
+							roll = MinRoll + bonus
+						AddedHP = int (roll / Divisor + 0.5)
+					else:
+						AddedHP = int (GemRB.Roll (rolls, sides, bonus) / Divisor + 0.5)
 				else:
-					CurrentHP += int ((rolls * HPTable.GetValue (level, 0) + bonus) / Divisor + 0.5)
+					AddedHP = int ((rolls * sides + bonus) / Divisor + 0.5)
 			else:
-				CurrentHP += int (bonus / Divisor + 0.5)
-			CurrentHP = int (CurrentHP)
+				AddedHP = int (bonus / Divisor + 0.5)
+			# ensure atleast 1hp is given
+			# this is safe for inactive dualclass levels too (handled above)
+			if AddedHP == 0:
+				AddedHP = 1
+			CurrentHP += AddedHP
 
 	#update our hp values
 	GemRB.SetPlayerStat (pc, IE_MAXHITPOINTS, CurrentHP+OldHP)
-	GemRB.SetPlayerStat (pc, IE_HITPOINTS, GemRB.GetPlayerStat (pc, IE_HITPOINTS, 1)+CurrentHP)
+	# HACK: account also for the new constitution bonus for the current hitpoints
+	GemRB.SetPlayerStat (pc, IE_HITPOINTS, GemRB.GetPlayerStat (pc, IE_HITPOINTS, 1)+CurrentHP+5)
 	return
+
+def ApplyFeats(MyChar):
+
+	#don't mess with feats outside of IWD2
+	if not GUICommon.GameIsIWD2():
+		return
+
+	#feats giving a single innate ability
+	SetSpell(MyChar, "SPIN111", FEAT_WILDSHAPE_BOAR)
+	SetSpell(MyChar, "SPIN197", FEAT_MAXIMIZED_ATTACKS)
+	SetSpell(MyChar, "SPIN231", FEAT_ENVENOM_WEAPON)
+	SetSpell(MyChar, "SPIN245", FEAT_WILDSHAPE_PANTHER)
+	SetSpell(MyChar, "SPIN246", FEAT_WILDSHAPE_SHAMBLER)
+	SetSpell(MyChar, "SPIN275", FEAT_POWER_ATTACK)
+	SetSpell(MyChar, "SPIN276", FEAT_EXPERTISE)
+	SetSpell(MyChar, "SPIN277", FEAT_ARTERIAL_STRIKE)
+	SetSpell(MyChar, "SPIN278", FEAT_HAMSTRING)
+	SetSpell(MyChar, "SPIN279", FEAT_RAPID_SHOT)
+
+	#extra rage
+	level = GemRB.GetPlayerStat(MyChar, IE_LEVELBARBARIAN)
+	if level>0:
+		if level>=15:
+			GemRB.RemoveSpell(MyChar, "SPIN236")
+			Spell = "SPIN260"
+		else:
+			GemRB.RemoveSpell(MyChar, "SPIN260")
+			Spell = "SPIN236"
+		cnt = GemRB.GetPlayerStat (MyChar, IE_FEAT_EXTRA_RAGE)+(level+3)/4
+		GUICommon.MakeSpellCount(MyChar, Spell, cnt)
+	else:
+		GemRB.RemoveSpell(MyChar, "SPIN236")
+		GemRB.RemoveSpell(MyChar, "SPIN260")
+
+	#TODO: extra shapeshifting
+	#GUICommon.MakeSpellCount(MyChar, "", cnt)
+
+	#extra smiting
+	level = GemRB.GetPlayerStat(MyChar, IE_LEVELPALADIN)
+	if level>1:
+		cnt = GemRB.GetPlayerStat (MyChar, IE_FEAT_EXTRA_SMITING) + 1
+		GUICommon.MakeSpellCount(MyChar, "SPIN152", cnt)
+	else:
+		GemRB.RemoveSpell(MyChar, "SPIN152")
+
+	#extra turning
+	level = GemRB.GetPlayerStat(MyChar, IE_TURNUNDEADLEVEL)
+	if level>0:
+		cnt = GUICommon.GetAbilityBonus(MyChar, IE_CHR) + 3
+		if cnt<1: cnt = 1
+		cnt += GemRB.GetPlayerStat (MyChar, IE_FEAT_EXTRA_TURNING)
+		GUICommon.MakeSpellCount(MyChar, "SPIN970", cnt)
+	else:
+		GemRB.RemoveSpell(MyChar, "SPIN970")
+
+	#stunning fist
+	if GemRB.HasFeat (MyChar, FEAT_STUNNING_FIST):
+		cnt = GemRB.GetPlayerStat(MyChar, IE_CLASSLEVELSUM) / 4
+		GUICommon.MakeSpellCount(MyChar, "SPIN232", cnt)
+	else:
+		GemRB.RemoveSpell(MyChar, "SPIN232")
+
+	#remove any previous SPLFOCUS
+	#GemRB.ApplyEffect(MyChar, "RemoveEffects",0,0,"SPLFOCUS")
+	#spell focus stats
+	SPLFocusTable = GemRB.LoadTable ("splfocus")
+	for i in range(SPLFocusTable.GetRowCount()):
+		Row = SPLFocusTable.GetRowName(i)
+		Stat = SPLFocusTable.GetValue(Row, "STAT", 2)
+		if Stat:
+			Column = GemRB.GetPlayerStat(MyChar, Stat)
+			if Column:
+				Value = SPLFocusTable.GetValue(i, Column)
+				if Value:
+					#add the effect, value could be 2 or 4, timing mode is 8 - so it is not saved
+					GemRB.ApplyEffect(MyChar, "SpellFocus", Value, i,"","","","SPLFOCUS", 8)
+	return
+
+def SetSpell(pc, SpellName, Feat):
+	if GemRB.HasFeat (pc, Feat):
+		GUICommon.MakeSpellCount(pc, SpellName, 1)
+	else:
+		GemRB.RemoveSpell(pc, SpellName)
+	return
+
+def AddPlayerStat(MyChar, stat, value):
+        value += GemRB.GetPlayerStat (MyChar, stat, 0)
+	#print "Set: ", stat," to ", value
+        GemRB.SetPlayerStat (MyChar, stat, value)
+        return
+
+def SetClassResistances(MyChar, clsstitle):
+        resistances = GemRB.LoadTable ("clssrsmd")
+        AddPlayerStat (MyChar, IE_RESISTFIRE, resistances.GetValue ( clsstitle, "FIRE") )
+        AddPlayerStat (MyChar, IE_RESISTCOLD, resistances.GetValue ( clsstitle, "COLD") )
+        AddPlayerStat (MyChar, IE_RESISTELECTRICITY, resistances.GetValue ( clsstitle, "ELEC") )
+        AddPlayerStat (MyChar, IE_RESISTACID, resistances.GetValue ( clsstitle, "ACID") )
+        AddPlayerStat (MyChar, IE_RESISTMAGIC, resistances.GetValue ( clsstitle, "SPELL") )
+        AddPlayerStat (MyChar, IE_RESISTMAGICFIRE, resistances.GetValue ( clsstitle, "MAGIC_FIRE") )
+        AddPlayerStat (MyChar, IE_RESISTMAGICCOLD, resistances.GetValue ( clsstitle, "MAGIC_COLD") )
+        AddPlayerStat (MyChar, IE_RESISTSLASHING, resistances.GetValue ( clsstitle, "SLASHING") )
+        AddPlayerStat (MyChar, IE_RESISTCRUSHING, resistances.GetValue ( clsstitle, "BLUDGEONING") )
+        AddPlayerStat (MyChar, IE_RESISTPIERCING, resistances.GetValue ( clsstitle, "PIERCING") )
+        AddPlayerStat (MyChar, IE_RESISTMISSILE, resistances.GetValue ( clsstitle, "MISSILE") )
+        return
+
