@@ -25,69 +25,94 @@ import GUICommon
 TextScreen = None
 TextArea = None
 Row = 1
-Position = 1
-Chapter = 0
 TableName = None
-BGTICKS = 300 # TODO: verify for suitability
-Ticks = IWDTICKS = 200 # TODO: verify for suitability in iwd1
+
+def FindTextRow (Table):
+	global Row
+
+	#this is still not the full implementation, but the original engine never used this stuff
+	Row = Table.GetRowIndex("DEFAULT")
+	if Table.GetValue (Row, 1)== -1:
+		if GemRB.GameGetReputation() >= 100:
+			Row = Table.GetRowIndex("GOOD_REPUTATION")
+		else:
+			Row = Table.GetRowIndex("BAD_REPUTATION")
+	return
 
 def StartTextScreen ():
-	global TextScreen, TextArea, Chapter, TableName, Row, Ticks
+	global TextScreen, TextArea, TableName, Row
+
+	GemRB.GamePause (1, 3)
+
+	ID = -1
+	MusicName = "*"
+	Message = 17556   # default: Paused for chapter text
+	TableName = GemRB.GetGameString (STR_TEXTSCREEN)
+
+	#iwd2/bg2 has no separate music
+	if GUICommon.GameIsIWD1():
+		if TableName == "":
+			MusicName = "chap0"
+		else:
+			MusicName = "chap1"
+		TableName = "chapters"
+	elif GUICommon.GameIsIWD2():
+		TableName = "chapters"
+
+	if TableName == "":
+		EndTextScreen ()
+		return
+	else:
+		TextTable = GemRB.LoadTable ("textscrn", 1)
+		if TextTable != None:
+			TxtRow = TextTable.GetRowIndex (TableName)
+			if TxtRow >= 0:
+				ID = TextTable.GetValue (TxtRow, 0)
+				MusicName = TextTable.GetValue (TxtRow, 1)
+				Message = TextTable.GetValue (TxtRow, 2)
+
+	if Message != "*":
+		GemRB.DisplayString (Message, 0xff0000)
 
 	if GUICommon.GameIsIWD2():
 		GemRB.LoadWindowPack ("GUICHAP", 800, 600)
 	else:
 		GemRB.LoadWindowPack ("GUICHAP", 640, 480)
-		if GUICommon.GameIsBG1() or GUICommon.GameIsBG2():
-			Ticks = BGTICKS
 
-	LoadPic = TableName = GemRB.GetGameString (STR_LOADMOS)
-	#if there is no preset loadpic, try to determine it from the chapter
-	#fixme: we always assume there isn't for non-bg2
 	if GUICommon.GameIsBG2():
-		if TableName == "":
-			Chapter = GemRB.GetGameVar ("CHAPTER") & 0x7fffffff
-			TableName = "CHPTXT"+str(Chapter)
 		ID = 62
-	else:
+	elif ID == -1:
+		#default: try to determine ID from current chapter
 		ID = GemRB.GetGameVar("CHAPTER") & 0x7fffffff
 		Chapter = ID + 1
 
-	#iwd2/bg2 has no separate music
-	if GUICommon.GameIsIWD1():
-		if LoadPic == "":
-			GemRB.LoadMusicPL ("chap0.mus")
-		else:
-			GemRB.LoadMusicPL ("chap1.mus")
-		TableName = "chapters"
-	elif GUICommon.GameIsIWD2():
+	if MusicName != "*":
+		GemRB.LoadMusicPL (MusicName + ".mus")
+	else:
 		GemRB.HardEndPL ()
-		TableName = "chapters"
-	elif GUICommon.GameIsBG1() and TableName[:6] == "chptxt":
-		GemRB.LoadMusicPL ("chapter.mus")
 
 	TextScreen = GemRB.LoadWindow (ID)
 	TextScreen.SetFrame ()
 
 	TextArea = TextScreen.GetControl (2)
 	TextArea.SetFlags (IE_GUI_TEXTAREA_SMOOTHSCROLL)
-	TextArea.SetEvent (IE_GUI_TEXTAREA_OUT_OF_TEXT, FeedScroll)
 
-	#caption
 	Table = GemRB.LoadTable (TableName)
 	if GUICommon.GameIsBG1():
 		#these suckers couldn't use a fix row
-		Row = Table.GetRowIndex("DEFAULT")
-		Value = Table.GetValue (Row, 0)
+		FindTextRow (Table)
 	elif GUICommon.GameIsBG2():
 		LoadPic = Table.GetValue (-1, -1)
 		if LoadPic != "":
 			TextScreen.SetPicture (LoadPic)
-		# TODO: verify
-		Value = 0
+		FindTextRow (Table)
 	else:
-		Value = Table.GetValue (Chapter, 0)
-	if Value:
+		Row = Chapter
+
+	#caption
+	Value = Table.GetValue (Row, 0)
+	#don't display the fake -1 string (No caption in toscst.2da)
+	if Value!="NONE" and Value>0 and TextScreen.HasControl(0x10000000):
 		Label=TextScreen.GetControl (0x10000000)
 		Label.SetText (Value)
 
@@ -102,55 +127,44 @@ def StartTextScreen ():
 	Button.SetText (16510)
 	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ReplayTextScreen)
 
+	#if this was opened from somewhere other than game control close that window
+	GUICommon.CloseOtherWindow(None)
 	GemRB.HideGUI ()
 	GUICommon.GameWindow.SetVisible(WINDOW_INVISIBLE) #removing the gamecontrol screen
 	TextScreen.SetVisible (WINDOW_VISIBLE)
 
-	TextArea.Rewind (Ticks)
-	GemRB.DisplayString (17556, 0xff0000) #Paused for chapter text
-	GemRB.GamePause (1, 1)
-	return
-
-def FeedScroll ():
-	global TextArea, Position
-
-	Table = GemRB.LoadTable (TableName)
-	if GUICommon.GameIsBG2():
-		#this is a rather primitive selection but works for the games
-		Value = Table.GetValue (0, 1)
-		if Value == "REPUTATION":
-			line = 2
-		else:
-			line = 1
-		Value = Table.GetValue (line, 1)
-	elif GUICommon.GameIsBG1():
-		Value = Table.GetValue (Row, Position)
-		if Value == 'NONE':
-			Position = 1
-		else:
-			Position = Position + 1
-	else:
-		Value = Table.GetValue (Chapter, 1)
-
-	TextArea.Append (Value, -1, 6)
+	ReplayTextScreen()
 	return
 
 def EndTextScreen ():
 	global TextScreen
 
-	TextScreen.SetVisible (WINDOW_INVISIBLE)
 	if TextScreen:
+		TextScreen.SetVisible (WINDOW_INVISIBLE)
 		TextScreen.Unload ()
 		GemRB.PlaySound(None, 0, 0, 4)
+
 	GUICommon.GameWindow.SetVisible(WINDOW_VISIBLE) # enable the gamecontrol screen
 	GemRB.UnhideGUI ()
-	GemRB.GamePause (0, 1)
+	GemRB.GamePause (0, 3)
 	return
 
 def ReplayTextScreen ():
-	global TextArea, Position
+	global TextArea, TableName, Row
+
+	TextArea.Rewind ()
+
+	Table = GemRB.LoadTable (TableName)
+	Count = Table.GetColumnCount (Row)
+
+	#hack for messy chapters.2da in IWD
+	#note: iwd doesn't use TextScreen, only IncrementChapter
+	if GUICommon.GameIsIWD1() or GUICommon.GameIsIWD2():
+		Count = 2
+
+	for i in range(1, Count):
+		TextArea.Append ("\n")
+		# flag value of 14 = IE_STR_SOUND|IE_STR_SPEECH/GEM_SND_SPEECH|GEM_SND_QUEUE
+		TextArea.Append (Table.GetValue (Row, i), -1, 14)
 
-	Position = 1
-	TextArea.SetEvent (IE_GUI_TEXTAREA_OUT_OF_TEXT, FeedScroll)
-	TextArea.Rewind (Ticks)
 	return
