@@ -1,8 +1,10 @@
 import GemRB
 from GUIDefines import *
 from ie_stats import *
+from ie_restype import RES_2DA
 import CharGenCommon
 import GUICommon
+import Spellbook
 import CommonTables
 import LUCommon
 import LUProfsSelection
@@ -83,15 +85,13 @@ def unsetAlignment():
 	GemRB.SetPlayerStat (MyChar, IE_ALIGNMENT,0)
 	
 def getAlignment(area):
-	AlignmentTable = GemRB.LoadTable("aligns")
-	
 	MyChar = GemRB.GetVar ("Slot")
 	AllignID = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
 	
 	area.Append(1049, -1)
 	area.Append(": ")
-	AllignIndex = AlignmentTable.FindValue(3,AllignID)
-	AllignCap = AlignmentTable.GetValue(AllignIndex,2)
+	AllignIndex = CommonTables.Aligns.FindValue (3, AllignID)
+	AllignCap = CommonTables.Aligns.GetValue (AllignIndex, 2)
 	area.Append(AllignCap)
 	area.Append("\n")
 
@@ -126,8 +126,7 @@ def unsetHateRace():
 
 def guardHateRace():
 	MyChar = GemRB.GetVar ("Slot")
-	Class = GemRB.GetPlayerStat(MyChar,IE_CLASS)
-	ClassName = CommonTables.ClassSkills.GetRowName(Class)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	TableName = CommonTables.ClassSkills.GetValue(ClassName, "HATERACE")
 	return TableName != "*"
 
@@ -148,12 +147,12 @@ def unsetMageSpells():
 	print("unsetMageSpells")
 	MyChar = GemRB.GetVar ("Slot")
 
-	GUICommon.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_WIZARD, 1, 5, 1)
+	Spellbook.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_WIZARD, 1, 5, 1)
 
 def guardMageSpells():
 	MyChar = GemRB.GetVar ("Slot")
-	Class = GemRB.GetPlayerStat(MyChar,IE_CLASS)
-	TableName = CommonTables.ClassSkills.GetValue(Class, 2)
+	ClassName = GUICommon.GetClassRowName (MyChar)
+	TableName = CommonTables.ClassSkills.GetValue(ClassName, "MAGESPELL")
 	return TableName != "*"
 
 def getMageSpells(TextAreaControl):
@@ -180,9 +179,7 @@ def guardSkills():
 	if Kit != 0: # luckily the first row is a dummy
 		KitName = CommonTables.KitList.GetValue(Kit, 0) #rowname is just a number
 	else:
-		ClassID = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-		ClassIndex = CommonTables.Classes.FindValue(5,ClassID)
-		KitName = CommonTables.Classes.GetRowName(ClassIndex)
+		KitName = GUICommon.GetClassRowName (MyChar)
 
 	for i in range(RowCount):
 		SkillName = SkillTable.GetRowName(i+2)
@@ -202,9 +199,7 @@ def getSkills(TextAreaControl):
 	# thieving and other skills
 	info = ""
 	SkillTable = GemRB.LoadTable ("skills")
-	ClassID = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, ClassID)
-	ClassName = CommonTables.Classes.GetRowName(Class)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	RangerSkills = CommonTables.ClassSkills.GetValue (ClassName, "RANGERSKILL")
 	BardSkills = CommonTables.ClassSkills.GetValue (ClassName, "BARDSKILL")
 	KitName = GUICommon.GetKitIndex (MyChar)
@@ -281,14 +276,12 @@ def getName(TextAreaControl):
 def setDivineSpells():
 	MyChar = GemRB.GetVar ("Slot")
 	
-	ClassID = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, ClassID)
-	ClassName = CommonTables.Classes.GetRowName(Class)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	
 	DruidTable = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL")
 	ClericTable = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL")
 	
-	print("CGG setDivineSpells: CP1",ClassID,Class,ClassName,DruidTable,ClericTable)
+	print("CGG setDivineSpells: CP1", ClassName, DruidTable, ClericTable)
 	
 	AllignID = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
 	
@@ -301,18 +294,18 @@ def setDivineSpells():
 
 def learnDivine(MyChar,ClassFlag,TableName,AllignID):
 	#print ("CGG setDivineSpells: CP2",MyChar,ClassFlag,TableName,AllignID )
-	GUICommon.SetupSpellLevels(MyChar, TableName, IE_SPELL_TYPE_PRIEST, 1)
-	Learnable = GUICommon.GetLearnablePriestSpells( ClassFlag, AllignID, 1)
+	Spellbook.SetupSpellLevels(MyChar, TableName, IE_SPELL_TYPE_PRIEST, 1)
+	Learnable = Spellbook.GetLearnablePriestSpells( ClassFlag, AllignID, 1)
 	for i in range(len(Learnable) ):
 		#print ("CGG setDivineSpells: CP3",Learnable[i])
-		if -1 == GUICommon.HasSpell(MyChar,IE_SPELL_TYPE_PRIEST,1,Learnable[i]):
+		if -1 == Spellbook.HasSpell(MyChar,IE_SPELL_TYPE_PRIEST,1,Learnable[i]):
 			#print ("CGG setDivineSpells: CP4",Learnable[i])
 			GemRB.LearnSpell (MyChar, Learnable[i], 0)
 
 def unsetDivineSpells():
 	print("unsetDivineSpells")
 	MyChar = GemRB.GetVar ("Slot")
-	GUICommon.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_PRIEST, 1, 1, 1)
+	Spellbook.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_PRIEST, 1, 1, 1)
 
 def getDivineSpells(TextAreaControl):
 	MyChar = GemRB.GetVar ("Slot")
@@ -333,15 +326,12 @@ def setAccept():
 	#set my character up
 	MyChar = GemRB.GetVar ("Slot")
 
-	ClassID = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, ClassID)
-	ClassName = CommonTables.Classes.GetRowName(Class)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	
 	#reputation
 	AllignID = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
 	
 	TmpTable=GemRB.LoadTable ("repstart")
-	AlignmentTable = GemRB.LoadTable ("aligns")
 	t = TmpTable.GetValue (AllignID,0) * 10
 	GemRB.SetPlayerStat (MyChar, IE_REPUTATION, t)
 
@@ -377,6 +367,27 @@ def setAccept():
 	#10 is a weapon slot (see slottype.2da row 10)
 	GemRB.CreateItem (MyChar, "staf01", 10, 1, 0, 0)
 	GemRB.SetEquippedQuickSlot (MyChar, 0)
+
+	# apply class/kit abilities
+	IsMulti = GUICommon.IsMultiClassed (MyChar, 1)
+	Levels = [GemRB.GetPlayerStat (MyChar, IE_LEVEL), GemRB.GetPlayerStat (MyChar, IE_LEVEL2), \
+			GemRB.GetPlayerStat (MyChar, IE_LEVEL3)]
+	KitIndex = GUICommon.GetKitIndex (MyChar)
+	if IsMulti[0]>1:
+		#get the class abilites for each class
+		for i in range (IsMulti[0]):
+			TmpClassName = GUICommon.GetClassRowName (IsMulti[i+1], "class")
+			ABTable = CommonTables.ClassSkills.GetValue (TmpClassName, "ABILITIES")
+			if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
+				GUICommon.AddClassAbilities (MyChar, ABTable, Levels[i], Levels[i])
+	else:
+		if KitIndex:
+			ABTable = CommonTables.KitList.GetValue (str(KitIndex), "ABILITIES")
+		else:
+			ABTable = CommonTables.ClassSkills.GetValue (ClassName, "ABILITIES")
+		if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
+			GUICommon.AddClassAbilities (MyChar, ABTable, Levels[0], Levels[0])
+
 	#LETS PLAY!!
 	playmode = GemRB.GetVar ("PlayMode")
 	
