@@ -19,6 +19,7 @@
 # character generation - ability; next skills/profs/spells (CharGen6)
 import GemRB
 import GUICommon
+import Spellbook
 import CommonTables
 import CharGenCommon
 import LUSkillsSelection
@@ -48,17 +49,17 @@ def OnLoad():
 	print "CharGen6 output:"
 
 	#remove all known spells and nullify the memorizable counts
-	GUICommon.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_WIZARD, 1,9, 1)
-	GUICommon.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_PRIEST, 1,7, 1)
+	Spellbook.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_WIZARD, 1,9, 1)
+	Spellbook.RemoveKnownSpells (MyChar, IE_SPELL_TYPE_PRIEST, 1,7, 1)
 
 	# learn divine spells if appropriate
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	TableName = CommonTables.ClassSkills.GetValue (Class, 1, 0) # cleric spells
+	ClassName = GUICommon.GetClassRowName (MyChar)
+	TableName = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL", 0) # cleric spells
 
 	if TableName == "*": # only druid spells or no spells at all
-		TableName = CommonTables.ClassSkills.GetValue (Class, 0, 0)
+		TableName = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL", 0)
 		ClassFlag = 0x8000
-	elif CommonTables.ClassSkills.GetValue (Class, 0, 0) != "*": # cleric and druid spells
+	elif CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL", 0) != "*": # cleric and druid spells
 		ClassFlag = 0
 	else: # only cleric spells
 		ClassFlag = 0x4000
@@ -77,7 +78,7 @@ def OnLoad():
 			#be looked at first in Cleric/Ranger multi's, which is correct
 			foundindex = 0
 			for i in range (IsMulti[0]):
-				ClassName = CommonTables.Classes.GetRowName (CommonTables.Classes.FindValue (5, IsMulti[i+1]) )
+				ClassName = GUICommon.GetClassRowName (IsMulti[i+1], "class")
 				for table in "CLERICSPELL", "DRUIDSPELL":
 					if CommonTables.ClassSkills.GetValue (ClassName, table) != "*":
 						index = i
@@ -87,12 +88,12 @@ def OnLoad():
 					break
 
 		#set our memorizable counts
-		GUICommon.SetupSpellLevels (MyChar, TableName, IE_SPELL_TYPE_PRIEST, Levels[index])
+		Spellbook.SetupSpellLevels (MyChar, TableName, IE_SPELL_TYPE_PRIEST, Levels[index])
 
 		#learn all our priest spells up to the level we can learn
 		for level in range (8):
 			if GemRB.GetMemorizableSpellsCount (MyChar, IE_SPELL_TYPE_PRIEST, level, 0) <= 0:
-				GUICommon.LearnPriestSpells (MyChar, level, ClassFlag)
+				Spellbook.LearnPriestSpells (MyChar, level, ClassFlag)
 				break
 	CharGenCommon.DisplayOverview (6)
 	return
