@@ -19,6 +19,7 @@
 # character generation end
 import GemRB
 import GUICommon
+import Spellbook
 import CommonTables
 import LUCommon
 from GUIDefines import *
@@ -31,9 +32,7 @@ def OnLoad():
 	# Lay on hands, turn undead and backstab multiplier get set by the core
 	# set my character up
 	MyChar = GemRB.GetVar ("Slot")
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	IsMulti = GUICommon.IsMultiClassed (MyChar, 1)
 	Levels = [GemRB.GetPlayerStat (MyChar, IE_LEVEL), GemRB.GetPlayerStat (MyChar, IE_LEVEL2), \
 			GemRB.GetPlayerStat (MyChar, IE_LEVEL3)]
@@ -52,23 +51,24 @@ def OnLoad():
 	LUCommon.SetupHP (MyChar)
 
 	# mage spells
-	TableName = CommonTables.ClassSkills.GetValue (Class, 2, 0)
+	TableName = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL", 0)
 	if TableName != "*":
 		index = 0
 		if IsMulti[0]>1:
 			#find out which class gets mage spells
 			for i in range (IsMulti[0]):
-				if CommonTables.ClassSkills.GetValue (IsMulti[i+1], 2, 0) != "*":
+				TmpClassName = GUICommon.GetClassRowName (IsMulti[i+1], "class")
+				if CommonTables.ClassSkills.GetValue (TmpClassName, "MAGESPELL", 0) != "*":
 					index = i
 					break
-		GUICommon.SetupSpellLevels(MyChar, TableName, IE_SPELL_TYPE_WIZARD, Levels[index])
+		Spellbook.SetupSpellLevels(MyChar, TableName, IE_SPELL_TYPE_WIZARD, Levels[index])
 
 	# apply class/kit abilities
 	KitIndex = GUICommon.GetKitIndex (MyChar)
 	if IsMulti[0]>1:
 		#get the class abilites for each class
 		for i in range (IsMulti[0]):
-			TmpClassName = CommonTables.Classes.GetRowName (CommonTables.Classes.FindValue (5, IsMulti[i+1]) )
+			TmpClassName = GUICommon.GetClassRowName (IsMulti[i+1], "class")
 			ABTable = CommonTables.ClassSkills.GetValue (TmpClassName, "ABILITIES")
 			if ABTable != "*" and GemRB.HasResource (ABTable, RES_2DA, 1):
 				GUICommon.AddClassAbilities (MyChar, ABTable, Levels[i], Levels[i])
@@ -82,14 +82,13 @@ def OnLoad():
 
 	# apply starting (alignment dictated) abilities
 	# pc, table, new level, level diff, alignment
-	AlignmentTable = GemRB.LoadTable ("aligns")
-	AlignmentAbbrev = AlignmentTable.FindValue (3, GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT))
+	AlignmentAbbrev = CommonTables.Aligns.FindValue (3, GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT))
 	GUICommon.AddClassAbilities (MyChar, "abstart", 6,6, AlignmentAbbrev)
 
 	# setup starting gold (uses a roll dictated by class
 	TmpTable = GemRB.LoadTable ("strtgold")
-	temp = GemRB.Roll (TmpTable.GetValue (Class, 1),TmpTable.GetValue (Class, 0), TmpTable.GetValue (Class, 2))
-	GemRB.SetPlayerStat (MyChar, IE_GOLD, temp * TmpTable.GetValue (Class, 3))
+	temp = GemRB.Roll (TmpTable.GetValue (ClassName, "ROLLS"), TmpTable.GetValue (ClassName, "SIDES"), TmpTable.GetValue (ClassName, "MODIFIER"))
+	GemRB.SetPlayerStat (MyChar, IE_GOLD, temp * TmpTable.GetValue (ClassName, "MULTIPLIER"))
 
 	# save the appearance
 	GUICommon.SetColorStat (MyChar, IE_HAIR_COLOR, GemRB.GetVar ("HairColor") )
@@ -110,8 +109,11 @@ def OnLoad():
 	GemRB.FillPlayerInfo (MyChar, LargePortrait, SmallPortrait)
 
 	if GUICommon.GameIsTOB():
-		# add the starting inventory for tob
-		GiveEquipment(MyChar, ClassName, KitIndex)
+		# will also add the starting inventory for tob
+		GemRB.GameSetExpansion (4)
+		# no torture, let's refresh all the spells, at least for sorcerers
+		# TODO: autopick memorisations for mages? Did they have a memorisation choice step like in bg1?
+		GemRB.ChargeSpells (MyChar)
 
 	playmode = GemRB.GetVar ("PlayMode")
 	if playmode >=0:
@@ -121,9 +123,6 @@ def OnLoad():
 		GemRB.EnterGame()
 		GemRB.ExecuteString ("EquipMostDamagingMelee()", MyChar)
 	else:
-		#leaving multi player pregen
-		if CharGenWindow:
-			CharGenWindow.Unload ()
 		#when export is done, go to start
 		if GUICommon.HasTOB():
 			GemRB.SetToken ("NextScript","Start2")
