@@ -20,7 +20,9 @@
 import GemRB
 from GUIDefines import *
 from ie_stats import *
+from ie_spells import LS_MEMO
 import GUICommon
+import Spellbook
 import CommonTables
 
 # HLA selection
@@ -43,7 +45,7 @@ def OpenHLAWindow (actor, numclasses, classes, levels):
 	global EnhanceGUI
 
 	#enhance GUI?
-	if (GemRB.GetVar("GUIEnhancements")):
+	if (GemRB.GetVar("GUIEnhancements")&GE_SCROLLBARS):
 		EnhanceGUI = 1
 
 	# save our variables 
@@ -124,9 +126,9 @@ def HLADonePress ():
 			GemRB.ApplySpell(pc, HLARef[3:])
 		elif HLARef[:2] == "GA":
 			# make sure it isn't already learned
-			SpellIndex = GUICommon.HasSpell (pc, HLAType, HLALevel, HLARef[3:])
+			SpellIndex = Spellbook.HasSpell (pc, HLAType, HLALevel, HLARef[3:])
 			if SpellIndex < 0: # gotta learn it
-				GemRB.LearnSpell (pc, HLARef[3:], 8)
+				GemRB.LearnSpell (pc, HLARef[3:], LS_MEMO)
 			else: # memorize it again
 				GemRB.MemorizeSpell (pc, HLAType, HLALevel, SpellIndex)
 
@@ -310,8 +312,7 @@ def GetHLAs ():
 
 	# get all the HLAs for each class
 	for i in range (NumClasses):
-		ClassIndex = CommonTables.Classes.FindValue (5, Classes[i])
-		ClassName = CommonTables.Classes.GetRowName (ClassIndex)
+		ClassName = GUICommon.GetClassRowName (Classes[i], "class")
 		CurrentLevel = Level[i]
 
 		if Kit != 0 and NumClasses == 1 and not IsDual: # kitted single-class
