@@ -27,6 +27,7 @@ from GUIDefines import *
 from ie_stats import *
 from ie_spells import LS_MEMO
 import GUICommon
+import Spellbook
 import CommonTables
 import LUSkillsSelection
 
@@ -66,7 +67,6 @@ KitDoneButton = 0
 
 AlignmentButton = 0
 AlignmentWindow = 0
-AlignmentTable = 0
 AlignmentTextArea = 0
 AlignmentDoneButton = 0
 
@@ -149,18 +149,18 @@ SoundIndex = 0
 VerbalConstants = None
 HasStrExtra = 0
 MyChar = 0
+ImportedChar = 0
 
 def OnLoad():
 	global CharGenWindow, CharGenState, TextArea, PortraitButton, AcceptButton
 	global GenderButton, RaceButton, ClassButton, AlignmentButton
 	global AbilitiesButton, SkillsButton, AppearanceButton, BiographyButton, NameButton
-	global KitTable, ProficienciesTable, AlignmentTable, RacialEnemyTable
+	global KitTable, ProficienciesTable, RacialEnemyTable
 	global AbilitiesTable, SkillsTable, PortraitsTable
-	global MyChar
+	global MyChar, ImportedChar
 
 	KitTable = GemRB.LoadTable ("magesch")
 	ProficienciesTable = GemRB.LoadTable ("weapprof")
-	AlignmentTable = GemRB.LoadTable ("aligns")
 	RacialEnemyTable = GemRB.LoadTable ("haterace")
 	AbilitiesTable = GemRB.LoadTable ("ability")
 	SkillsTable = GemRB.LoadTable ("skills")
@@ -170,6 +170,7 @@ def OnLoad():
 	CharGenWindow.SetFrame ()
 	CharGenState = 0
 	MyChar = GemRB.GetVar ("Slot")
+	ImportedChar = 0
 
 	GenderButton = CharGenWindow.GetControl (0)
 	GenderButton.SetState (IE_GUI_BUTTON_ENABLED)
@@ -313,27 +314,30 @@ def AcceptPress():
 	#mage spells
 	Kit = GemRB.GetPlayerStat (MyChar, IE_KIT)
 	KitIndex = KitTable.FindValue (3, Kit)
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	t = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
-	TableName = CommonTables.ClassSkills.GetValue (Class, 2, 0)
+	TableName = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL", 0)
 	if TableName != "*":
 		#todo: set up ALL spell levels not just level 1
-		GUICommon.SetupSpellLevels (MyChar, TableName, IE_SPELL_TYPE_WIZARD, 1)
-		Learnable = GUICommon.GetLearnableMageSpells (KitIndex, t, 1)
+		Spellbook.SetupSpellLevels (MyChar, TableName, IE_SPELL_TYPE_WIZARD, 1)
+		Learnable = Spellbook.GetLearnableMageSpells (KitIndex, t, 1)
 		SpellBook = GemRB.GetVar ("MageSpellBook")
 		MemoBook = GemRB.GetVar ("MageMemorized")
 		j=1
 		for i in range (len(Learnable) ):
 			if SpellBook & j:
 				if MemoBook & j:
-					memorize = 8
+					memorize = LS_MEMO
 				else:
 					memorize = 0
 				GemRB.LearnSpell (MyChar, Learnable[i], memorize)
 			j=j<<1
 
 	#priest spells
-	TableName = CommonTables.ClassSkills.GetValue (Class, 1, 0)
+	TableName = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL", 0)
+	# druids and rangers have a column of their own
+	if TableName == "*":
+		TableName = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL", 0)
 	if TableName != "*":
 		if TableName == "MXSPLPRS" or TableName == "MXSPLPAL":
 			ClassFlag = 0x8000
@@ -346,21 +350,26 @@ def AcceptPress():
 		else:
 			ClassFlag = 0
 		#todo: set up ALL spell levels not just level 1
-		GUICommon.SetupSpellLevels (MyChar, TableName, IE_SPELL_TYPE_PRIEST, 1)
-		Learnable = GUICommon.GetLearnablePriestSpells (ClassFlag, t, 1)
+		Spellbook.SetupSpellLevels (MyChar, TableName, IE_SPELL_TYPE_PRIEST, 1)
+		Learnable = Spellbook.GetLearnablePriestSpells (ClassFlag, t, 1)
+		PriestMemorized = GemRB.GetVar ("PriestMemorized")
+		j = 1
+		while (PriestMemorized and PriestMemorized != 1<<(j-1)):
+			j = j + 1
 		for i in range (len(Learnable) ):
 			GemRB.LearnSpell (MyChar, Learnable[i], 0)
+		GemRB.MemorizeSpell (MyChar, IE_SPELL_TYPE_PRIEST, 0, j, 1)
 
 	# ranger tracking is a hardcoded innate
 	if GUICommon.HasHOW():
-		if CommonTables.ClassSkills.GetValue (GemRB.GetPlayerStat (MyChar, IE_CLASS), 0) != "*":
+		if CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL") != "*":
 			GemRB.LearnSpell (MyChar, "spin139", LS_MEMO)
 
 	# save all the skills
 	LUSkillsSelection.SkillsSave (MyChar)
 
 	TmpTable = GemRB.LoadTable ("repstart")
-	t = AlignmentTable.FindValue (3, t)
+	t = CommonTables.Aligns.FindValue (3, t)
 	t = TmpTable.GetValue (t, 0) * 10
 	GemRB.SetPlayerStat (MyChar, IE_REPUTATION, t)
 	# set the party rep if this in the main char
@@ -369,11 +378,11 @@ def AcceptPress():
 
 	print "Reputation", t
 	TmpTable = GemRB.LoadTable ("strtgold")
-	a = TmpTable.GetValue (Class, 1) #number of dice
-	b = TmpTable.GetValue (Class, 0) #size
-	c = TmpTable.GetValue (Class, 2) #adjustment
-	d = TmpTable.GetValue (Class, 3) #external multiplier
-	e = TmpTable.GetValue (Class, 4) #level bonus rate
+	a = TmpTable.GetValue (ClassName, "ROLLS") #number of dice
+	b = TmpTable.GetValue (ClassName, "SIDES") #size
+	c = TmpTable.GetValue (ClassName, "MODIFIER") #adjustment
+	d = TmpTable.GetValue (ClassName, "MULTIPLIER") #external multiplier
+	e = TmpTable.GetValue (ClassName, "BONUS_PER_LEVEL") #level bonus rate (iwd only!)
 	t = GemRB.GetPlayerStat (MyChar, IE_LEVEL) #FIXME: calculate multiclass average
 	if t>1:
 		e=e*(t-1)
@@ -398,7 +407,9 @@ def AcceptPress():
 
 	GemRB.SetPlayerName (MyChar, GemRB.GetToken ("CHARNAME"), 0)
 	GemRB.SetToken ("CHARNAME","")
-	GemRB.SetPlayerStat (MyChar, IE_XP, CommonTables.ClassSkills.GetValue (Class, 3) )
+	# don't reset imported char's xp back to start
+	if not ImportedChar:
+		GemRB.SetPlayerStat (MyChar, IE_XP, CommonTables.ClassSkills.GetValue (ClassName, "STARTXP"))
 
 	GUICommon.SetColorStat (MyChar, IE_SKIN_COLOR, GemRB.GetVar ("SkinColor") )
 	GUICommon.SetColorStat (MyChar, IE_HAIR_COLOR, GemRB.GetVar ("HairColor") )
@@ -440,14 +451,14 @@ def SetCharacterDescription():
 		else:
 			TextArea.Append (1051)
 	if CharGenState > 2:
-		Class = CommonTables.Classes.FindValue (5, GemRB.GetPlayerStat (MyChar, IE_CLASS) )
+		ClassName = GUICommon.GetClassRowName (MyChar)
 		TextArea.Append (12136, -1)
 		TextArea.Append (": ")
 		#this is only mage school in iwd
 		Kit = GemRB.GetPlayerStat (MyChar, IE_KIT)
 		KitIndex = KitTable.FindValue (3, Kit)
 		if KitIndex <= 0:
-			ClassTitle = CommonTables.Classes.GetValue (Class, 2)
+			ClassTitle = CommonTables.Classes.GetValue (ClassName, "CAP_REF")
 		else:
 			ClassTitle = KitTable.GetValue (KitIndex, 2)
 		TextArea.Append (ClassTitle)
@@ -461,8 +472,8 @@ def SetCharacterDescription():
 	if CharGenState > 3:
 		TextArea.Append (1049, -1)
 		TextArea.Append (": ")
-		Alignment = AlignmentTable.FindValue (3, GemRB.GetPlayerStat(MyChar, IE_ALIGNMENT) )
-		TextArea.Append (AlignmentTable.GetValue (Alignment, 2) )
+		Alignment = CommonTables.Aligns.FindValue (3, GemRB.GetPlayerStat(MyChar, IE_ALIGNMENT))
+		TextArea.Append (CommonTables.Aligns.GetValue (Alignment, 2))
 	if CharGenState > 4:
 		strextra = GemRB.GetPlayerStat (MyChar, IE_STREXTRA)
 		TextArea.Append ("", -1)
@@ -476,13 +487,11 @@ def SetCharacterDescription():
 			else:
 				TextArea.Append (str(stat) )
 	if CharGenState > 5:
-		ClassName = CommonTables.Classes.GetRowName (Class)
-		Row = CommonTables.Classes.GetValue (Class, 5)
-		IsRanger = CommonTables.ClassSkills.GetValue (Row, 0)
-		IsArcane = CommonTables.ClassSkills.GetValue (Row, 1)
-		IsMage = CommonTables.ClassSkills.GetValue (Row, 2)
-		IsBard = CommonTables.ClassSkills.GetValue (Row, 4)
-		IsThief = CommonTables.ClassSkills.GetValue (Row, 5)
+		DruidSpell = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL")
+		PriestSpell = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL")
+		MageSpell = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL")
+		IsBard = CommonTables.ClassSkills.GetValue (ClassName, "BARDSKILL")
+		IsThief = CommonTables.ClassSkills.GetValue (ClassName, "THIEFSKILL")
 
 		if IsThief!="*":
 			TextArea.Append ("", -1)
@@ -493,7 +502,7 @@ def SetCharacterDescription():
 				TextArea.Append (": " )
 				TextArea.Append (str(GemRB.GetPlayerStat (MyChar, StatID)) )
 				TextArea.Append ("%" )
-		elif IsRanger!="*":
+		elif DruidSpell!="*":
 			TextArea.Append ("", -1)
 			TextArea.Append (8442, -1)
 			for i in range (4):
@@ -534,12 +543,12 @@ def SetCharacterDescription():
 					TextArea.Append ("+")
 					j = j + 1
 
-		if IsMage !="*":
+		if MageSpell !="*":
 			TextArea.Append ("", -1)
 			TextArea.Append (11027, -1)
 			TextArea.Append (": " )
 			t = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
-			Learnable = GUICommon.GetLearnableMageSpells (GemRB.GetPlayerStat (MyChar, IE_KIT), t,1)
+			Learnable = Spellbook.GetLearnableMageSpells (GemRB.GetPlayerStat (MyChar, IE_KIT), t,1)
 			MageSpellBook = GemRB.GetVar ("MageSpellBook")
 			MageMemorized = GemRB.GetVar ("MageMemorized")
 			for i in range (len(Learnable)):
@@ -550,19 +559,21 @@ def SetCharacterDescription():
 						TextArea.Append (" +")
 					TextArea.Append (" ")
 
-		if IsArcane!="*":
+		if PriestSpell == "*":
+			PriestSpell = DruidSpell
+		if PriestSpell!="*":
 			TextArea.Append ("", -1)
 			TextArea.Append (11028, -1)
 			TextArea.Append (": " )
 			t = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
-			if IsArcane == "MXSPLPRS" or IsArcane == "MXSPLPAL":
+			if PriestSpell == "MXSPLPRS" or PriestSpell == "MXSPLPAL":
 				ClassFlag = 0x4000
-			elif IsArcane == "MXSPLDRU" or IsArcane == "MXSPLRAN":
+			elif PriestSpell == "MXSPLDRU" or PriestSpell == "MXSPLRAN":
 				ClassFlag = 0x8000
 			else:
 				ClassFlag = 0
 
-			Learnable = GUICommon.GetLearnablePriestSpells( ClassFlag, t, 1)
+			Learnable = Spellbook.GetLearnablePriestSpells( ClassFlag, t, 1)
 			PriestMemorized = GemRB.GetVar ("PriestMemorized")
 			for i in range (len(Learnable)):
 				if (1 << i) & PriestMemorized:
@@ -961,8 +972,9 @@ def ClassPress():
 	HasMulti = 0
 	j = 2
 	for i in range (ClassCount):
-		Allowed = CommonTables.Classes.GetValue (CommonTables.Classes.GetRowName (i), RaceName)
-		if CommonTables.Classes.GetValue (i, 4):
+		ClassRowName = CommonTables.Classes.GetRowName (i)
+		Allowed = CommonTables.Classes.GetValue (ClassRowName, RaceName)
+		if CommonTables.Classes.GetValue (ClassRowName, "MULTI"):
 			if Allowed != 0:
 				HasMulti = 1
 		else:
@@ -973,7 +985,7 @@ def ClassPress():
 			else:
 				ClassSelectButton.SetState (IE_GUI_BUTTON_DISABLED)
 			ClassSelectButton.SetEvent (IE_GUI_BUTTON_ON_PRESS,  ClassSelectPress)
-			ClassSelectButton.SetText (CommonTables.Classes.GetValue (i, 0) )
+			ClassSelectButton.SetText (CommonTables.Classes.GetValue (ClassRowName, "NAME_REF"))
 			ClassSelectButton.SetVarAssoc ("Class", i + 1)
 
 	ClassMultiButton = ClassWindow.GetControl (10)
@@ -1015,8 +1027,8 @@ def ClassPress():
 def ClassSelectPress():
 	global ClassWindow, ClassTextArea, ClassDoneButton
 
-	Class = GemRB.GetVar ("Class") - 1
-	ClassTextArea.SetText (CommonTables.Classes.GetValue (Class, 1) )
+	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
+	ClassTextArea.SetText (CommonTables.Classes.GetValue (ClassName, "DESC_REF"))
 	ClassDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
 	return
 
@@ -1045,7 +1057,7 @@ def ClassMultiPress():
 			else:
 				ClassMultiSelectButton.SetState (IE_GUI_BUTTON_DISABLED)
 			ClassMultiSelectButton.SetEvent (IE_GUI_BUTTON_ON_PRESS,  ClassMultiSelectPress)
-			ClassMultiSelectButton.SetText (CommonTables.Classes.GetValue (i, 0) )
+			ClassMultiSelectButton.SetText (CommonTables.Classes.GetValue (ClassName, "NAME_REF"))
 			ClassMultiSelectButton.SetVarAssoc ("Class", i + 1)
 
 	ClassMultiTextArea = ClassMultiWindow.GetControl (12)
@@ -1069,8 +1081,8 @@ def ClassMultiPress():
 def ClassMultiSelectPress():
 	global ClassMultiWindow, ClassMultiTextArea, ClassMultiDoneButton
 
-	Class = GemRB.GetVar ("Class") - 1
-	ClassMultiTextArea.SetText (CommonTables.Classes.GetValue (Class, 1) )
+	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
+	ClassMultiTextArea.SetText (CommonTables.Classes.GetValue (ClassName, "DESC_REF"))
 	ClassMultiDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
 	return
 
@@ -1161,8 +1173,8 @@ def ClassDonePress():
 	AlignmentButton.SetState (IE_GUI_BUTTON_ENABLED)
 	AlignmentButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
-	ClassIndex = GemRB.GetVar ("Class")-1
-	Class = CommonTables.Classes.GetValue (ClassIndex, 5)
+	ClassName = GUICommon.GetClassRowName (GemRB.GetVar ("Class")-1, "index")
+	Class = CommonTables.Classes.GetValue (ClassName, "ID")
 	GemRB.SetPlayerStat (MyChar, IE_CLASS, Class)
 
 	Kit = KitTable.GetValue (GemRB.GetVar ("MAGESCHOOL"), 3 )
@@ -1192,9 +1204,7 @@ def AlignmentPress():
 	CharGenWindow.SetVisible (WINDOW_INVISIBLE)
 	AlignmentWindow = GemRB.LoadWindow (3)
 	ClassAlignmentTable = GemRB.LoadTable ("alignmnt")
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	GemRB.SetVar ("Alignment", 0)
 
 	for i in range (2, 11):
@@ -1203,12 +1213,12 @@ def AlignmentPress():
 
 	for i in range (9):
 		AlignmentSelectButton = AlignmentWindow.GetControl (i + 2)
-		if ClassAlignmentTable.GetValue (ClassName, AlignmentTable.GetValue(i, 4)) == 0:
+		if ClassAlignmentTable.GetValue (ClassName, CommonTables.Aligns.GetValue(i, 4)) == 0:
 			AlignmentSelectButton.SetState (IE_GUI_BUTTON_DISABLED)
 		else:
 			AlignmentSelectButton.SetState (IE_GUI_BUTTON_ENABLED)
 		AlignmentSelectButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, AlignmentSelectPress)
-		AlignmentSelectButton.SetText (AlignmentTable.GetValue (i, 0) )
+		AlignmentSelectButton.SetText (CommonTables.Aligns.GetValue (i, 0))
 		AlignmentSelectButton.SetVarAssoc ("Alignment", i + 1)
 
 	AlignmentTextArea = AlignmentWindow.GetControl (11)
@@ -1233,7 +1243,7 @@ def AlignmentSelectPress():
 	global AlignmentWindow, AlignmentTextArea, AlignmentDoneButton
 
 	Alignment = GemRB.GetVar ("Alignment") - 1
-	AlignmentTextArea.SetText (AlignmentTable.GetValue (Alignment, 1))
+	AlignmentTextArea.SetText (CommonTables.Aligns.GetValue (Alignment, 1))
 	AlignmentDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
 	return
 
@@ -1248,7 +1258,7 @@ def AlignmentDonePress():
 	AbilitiesButton.SetState (IE_GUI_BUTTON_ENABLED)
 	AbilitiesButton.SetFlags (IE_GUI_BUTTON_DEFAULT, OP_OR)
 
-	Alignment = AlignmentTable.GetValue (GemRB.GetVar ("Alignment")-1, 3)
+	Alignment = CommonTables.Aligns.GetValue (GemRB.GetVar ("Alignment")-1, 3)
 	GemRB.SetPlayerStat (MyChar, IE_ALIGNMENT, Alignment )
 
 	CharGenState = 4
@@ -1282,9 +1292,8 @@ def AbilitiesPress():
 	PointsLeftLabel = AbilitiesWindow.GetControl (0x10000002)
 	PointsLeftLabel.SetUseRGB (1)
 
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	Class = CommonTables.Classes.FindValue (5, Class)
-	HasStrExtra = CommonTables.Classes.GetValue (Class, 3)=="SAVEWAR"
+	ClassName = GUICommon.GetClassRowName (MyChar)
+	HasStrExtra = CommonTables.Classes.GetValue (ClassName, "SAVE") == "SAVEWAR"
 
 	for i in range (6):
 		AbilitiesLabelButton = AbilitiesWindow.GetControl (30 + i)
@@ -1350,8 +1359,7 @@ def AbilitiesCalcLimits(Index):
 	AbilitiesMaximum = AbilitiesRaceReqTable.GetValue (Race, Index * 2 + 1)
 	AbilitiesModifier = AbilitiesRaceAddTable.GetValue (Race, Index)
 
-	Class = CommonTables.Classes.FindValue (5, GemRB.GetPlayerStat (MyChar, IE_CLASS) )
-	ClassName = CommonTables.Classes.GetRowName (Class)
+	ClassName = GUICommon.GetClassRowName (MyChar)
 	ClassIndex = AbilitiesClassReqTable.GetRowIndex (ClassName)
 	Min = AbilitiesClassReqTable.GetValue (ClassIndex, Index)
 	if Min > 0 and AbilitiesMinimum < Min:
@@ -1540,18 +1548,18 @@ def SkillsPress():
 
 	Level = 1
 	SpellLevel = 1
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	IsRanger = CommonTables.ClassSkills.GetValue (Class, 0)
-	IsArcane = CommonTables.ClassSkills.GetValue (Class, 1)
-	IsMage = CommonTables.ClassSkills.GetValue (Class, 2)
-	IsBard = CommonTables.ClassSkills.GetValue (Class, 4)
-	IsThief = CommonTables.ClassSkills.GetValue (Class, 5)
+	ClassName = GUICommon.GetClassRowName (MyChar)
+	DruidSpell = CommonTables.ClassSkills.GetValue (ClassName, "DRUIDSPELL")
+	PriestSpell = CommonTables.ClassSkills.GetValue (ClassName, "CLERICSPELL")
+	MageSpell = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL")
+	IsBard = CommonTables.ClassSkills.GetValue (ClassName, "BARDSKILL")
+	IsThief = CommonTables.ClassSkills.GetValue (ClassName, "THIEFSKILL")
 
 	if SkillsState == 0:
 		GemRB.SetVar ("HatedRace", 0)
 		if IsThief!="*":
 			SkillsSelect()
-		elif IsRanger!="*":
+		elif DruidSpell!="*":
 			Skill = GemRB.LoadTable("SKILLRNG").GetValue(str(Level), "STEALTH")
 			GemRB.SetPlayerStat (MyChar, IE_STEALTH, Skill)
 			RacialEnemySelect()
@@ -1566,27 +1574,27 @@ def SkillsPress():
 		ProficienciesSelect()
 
 	if SkillsState == 2:
-		if IsMage!="*":
-			MageSpellsSelect(IsMage, Level, SpellLevel)
+		if MageSpell!="*":
+			MageSpellsSelect(MageSpell, Level, SpellLevel)
 		else:
 			SkillsState = 3
 
 	if SkillsState == 3:
-		if IsMage!="*":
-			MageSpellsMemorize(IsMage, Level, SpellLevel)
+		if MageSpell!="*":
+			MageSpellsMemorize(MageSpell, Level, SpellLevel)
 		else:
 			SkillsState = 4
 
 	if SkillsState == 4:
-		if IsArcane=="MXSPLPRS" or IsArcane =="MXSPLPAL":
+		if PriestSpell=="MXSPLPRS" or PriestSpell =="MXSPLPAL":
 			ClassFlag = 0x4000
-			PriestSpellsMemorize(IsArcane, Level, SpellLevel)
-		elif IsArcane=="MXSPLDRU" or IsArcane =="MXSPLRAN":
+			PriestSpellsMemorize(PriestSpell, Level, SpellLevel)
+		elif DruidSpell=="MXSPLDRU" or DruidSpell =="MXSPLRAN":
 			#no separate spell progression
-			if IsArcane == "MXSPLDRU":
-				IsArcane = "MXSPLPRS"
+			if DruidSpell == "MXSPLDRU":
+				DruidSpell = "MXSPLPRS"
 			ClassFlag = 0x8000
-			PriestSpellsMemorize(IsArcane, Level, SpellLevel)
+			PriestSpellsMemorize(DruidSpell, Level, SpellLevel)
 		else:
 			SkillsState = 5
 
@@ -1766,11 +1774,8 @@ def ProficienciesSelect():
 	ProfsMaxTable = GemRB.LoadTable ("profsmax")
 	ClassWeaponsTable = GemRB.LoadTable ("clasweap")
 
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
-	Class = ProfsTable.GetRowIndex (ClassName)
-	ProficienciesPointsLeft = ProfsTable.GetValue (Class, 0)
+	ClassName = GUICommon.GetClassRowName (MyChar)
+	ProficienciesPointsLeft = ProfsTable.GetValue (ClassName, "FIRST_LEVEL")
 	PointsLeftLabel = ProficienciesWindow.GetControl (0x10000009)
 	PointsLeftLabel.SetUseRGB (1)
 	PointsLeftLabel.SetText (str(ProficienciesPointsLeft))
@@ -1874,11 +1879,8 @@ def ProficienciesPlusPress():
 
 	ProficienciesIndex = GemRB.GetVar ("ProficienciesIndex") - 1
 	ProficienciesValue = GemRB.GetVar ("Proficiency" + str(ProficienciesIndex) )
-	Class = GemRB.GetPlayerStat (MyChar, IE_CLASS)
-	ClassIndex = CommonTables.Classes.FindValue (5, Class)
-	ClassName = CommonTables.Classes.GetRowName (ClassIndex)
-	Class = ProfsMaxTable.GetRowIndex (ClassName)
-	if ProficienciesPointsLeft > 0 and ProficienciesValue < ProfsMaxTable.GetValue (Class, 0):
+	ClassName = GUICommon.GetClassRowName (MyChar)
+	if ProficienciesPointsLeft > 0 and ProficienciesValue < ProfsMaxTable.GetValue (ClassName, "FIRST_LEVEL"):
 		ProficienciesPointsLeft = ProficienciesPointsLeft - 1
 		PointsLeftLabel = ProficienciesWindow.GetControl (0x10000009)
 		PointsLeftLabel.SetText (str(ProficienciesPointsLeft))
@@ -1949,7 +1951,7 @@ def MageSpellsSelect(SpellTable, Level, SpellLevel):
 	#kit (school), alignment, level
 	k = GemRB.GetPlayerStat (MyChar, IE_KIT)
 	t = GemRB.GetPlayerStat (MyChar, IE_ALIGNMENT)
-	Learnable = GUICommon.GetLearnableMageSpells(k, t, SpellLevel)
+	Learnable = Spellbook.GetLearnableMageSpells(k, t, SpellLevel)
 	GemRB.SetVar ("MageSpellBook", 0)
 	GemRB.SetVar ("SpellMask", 0)
 
@@ -2187,8 +2189,8 @@ def PriestSpellsMemorize(SpellTable, Level, SpellLevel):
 
 	CharGenWindow.SetVisible (WINDOW_INVISIBLE)
 	PriestMemorizeWindow = GemRB.LoadWindow (17)
-	t = AlignmentTable.GetValue ( GemRB.GetVar ("Alignment")-1, 3)
-	Learnable = GUICommon.GetLearnablePriestSpells( ClassFlag, t, SpellLevel)
+	t = CommonTables.Aligns.GetValue (GemRB.GetVar ("Alignment")-1, 3)
+	Learnable = Spellbook.GetLearnablePriestSpells( ClassFlag, t, SpellLevel)
 
 	MaxSpellsPriestTable = GemRB.LoadTable (SpellTable)
 	GemRB.SetVar ("PriestMemorized", 0)
@@ -2372,8 +2374,7 @@ def DrawAvatar():
 	lookup = CommonTables.Races.GetRowName (lookup)
 	AvatarID = AvatarID+table.GetValue (lookup, "RACE")
 	table = GemRB.LoadTable ("avprefc")
-	lookup = CommonTables.Classes.FindValue (5, GemRB.GetPlayerStat(MyChar, IE_CLASS))
-	lookup = CommonTables.Classes.GetRowName (lookup)
+	lookup = GUICommon.GetClassRowName (MyChar)
 	AvatarID = AvatarID+table.GetValue (lookup, "PREFIX")
 	table = GemRB.LoadTable ("avprefg")
 	AvatarID = AvatarID+table.GetValue (GemRB.GetPlayerStat(MyChar,IE_SEX),0)
@@ -2560,6 +2561,7 @@ def BiographyPress():
 
 	BiographyField = BiographyWindow.GetControl (4)
 	BiographyField.SetText (19423)
+	BiographyField.SetBackground("")
 
 	BiographyClearButton = BiographyWindow.GetControl (5)
 	BiographyClearButton.SetState (IE_GUI_BUTTON_ENABLED)
@@ -2699,7 +2701,7 @@ def ImportPress():
 
 def ImportDonePress():
 	global CharGenWindow, ImportWindow, CharImportList
-	global CharGenState, SkillsState, Portrait
+	global CharGenState, SkillsState, Portrait, ImportedChar
 
 	# Import the character from the chosen name
 	GemRB.CreatePlayer (CharImportList.QueryText(), MyChar|0x8000, 1)
@@ -2711,6 +2713,7 @@ def ImportDonePress():
 	PortraitButton.SetPicture (PortraitName)
 	Portrait = -1
 
+	ImportedChar = 1
 	CharGenState = 7
 	SkillsState = 5
 	SetCharacterDescription ()
