@@ -299,12 +299,17 @@ def MoveToNewArea ():
 	global WorldMapWindow, WorldMapControl
 
 	tmp = WorldMapControl.GetDestinationArea (1)
-	if tmp["Distance"]==-1:
+	CloseWorldMapWindow ()
+
+	if tmp["Destination"].lower() == GemRB.GetGameString(STR_AREANAME).lower():
+		return
+	elif tmp["Distance"] == -1:
 		print "Invalid target", tmp
 		return
 
-	CloseWorldMapWindow ()
 	GemRB.CreateMovement (tmp["Destination"], tmp["Entrance"], tmp["Direction"])
+	# distance is stored in hours, but the action needs seconds
+	GemRB.ExecuteString ("AdvanceTime(%d)"%(tmp["Distance"]*300))
 	return
 
 def ChangeTooltip ():
@@ -312,7 +317,7 @@ def ChangeTooltip ():
 	global str
 
 	tmp = WorldMapControl.GetDestinationArea ()
-	if (tmp):
+	if tmp and tmp["Distance"] >= 0:
 		str = "%s: %d"%(GemRB.GetString(23084),tmp["Distance"])
 	else:
 		str=""
@@ -380,6 +385,7 @@ def WorldMapWindowCommon (Travel):
 	else:
 		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenMapWindow)
 	Window.SetVisible (WINDOW_VISIBLE)
+	WorldMapControl.SetStatus (IE_GUI_CONTROL_FOCUSED)
 
 ###################################################
 # End of file GUIMA.py
