@@ -35,6 +35,13 @@ InventoryWindow = None
 ItemAmountWindow = None
 OverSlot = None
 
+def OpenInventoryWindowClick ():
+	tmp = GemRB.GetVar ("PressedPortrait")
+	GemRB.GameSelectPC (tmp, True, SELECT_REPLACE)
+	OpenInventoryWindow ()
+	return
+
+
 ItemHash = {}
 
 # Control ID's:
@@ -192,7 +199,6 @@ def RefreshInventoryWindow ():
 
 	# armor class
 	ac = GemRB.GetPlayerStat (pc, IE_ARMORCLASS)
-	ac += GemRB.GetAbilityBonus (IE_DEX, 2, GemRB.GetPlayerStat (pc, IE_DEX) )
 	Label = Window.GetControl (0x1000003a)
 	Label.SetText (str (ac))
 
@@ -211,16 +217,12 @@ def RefreshInventoryWindow ():
 	Label.SetText (str (GemRB.GameGetPartyGold ()))
 
 	# class
-	text = CommonTables.Classes.GetValue (GemRB.GetPlayerStat (pc, IE_CLASS) - 1, 0)
+	text = CommonTables.Classes.GetValue (GUICommon.GetClassRowName (pc), "NAME_REF")
 
 	Label = Window.GetControl (0x1000003f)
 	Label.SetText (text)
 
-	held = GemRB.GetPlayerStat (pc, IE_HELD) + GemRB.GetPlayerStat (pc, IE_CASTERHOLD)
-	if held == 0 and GemRB.GetPlayerStat (pc, IE_STATE_ID) & STATE_DEAD == 0:
-		Window.SetVisible (WINDOW_VISIBLE)
-	else:
-		Window.SetVisible (WINDOW_GRAYED)
+	GUICommon.AdjustWindowVisibility (Window, pc, False)
 
 	# update ground inventory slots
 	Container = GemRB.GetContainer(pc, 1)
@@ -228,7 +230,7 @@ def RefreshInventoryWindow ():
 	for i in range (10):
 		Button = Window.GetControl (i+47)
 		if GemRB.IsDraggingItem ():
-			Button.SetState (IE_GUI_BUTTON_SECOND)
+			Button.SetState (IE_GUI_BUTTON_FAKEPRESSED)
 		else:
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, InventoryCommon.OnDragItemGround)
@@ -352,15 +354,15 @@ def UpdateSlot (pc, i):
 		if (SlotType["Type"]&SLOT_INVENTORY) or not GemRB.CanUseItemType (SlotType["Type"], itemname):
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		else:
-			Button.SetState (IE_GUI_BUTTON_SECOND)
+			Button.SetState (IE_GUI_BUTTON_FAKEPRESSED)
 
 		if slot_item and (GemRB.GetEquippedQuickSlot (pc)==slot or GemRB.GetEquippedAmmunition (pc)==slot):
-			Button.SetState (IE_GUI_BUTTON_THIRD)
+			Button.SetState (IE_GUI_BUTTON_FAKEDISABLED)
 	return
 
 def DefaultWeapon ():
 	pc = GemRB.GameGetFirstSelectedActor ()
-	GemRB.SetEquippedQuickSlot (pc, 1000, -1, 1)
+	GemRB.SetEquippedQuickSlot (pc, 1000, -1)
 	UpdateInventoryWindow ()
 	return
 
