@@ -28,6 +28,8 @@
 #include "Sprite2D.h"
 #include "Video.h"
 
+namespace GemRB {
+
 Animation::Animation(int count)
 {
 	frames = (Sprite2D **) calloc(count, sizeof(Sprite2D *));
@@ -106,8 +108,8 @@ unsigned int Animation::GetCurrentFrame() const
 
 Sprite2D* Animation::LastFrame(void)
 {
-	if (!Flags&A_ANI_ACTIVE) {
-		print("Frame fetched while animation is inactive!\n");
+	if (!(Flags&A_ANI_ACTIVE)) {
+		Log(MESSAGE, "Sprite2D", "Frame fetched while animation is inactive1!");
 		return NULL;
 	}
 	if (gameAnimation) {
@@ -125,8 +127,8 @@ Sprite2D* Animation::LastFrame(void)
 
 Sprite2D* Animation::NextFrame(void)
 {
-	if (!Flags&A_ANI_ACTIVE) {
-		print("Frame fetched while animation is inactive!\n");
+	if (!(Flags&A_ANI_ACTIVE)) {
+		Log(MESSAGE, "Sprite2D", "Frame fetched while animation is inactive2!");
 		return NULL;
 	}
 	if (starttime == 0) {
@@ -142,7 +144,7 @@ Sprite2D* Animation::NextFrame(void)
 	else
 		ret = frames[pos];
 
-	if (endReached && (Flags&A_ANI_PLAYONCE) )
+	if (endReached && (Flags&A_ANI_PLAYONCE))
 		return ret;
 
 	unsigned long time;
@@ -179,8 +181,8 @@ Sprite2D* Animation::NextFrame(void)
 
 Sprite2D* Animation::GetSyncedNextFrame(Animation* master)
 {
-	if (!Flags&A_ANI_ACTIVE) {
-		print("Frame fetched while animation is inactive!\n");
+	if (!(Flags&A_ANI_ACTIVE)) {
+		Log(MESSAGE, "Sprite2D", "Frame fetched while animation is inactive!");
 		return NULL;
 	}
 	Sprite2D* ret;
@@ -190,9 +192,11 @@ Sprite2D* Animation::GetSyncedNextFrame(Animation* master)
 		ret = frames[pos];
 
 	starttime = master->starttime;
-	pos = master->pos;
 	endReached = master->endReached;
 
+	//return a valid frame even if the master is longer (e.g. ankhegs)
+	pos = master->pos % indicesCount;
+
 	return ret;
 }
 
@@ -259,3 +263,5 @@ void Animation::AddAnimArea(Animation* slave)
 		animArea.h = y+h-animArea.y;
 	}
 }
+
+}
