@@ -26,6 +26,8 @@
 #include "Sprite2D.h"
 #include "Video.h"
 
+namespace GemRB {
+
 AnimationFactory::AnimationFactory(const char* ResRef)
 	: FactoryObject( ResRef, IE_BAM_CLASS_ID )
 {
@@ -44,7 +46,7 @@ AnimationFactory::~AnimationFactory(void)
 
 	// FIXME: track down where sprites are being leaked
 	if (datarefcount) {
-		fprintf(stderr, "AnimationFactory %s has refcount %d\n", ResRef, datarefcount);
+		Log(ERROR, "AnimationFactory", "AnimationFactory %s has refcount %d", ResRef, datarefcount);
 		//assert(datarefcount == 0);
 	}
 	if (FrameData)
@@ -167,3 +169,5 @@ void AnimationFactory::DecDataRefCount()
 	assert(datarefcount > 0);
 	--datarefcount;
 }
+
+}
