@@ -32,11 +32,15 @@
 #include "Palette.h"
 #include "Video.h"
 
+namespace GemRB {
+
 static int AvatarsCount = 0;
 static AvatarStruct *AvatarTable = NULL;
 static const ieByte SixteenToNine[16]={0,1,2,3,4,5,6,7,8,7,6,5,4,3,2,1};
 static const ieByte SixteenToFive[16]={0,0,1,1,2,2,3,3,4,4,3,3,2,2,1,1};
 
+static const int zOrder_TwoPiece[2] = { 1, 0 };
+
 static const int zOrder_Mirror16[16][4] = {
 	{ 0, 3, 2, 1 },
 	{ 0, 3, 2, 1 },
@@ -60,8 +64,8 @@ static const int zOrder_8[8][4] = {
 	{ 0, 3, 2, 1 },
 	{ 0, 3, 1, 2 },
 	{ 1, 0, 3, 2 },
-	{ 1, 0, 3, 2 },
-	{ 1, 0, 3, 2 },
+	{ 1, 2, 0, 3 },
+	{ 1, 2, 0, 3 },
 	{ 2, 0, 3, 1 },
 	{ 2, 0, 3, 1 },
 	{ 2, 0, 3, 1 }
@@ -126,6 +130,18 @@ int CharAnimations::GetBloodColor() const
 	return AvatarTable[AvatarsRowNum].BloodColor;
 }
 
+unsigned int CharAnimations::GetFlags() const
+{
+	if(AvatarsRowNum==~0u) return 0;
+	return AvatarTable[AvatarsRowNum].Flags;
+}
+
+unsigned char CharAnimations::MaybeOverrideStance(unsigned char stance) const
+{
+	if(AvatarsRowNum==~0u) return stance;
+	return AvatarTable[AvatarsRowNum].StanceOverride[stance];
+}
+
 static ieResRef EmptySound={0};
 
 const ieResRef &CharAnimations::GetWalkSound() const
@@ -148,6 +164,8 @@ int CharAnimations::GetActorPartCount() const
 		return 9;
 	case IE_ANI_FOUR_FRAMES: //wyvern animations
 		return 4;
+	case IE_ANI_TWO_PIECE:   //ankheg animations
+		return 2;
 	case IE_ANI_PST_GHOST:   //special pst anims
 		if (AvatarTable[AvatarsRowNum].Prefixes[1][0]=='*') {
 			return 1;
@@ -180,6 +198,15 @@ int CharAnimations::GetTotalPartCount() const
 	}
 }
 
+const ieResRef& CharAnimations::GetArmourLevel(int ArmourLevel) const
+{
+	//ignore ArmourLevel for the static pst anims (all sprites are displayed)
+	if (AvatarTable[AvatarsRowNum].AnimationType == IE_ANI_PST_GHOST) {
+		ArmourLevel = 0;
+	}
+	return AvatarTable[AvatarsRowNum].Prefixes[ArmourLevel];
+}
+
 void CharAnimations::SetArmourLevel(int ArmourLevel)
 {
 	if (AvatarsRowNum==~0u) return;
@@ -187,8 +214,7 @@ void CharAnimations::SetArmourLevel(int ArmourLevel)
 	if (AvatarTable[AvatarsRowNum].AnimationType == IE_ANI_PST_GHOST) {
 		ArmourLevel = 0;
 	}
-	strncpy( ResRef, AvatarTable[AvatarsRowNum].Prefixes[ArmourLevel], 8 );
-	ResRef[8]=0;
+	CopyResRef( ResRef, AvatarTable[AvatarsRowNum].Prefixes[ArmourLevel] );
 	DropAnims();
 }
 
@@ -295,6 +321,8 @@ void CharAnimations::CheckColorMod()
 		  }
 		}
 	}
+	//this is set by sanctuary and stoneskin (override global colors)
+	lockPalette = false;
 }
 
 void CharAnimations::SetupColors(PaletteType type)
@@ -429,9 +457,11 @@ Palette* CharAnimations::GetPartPalette(int part)
 	int actorPartCount = GetActorPartCount();
 	PaletteType type = PAL_MAIN;
 
-	if (part == actorPartCount) type = PAL_WEAPON;
-	if (part == actorPartCount+1) type = PAL_OFFHAND;
-	if (part == actorPartCount+2) type = PAL_HELMET;
+	// always use unmodified BAM palette for the supporting part
+	if (GetAnimType() == IE_ANI_TWO_PIECE && part == 1) return NULL;
+	else if (part == actorPartCount) type = PAL_WEAPON;
+	else if (part == actorPartCount+1) type = PAL_OFFHAND;
+	else if (part == actorPartCount+2) type = PAL_HELMET;
 
 	if (modifiedPalette[(int)type])
 		return modifiedPalette[(int)type];
@@ -483,6 +513,9 @@ void CharAnimations::InitAvatarsTable()
 		AvatarTable[i].WalkScale = 0;
 		AvatarTable[i].RunScale = 0;
 		AvatarTable[i].Bestiary = -1;
+		
+		for (int j = 0; j < MAX_ANIMS; j++)
+			AvatarTable[i].StanceOverride[j] = j;
 
 		if (resdata) {
 			char section[12];
@@ -505,22 +538,24 @@ void CharAnimations::InitAvatarsTable()
 		int rows = blood->GetRowCount();
 		for(int i=0;i<rows;i++) {
 			unsigned long value = 0;
+			unsigned long flags = 0;
 			unsigned long rmin = 0;
 			unsigned long rmax = 0xffff;
 
 			valid_number(blood->QueryField(i,0), (long &)value);
 			valid_number(blood->QueryField(i,1), (long &)rmin);
 			valid_number(blood->QueryField(i,2), (long &)rmax);
-			if (value>255 || rmin>0xffff || rmax>0xffff) {
-				printMessage("CharAnimations", "bloodclr entry: %02x %04x-%04x ", LIGHT_RED,
+			valid_number(blood->QueryField(i,3), (long &)flags);
+			if (value>255 || rmin>rmax || rmax>0xffff) {
+				Log(ERROR, "CharAnimations", "Invalid bloodclr entry: %02x %04x-%04x ",
 						(unsigned int) value, (unsigned int) rmin, (unsigned int) rmax);
-				printStatus("Invalid value!", LIGHT_RED);
 				continue;
 			}
 			for(int j=0;j<AvatarsCount;j++) {
 				if (rmax<AvatarTable[j].AnimID) break;
 				if (rmin>AvatarTable[j].AnimID) continue;
-				AvatarTable[j].BloodColor = value;
+				AvatarTable[j].BloodColor = (char) value;
+				AvatarTable[j].Flags = (unsigned int) flags;
 			}
 		}
 	}
@@ -532,7 +567,7 @@ void CharAnimations::InitAvatarsTable()
 			ieResRef value;
 			unsigned long rmin = 0;
 			unsigned long rmax = 0xffff;
-			unsigned int range = 0;
+			unsigned long range = 0;
 
 			strnuprcpy(value, walk->QueryField(i,0), 8);
 			valid_number(walk->QueryField(i,1), (long &)rmin);
@@ -542,11 +577,41 @@ void CharAnimations::InitAvatarsTable()
 				value[0]=0;
 				range = 0;
 			}
+			if (range>255 || rmin>rmax || rmax>0xffff) {
+				Log(ERROR, "CharAnimations", "Invalid walksnd entry: %02x %04x-%04x ",
+						(unsigned int) range, (unsigned int) rmin, (unsigned int) rmax);
+				continue;
+			}
 			for(int j=0;j<AvatarsCount;j++) {
 				if (rmax<AvatarTable[j].AnimID) break;
 				if (rmin>AvatarTable[j].AnimID) continue;
 				memcpy(AvatarTable[j].WalkSound, value, sizeof(ieResRef) );
-				AvatarTable[j].WalkSoundCount = range;
+				AvatarTable[j].WalkSoundCount = (unsigned int) range;
+			}
+		}
+	}
+
+	AutoTable stances("stances", true);
+	if (stances) {
+		int rows = stances->GetRowCount();
+		for (int i = 0; i < rows; i++) {
+			unsigned long id = 0, s1 = 0, s2 = 0;
+			valid_number(stances->GetRowName(i), (long &)id);
+			valid_number(stances->QueryField(i, 0), (long &)s1);
+			valid_number(stances->QueryField(i, 1), (long &)s2);
+
+			if (s1 >= MAX_ANIMS || s2 >= MAX_ANIMS) {
+				Log(ERROR, "CharAnimations", "Invalid stances entry: %04x %d %d",
+						(unsigned int) id, (unsigned int) s1, (unsigned int) s2);
+				continue;
+			}
+
+			for (int j = 0; j < AvatarsCount; j++) {
+				if (id < AvatarTable[j].AnimID) break;
+				if (id == AvatarTable[j].AnimID) {
+					AvatarTable[j].StanceOverride[s1] = s2;
+					break;
+				}
 			}
 		}
 	}
@@ -610,7 +675,7 @@ CharAnimations::CharAnimations(unsigned int AnimID, ieDword ArmourLevel)
 		}
 	}
 	ResRef[0]=0;
-	printMessage("CharAnimations", "Invalid or nonexistent avatar entry:%04X\n", LIGHT_RED, AnimID);
+	Log(ERROR, "CharAnimations", "Invalid or nonexistent avatar entry:%04X", AnimID);
 }
 
 //we have to drop them when armourlevel changes
@@ -685,6 +750,11 @@ IE_ANI_FOUR_FILES:	The Animation is coded in Four Files. Probably it is an old T
 
 IE_ANI_FOUR_FILES_2:	Like IE_ANI_FOUR_FILES but with only 16 cycles per frame.
 
+IE_ANI_FOUR_FILES_3: A variation of the four files animation without equipment, and
+			with even and odd orientations split across two files, plus the standard
+			separate eastern parts, so the layout is
+			[NAME][H|L]G1[/E]
+
 IE_ANI_TWENTYTWO:	This Animation Type stores the Animation in the following format
 			[NAME][ACTIONCODE][/E]
 			ACTIONCODE=A1-6, CA, SX, SA (sling is A1)
@@ -729,6 +799,9 @@ IE_ANI_TWO_FILES_3B:	Animations using this type are stored using the following t
 			This is the standard IWD animation, but BG2 also has it.
 			See MOR2
 
+IE_ANI_TWO_PIECE: This is a modified IE_ANI_SIX_FILES with supporting still frames (using a
+			different palette) stored in a second set of files. Currently only used by MAKH
+
 IE_ANI_FOUR_FRAMES:	These animations are large, four bams make a frame.
 
 
@@ -786,7 +859,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 	}
 
 	//for paletted dragon animations, we need the stance id
-	StanceID  = nextStanceID = Stance;
+	StanceID = nextStanceID = Stance;
 	int AnimType = GetAnimType();
 
 	//alter stance here if it is missing and you know a substitute
@@ -820,6 +893,8 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 		}
 	}
 
+	StanceID = MaybeOverrideStance(StanceID);
+
 	//TODO: Implement Auto Resource Loading
 	//setting up the sequencing of animation cycles
 	autoSwitchOnEnd = false;
@@ -863,7 +938,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 			autoSwitchOnEnd = true;
 			break;
 		default:
-			print ("Invalid Stance: %d\n", StanceID);
+			Log(MESSAGE, "CharAnimation", "Invalid Stance: %d", StanceID);
 			break;
 	}
 	Animation** anims = Anims[StanceID][Orient];
@@ -874,7 +949,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 
 	int partCount = GetTotalPartCount();
 	int actorPartCount = GetActorPartCount();
-	if (partCount < 0) return 0;
+	if (partCount <= 0) return 0;
 	anims = new Animation*[partCount];
 
 	EquipResRefData* equipdat = 0;
@@ -893,7 +968,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 			if (equipdat) delete equipdat;
 
 			//we need this long for special anims
-			strncpy( NewResRef, ResRef, 8 );
+			strlcpy( NewResRef, ResRef, sizeof(ieResRef) );
 			GetAnimResRef( StanceID, Orient, NewResRef, Cycle, part, equipdat);
 		} else {
 			// Equipment animation parts
@@ -930,8 +1005,8 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 
 		if (!af) {
 			if (part < actorPartCount) {
-				printMessage("CharAnimations", "Couldn't create animationfactory: %s (%04x)\n",
-					LIGHT_RED, NewResRef, GetAnimationID());;
+				Log(ERROR, "CharAnimations", "Couldn't create animationfactory: %s (%04x)",
+						NewResRef, GetAnimationID());;
 				for (int i = 0; i < part; ++i)
 					delete anims[i];
 				delete[] anims;
@@ -948,7 +1023,7 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 
 		if (!a) {
 			if (part < actorPartCount) {
-				printMessage("CharAnimations", "Couldn't load animation: %s, cycle %d\n", LIGHT_RED,
+				Log(ERROR, "CharAnimations", "Couldn't load animation: %s, cycle %d",
 						 NewResRef, Cycle);
 				for (int i = 0; i < part; ++i)
 					delete anims[i];
@@ -1060,11 +1135,22 @@ Animation** CharAnimations::GetAnimation(unsigned char Stance, unsigned char Ori
 		case IE_ANI_FOUR_FILES:
 		case IE_ANI_FOUR_FILES_2:
 		case IE_ANI_SIX_FILES_2:
+		case IE_ANI_TWO_PIECE:
 		case IE_ANI_FRAGMENT:
 			Orient&=~1;
 			Anims[StanceID][Orient] = anims;
 			Anims[StanceID][Orient + 1] = anims;
 			break;
+		case IE_ANI_FOUR_FILES_3:
+			//only 8 orientations for WALK
+			if (StanceID == IE_ANI_WALK) {
+				Orient&=~1;
+				Anims[StanceID][Orient] = anims;
+				Anims[StanceID][Orient + 1] = anims;
+			} else {
+				Anims[StanceID][Orient] = anims;
+			}
+			break;
 
 		case IE_ANI_PST_ANIMATION_3: //no stc just std
 		case IE_ANI_PST_ANIMATION_2: //no std just stc
@@ -1124,7 +1210,8 @@ void CharAnimations::GetAnimResRef(unsigned char StanceID,
 			break;
 
 		case IE_ANI_BIRD:
-			Cycle = (ieByte) ((StanceID&1) * 9 + SixteenToNine[Orient]);
+			// TODO: use 0-8 for gliding; those only have a single frame
+			Cycle = 9 + SixteenToNine[Orient];
 			break;
 
 		case IE_ANI_FRAGMENT:
@@ -1167,10 +1254,18 @@ void CharAnimations::GetAnimResRef(unsigned char StanceID,
 			AddLRSuffix2( NewResRef, StanceID, Cycle, Orient, EquipData );
 			break;
 
+		case IE_ANI_FOUR_FILES_3:
+			AddHLSuffix( NewResRef, StanceID, Cycle, Orient );
+			break;
+
 		case IE_ANI_SIX_FILES_2: //MOGR (variant of FOUR_FILES)
 			AddLR3Suffix( NewResRef, StanceID, Cycle, Orient );
 			break;
 
+		case IE_ANI_TWO_PIECE: //MAKH
+			AddTwoPieceSuffix( NewResRef, StanceID, Cycle, Orient, Part );
+			break;
+
 		case IE_ANI_CODE_MIRROR_2: //9 orientations
 			AddVHR2Suffix( NewResRef, StanceID, Cycle, Orient );
 			break;
@@ -1228,6 +1323,8 @@ const int* CharAnimations::GetZOrder(unsigned char Orient)
 			return zOrder_8[Orient/2];
 		case IE_ANI_FOUR_FILES:
 			return 0; // FIXME
+		case IE_ANI_TWO_PIECE:
+			return zOrder_TwoPiece;
 		default:
 			return 0;
 	}
@@ -1305,6 +1402,7 @@ void CharAnimations::AddVHR2Suffix(char* ResRef, unsigned char StanceID,
 		case IE_ANI_ATTACK: //temporarily
 		case IE_ANI_ATTACK_BACKSLASH:
 			strcat( ResRef, "g21" );
+			Cycle+=9;
 			break;
 
 		case IE_ANI_ATTACK_SLASH:
@@ -1312,8 +1410,8 @@ void CharAnimations::AddVHR2Suffix(char* ResRef, unsigned char StanceID,
 			break;
 
 		case IE_ANI_ATTACK_JAB:
-			strcat( ResRef, "g26" );
-			Cycle+=45;
+			strcat( ResRef, "g22" );
+			Cycle+=18;
 			break;
 
 		case IE_ANI_CAST: //looping
@@ -1506,6 +1604,7 @@ void CharAnimations::AddFFSuffix(char* ResRef, unsigned char StanceID,
 			Cycle += 48;
 			break;
 
+		case IE_ANI_SLEEP:
 		case IE_ANI_TWITCH:
 			strcat( ResRef, "g2" );
 			Cycle += 64;
@@ -2022,6 +2121,65 @@ void CharAnimations::AddLRSuffix2( char* ResRef, unsigned char StanceID,
 	EquipData->Cycle = Cycle;
 }
 
+void CharAnimations::AddTwoPieceSuffix(char* ResRef, unsigned char StanceID,
+	unsigned char& Cycle, unsigned char Orient, int Part)
+{
+	if (Part == 1) {
+		strcat( ResRef, "d" );
+	}
+
+	switch (StanceID) {
+		case IE_ANI_DIE:
+			strcat( ResRef, "g1" );
+			Cycle = 8 + Orient / 2;
+			break;
+		case IE_ANI_TWITCH:
+		case IE_ANI_SLEEP:
+			strcat( ResRef, "g1" );
+			Cycle = 16 + Orient / 2;
+			break;
+		case IE_ANI_READY:
+		case IE_ANI_HEAD_TURN:
+		case IE_ANI_AWAKE:
+		case IE_ANI_DAMAGE:
+			strcat( ResRef, "g1" );
+			Cycle = 24 + Orient / 2;
+			break;
+		case IE_ANI_WALK:
+			strcat( ResRef, "g2" );
+			Cycle = Orient / 2;
+			break;
+		case IE_ANI_GET_UP:
+		case IE_ANI_EMERGE:
+			strcat( ResRef, "g2" );
+			Cycle = 8 + Orient / 2;
+			break;
+		case IE_ANI_HIDE:
+			strcat( ResRef, "g2" );
+			Cycle = 16 + Orient / 2;
+			break;
+		case IE_ANI_ATTACK:
+		case IE_ANI_ATTACK_BACKSLASH:
+			strcat( ResRef, "g3" );
+			Cycle = Orient / 2;
+			break;
+		case IE_ANI_ATTACK_SLASH:
+		case IE_ANI_ATTACK_JAB:
+		case IE_ANI_CAST:
+		case IE_ANI_CONJURE:
+		case IE_ANI_SHOOT:
+			strcat( ResRef, "g3" );
+			Cycle = 8 + Orient / 2;
+			break;
+		default:
+			error("CharAnimation", "Two-piece Animation: unhandled stance: %s %d", ResRef, StanceID);
+			break;
+	}
+	if (Orient > 9) {
+		strcat( ResRef, "e" );
+	}
+}
+
 void CharAnimations::AddLRSuffix( char* ResRef, unsigned char StanceID,
 	unsigned char& Cycle, unsigned char Orient, EquipResRefData *&EquipData)
 {
@@ -2035,23 +2193,18 @@ void CharAnimations::AddLRSuffix( char* ResRef, unsigned char StanceID,
 			Cycle = Orient / 2;
 			break;
 		case IE_ANI_ATTACK_SLASH:
+		case IE_ANI_CAST:
+		case IE_ANI_CONJURE:
 			strcat( ResRef, "g2" );
 			strcpy( EquipData->Suffix, "g2" );
 			Cycle = 8 + Orient / 2;
 			break;
 		case IE_ANI_ATTACK_JAB:
+		case IE_ANI_SHOOT:
 			strcat( ResRef, "g2" );
 			strcpy( EquipData->Suffix, "g2" );
 			Cycle = 16 + Orient / 2;
 			break;
-		case IE_ANI_CAST:
-		case IE_ANI_CONJURE:
-		case IE_ANI_SHOOT:
-			//these animations are missing
-			strcat( ResRef, "g2" );
-			strcpy( EquipData->Suffix, "g2" );
-			Cycle = Orient / 2;
-			break;
 		case IE_ANI_WALK:
 			strcat( ResRef, "g1" );
 			strcpy( EquipData->Suffix, "g1" );
@@ -2329,6 +2482,65 @@ void CharAnimations::AddMMRSuffix(char* ResRef, unsigned char StanceID,
 	}
 }
 
+void CharAnimations::AddHLSuffix(char* ResRef, unsigned char StanceID,
+	unsigned char& Cycle, unsigned char Orient)
+{
+	//even orientations in 'h', odd in 'l', and since the WALK animation
+	//with fewer orientations is first in h, all other stances in that
+	//file need to be offset by those cycles
+	int offset = ((Orient % 2)^1) * 8;
+
+	switch (StanceID) {
+
+		case IE_ANI_WALK:
+			//only available in 8 orientations instead of the usual 16
+			Cycle = 0 + Orient / 2;
+			offset = 1;
+			break;
+
+		case IE_ANI_HEAD_TURN:
+			Cycle = offset + Orient / 2;
+			break;
+
+		case IE_ANI_AWAKE:
+		case IE_ANI_READY:
+		//the following are not available
+		case IE_ANI_CAST:
+		case IE_ANI_CONJURE:
+		case IE_ANI_HIDE:
+		case IE_ANI_SHOOT:
+		case IE_ANI_ATTACK:
+		case IE_ANI_ATTACK_SLASH:
+		case IE_ANI_ATTACK_BACKSLASH:
+		case IE_ANI_ATTACK_JAB:
+			Cycle = 8 + offset + Orient / 2;
+			break;
+
+		case IE_ANI_DAMAGE:
+			Cycle = 16 + offset + Orient / 2;
+			break;
+
+		case IE_ANI_DIE:
+		case IE_ANI_GET_UP:
+		case IE_ANI_EMERGE:
+			Cycle = 24 + offset + Orient / 2;
+			break;
+
+		case IE_ANI_SLEEP:
+		case IE_ANI_TWITCH:
+			Cycle = 32 + offset + Orient / 2;
+			break;
+
+		default:
+			error("CharAnimation", "HL Animation: unhandled stance: %s %d", ResRef, StanceID);
+			break;
+	}
+	strcat(ResRef, offset ? "hg1" : "lg1");
+	if (Orient > 9) {
+		strcat(ResRef, "e");
+	}
+}
+
 void CharAnimations::PulseRGBModifiers()
 {
 	unsigned long time = core->GetGame()->Ticks;
@@ -2389,3 +2601,12 @@ void CharAnimations::PulseRGBModifiers()
 
 	lastModUpdate += inc*40;
 }
+
+void CharAnimations::DebugDump()
+{
+	Log (DEBUG, "CharAnimations", "Anim ID   : %04x", GetAnimationID() );
+	Log (DEBUG, "CharAnimations", "BloodColor: %d", GetBloodColor() );
+	Log (DEBUG, "CharAnimations", "Flags     : %04x", GetFlags() );
+}
+
+}
