@@ -30,6 +30,8 @@
 
 #include <vector>
 
+namespace GemRB {
+
 #define AV_PREFIX1      0
 #define AV_PREFIX2      1
 #define AV_PREFIX3      2
@@ -80,6 +82,8 @@
 #define IE_ANI_FOUR_FILES_2		14 //METT
 #define IE_ANI_CODE_MIRROR_3	15 //MSPS
 #define IE_ANI_TWO_FILES_3B		16    //iwd animations (eg. MBBM)
+#define IE_ANI_TWO_PIECE		17    //MAKH
+#define IE_ANI_FOUR_FILES_3		18    //mostly civilians
 
 //PST animation types
 #define IE_ANI_PST_ANIMATION_1		56   //full animation
@@ -102,6 +106,9 @@
 #define IE_ANI_RANGED_XBOW		1
 #define IE_ANI_RANGED_THROW		2
 
+//special flags
+#define AV_NO_BODY_HEAT                 1
+
 struct AvatarStruct {
 	/* entries from avatars.2da */
 	unsigned int AnimID;
@@ -112,7 +119,8 @@ struct AvatarStruct {
 	char Size;
 
 	/* comes from bloodclr.2da */
-	char BloodColor;	
+	char BloodColor;
+	unsigned int Flags;
 	
 	/* resdata.ini entries */
 	unsigned int WalkScale; /* 1000 / walkscale */
@@ -122,6 +130,9 @@ struct AvatarStruct {
 	/* comes from walksnd.2da */
 	ieResRef WalkSound;
 	ieByte WalkSoundCount;
+
+	/* comes from stances.2da */
+	unsigned char StanceOverride[MAX_ANIMS];
 };
 
 struct EquipResRefData;
@@ -180,10 +191,12 @@ public: //attribute functions
 	int GetAnimType() const;
 	int GetSize() const;
 	int GetBloodColor() const;
+	unsigned int GetFlags() const;
 	const ieResRef &GetWalkSound() const;
 	int GetWalkSoundCount() const;
+	const ieResRef &GetArmourLevel(int ArmourLevel) const;
 	void PulseRGBModifiers();
-
+	void DebugDump();
 private:
 	void DropAnims();
 	void InitAvatarsTable();
@@ -192,6 +205,8 @@ private:
 		unsigned char& Cycle, unsigned char Orient);
 	void AddFFSuffix(char* ResRef, unsigned char AnimID,
 		unsigned char& Cycle, unsigned char Orient, int Part);
+	void AddHLSuffix(char* ResRef, unsigned char AnimID,
+		unsigned char& Cycle, unsigned char Orient);
 	void AddNFSuffix(char* ResRef, unsigned char AnimID,
 		unsigned char& Cycle, unsigned char Orient, int Part);
 	void AddVHR2Suffix(char* ResRef, unsigned char AnimID,
@@ -204,6 +219,8 @@ private:
 		const char* equipRef, bool offhand, EquipResRefData* equip);
 	void AddSixSuffix(char* ResRef, unsigned char AnimID,
 		unsigned char& Cycle, unsigned char Orient);
+	void AddTwoPieceSuffix(char* ResRef, unsigned char AnimID,
+		unsigned char& Cycle, unsigned char Orient, int Part);
 	void AddMHRSuffix(char* ResRef, unsigned char AnimID,
 		unsigned char& Cycle, unsigned char Orient, EquipResRefData*& equip);
 	void GetMHREquipmentRef(char* ResRef, unsigned char& Cycle,
@@ -228,6 +245,9 @@ private:
 		char* ResRef, unsigned char& Cycle, int Part, EquipResRefData*& equip);
 	void GetEquipmentResRef(const char* equipRef, bool offhand,
 		char* ResRef, unsigned char& Cycle, EquipResRefData* equip);
+	unsigned char MaybeOverrideStance(unsigned char stance) const;
 };
 
+}
+
 #endif
