@@ -31,6 +31,8 @@
 
 #include "Region.h"
 
+namespace GemRB {
+
 class Actor;
 
 //local variables in creatures are stored in fake opcodes
@@ -70,7 +72,10 @@ class Actor;
 //#define FX_CAN_RESIST_NO_DISPEL     2   //same as 0 (not resistable, not dispellable)
 #define FX_NO_RESIST_CAN_DISPEL    3
 #define FX_CAN_DISPEL               1
-#define FX_CAN_RESIST               3
+
+// Effect save flags (ToBEx)
+#define SF_BYPASS_MIRROR_IMAGE 0x1000000
+#define SF_IGNORE_DIFFICULTY   0x2000000
 
 /**
  * @class Effect
@@ -88,8 +93,8 @@ struct Effect {
 	ieWord unknown2;
 	ieDword Resistance;
 	ieDword Duration;
-	ieWord Probability1;
-	ieWord Probability2;
+	ieWord ProbabilityRangeMax;
+	ieWord ProbabilityRangeMin;
 	//keep these four in one bunch, VariableName will
 	//spread across them
 	ieResRef Resource;
@@ -109,6 +114,8 @@ struct Effect {
 	ieDword MaxAffectedLevel;
 	ieDword Parameter3;
 	ieDword Parameter4;
+	ieDword Parameter5;
+	ieDword Parameter6;
 	ieDword PosX, PosY;
 	ieDword SourceType; //1-item, 2-spell
 	ieResRef Source;
@@ -136,4 +143,6 @@ public:
 // FIXME: what about area spells? They can have map & coordinates as target
 //void AddEffect(Effect* fx, Actor* self, Actor* pretarget);
 
+}
+
 #endif  // ! EFFECT_H
