@@ -36,9 +36,12 @@
 #include <cstdlib>
 #include <list>
 
+namespace GemRB {
+
 class Actor;
 class Map;
 class Scriptable;
+class StringBuffer;
 
 /** Maximum number of different Effect opcodes */
 #define MAX_EFFECTS 512
@@ -89,6 +92,9 @@ class Scriptable;
 #define IMM_SECTYPE_DEC 0x1000
 #define IMM_RESOURCE_DEC 0x2000
 
+//pst immunities
+#define IMM_GUARDIAN  0x80000000
+
 // FIXME: Dice roll should be probably done just once, e.g. when equipping
 // the item, not each time the fx are applied
 // <avenger> the dice values are actually level limits, except in 3 hp modifier functions
@@ -213,6 +219,8 @@ public:
 
 	/* directly removes effects with specified opcode, use effect_reference when you can */
 	void RemoveAllEffects(ieDword opcode) const;
+
+	/* directly removes effects with specified opcode and resource (used by IWD) */
 	void RemoveAllEffectsWithResource(ieDword opcode, const ieResRef resource) const;
 
 	/* removes any effects (delayed or not) which were using projectile */
@@ -233,6 +241,7 @@ public:
 	void RemoveAllDetrimentalEffects(EffectRef &effect_reference, ieDword current) const;
 	void RemoveAllEffectsWithParam(EffectRef &effect_reference, ieDword param2) const;
 	void RemoveAllEffectsWithResource(EffectRef &effect_reference, const ieResRef resource) const;
+	void RemoveAllEffectsWithParamAndResource(EffectRef &effect_reference, ieDword param2, const ieResRef resource) const;
 	void RemoveLevelEffects(ieResRef &Removed, ieDword level, ieDword flags, ieDword match) const;
 
 	/* returns true if the timing method supports simplified duration */
@@ -262,16 +271,18 @@ public:
 	Effect *HasEffectWithParam(EffectRef &effect_reference, ieDword param2) const;
 	Effect *HasEffectWithParamPair(EffectRef &effect_reference, ieDword param1, ieDword param2) const;
 	Effect *HasEffectWithResource(EffectRef &effect_reference, const ieResRef resource) const;
+	Effect *HasSource(const ieResRef source) const;
 	Effect *HasEffectWithSource(EffectRef &effect_reference, const ieResRef source) const;
 	void DecreaseParam1OfEffect(EffectRef &effect_reference, ieDword amount) const;
-	int SpecificDamageBonus(ieDword damage_type) const;
+	int DecreaseParam3OfEffect(EffectRef &effect_reference, ieDword amount, ieDword param2) const;
+	//int SpecificDamageBonus(ieDword damage_type) const;
+	int BonusForParam2(EffectRef &effect_reference, ieDword param2) const;
 	bool HasAnyDispellableEffect() const;
-	//transforming timing modes
-	static void TransformToDelay(ieByte &TimingMode);
 	//getting summarised effects
 	int BonusAgainstCreature(EffectRef &effect_reference, Actor *actor) const;
 	//getting weapon immunity flag
 	bool WeaponImmunity(int enchantment, ieDword weapontype) const;
+	int SumDamageReduction(EffectRef &effect_reference, ieDword weaponEnchantment, int &total) const;
 	//melee and ranged effects
 	void AddWeaponEffects(EffectQueue *fxqueue, EffectRef &fx_ref) const;
 	// checks if spells of type "types" are disabled (usually by armor)
@@ -286,6 +297,7 @@ public:
 	void AffectAllInRange(Map *map, const Point &pos, int idstype, int idsvalue, unsigned int range, Actor *except);
 	/** Lists contents of the queue on a terminal for debugging */
 	void dump() const;
+	void dump(StringBuffer&) const;
 	//resolve effect
 	static int ResolveEffect(EffectRef &effect_reference);
 	static bool match_ids(Actor *target, int table, ieDword value);
@@ -300,15 +312,19 @@ private:
 	static Effect *CreateEffectCopy(Effect *oldfx, ieDword opcode, ieDword param1, ieDword param2);
 	void RemoveAllDetrimentalEffects(ieDword opcode, ieDword current) const;
 	void RemoveAllEffectsWithParam(ieDword opcode, ieDword param2) const;
+	void RemoveAllEffectsWithParamAndResource(ieDword opcode, ieDword param2, const ieResRef resource) const;
 	Effect *HasOpcode(ieDword opcode) const;
 	Effect *HasOpcodeWithParam(ieDword opcode, ieDword param2) const;
 	Effect *HasOpcodeWithParamPair(ieDword opcode, ieDword param1, ieDword param2) const;
 	Effect *HasOpcodeWithResource(ieDword opcode, const ieResRef resource) const;
 	Effect *HasOpcodeWithSource(ieDword opcode, const ieResRef source) const;
 	void DecreaseParam1OfEffect(ieDword opcode, ieDword amount) const;
-	int SpecificDamageBonus(ieDword opcode, ieDword param2) const;
+	int DecreaseParam3OfEffect(ieDword opcode, ieDword amount, ieDword param2) const;
+	int BonusForParam2(ieDword opcode, ieDword param2) const;
 	int BonusAgainstCreature(ieDword opcode, Actor *actor) const;
 	bool WeaponImmunity(ieDword opcode, int enchantment, ieDword weapontype) const;
 };
 
+}
+
 #endif // ! EFFECTQUEUE_H
