@@ -18,9 +18,6 @@
  *
  */
 
-//This class represents game fonts. Fonts are special .bam files.
-//Each cycle stands for a letter. 
-
 #include "Font.h"
 
 #include "win32def.h"
@@ -33,93 +30,71 @@
 
 #include <cassert>
 
-unsigned int lastX = 0;
+namespace GemRB {
 
-#define PARAGRAPH_START_X 5;
+#define SET_BLIT_PALETTE( palette )\
+if (palette != NULL) ((Palette*)palette)->IncRef();\
+if (blitPalette != NULL) blitPalette->Release();\
+blitPalette = palette;
 
-static const Color black = {0, 0, 0, 0};
-
-inline size_t mystrlen(const char* string)
+Font::Font()
+: resRefs(NULL), numResRefs(0), palette(NULL), maxHeight(0)
 {
-	if (!string) {
-		return ( size_t ) 0;
-	}
-	const char* tmp = string;
-	size_t count = 0;
-	while (*tmp != 0) {
-		if (( ( unsigned char ) * tmp ) >= 0xf0) {
-			tmp += 3;
-			count += 3;
+	name[0] = '\0';
+	multibyte = false;
+
+	// TODO: list incomplete
+	// maybe want to externalize this
+	// list compiled form wiki: http://www.gemrb.org/wiki/doku.php?id=engine:encodings
+	const char* multibyteEncodings[] = {
+										// Chinese
+										"GBK", "BIG5",
+										// Korean
+										"EUCKR",
+										// Japanese
+										"SJIS"
+										};
+	const size_t listSize = sizeof(multibyteEncodings) / sizeof(multibyteEncodings[0]);
+	const char* encoding = core->TLKEncoding.c_str();
+
+	for (size_t i = 0; i < listSize; i++) {
+		if (stricmp(encoding, multibyteEncodings[i]) == 0) {
+			multibyte = true;
+			break;
 		}
-		count++;
-		tmp++;
 	}
-	return count;
-}
-
-Font::Font(int w, int h, Palette* pal)
-{
-	lastX = 0;
-	count = 0;
-	FirstChar = 0;
-	sprBuffer = 0;
-
-	width = w;
-	height = h;
-	tmpPixels = (unsigned char*)malloc(width*height);
-
-	memset( xPos, 0, sizeof( xPos) );
-	memset( yPos, 0, sizeof( yPos) );
-
-	pal->IncRef();
-	palette = pal;
-	maxHeight = h;
 }
 
 Font::~Font(void)
 {
-	Video *video = core->GetVideoDriver();
-	gamedata->FreePalette( palette );
-	video->FreeSprite( sprBuffer );
+	blank->release();
+	SetPalette(NULL);
+	free(resRefs);
 }
 
-void Font::FinalizeSprite(bool cK, int index)
+bool Font::AddResRef(const ieResRef resref)
 {
-	sprBuffer = core->GetVideoDriver()->CreateSprite8( width, height, 8, tmpPixels, palette ? palette->col : 0, cK, index );
-	tmpPixels = 0;
+	if (resref) {
+		resRefs = (ieResRef*)realloc(resRefs, sizeof(ieResRef) * ++numResRefs);
+		strnlwrcpy( resRefs[numResRefs - 1], resref, sizeof(ieResRef)-1);
+		return true;
+	}
+	return false;
 }
 
-void Font::AddChar(unsigned char* spr, int w, int h, short xPos, short yPos)
+bool Font::MatchesResRef(const ieResRef resref)
 {
-	if (!spr) {
-		size[count].x = 0;
-		size[count].y = 0;
-		size[count].w = 0;
-		size[count].h = 0;
-		this->xPos[count] = 0;
-		this->yPos[count] = 0;
-		count++;
-		return;
+	for (int i=0; i < numResRefs; i++)
+	{
+		if (strnicmp( resref, resRefs[i], sizeof(ieResRef)-1) == 0){
+			return true;
+		}
 	}
-	unsigned char * currPtr = tmpPixels + lastX;
-	unsigned char * srcPtr = ( unsigned char * ) spr;
-	for (int y = 0; y < h; y++) {
-		memcpy( currPtr, srcPtr, w );
-		srcPtr += w;
-		currPtr += width;
-	}
-	size[count].x = lastX;
-	size[count].y = 0;
-	size[count].w = w;
-	size[count].h = h;
-	this->xPos[count] = xPos;
-	this->yPos[count] = yPos;
-	count++;
-	lastX += w;
+	return false;
 }
 
 void Font::PrintFromLine(int startrow, Region rgn, const unsigned char* string,
-	Palette* hicolor, unsigned char Alignment, Font* initials,
+	Palette* hicolor, ieByte Alignment, Font* initials,
 	Sprite2D* cursor, unsigned int curpos, bool NoColor) const
 {
 	bool enablecap=false;
@@ -128,55 +103,53 @@ void Font::PrintFromLine(int startrow, Region rgn, const unsigned char* string,
 	int last_initial_row = 0;
 	int initials_x = 0;
 	int initials_row = 0;
-	unsigned char currCap = 0;
+	ieWord currCap = 0;
+	ieWord* tmp = NULL;
+	size_t len = GetDoubleByteString(string, tmp);
 
-	if (initials)
+	int num_empty_rows = 0;
+
+	if (initials && initials != this)
 	{
 		capital=1;
 		enablecap=true;
-		initials_rows = ((initials->maxHeight - 1) / maxHeight) - (startrow - 1);
-
-		if (startrow > 0 && initials_rows > 0) { // we need to look back to get the cap
-			currCap = string[0] - 1;
-			last_initial_row = (startrow - 1);
-			initials_rows--;
+		initials_rows = 1 + ((initials->maxHeight - 1) / maxHeight); // ceiling
+		currCap = string[0];
+		if ((startrow > 0 && initials_rows > 0) || (len > 0 && isspace(currCap))) { // we need to look back to get the cap
+			while(isspace(currCap) && num_empty_rows < (int)len){//we cant cap whiteSpace so keep looking
+				currCap = string[++num_empty_rows];
+				// WARNING: this assumes all preceeding whiteSpace is an empty line
+			}
+			last_initial_row = startrow - 1; // always the row before current since this cannot be the first row
+			initials_rows = initials_rows - (startrow + 1) + num_empty_rows; // startrow + 1 because start row is 0 based, but initials_rows is 1 based
 		}
 	}
 
-	unsigned int psx = PARAGRAPH_START_X;
+	unsigned int psx = IE_FONT_PADDING;
 	Palette *pal = hicolor;
 	if (!pal) {
 		pal = palette;
 	}
 
-	if (initials==this) {
-		enablecap=false;
-	}
+	Palette* blitPalette = NULL;
+	SET_BLIT_PALETTE(pal);
 
-	sprBuffer->SetPalette( pal );
-	size_t len = strlen( ( char* ) string );
-	char* tmp = ( char* ) malloc( len + 1 );
-	memcpy( tmp, ( char * ) string, len + 1 );
 	SetupString( tmp, rgn.w, NoColor, initials, enablecap );
 
 	if (startrow) enablecap=false;
-	int ystep = 0;
+	int ystep;
 	if (Alignment & IE_FONT_SINGLE_LINE) {
-		for (size_t i = 0; i < len; i++) {
-			int height = yPos[( unsigned char ) tmp[i] - 1];
-			if (ystep < height)
-				ystep = height;
-		}
+		ystep = CalcStringHeight(tmp, len, NoColor);
+		if (!ystep) ystep = maxHeight;
 	} else {
-		ystep = size[1].h;
+		ystep = maxHeight;
 	}
-	if (!ystep) ystep = maxHeight;
 	int x = psx, y = ystep;
 	int w = CalcStringWidth( tmp, NoColor );
 	if (Alignment & IE_FONT_ALIGN_CENTER) {
 		x = ( rgn.w - w) / 2;
 	} else if (Alignment & IE_FONT_ALIGN_RIGHT) {
-		x = ( rgn.w - w );
+		x = ( rgn.w - w ) - IE_FONT_PADDING;
 	}
 	if (Alignment & IE_FONT_ALIGN_MIDDLE) {
 		int h = 0;
@@ -187,26 +160,28 @@ void Font::PrintFromLine(int startrow, Region rgn, const unsigned char* string,
 		h = h * ystep;
 		y += ( rgn.h - h ) / 2;
 	} else if (Alignment & IE_FONT_ALIGN_BOTTOM) {
-		int h = 1;
+		int h = 0;
 		for (size_t i = 0; i <= len; i++) {
 			if (( tmp[i] == 0 ) || ( tmp[i] == '\n' ))
 				h++;
 		}
 		h = h * ystep;
-		y += ( rgn.h - h );
+		y += ( rgn.h - h ) - IE_FONT_PADDING;
 	} else if (Alignment & IE_FONT_ALIGN_TOP) {
-		y += 5;
+		y += IE_FONT_PADDING;
 	}
 
 	Video* video = core->GetVideoDriver();
+	const Sprite2D* currGlyph;
+	ieWord currChar = '\0';
 	int row = 0;
 	for (size_t i = 0; i < len; i++) {
-		if (( ( unsigned char ) tmp[i] ) == '[' && !NoColor) {
+		if (( tmp[i] ) == '[' && !NoColor) {
 			i++;
 			char tag[256];
 			tag[0]=0;
 
-			for (int k = 0; k < 256 && i<len; k++) {
+			for (size_t k = 0; k < 256 && i<len; k++) {
 				if (tmp[i] == ']') {
 					tag[k] = 0;
 					break;
@@ -226,14 +201,14 @@ void Font::PrintFromLine(int startrow, Region rgn, const unsigned char* string,
 				unsigned int r,g,b;
 				if (sscanf( tag, "color=%02X%02X%02X", &r, &g, &b ) != 3)
 					continue;
-				const Color c = {(unsigned char) r,(unsigned char)g, (unsigned char)b, 0};
+				const Color c = {(unsigned char)r, (unsigned char)g, (unsigned char)b, 0};
 				Palette* newPal = core->CreatePalette( c, palette->back );
-				sprBuffer->SetPalette( newPal );
+				SET_BLIT_PALETTE(newPal);
 				gamedata->FreePalette( newPal );
 				continue;
 			}
 			if (stricmp( tag, "/color" ) == 0) {
-				sprBuffer->SetPalette( pal );
+				SET_BLIT_PALETTE(pal);
 				continue;
 			}
 			if (stricmp( "p", tag ) == 0) {
@@ -241,7 +216,7 @@ void Font::PrintFromLine(int startrow, Region rgn, const unsigned char* string,
 				continue;
 			}
 			if (stricmp( "/p", tag ) == 0) {
-				psx = PARAGRAPH_START_X;
+				psx = IE_FONT_PADDING;
 			}
 			continue;
 		}
@@ -268,62 +243,72 @@ void Font::PrintFromLine(int startrow, Region rgn, const unsigned char* string,
 			}
 			continue;
 		}
-		unsigned char currChar = ( unsigned char ) tmp[i] - 1;
+		currChar = tmp[i];
 		if (initials && capital && enablecap) {
 			currCap = currChar;
 			x = initials->PrintInitial( x, y, rgn, currChar );
 			initials_x = x;
 
 			//how many more lines to be indented (one was already indented)
-			initials_rows = (initials->maxHeight-1)/maxHeight;
+			initials_rows = (initials->maxHeight - 1) / maxHeight;
+			initials_rows += num_empty_rows;
 			initials_row = row;
 			last_initial_row = initials_row;
 
 			enablecap = false;
 			continue;
-		}else if(currCap && row > last_initial_row && (row - initials_row) <= ((initials->maxHeight-1)/maxHeight)){
+		} else if (initials && currCap
+				   && row > last_initial_row
+				   && (row - num_empty_rows - initials_row) <= ((initials->maxHeight-1)/maxHeight)){
 			// means this row doesnt have a cap, but a preceeding one did and its overlapping this row
-			x = initials->PrintInitial( x, y - (maxHeight * (row - initials_row)), rgn, currCap );
+			int initY = y;
+			if (!num_empty_rows || row > num_empty_rows) {// num_empty_rows is for scrolling text areas
+				initY = (y - (maxHeight * (row - initials_row - num_empty_rows)));
+			}
+			x = initials->PrintInitial( x, initY, rgn, currCap );
 			initials_x = x;
-			x += psx;
 			last_initial_row++;
+			if (num_empty_rows && row <= num_empty_rows) continue;
+			else x += psx;
 		}
-		video->BlitSpriteRegion( sprBuffer, size[currChar],
-			x + rgn.x, y + rgn.y - yPos[currChar], true, &rgn );
+		if (i > 0) {
+			// kerning
+			x -= GetKerningOffset(tmp[i-1], currChar);
+		}
+		currGlyph = GetCharSprite(currChar);
+		video->BlitSprite(currGlyph, x + rgn.x, y + rgn.y, true, &rgn, blitPalette);
 		if (cursor && ( i == curpos )) {
-			video->BlitSprite( cursor, x + rgn.x,
-				y + rgn.y, true, &rgn );
+			video->BlitSprite( cursor, x + rgn.x, y + rgn.y, true, &rgn );
 		}
-		x += size[currChar].w;
+		x += currGlyph->Width;
 	}
 	if (cursor && ( curpos == len )) {
-		video->BlitSprite( cursor, x + rgn.x,
-			y + rgn.y, true, &rgn );
+		video->BlitSprite( cursor, x + rgn.x, y + rgn.y, true, &rgn );
 	}
+	SET_BLIT_PALETTE(NULL);
 	free( tmp );
 }
 
 void Font::Print(Region rgn, const unsigned char* string, Palette* hicolor,
-	unsigned char Alignment, bool anchor, Font* initials,
+	ieByte Alignment, bool anchor, Font* initials,
 	Sprite2D* cursor, unsigned int curpos, bool NoColor) const
 {
-	Print(rgn, rgn, string, hicolor, Alignment, anchor, initials, cursor, curpos, NoColor);
+	Region cliprgn = rgn;
+	if (!anchor) {
+		Region Viewport = core->GetVideoDriver()->GetViewport();
+		cliprgn.x -= Viewport.x;
+		cliprgn.y -= Viewport.y;
+	}
+	Print(cliprgn, rgn, string, hicolor, Alignment, anchor, initials, cursor, curpos, NoColor);
 }
 
 void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
-	Palette* hicolor, unsigned char Alignment, bool anchor, Font* initials,
+	Palette* hicolor, ieByte Alignment, bool anchor, Font* initials,
 	Sprite2D* cursor, unsigned int curpos, bool NoColor) const
 {
-	bool enablecap=false;
-	int capital = 0;
-	if (initials)
-	{
-		capital=1;
-		enablecap=true;
-	}
-	(void)enablecap; //HACK: shut up unused-but-set warnings, until the var is reused
+	int capital = (initials) ? 1 : 0;
 
-	unsigned int psx = PARAGRAPH_START_X;
+	unsigned int psx = IE_FONT_PADDING;
 	Palette* pal = hicolor;
 	if (!pal) {
 		pal = palette;
@@ -332,10 +317,11 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 		initials = NULL;
 	}
 
-	sprBuffer->SetPalette( pal );
-	size_t len = strlen( ( char* ) string );
-	char* tmp = ( char* ) malloc( len + 1 );
-	memcpy( tmp, ( char * ) string, len + 1 );
+	Palette* blitPalette = NULL;
+	SET_BLIT_PALETTE( pal );
+
+	ieWord* tmp = NULL;
+	size_t len = GetDoubleByteString(string, tmp);
 	while (len > 0 && (tmp[len - 1] == '\n' || tmp[len - 1] == '\r')) {
 		// ignore trailing newlines
 		tmp[len - 1] = 0;
@@ -343,19 +329,13 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 	}
 
 	SetupString( tmp, rgn.w, NoColor, initials, capital );
-	int ystep = 0;
+	int ystep;
 	if (Alignment & IE_FONT_SINGLE_LINE) {
-		
-		for (size_t i = 0; i < len; i++) {
-			if (tmp[i] == 0) continue;
-			int height = yPos[( unsigned char ) tmp[i] - 1];
-			if (ystep < height)
-				ystep = height;
-		}
+		ystep = CalcStringHeight(tmp, len, NoColor);
+		if (!ystep) ystep = maxHeight;
 	} else {
-		ystep = size[1].h;
+		ystep = maxHeight;
 	}
-	if (!ystep) ystep = maxHeight;
 	int x = psx, y = ystep;
 	Video* video = core->GetVideoDriver();
 
@@ -364,7 +344,7 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 		x = ( rgn.w - w ) / 2;
 	} else if (Alignment & IE_FONT_ALIGN_RIGHT) {
 		int w = CalcStringWidth( tmp, NoColor );
-		x = ( rgn.w - w );
+		x = ( rgn.w - w ) - IE_FONT_PADDING;
 	}
 
 	if (Alignment & IE_FONT_ALIGN_MIDDLE) {
@@ -376,22 +356,25 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 		h = h * ystep;
 		y += ( rgn.h - h ) / 2;
 	} else if (Alignment & IE_FONT_ALIGN_BOTTOM) {
-		int h = 1;
+		int h = 0;
 		for (size_t i = 0; i <= len; i++) {
 			if (tmp[i] == 0)
 				h++;
 		}
 		h = h * ystep;
-		y += ( rgn.h - h );
+		y += ( rgn.h - h ) - IE_FONT_PADDING;
 	} else if (Alignment & IE_FONT_ALIGN_TOP) {
-		y += 5;
+		y += IE_FONT_PADDING;
 	}
+
+	ieWord currChar = '\0';
+	const Sprite2D* currGlyph = NULL;
 	for (size_t i = 0; i < len; i++) {
-		if (( ( unsigned char ) tmp[i] ) == '[' && !NoColor) {
+		if (( tmp[i] ) == '[' && !NoColor) {
 			i++;
 			char tag[256];
 			tag[0]=0;
-			for (int k = 0; k < 256 && i<len; k++) {
+			for (size_t k = 0; k < 256 && i<len; k++) {
 				if (tmp[i] == ']') {
 					tag[k] = 0;
 					break;
@@ -401,24 +384,21 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 
 			if (strnicmp( tag, "capital=",8)==0) {
 				sscanf( tag, "capital=%d", &capital);
-				if (capital) {
-					enablecap=true;
-				}
 				continue;
 			}
-			
+
 			if (strnicmp( tag, "color=", 6 ) == 0) {
 				unsigned int r,g,b;
 				if (sscanf( tag, "color=%02X%02X%02X", &r, &g, &b ) != 3)
 					continue;
-				const Color c = {(unsigned char) r,(unsigned char) g,(unsigned char)  b, 0};
+				const Color c = {(unsigned char)r, (unsigned char)g, (unsigned char)b, 0};
 				Palette* newPal = core->CreatePalette( c, palette->back );
-				sprBuffer->SetPalette( newPal );
+				SET_BLIT_PALETTE(newPal);
 				gamedata->FreePalette( newPal );
 				continue;
 			}
 			if (stricmp( tag, "/color" ) == 0) {
-				sprBuffer->SetPalette( pal );
+				SET_BLIT_PALETTE(pal);
 				continue;
 			}
 			if (stricmp( "p", tag ) == 0) {
@@ -426,7 +406,7 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 				continue;
 			}
 			if (stricmp( "/p", tag ) == 0) {
-				psx = PARAGRAPH_START_X;
+				psx = IE_FONT_PADDING;
 				continue;
 			}
 			continue;
@@ -443,52 +423,90 @@ void Font::Print(Region cliprgn, Region rgn, const unsigned char* string,
 			}
 			continue;
 		}
-		unsigned char currChar = ( unsigned char ) tmp[i] - 1;
+		currChar = tmp[i];
+		currGlyph = GetCharSprite(currChar);
 		if (initials && capital) {
 			x = initials->PrintInitial( x, y, rgn, currChar );
-			enablecap=false;
 			continue;
 		}
-		video->BlitSpriteRegion( sprBuffer, size[currChar],
-			x + rgn.x, y + rgn.y - yPos[currChar],
-			anchor, &cliprgn );
+
+		if (i > 0) {
+			// kerning
+			x -= GetKerningOffset(tmp[i-1], currChar);
+		}
+
+		video->BlitSprite(currGlyph, x + rgn.x, y + rgn.y, anchor, &cliprgn, blitPalette);
+
 		if (cursor && ( curpos == i ))
 			video->BlitSprite( cursor, x + rgn.x, y + rgn.y, anchor, &cliprgn );
-		x += size[currChar].w;
+		x += currGlyph->Width;
 	}
 	if (cursor && ( curpos == len )) {
 		video->BlitSprite( cursor, x + rgn.x, y + rgn.y, anchor, &cliprgn );
 	}
+	SET_BLIT_PALETTE(NULL);
 	free( tmp );
 }
 
-int Font::PrintInitial(int x, int y, const Region &rgn, unsigned char currChar) const
+int Font::PrintInitial(int x, int y, const Region &rgn, ieWord currChar) const
 {
-	Video *video = core->GetVideoDriver();
-	video->BlitSpriteRegion( sprBuffer, size[currChar],
-		x + rgn.x, y + rgn.y - yPos[currChar], true, &rgn );
-	x += size[currChar].w;
+	const Sprite2D* glyph = GetCharSprite(currChar);
+	core->GetVideoDriver()->BlitSprite(glyph, x + rgn.x, y + rgn.y, true, &rgn);
+
+	x += glyph->Width;
 	return x;
 }
 
-int Font::CalcStringWidth(const char* string, bool NoColor) const
+int Font::CalcStringWidth(const unsigned char* string, bool NoColor) const
 {
-	size_t ret = 0, len = strlen( string );
+	ieWord* tmp = NULL;
+	GetDoubleByteString(string, tmp);
+	int width = CalcStringWidth(tmp, NoColor);
+	free(tmp);
+	return width;
+}
+
+int Font::CalcStringWidth(const ieWord* string, bool NoColor) const
+{
+	size_t ret = 0, len = dbStrLen(string);
 	for (size_t i = 0; i < len; i++) {
-		if (( ( unsigned char ) string[i] ) == '[' && !NoColor) {
-			while(i<len && ((unsigned char) string[i]) != ']') {
+		if (( string[i] ) == '[' && !NoColor) {
+			i++; // cannot be ']' when it is '['
+			while(i<len && (string[i]) != ']') {
 				i++;
 			}
+		} else {
+			ret += GetCharSprite(string[i])->Width;
 		}
-		ret += size[( unsigned char ) string[i] - 1].w;
 	}
 	return ( int ) ret;
 }
 
-void Font::SetupString(char* string, unsigned int width, bool NoColor, Font *initials, bool enablecap) const
+int Font::CalcStringHeight(const ieWord* string, unsigned int len, bool NoColor) const
 {
-	size_t len = strlen( string );
-	unsigned int psx = PARAGRAPH_START_X;
+	int h, max = 0;
+	for (unsigned int i = 0; i < len; i++) {
+		if (( string[i] ) == '[' && !NoColor) {
+			i++; // cannot be ']' when it is '['
+			while(i<len && (string[i]) != ']') {
+				i++;
+			}
+		} else {
+			h = GetCharSprite(string[i])->Height;
+			//the space check is here to hack around overly high frames
+			//in some bg1 fonts that throw vertical alignment off
+			if (h > max && string[i] != ' ') {
+				max = h;
+			}
+		}
+	}
+	return max;
+}
+
+void Font::SetupString(ieWord* string, unsigned int width, bool NoColor, Font *initials, bool enablecap) const
+{
+	size_t len = dbStrLen(string);
+	unsigned int psx = IE_FONT_PADDING;
 	int lastpos = 0;
 	unsigned int x = psx, wx = 0;
 	bool endword = false;
@@ -526,7 +544,7 @@ void Font::SetupString(char* string, unsigned int width, bool NoColor, Font *ini
 			endword = true;
 			continue;
 		}
-		if (( ( unsigned char ) string[pos] ) == '[' && !NoColor) {
+		if (( string[pos] ) == '[' && !NoColor) {
 			pos++;
 			if (pos>=len)
 				break;
@@ -552,24 +570,21 @@ void Font::SetupString(char* string, unsigned int width, bool NoColor, Font *ini
 				continue;
 			}
 			if (stricmp( "/p", tag ) == 0) {
-				psx = PARAGRAPH_START_X;
+				psx = IE_FONT_PADDING;
 				continue;
 			}
 			continue;
 		}
 
-		if (string[pos] && string[pos] != ' ') {
-			string[pos] = ( unsigned char ) (string[pos] - FirstChar);
-		}
-
-		wx += size[( unsigned char ) string[pos] - 1].w;
 		if (initials && enablecap) {
-			wx += initials->size[(unsigned char) string[pos] - 1].w;
+			wx += initials->GetCharSprite(string[pos])->Width;
 			enablecap=false;
-			initials_x = wx;
+			initials_x = wx + psx;
 			//how many more lines to be indented (one was already indented)
-			initials_rows = (initials->maxHeight-1)/maxHeight;
+			initials_rows = (initials->maxHeight - 1) / maxHeight;
 			continue;
+		} else {
+			wx += GetCharSprite(string[pos])->Width;
 		}
 		if (( string[pos] == ' ' ) || ( string[pos] == '-' )) {
 			x += wx;
@@ -580,6 +595,42 @@ void Font::SetupString(char* string, unsigned int width, bool NoColor, Font *ini
 	}
 }
 
+size_t Font::GetDoubleByteString(const unsigned char* string, ieWord* &dbString) const
+{
+	size_t len = strlen((char*)string);
+	dbString = (ieWord*)malloc((len+1) * sizeof(ieWord));
+	size_t dbLen = 0;
+	for(size_t i=0; i<len; ++i)
+	{
+		// we are assuming that every multibyte encoding uses single bytes for chars 32 - 127
+		if( multibyte && (i+1 < len) && (string[i] >= 128 || string[i] < 32)) { // this is a double byte char
+			dbString[dbLen] = (string[i+1] << 8) + string[i];
+			++i;
+		} else
+			dbString[dbLen] = string[i];
+		assert(dbString[dbLen] != 0);
+		++dbLen;
+	}
+	dbString[dbLen] = '\0';
+
+	// we dont always use everything we allocated.
+	// realloc in this case to avoid static anylizer warnings about "garbage values"
+	// since this realloc always truncates it *should* be quick
+	dbString = (ieWord*)realloc(dbString, (dbLen+1) * sizeof(ieWord));
+
+	return dbLen;
+}
+
+void Font::SetName(const char* newName)
+{
+	strnlwrcpy( name, newName, sizeof(name)-1);
+
+	if (strnicmp(name, "STATES", 6) == 0) {
+		// state fonts are NEVER multibyte; regardless of TKL encoding.
+		multibyte = false;
+	}
+}
+
 Palette* Font::GetPalette() const
 {
 	assert(palette);
@@ -589,12 +640,19 @@ Palette* Font::GetPalette() const
 
 void Font::SetPalette(Palette* pal)
 {
+	if (pal) pal->IncRef();
 	if (palette) palette->Release();
-	pal->IncRef();
 	palette = pal;
 }
 
-void Font::SetFirstChar( unsigned char first)
+int Font::dbStrLen(const ieWord* string) const
 {
-	FirstChar = first;
+	if (string == NULL) return 0;
+	int count = 0;
+	for ( ; string[count] != 0; count++)
+		continue; // intentionally empty loop
+	return count;
+}
+
+#undef SET_BLIT_PALETTE
 }
