@@ -32,19 +32,18 @@
 
 #include <vector>
 
+namespace GemRB {
+
+enum FontStyle {
+	NORMAL = 0x00,
+	BOLD = 0x01,
+	ITALIC = 0x02,
+	UNDERLINE = 0x04
+};
+
 class Palette;
 class Sprite2D;
 
-struct StringList {
-	Sprite2D*** strings;
-	unsigned int* heights;
-	unsigned int* lengths;
-	int StringCount;
-	int starty;
-	int curx;
-	int cury;
-};
-
 #define IE_FONT_ALIGN_LEFT   0x00
 #define IE_FONT_ALIGN_CENTER 0x01
 #define IE_FONT_ALIGN_RIGHT  0x02
@@ -53,60 +52,74 @@ struct StringList {
 #define IE_FONT_ALIGN_MIDDLE 0x20 //Only for single line Text
 #define IE_FONT_SINGLE_LINE  0x40
 
+#define IE_FONT_PADDING 5
+
 /**
  * @class Font
  * Class for using and manipulating images serving as fonts
  */
 
 class GEM_EXPORT Font {
-private:
-	int count;
+protected:
+	ieResRef* resRefs;
+	int numResRefs;
+	char name[20];
+
 	Palette* palette;
-	Sprite2D* sprBuffer;
-	unsigned char FirstChar;
+	Sprite2D* blank;
 
-	short xPos[256];
-	short yPos[256];
+	bool multibyte;
 
-	// For the temporary bitmap
-	unsigned char* tmpPixels;
-	unsigned int width, height;
 public:
-	/** ResRef of the Font image */
-	ieResRef ResRef;
 	int maxHeight;
-	Region size[256];
-public:
-	Font(int w, int h, Palette* palette);
-	~Font(void);
-	void AddChar(unsigned char* spr, int w, int h, short xPos, short yPos);
-	/** Call this after adding all characters */
-	void FinalizeSprite(bool cK, int index);
 
-	void Print(Region cliprgn, Region rgn, const unsigned char* string,
-		Palette* color, unsigned char Alignment, bool anchor = false,
-		Font* initials = NULL, Sprite2D* cursor = NULL,
-		unsigned int curpos = 0, bool NoColor = false) const;
-	void Print(Region rgn, const unsigned char* string, Palette* color,
-		unsigned char Alignment, bool anchor = false,
-		Font* initials = NULL, Sprite2D* cursor = NULL,
-		unsigned int curpos = 0, bool NoColor = false) const;
-	void PrintFromLine(int startrow, Region rgn, const unsigned char* string,
-		Palette* color, unsigned char Alignment,
-		Font* initials = NULL, Sprite2D* cursor = NULL,
-		unsigned int curpos = 0, bool NoColor = false) const;
+public:
+	Font();
+	virtual ~Font(void);
+
+	//allow reading but not setting glyphs
+	virtual const Sprite2D* GetCharSprite(ieWord chr) const = 0;
+
+	bool AddResRef(const ieResRef resref);
+	bool MatchesResRef(const ieResRef resref);
+
+	const char* GetName() const {return name;};
+	void SetName(const char* newName);
+
+	virtual ieWord GetPointSize() const {return 0;};
+	virtual FontStyle GetStyle() const {return NORMAL;};
 
 	Palette* GetPalette() const;
 	void SetPalette(Palette* pal);
-	/** Returns width of the string rendered in this font in pixels */
-	int CalcStringWidth(const char* string, bool NoColor = false) const;
-	void SetupString(char* string, unsigned int width, bool NoColor = false, Font *initials = NULL, bool enablecap = false) const;
-	/** Sets ASCII code of the first character in the font.
-	 * (it allows remapping numeric fonts from \000 to '0') */
-	void SetFirstChar(unsigned char first);
 
+	// Printing methods
+	void Print(Region cliprgn, Region rgn, const unsigned char* string,
+		Palette* color, ieByte Alignment, bool anchor = false,
+		Font* initials = NULL, Sprite2D* cursor = NULL,
+		unsigned int curpos = 0, bool NoColor = false) const;
+	void Print(Region rgn, const unsigned char* string, Palette* color,
+		ieByte Alignment, bool anchor = false,
+		Font* initials = NULL, Sprite2D* cursor = NULL,
+		unsigned int curpos = 0, bool NoColor = false) const;
+	void PrintFromLine(int startrow, Region rgn, const unsigned char* string,
+		Palette* color, ieByte Alignment,
+		Font* initials = NULL, Sprite2D* cursor = NULL,
+		unsigned int curpos = 0, bool NoColor = false) const;
+
+	/** Returns width of the string rendered in this font in pixels */
+	int CalcStringWidth(const unsigned char* string, bool NoColor = false) const;
+	void SetupString(ieWord* string, unsigned int width, bool NoColor = false, Font *initials = NULL, bool enablecap = false) const;
+	size_t GetDoubleByteString(const unsigned char* string, ieWord* &dbString) const;
+
+protected:
+	virtual int GetKerningOffset(ieWord /*leftChr*/, ieWord /*rightChr*/) const {return 0;};
 private:
-	int PrintInitial(int x, int y, const Region &rgn, unsigned char currChar) const;
+	int PrintInitial(int x, int y, const Region &rgn, ieWord currChar) const;
+	int CalcStringWidth(const ieWord* string, bool NoColor = false) const;
+	int CalcStringHeight(const ieWord* string, unsigned int len, bool NoColor) const;
+	int dbStrLen(const ieWord* string) const;
 };
 
+}
+
 #endif
