@@ -35,6 +35,8 @@
 #include "Variables.h"
 #include "Video.h"
 
+namespace GemRB {
+
 Button::Button()
 {
 	Unpressed = Pressed = Selected = Disabled = NULL;
@@ -68,6 +70,7 @@ Button::Button()
 	memset( borders, 0, sizeof( borders ));
 	starttime = 0;
 	Anchor.null();
+	PushOffset = Point(2, 2);
 }
 Button::~Button()
 {
@@ -100,7 +103,7 @@ void Button::SetImage(unsigned char type, Sprite2D* img)
 			Unpressed = img;
 			break;
 
-		case IE_GUI_BUTTON_SECOND:
+		case IE_GUI_BUTTON_FAKEPRESSED:
 		case IE_GUI_BUTTON_PRESSED:
 			core->GetVideoDriver()->FreeSprite( Pressed );
 			Pressed = img;
@@ -112,7 +115,7 @@ void Button::SetImage(unsigned char type, Sprite2D* img)
 			break;
 
 		case IE_GUI_BUTTON_DISABLED:
-		case IE_GUI_BUTTON_THIRD:
+		case IE_GUI_BUTTON_FAKEDISABLED:
 			core->GetVideoDriver()->FreeSprite( Disabled );
 			Disabled = img;
 			break;
@@ -133,17 +136,23 @@ void Button::CloseUpColor()
 	if (newtime<starttime) {
 		return;
 	}
-	SourceRGB.r = (SourceRGB.r + DestRGB.r) / 2;
-	SourceRGB.g = (SourceRGB.g + DestRGB.g) / 2;
-	SourceRGB.b = (SourceRGB.b + DestRGB.b) / 2;
-	SourceRGB.a = (SourceRGB.a + DestRGB.a) / 2;
-	if (SourceRGB.r == DestRGB.r &&
-		SourceRGB.g == DestRGB.g &&
-		SourceRGB.b == DestRGB.b &&
-		SourceRGB.a == DestRGB.a) {
+
+	Color nc;
+
+	nc.r = (SourceRGB.r + DestRGB.r) / 2;
+	nc.g = (SourceRGB.g + DestRGB.g) / 2;
+	nc.b = (SourceRGB.b + DestRGB.b) / 2;
+	nc.a = (SourceRGB.a + DestRGB.a) / 2;
+	if (SourceRGB.r == nc.r &&
+		SourceRGB.g == nc.g &&
+		SourceRGB.b == nc.b &&
+		SourceRGB.a == nc.a) {
+		SourceRGB = DestRGB;
 		starttime = 0;
 		return;
 	}
+
+	SourceRGB = nc;
 	starttime = newtime + 40;
 }
 
@@ -171,7 +180,7 @@ void Button::Draw(unsigned short x, unsigned short y)
 				Image = Unpressed;
 				break;
 
-			case IE_GUI_BUTTON_SECOND:
+			case IE_GUI_BUTTON_FAKEPRESSED:
 			case IE_GUI_BUTTON_PRESSED:
 				Image = Pressed;
 				if (! Image)
@@ -185,7 +194,7 @@ void Button::Draw(unsigned short x, unsigned short y)
 				break;
 
 			case IE_GUI_BUTTON_DISABLED:
-			case IE_GUI_BUTTON_THIRD:
+			case IE_GUI_BUTTON_FAKEDISABLED:
 				Image = Disabled;
 				if (! Image)
 					Image = Unpressed;
@@ -202,28 +211,56 @@ void Button::Draw(unsigned short x, unsigned short y)
 
 	if (State == IE_GUI_BUTTON_PRESSED) {
 		//shift the writing/border a bit
-		x+= 2;
-		y+= 2;
+		x+= PushOffset.x;
+		y+= PushOffset.y;
 	}
 
 	// Button picture
+	if (AnimPicture) {
+		int xOffs = ( Width / 2 ) - ( AnimPicture->Width / 2 );
+		int yOffs = ( Height / 2 ) - ( AnimPicture->Height / 2 );
+		Region r( x + XPos + xOffs, y + YPos + yOffs, (int)(AnimPicture->Width * Clipping), AnimPicture->Height );
+
+		if (Flags & IE_GUI_BUTTON_CENTER_PICTURES) {
+			video->BlitSprite( AnimPicture, x + XPos + xOffs + AnimPicture->XPos, y + YPos + yOffs + AnimPicture->YPos, true, &r );
+		} else {
+			video->BlitSprite( AnimPicture, x + XPos + xOffs, y + YPos + yOffs, true, &r );
+		}
+	}
+
+	// Button picture
+	int picXPos = 0, picYPos = 0;
 	if (Picture  && (Flags & IE_GUI_BUTTON_PICTURE) ) {
 		// Picture is drawn centered
-		int xOffs = ( Width / 2 ) - ( Picture->Width / 2 );
-		int yOffs = ( Height / 2 ) - ( Picture->Height / 2 );
+		picXPos = ( Width / 2 ) - ( Picture->Width / 2 ) + x + XPos;
+		picYPos = ( Height / 2 ) - ( Picture->Height / 2 ) + y + YPos;
 		if (Flags & IE_GUI_BUTTON_HORIZONTAL) {
-			xOffs += x + XPos + Picture->XPos;
-			yOffs += y + YPos + Picture->YPos;
-			video->BlitSprite( Picture, xOffs, yOffs, true );
-			Region r = Region( xOffs, yOffs + (int) (Picture->Height * Clipping), Picture->Width, (int) (Picture->Height*(1.0 - Clipping)) );
-			video->DrawRect( r, SourceRGB, true );
+			picXPos += Picture->XPos;
+			picYPos += Picture->YPos;
+
+			// Clipping: 0 = overlay over full button, 1 = no overlay
+			int overlayHeight = Picture->Height * (1.0 - Clipping);
+			if (overlayHeight < 0)
+				overlayHeight = 0;
+			if (overlayHeight >= Picture->Height)
+				overlayHeight = Picture->Height;
+			int buttonHeight = Picture->Height - overlayHeight;
+
+			Region rb = Region(picXPos, picYPos, Picture->Width, buttonHeight);
+			Region ro = Region(picXPos, picYPos + buttonHeight, Picture->Width, overlayHeight);
+
+			video->BlitSprite( Picture, picXPos, picYPos, true, &rb );
+
+			// TODO: Add an option to add BLIT_GREY to the flags
+			video->BlitGameSprite( Picture, picXPos, picYPos, BLIT_TINTED, SourceRGB, 0, 0, &ro, true);
+
 			// do NOT uncomment this, you can't change Changed or invalidate things from
 			// the middle of Window::DrawWindow() -- it needs moving to somewhere else
 			//CloseUpColor();
 		}
 		else {
-			Region r( x + XPos + xOffs, y + YPos + yOffs, (int)(Picture->Width * Clipping), Picture->Height );
-			video->BlitSprite( Picture, x + XPos + xOffs + Picture->XPos, y + YPos + yOffs + Picture->YPos, true, &r );
+			Region r( picXPos, picYPos, (int)(Picture->Width * Clipping), Picture->Height );
+			video->BlitSprite( Picture, picXPos + Picture->XPos, picYPos + Picture->YPos, true, &r );
 		}
 	}
 
@@ -250,19 +287,6 @@ void Button::Draw(unsigned short x, unsigned short y)
 		}
 	}
 
-	// Button picture
-	if (AnimPicture) {
-		int xOffs = ( Width / 2 ) - ( AnimPicture->Width / 2 );
-		int yOffs = ( Height / 2 ) - ( AnimPicture->Height / 2 );
-		Region r( x + XPos + xOffs, y + YPos + yOffs, (int)(AnimPicture->Width * Clipping), AnimPicture->Height );
-
-		if (Flags & IE_GUI_BUTTON_CENTER_PICTURES) {
-			video->BlitSprite( AnimPicture, x + XPos + xOffs + AnimPicture->XPos, y + YPos + yOffs + AnimPicture->YPos, true, &r );
-		} else {
-			video->BlitSprite( AnimPicture, x + XPos + xOffs, y + YPos + yOffs, true, &r );
-		}
-	}
-
 	// Button label
 	if (hasText && ! ( Flags & IE_GUI_BUTTON_NO_TEXT )) {
 		Palette* ppoi = normal_palette;
@@ -291,9 +315,18 @@ void Button::Draw(unsigned short x, unsigned short y)
 		if (! (Flags & IE_GUI_BUTTON_MULTILINE)) {
 			align |= IE_FONT_SINGLE_LINE;
 		}
-		font->Print( Region( x + XPos, y + YPos, Width - 2, Height - 2),
-			( unsigned char * ) Text, ppoi,
-			(ieByte) align, true );
+
+		Region r;
+		if (Picture && (Flags & IE_GUI_BUTTON_PORTRAIT) == IE_GUI_BUTTON_PORTRAIT) {
+			// constrain the label (status icons) to the picture bounds
+			// we are subtracting IE_FONT_PADDING because Font indents 5px, but we dont want that here
+			r = Region(picXPos - IE_FONT_PADDING, picYPos + IE_FONT_PADDING,
+					   Picture->Width + IE_FONT_PADDING, Picture->Height);
+		} else {
+			r = Region( x + XPos, y + YPos, Width - 2, Height - 2);
+		}
+
+		font->Print( r, ( unsigned char * ) Text, ppoi, (ieByte) align, true );
 	}
 
 	if (! (Flags&IE_GUI_BUTTON_NO_IMAGE)) {
@@ -349,23 +382,23 @@ void Button::SetFont(Font* newfont)
 	font = newfont;
 }
 /** Handling The default button (enter) */
-void Button::OnSpecialKeyPress(unsigned char Key)
+bool Button::OnSpecialKeyPress(unsigned char Key)
 {
 	if (State != IE_GUI_BUTTON_DISABLED && State != IE_GUI_BUTTON_LOCKED) {
 		if (Key == GEM_RETURN) {
 			if (Flags & IE_GUI_BUTTON_DEFAULT ) {
 				RunEventHandler( ButtonOnPress );
-				return;
+				return true;
 			}
 		}
 		else if (Key == GEM_ESCAPE) {
 			if (Flags & IE_GUI_BUTTON_CANCEL ) {
 				RunEventHandler( ButtonOnPress );
-				return;
+				return true;
 			}
 		}
 	}
-	Control::OnSpecialKeyPress(Key);
+	return Control::OnSpecialKeyPress(Key);
 }
 
 /** Mouse Button Down */
@@ -408,7 +441,6 @@ void Button::OnMouseDown(unsigned short x, unsigned short y,
 		}
 		if ((Button & GEM_MB_DOUBLECLICK) && ButtonOnDoublePress) {
 			RunEventHandler( ButtonOnDoublePress );
-			printMessage("Button","Doubleclick detected\n",GREEN);
 		}
 		break;
 	case GEM_MB_SCRLUP:
@@ -528,7 +560,7 @@ void Button::OnMouseOver(unsigned short x, unsigned short y)
 		return;
 	}
 
-	if ( RunEventHandler( MouseOverButton )) {
+	if ( RunEventHandler( MouseOverButton )<0) {
 		//event handler destructed this object
 		return;
 	}
@@ -600,9 +632,9 @@ void Button::SetText(const char* string)
 	} else {
 		Text = strndup( string, 255 );
 		if (Flags&IE_GUI_BUTTON_LOWERCASE)
-			strlwr( Text );
+			strtolower( Text );
 		else if (Flags&IE_GUI_BUTTON_CAPS)
-			strupr( Text );
+			strtoupper( Text );
 		hasText = true;
 	}
 	Changed = true;
@@ -651,8 +683,8 @@ bool Button::SetEvent(int eventType, EventHandler handler)
 	return true;
 }
 
-/** Redraws a button from a given radio button group */
-void Button::RedrawButton(const char* VariableName, unsigned int Sum)
+/** Refresh a button from a given radio button group */
+void Button::UpdateState(const char* VariableName, unsigned int Sum)
 {
 	if (strnicmp( VarName, VariableName, MAX_VARIABLE_LENGTH )) {
 		return;
@@ -712,7 +744,9 @@ bool Button::IsPixelTransparent(unsigned short x, unsigned short y)
 	// some buttons have hollow Image frame filled w/ Picture
 	// some buttons in BG2 are text only (if BAM == 'GUICTRL')
 	if (Picture || PictureList.size() || ! Unpressed) return false;
-	return Unpressed->IsPixelTransparent(x, y);
+	int xOffs = ( Width / 2 ) - ( Unpressed->Width / 2 );
+	int yOffs = ( Height / 2 ) - ( Unpressed->Height / 2 );
+	return Unpressed->IsPixelTransparent(x - xOffs, y - yOffs);
 }
 
 // Set palette used for drawing button label in normal state
@@ -723,14 +757,21 @@ void Button::SetTextColor(const Color &fore, const Color &back)
 	Changed = true;
 }
 
-void Button::SetHorizontalOverlay(double clip, const Color &src, const Color &dest)
+void Button::SetHorizontalOverlay(double clip, const Color &/*src*/, const Color &dest)
 {
 	if ((Clipping>clip) || !(Flags&IE_GUI_BUTTON_HORIZONTAL) ) {
 		Flags |= IE_GUI_BUTTON_HORIZONTAL;
+#if 0
+		// FIXME: This doesn't work while CloseUpColor isn't being called
+		// (see Draw)
 		SourceRGB=src;
 		DestRGB=dest;
 		starttime = GetTickCount();
 		starttime += 40;
+#else
+		SourceRGB = DestRGB = dest;
+		starttime = 0;
+#endif
 	}
 	Clipping = clip;
 	Changed = true;
@@ -740,3 +781,10 @@ void Button::SetAnchor(ieWord x, ieWord y)
 {
 	Anchor = Point(x,y);
 }
+
+void Button::SetPushOffset(ieWord x, ieWord y)
+{
+	PushOffset = Point(x,y);
+}
+
+}
