@@ -29,6 +29,8 @@
 #include "Video.h"
 #include "GUI/EventMgr.h"
 
+namespace GemRB {
+
 Console::Console(void)
 {
 	Cursor = NULL;
@@ -67,7 +69,7 @@ void Console::Draw(unsigned short x, unsigned short y)
 	Color black = {
 		0x00, 0x00, 0x00, 0xff
 	};
-	Region r( x + XPos, y + YPos, Width, Height );
+	Region r( (short)x + XPos, (short)y + YPos, Width, Height );
 	core->GetVideoDriver()->DrawRect( r, black );
 	font->Print( r, Buffer, palette,
 			IE_FONT_ALIGN_LEFT | IE_FONT_ALIGN_MIDDLE, true, NULL,
@@ -96,10 +98,10 @@ void Console::SetBackGround(Sprite2D* back)
 /** Sets the Text of the current control */
 void Console::SetText(const char* string)
 {
-	strncpy( ( char * ) Buffer, string, max );
+	strlcpy( ( char * ) Buffer, string, max );
 }
 /** Key Press Event */
-void Console::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
+bool Console::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 {
 	if (Key >= 0x20) {
 		size_t len = strlen( ( char* ) Buffer );
@@ -110,10 +112,12 @@ void Console::OnKeyPress(unsigned char Key, unsigned short /*Mod*/)
 			Buffer[CurPos++] = Key;
 			Buffer[len + 1] = 0;
 		}
+		return true;
 	}
+	return false;
 }
 /** Special Key Press */
-void Console::OnSpecialKeyPress(unsigned char Key)
+bool Console::OnSpecialKeyPress(unsigned char Key)
 {
 	size_t len;
 
@@ -167,6 +171,7 @@ void Console::OnSpecialKeyPress(unsigned char Key)
 			Changed = true;
 			break;
 	}
+	return true;
 }
 
 //ctrl-up
@@ -227,3 +232,5 @@ bool Console::SetEvent(int /*eventType*/, EventHandler /*handler*/)
 {
 	return false;
 }
+
+}
