@@ -30,6 +30,8 @@
 #include "GUI/Control.h"
 #include "GUI/TextArea.h"
 
+namespace GemRB {
+
 class Palette;
 
 /**
@@ -80,9 +82,9 @@ private:
 
 public: //Events
 	/** Key Press Event */
-	void OnKeyPress(unsigned char Key, unsigned short Mod);
+	bool OnKeyPress(unsigned char Key, unsigned short Mod);
 	/** Special Key Press */
-	void OnSpecialKeyPress(unsigned char Key);
+	bool OnSpecialKeyPress(unsigned char Key);
 	void SetFocus(bool focus);
 	bool SetEvent(int eventType, EventHandler handler);
 private:
@@ -91,4 +93,6 @@ private:
 	void HistoryAdd(bool force);
 };
 
+}
+
 #endif
