@@ -28,6 +28,8 @@
 
 #include <cstring>
 
+namespace GemRB {
+
 Progressbar::Progressbar( unsigned short KnobStepsCount, bool Clear)
 {
 	BackGround = NULL;
@@ -121,12 +123,12 @@ void Progressbar::SetPosition(unsigned int pos)
 	Changed = true;
 }
 
-void Progressbar::RedrawProgressbar(const char* VariableName, int Sum)
+void Progressbar::UpdateState(const char* VariableName, unsigned int Sum)
 {
 	if (strnicmp( VarName, VariableName, MAX_VARIABLE_LENGTH )) {
 		return;
 	}
-	SetPosition((unsigned int) Sum);
+	SetPosition(Sum);
 	if((Value==100) && Changed)
 		RunEventHandler( EndReached );
 }
@@ -177,3 +179,5 @@ bool Progressbar::SetEvent(int eventType, EventHandler handler)
 
 	return true;
 }
+
+}
