@@ -34,6 +34,8 @@
 
 #include "Sprite2D.h"
 
+namespace GemRB {
+
 // !!! Keep these synchronized with GUIDefines.py !!!
 #define IE_GUI_SLIDER_ON_CHANGE    0x02000000
 
@@ -62,8 +64,8 @@ public:
 	void SetImage(unsigned char type, Sprite2D * img);
 	/** Sets the State of the Slider */
 	void SetState(int arg) { State=(unsigned char) arg; }
-	/** Redraws a slider which is associated with VariableName */
-	void RedrawSlider(const char *VariableName, int Sum);
+	/** Refreshes a slider which is associated with VariableName */
+	void UpdateState(const char *VariableName, unsigned int Sum);
 
 private: // Private attributes
 	/** BackGround Image. If smaller than the Control Size, the image will be tiled. */
@@ -101,4 +103,6 @@ public: // Public Events
 	EventHandler SliderOnChange;
 };
 
+}
+
 #endif
