@@ -39,6 +39,9 @@
 #include "GameScript/GameScript.h"
 #include "GUI/GameControl.h"
 #include "System/DataStream.h"
+#include "System/StringBuffer.h"
+
+namespace GemRB {
 
 #define MAX_MAPS_LOADED 1
 
@@ -50,6 +53,7 @@ Game::Game(void) : Scriptable( ST_GLOBAL )
 	version = 0;
 	Expansion = 0;
 	LoadMos[0] = 0;
+	TextScreen[0] = 0;
 	SelectedSingle = 1; //the PC we are looking at (inventory, shop)
 	PartyGold = 0;
 	SetScript( core->GlobalScript, 0 );
@@ -88,8 +92,8 @@ Game::Game(void) : Scriptable( ST_GLOBAL )
 
 	//loading rest/daylight switching movies (only bg2 has them)
 	memset(restmovies,'*',sizeof(restmovies));
-	memset(daymovies,'*',sizeof(restmovies));
-	memset(nightmovies,'*',sizeof(restmovies));
+	memset(daymovies,'*',sizeof(daymovies));
+	memset(nightmovies,'*',sizeof(nightmovies));
 	if (table.load("restmov")) {
 		for(int i=0;i<8;i++) {
 			strnuprcpy(restmovies[i],table->QueryField(i,0),8);
@@ -101,25 +105,25 @@ Game::Game(void) : Scriptable( ST_GLOBAL )
 	//loading npc starting levels
 	ieResRef tn;
 	if (Expansion == 5) { // tob is special
-		strncpy(tn, "npclvl25", sizeof(ieResRef));
+		CopyResRef(tn, "npclvl25");
 	} else {
-		strncpy(tn, "npclevel", sizeof(ieResRef));
+		CopyResRef(tn, "npclevel");
 	}
 	if (table.load(tn)) {
 		int cols = table->GetColumnCount();
 		int rows = table->GetRowCount();
 		int i, j;
-		npclevels.reserve(cols*rows);
+		npclevels.reserve(rows);
 		for (i = 0; i < rows; i++) {
-			npclevels.push_back (std::vector<const char *>());
+			npclevels.push_back (std::vector<char *>(cols+1));
 			for(j = -1; j < cols; j++) {
 				char *ref = new char[9];
 				if (j == -1) {
-					strncpy(ref, table->GetRowName(i), sizeof(ieResRef));
-					npclevels[i].push_back (ref);
+					CopyResRef(ref, table->GetRowName(i));
+					npclevels[i][j+1] = ref;
 				} else {
-					strncpy(ref, table->QueryField(i, j), sizeof(ieResRef));
-					npclevels[i].push_back (ref);
+					CopyResRef(ref, table->QueryField(i, j));
+					npclevels[i][j+1] = ref;
 				}
 			}
 		}
@@ -182,7 +186,10 @@ Game::~Game(void)
 
 	i = npclevels.size();
 	while (i--) {
-		npclevels[i].clear();
+		size_t j = npclevels[i].size();
+		while(j--) {
+			delete [] npclevels[i][j];
+		}
 	}
 }
 
@@ -381,6 +388,9 @@ int Game::LeaveParty (Actor* actor)
 
 	if (core->HasFeature( GF_HAS_DPLAYER )) {
 		actor->SetScript( "", SCR_DEFAULT );
+		if (actor->GetBase(IE_MC_FLAGS) & MC_EXPORTABLE) {
+			actor->SetDialog("MULTIJ");
+		}
 	}
 	actor->SetBase( IE_EA, EA_NEUTRAL );
 	return ( int ) NPCs.size() - 1;
@@ -427,6 +437,8 @@ void Game::InitActorPos(Actor *actor)
 
 	actor->Pos.x = actor->Destination.x = (short) atoi( strta->QueryField( strta->GetRowIndex(xpos), ip ) );
 	actor->Pos.y = actor->Destination.y = (short) atoi( strta->QueryField( strta->GetRowIndex(ypos), ip ) );
+	actor->HomeLocation.x = actor->Pos.x;
+	actor->HomeLocation.y = actor->Pos.y;
 	actor->SetOrientation( atoi( strta->QueryField( strta->GetRowIndex(rot), ip) ), false );
 
 	if (strta.load("startare")) {
@@ -521,7 +533,7 @@ void Game::SetHotKey(unsigned long Key)
 bool Game::SelectPCSingle(int index)
 {
 	Actor* actor = FindPC( index );
-	if (!actor || ! actor->ValidTarget( GA_NO_HIDDEN ))
+	if (!actor)
 		return false;
 
 	SelectedSingle = index;
@@ -624,19 +636,22 @@ bool Game::SelectActor(Actor* actor, bool select, unsigned flags)
 	return true;
 }
 
-// Gets average party level, of onlyalive is true, then counts only living PCs
+// Gets sum of party level, if onlyalive is true, then counts only living PCs
+// If you need average party level, divide this with GetPartySize
 int Game::GetPartyLevel(bool onlyalive) const
 {
-	int count = 0;
+	int amount = 0;
+
 	for (unsigned int i = 0; i<PCs.size(); i++) {
 			if (onlyalive) {
 				if (PCs[i]->GetStat(IE_STATE_ID)&STATE_DEAD) {
 					continue;
 				}
 			}
-			count += PCs[i]->GetXPLevel(0);
+			amount += PCs[i]->GetXPLevel(0);
 	}
-	return count;
+
+	return amount;
 }
 
 // Returns map structure (ARE) if it is already loaded in memory
@@ -737,7 +752,7 @@ int Game::DelMap(unsigned int index, int forced)
 
 
 	if (!map) { //this shouldn't happen, i guess
-		printMessage("Game","Erased NULL Map\n",YELLOW);
+		Log(WARNING, "Game", "Erased NULL Map");
 		Maps.erase( Maps.begin()+index);
 		if (MapIndex>(int) index) {
 			MapIndex--;
@@ -765,6 +780,17 @@ int Game::DelMap(unsigned int index, int forced)
 		{
 			return 1;
 		}
+		//if there are still selected actors on the map (e.g. summons)
+		//unselect them now before they get axed
+		std::vector< Actor*>::iterator m;
+		for (m = selected.begin(); m != selected.end();) {
+			if (!(*m)->InParty && !stricmp(Maps[index]->GetScriptName(), (*m)->Area)) {
+				m = selected.erase(m);
+			} else {
+				++m;
+			}
+		}
+
 		//remove map from memory
 		core->SwapoutArea(Maps[index]);
 		delete( Maps[index] );
@@ -779,10 +805,30 @@ int Game::DelMap(unsigned int index, int forced)
 	return 0;
 }
 
+void Game::PlacePersistents(Map *newMap, const char *ResRef)
+{
+	unsigned int i, last;
+
+	// count the number of replaced actors, so we don't need to recheck them
+	// if their max level is still lower than ours, each check would also result in a substitution
+	last = NPCs.size()-1;
+	for (i = 0; i < NPCs.size(); i++) {
+		if (stricmp( NPCs[i]->Area, ResRef ) == 0) {
+			if (i < last && CheckForReplacementActor(i)) {
+				i--;
+				last--;
+				continue;
+			}
+			newMap->AddActor( NPCs[i], false );
+			NPCs[i]->SetMap(newMap);
+		}
+	}
+}
+
 /* Loads an area */
 int Game::LoadMap(const char* ResRef, bool loadscreen)
 {
-	unsigned int i;
+	unsigned int i, ret;
 	Map *newMap;
 	PluginHolder<MapMgr> mM(IE_ARE_CLASS_ID);
 	ScriptEngine *sE = core->GetGUIScriptEngine();
@@ -794,6 +840,7 @@ int Game::LoadMap(const char* ResRef, bool loadscreen)
 
 	int index = FindMap(ResRef);
 	if (index>=0) {
+		PlacePersistents(GetMap(index), ResRef);
 		return index;
 	}
 
@@ -817,39 +864,56 @@ int Game::LoadMap(const char* ResRef, bool loadscreen)
 
 	core->LoadProgress(100);
 
+	ret = AddMap( newMap );
+
+	//spawn creatures on a map already in the game
+	//this feature exists in all blackisle games but not in bioware games
+	if (core->HasFeature(GF_SPAWN_INI)) {
+		newMap->LoadIniSpawn();
+	}
+
 	for (i = 0; i < PCs.size(); i++) {
 		if (stricmp( PCs[i]->Area, ResRef ) == 0) {
-			newMap->AddActor( PCs[i] );
-		}
-	}
-	for (i = 0; i < NPCs.size(); i++) {
-		if (stricmp( NPCs[i]->Area, ResRef ) == 0) {
-			if (!CheckForReplacementActor(i)) {
-				newMap->AddActor( NPCs[i] );
-			} // when it happens, it will be handled by the loop limit reevaluation
+			newMap->AddActor( PCs[i], false );
 		}
 	}
+
+	PlacePersistents(newMap, ResRef);
+
 	if (hide) {
 		core->UnhideGCWindow();
 	}
-	return AddMap( newMap );
+	newMap->InitActors();
+
+	return ret;
 failedload:
-	if (hide) 
+	if (hide) {
 		core->UnhideGCWindow();
+	}
 	core->LoadProgress(100);
 	return -1;
 }
 
 // check if the actor is in npclevel.2da and replace accordingly
-bool Game::CheckForReplacementActor(int i) {
+bool Game::CheckForReplacementActor(int i)
+{
+	if (core->InCutSceneMode() || npclevels.empty()) {
+		return false;
+	}
+
 	Actor* act = NPCs[i];
 	ieDword level = GetPartyLevel(false) / GetPartySize(false);
-	if (! npclevels.empty() && !(act->Modified[IE_MC_FLAGS]&MC_BEENINPARTY) && !(act->Modified[IE_STATE_ID]&STATE_DEAD) && act->GetXPLevel(false) < level) {
+	if (!(act->Modified[IE_MC_FLAGS]&MC_BEENINPARTY) && !(act->Modified[IE_STATE_ID]&STATE_DEAD) && act->GetXPLevel(false) < level) {
 		ieResRef newcre = "****"; // default table value
-		std::vector<std::vector<const char *> >::iterator it;
+		std::vector<std::vector<char *> >::iterator it;
 		for (it = npclevels.begin(); it != npclevels.end(); it++) {
-			if (!stricmp((*it)[0], act->GetScriptName())) {
-				strncpy(newcre, (*it)[level-2], sizeof(ieResRef));
+			if (!stricmp((*it)[0], act->GetScriptName()) && (level > 2)) {
+				// the tables have entries only up to level 24
+				ieDword safeLevel = npclevels[0].size() - 1;
+				if (level < safeLevel) {
+					safeLevel = level;
+				}
+				CopyResRef(newcre, (*it)[safeLevel-2]);
 				break;
 			}
 		}
@@ -863,6 +927,8 @@ bool Game::CheckForReplacementActor(int i) {
 				if (!newact) {
 					error("Game::CheckForReplacementActor", "GetNPC failed: cannot find act!\n");
 				} else {
+					newact->Pos = act->Pos; // the map is not loaded yet, so no SetPosition
+					CopyResRef(newact->Area, act->Area);
 					DelNPC(InStore(act));
 					return true;
 				}
@@ -1093,14 +1159,17 @@ int Game::GetXPFromCR(int cr)
 {
 	if (!crtable) LoadCRTable();
 	if (crtable) {
-		int level = GetPartyLevel(true);
-		if (cr>=MAX_CRLEVEL) {
-			cr=MAX_CRLEVEL-1;
+		int size = GetPartySize(true);
+		if (!size) return 0; // everyone just died anyway
+		int level = GetPartyLevel(true) / size;
+		if (cr+1>=MAX_CRLEVEL) {
+			cr=MAX_CRLEVEL-2;
 		}
-		print("Challenge Rating: %d, party level: %d ", cr, level);
-		return crtable[level][cr];
+		Log(MESSAGE, "Game", "Challenge Rating: %d, party level: %d", cr, level);
+		// it also has a column for cr 0.25 and 0.5
+		return crtable[level-1][cr+1];
 	}
-	printMessage("Game","Cannot find moncrate.2da!\n", LIGHT_RED);
+	Log(ERROR, "Game", "Cannot find moncrate.2da!");
 	return 0;
 }
 
@@ -1135,7 +1204,7 @@ void Game::ShareXP(int xp, int flags)
 		if (PCs[i]->GetStat(IE_STATE_ID)&STATE_DEAD) {
 			continue;
 		}
-		PCs[i]->AddExperience(individual);
+		PCs[i]->AddExperience(individual, flags&SX_COMBAT);
 	}
 }
 
@@ -1173,7 +1242,7 @@ bool Game::EveryoneNearPoint(Map *area, const Point &p, int flags) const
 			return false;
 		}
 		if (Distance(p,PCs[i])>MAX_TRAVELING_DISTANCE) {
-			print("Actor %s is not near!\n", PCs[i]->LongName);
+			Log(MESSAGE, "Game", "Actor %s is not near!", PCs[i]->LongName);
 			return false;
 		}
 	}
@@ -1265,6 +1334,8 @@ void Game::AdvanceTime(ieDword add)
 	if (h!=GameTime/(300*AI_UPDATE_TIME)) {
 		//asking for a new weather when the hour changes
 		WeatherBits&=~WB_HASWEATHER;
+		//update clock display
+		core->GetGUIScriptEngine()->RunFunction("GUICommonWindows", "UpdateClock");
 	}
 	Ticks+=add*interval;
 	//change the tileset if needed
@@ -1276,7 +1347,6 @@ void Game::AdvanceTime(ieDword add)
 		int areatype = (area->AreaType&(AT_FOREST|AT_CITY|AT_DUNGEON))>>3;
 		ieResRef *res;
 
-		printMessage("Game","Switching DayLight\n",GREEN);
 		if (IsDay()) {
 			res=&nightmovies[areatype];
 		} else {
@@ -1460,20 +1530,53 @@ void Game::PlayerDream()
 	delete( gs );
 }
 
+//Start a TextScreen dream for the protagonist
+void Game::TextDream()
+{
+	ieDword dream, chapter;
+	locals->Lookup("CHAPTER", chapter);
+	if (!locals->Lookup("DREAM", dream)) {
+		dream = 1;
+	}
+	snprintf(TextScreen, sizeof(ieResRef)-1, "drmtxt%d", dream+1);
+	if ((chapter > dream) && (core->Roll(1, 100, 0) <= 33)
+		&& gamedata->Exists(TextScreen, IE_2DA_CLASS_ID)) {
+
+		// give innate spell to protagonist
+		AutoTable drm(TextScreen);
+		if (drm) {
+			const char *repLabel;
+			if (Reputation >= 100)
+				repLabel = "GOOD_POWER";
+			else
+				repLabel = "BAD_POWER";
+			int row = drm->GetRowIndex(repLabel);
+			if (row != -1) {
+				Actor *actor = GetPC(0, false);
+				actor->LearnSpell(drm->QueryField(row, 0), LS_MEMO|LS_LEARN);
+			}
+		}
+
+		locals->SetAt("DREAM", dream+1);
+		core->SetEventFlag(EF_TEXTSCREEN);
+	}
+}
+
 //noareacheck = no random encounters
 //dream = 0 - based on area non-0 - select from list
 //-1 no dream
 //hp is how much hp the rest will heal
-void Game::RestParty(int checks, int dream, int hp)
+//returns true if a cutscene dream is about to be played
+bool Game::RestParty(int checks, int dream, int hp)
 {
 	if (!(checks&REST_NOMOVE) ) {
 		if (!EveryoneStopped()) {
-			return;
+			return false;
 		}
 	}
 	Actor *leader = GetPC(0, true);
 	if (!leader) {
-		return;
+		return false;
 	}
 
 	Map *area = leader->GetCurrentArea();
@@ -1482,7 +1585,7 @@ void Game::RestParty(int checks, int dream, int hp)
 		if (!EveryoneNearPoint( area, leader->Pos, 0 ) ) {
 			//party too scattered
 			displaymsg->DisplayConstantString( STR_SCATTERED, DMC_RED );
-			return;
+			return false;
 		}
 	}
 
@@ -1490,36 +1593,52 @@ void Game::RestParty(int checks, int dream, int hp)
 		//don't allow resting while in combat
 		if (AnyPCInCombat()) {
 			displaymsg->DisplayConstantString( STR_CANTRESTMONS, DMC_RED );
-			return;
+			return false;
 		}
 		//don't allow resting if hostiles are nearby
 		if (area->AnyEnemyNearPoint(leader->Pos)) {
 			displaymsg->DisplayConstantString( STR_CANTRESTMONS, DMC_RED );
-			return;
+			return false;
 		}
 	}
 
 	//rest check, if PartyRested should be set, area should return true
-	//area should advance gametime too (so partial rest is possible)
+	// TODO: implement "rest until healed", it's an option in some games
 	int hours = 8;
+	int hoursLeft = 0;
 	if (!(checks&REST_NOAREA) ) {
 		//you cannot rest here
-		if (area->AreaFlags&1) {
+		if (area->AreaFlags&AF_NOSAVE) {
 			displaymsg->DisplayConstantString( STR_MAYNOTREST, DMC_RED );
-			return;
+			return false;
 		}
 		//you may not rest here, find an inn
 		if (!(area->AreaType&(AT_OUTDOOR|AT_FOREST|AT_DUNGEON|AT_CAN_REST) ))
 		{
 			displaymsg->DisplayConstantString( STR_MAYNOTREST, DMC_RED );
-			return;
+			return false;
 		}
 		//area encounters
-		if(area->Rest( leader->Pos, 8, (GameTime/AI_UPDATE_TIME)%7200/3600) ) {
-			return;
+		// also advances gametime (so partial rest is possible)
+		hoursLeft = area->CheckRestInterruptsAndPassTime( leader->Pos, hours, (GameTime/AI_UPDATE_TIME)%7200/3600);
+		if (hoursLeft) {
+			// partial rest only, so adjust the parameters for the loop below
+			if (hp) {
+				hp = hp * (hours - hoursLeft) / hours;
+				// 0 means full heal, so we need to cancel it if we rounded to 0
+				if (!hp) {
+					hp = 1;
+				}
+			}
+			hours -= hoursLeft;
+			// the interruption occured before any resting could be done, so just bail out
+			if (!hours) {
+				return false;
+			}
 		}
+	} else {
+		AdvanceTime(hours*300*AI_UPDATE_TIME);
 	}
-	AdvanceTime(2400*AI_UPDATE_TIME);
 
 	int i = GetPartySize(true); // party size, only alive
 
@@ -1531,15 +1650,26 @@ void Game::RestParty(int checks, int dream, int hp)
 		//if hp = 0, then healing will be complete
 		tar->Heal(hp);
 		//removes fatigue, recharges spells
-		tar->Rest(0);
-		tar->PartyRested();
+		tar->Rest(hours);
+		if (!hoursLeft)
+			tar->PartyRested();
 	}
 
-	//movie and cutscene dreams
+	// abort the partial rest; we got what we wanted
+	if (hoursLeft) {
+		return false;
+	}
+
+	//movie, cutscene, and still frame dreams
+	bool cutscene = false;
 	if (dream>=0) {
 		//cutscene dreams
-		if (gamedata->Exists("player1d",IE_BCS_CLASS_ID, true))
+		if (gamedata->Exists("player1d",IE_BCS_CLASS_ID, true)) {
+			cutscene = true;
 			PlayerDream();
+		} else if (gamedata->Exists("drmtxt2", IE_2DA_CLASS_ID, true)) {
+			TextDream();
+		}
 
 		//select dream based on area
 		ieResRef *movie;
@@ -1558,30 +1688,25 @@ void Game::RestParty(int checks, int dream, int hp)
 	area->PartyRested();
 	core->SetEventFlag(EF_ACTION);
 
-	//restindex will be -1 in the case of PST
-	//FIXME: I don't quite see why we can't sumply use the same strings.2da entry
-	//It seems we could reduce complexity here, and free up 2-3 string slots too
+	//bg1 has "You have rested for <DURATION>" while pst has "You have
+	//rested for <HOUR> <DURATION>" and then bg1 has "<HOUR> hours" while
+	//pst just has "Hours", so this works for both
 	int restindex = displaymsg->GetStringReference(STR_REST);
-	int strindex;
+	int hrsindex = displaymsg->GetStringReference(STR_HOURS);
 	char* tmpstr = NULL;
 
 	core->GetTokenDictionary()->SetAtCopy("HOUR", hours);
-	if (restindex != -1) {
-		strindex = displaymsg->GetStringReference(STR_HOURS);
-	} else {
-		strindex = displaymsg->GetStringReference(STR_PST_HOURS);
-		restindex = displaymsg->GetStringReference(STR_PST_REST);
-	}
 
 	//this would be bad
-	if (strindex == -1 || restindex == -1) return;
-	tmpstr = core->GetString(strindex, 0);
+	if (hrsindex == -1 || restindex == -1) return cutscene;
+	tmpstr = core->GetString(hrsindex, 0);
 	//as would this
-	if (!tmpstr) return;
+	if (!tmpstr) return cutscene;
 
 	core->GetTokenDictionary()->SetAtCopy("DURATION", tmpstr);
 	core->FreeString(tmpstr);
 	displaymsg->DisplayString(restindex, DMC_WHITE, 0);
+	return cutscene;
 }
 
 //timestop effect
@@ -1606,7 +1731,10 @@ bool Game::TimeStoppedFor(const Actor* target)
 //recalculate the party's infravision state
 void Game::Infravision()
 {
+	ieDword tmp = 0;
 	hasInfra = false;
+	core->GetDictionary()->Lookup("infravision", tmp);
+	if (!tmp) return;
 	Map *map = GetCurrentArea();
 	if (!map) return;
 	for(size_t i=0;i<PCs.size();i++) {
@@ -1689,7 +1817,7 @@ void Game::ChangeSong(bool always, bool force)
 }
 
 /* this method redraws weather. If update is false,
-   then the weather particles won't change (game paused)
+// then the weather particles won't change (game paused)
 */
 void Game::DrawWeather(const Region &screen, bool update)
 {
@@ -1710,12 +1838,10 @@ void Game::DrawWeather(const Region &screen, bool update)
 			weather->SetPhase(P_FADE);
 		}
 	}
-	//if (GameTime&1) {
-		int drawn = weather->Update();
-		if (drawn) {
-			WeatherBits &= ~WB_START;
-		}
-	//}
+	int drawn = weather->Update();
+	if (drawn) {
+		WeatherBits &= ~WB_INCREASESTORM;
+	}
 
 	if (WeatherBits&WB_HASWEATHER) {
 		return;
@@ -1732,8 +1858,8 @@ void Game::StartRainOrSnow(bool conditional, int w)
 	}
 	// whatever was responsible for calling this, we now have some set weather
 	WeatherBits = w | WB_HASWEATHER;
-	if (w & WB_LIGHTNING) {
-		if (WeatherBits&WB_START) {
+	if (w & WB_LIGHTNINGMASK) {
+		if (WeatherBits&WB_INCREASESTORM) {
 			//already raining
 			if (GameTime&1) {
 				core->PlaySound(DS_LIGHTNING1);
@@ -1764,17 +1890,19 @@ void Game::StartRainOrSnow(bool conditional, int w)
 
 void Game::SetExpansion(ieDword value)
 {
-	if (Expansion>=value) {
-		return;
+	if (value) {
+		if (Expansion>=value) {
+			return;
+		}
+		Expansion = value;
 	}
-	Expansion = value;
 
-	switch(Expansion) {
+	core->SetEventFlag(EF_EXPANSION);
+	switch(value) {
 	default:
-		core->SetEventFlag(EF_EXPANSION);
 		break;
 	//TODO: move this hardcoded hack to the scripts
-	case 5:
+	case 0:
 		core->GetDictionary()->SetAt( "PlayMode", 2 );
 
 		int i = GetPartySize(false);
@@ -1785,26 +1913,30 @@ void Game::SetExpansion(ieDword value)
 	}
 }
 
-void Game::DebugDump()
+void Game::dump() const
 {
+	StringBuffer buffer;
 	size_t idx;
 
-	print("Currently loaded areas:\n");
+	buffer.append("Currently loaded areas:\n");
 	for(idx=0;idx<Maps.size();idx++) {
 		Map *map = Maps[idx];
 
-		print("%s\n",map->GetScriptName());
+		print("%s", map->GetScriptName());
 	}
-	print("Current area: %s   Previous area: %s\n", CurrentArea, PreviousArea);
-	print("Global script: %s\n", Scripts[0]->GetName());
-	print("CombatCounter: %d\n", (int) CombatCounter);
+	buffer.appendFormatted("Current area: %s   Previous area: %s\n", CurrentArea, PreviousArea);
+	buffer.appendFormatted("Global script: %s\n", Scripts[0]->GetName());
+	int hours = GameTime/AI_UPDATE_TIME/300;
+	buffer.appendFormatted("Game time: %d (%d days, %d hours)\n", GameTime, hours/24, hours%24);
+	buffer.appendFormatted("CombatCounter: %d\n", (int) CombatCounter);
 
-	print("Party size: %d\n", (int) PCs.size());
+	buffer.appendFormatted("Party size: %d\n", (int) PCs.size());
 	for(idx=0;idx<PCs.size();idx++) {
 		Actor *actor = PCs[idx];
 
-		print("Name: %s Order %d %s\n",actor->ShortName, actor->InParty, actor->Selected?"x":"-");
+		buffer.appendFormatted("Name: %s Order %d %s\n",actor->ShortName, actor->InParty, actor->Selected?"x":"-");
 	}
+	Log(DEBUG, "Game", buffer);
 }
 
 Actor *Game::GetActorByGlobalID(ieDword globalID)
@@ -1834,12 +1966,10 @@ bool Game::IsTimestopActive() const
 
 bool Game::RandomEncounter(ieResRef &BaseArea)
 {
-	displaymsg->DisplayConstantString(STR_AMBUSH, DMC_BG2XPGREEN);
-
 	if (bntrows<0) {
 		AutoTable table;
 
-	        if (table.load("bntychnc")) {
+		if (table.load("bntychnc")) {
 			bntrows = table->GetRowCount();
 			bntchnc = (int *) calloc(sizeof(int),bntrows);
 			for(int i = 0; i<bntrows; i++) {
@@ -1857,3 +1987,12 @@ bool Game::RandomEncounter(ieResRef &BaseArea)
 	memcpy(BaseArea+4,"10",3);
 	return gamedata->Exists(BaseArea, IE_ARE_CLASS_ID);
 }
+
+void Game::ResetPartyCommentTimes()
+{
+	for (unsigned int slot=0; slot<PCs.size(); slot++) {
+		PCs[slot]->ResetCommentTime();
+	}
+}
+
+}
