@@ -46,6 +46,8 @@
 
 #include <cstdio>
 
+namespace GemRB {
+
 static void ReleaseItem(void *poi)
 {
 	delete ((Item *) poi);
@@ -154,15 +156,15 @@ int GameData::LoadCreature(const char* ResRef, unsigned int PartySlot, bool char
 }
 
 /** Loads a 2DA Table, returns -1 on error or the Table Index on success */
-int GameData::LoadTable(const ieResRef ResRef)
+int GameData::LoadTable(const ieResRef ResRef, bool silent)
 {
 	int ind = GetTableIndex( ResRef );
 	if (ind != -1) {
 		tables[ind].refcount++;
 		return ind;
 	}
-	//print("(%s) Table not found... Loading from file\n", ResRef);
-	DataStream* str = GetResource( ResRef, IE_2DA_CLASS_ID );
+	//print("(%s) Table not found... Loading from file", ResRef);
+	DataStream* str = GetResource( ResRef, IE_2DA_CLASS_ID, silent );
 	if (!str) {
 		return -1;
 	}
@@ -287,13 +289,13 @@ void GameData::FreePalette(Palette *&pal, const ieResRef name)
 	pal = NULL;
 }
 
-Item* GameData::GetItem(const ieResRef resname)
+Item* GameData::GetItem(const ieResRef resname, bool silent)
 {
 	Item *item = (Item *) ItemCache.GetResource(resname);
 	if (item) {
 		return item;
 	}
-	DataStream* str = GetResource( resname, IE_ITM_CLASS_ID );
+	DataStream* str = GetResource(resname, IE_ITM_CLASS_ID, silent);
 	PluginHolder<ItemMgr> sm(IE_ITM_CLASS_ID);
 	if (!sm) {
 		delete ( str );
@@ -362,9 +364,8 @@ void GameData::FreeSpell(Spell *spl, const ieResRef name, bool free)
 
 	res=SpellCache.DecRef((void *) spl, name, free);
 	if (res<0) {
-		printMessage("Core", "Corrupted Spell cache encountered (reference count went below zero), Spell name is: %.8s or %.8s\n", LIGHT_RED,
+		error("Core", "Corrupted Spell cache encountered (reference count went below zero), Spell name is: %.8s or %.8s\n",
 			name, spl->Name);
-		abort();
 	}
 	if (res) return;
 	if (free) delete spl;
@@ -432,11 +433,11 @@ ScriptedAnimation* GameData::GetScriptedAnimation( const char *effect, bool doub
 
 // Return single BAM frame as a sprite. Use if you want one frame only,
 // otherwise it's not efficient
-Sprite2D* GameData::GetBAMSprite(const ieResRef ResRef, int cycle, int frame)
+Sprite2D* GameData::GetBAMSprite(const ieResRef ResRef, int cycle, int frame, bool silent)
 {
 	Sprite2D *tspr;
 	AnimationFactory* af = ( AnimationFactory* )
-		GetFactoryResource( ResRef, IE_BAM_CLASS_ID, IE_NORMAL );
+		GetFactoryResource( ResRef, IE_BAM_CLASS_ID, IE_NORMAL, silent );
 	if (!af) return 0;
 	if (cycle == -1)
 		tspr = af->GetFrameWithoutCycle( (unsigned short) frame );
@@ -475,7 +476,7 @@ void* GameData::GetFactoryResource(const char* resname, SClass_ID type,
 	}
 	case IE_BMP_CLASS_ID:
 	{
-		ResourceHolder<ImageMgr> img(resname);
+		ResourceHolder<ImageMgr> img(resname, silent);
 		if (img) {
 			ImageFactory* fact = img->GetImageFactory( resname );
 			factory->AddFactoryObject( fact );
@@ -485,8 +486,7 @@ void* GameData::GetFactoryResource(const char* resname, SClass_ID type,
 		return NULL;
 	}
 	default:
-		print( "\n" );
-		printMessage("KEYImporter", "%s files are not supported.\n", WHITE,
+		Log(MESSAGE, "KEYImporter", "%s files are not supported.",
 			core->TypeExt(type));
 		return NULL;
 	}
@@ -553,3 +553,5 @@ void GameData::SaveAllStores()
 		SaveStore(stores.begin()->second);
 	}
 }
+
+}
