@@ -37,6 +37,8 @@
 #include "TableMgr.h"
 #endif
 
+namespace GemRB {
+
 class Actor;
 struct Effect;
 class Factory;
@@ -72,7 +74,7 @@ public:
 	// (See also the AutoTable class)
 
 	/** Loads a 2DA Table, returns -1 on error or the Table Index on success */
-	int LoadTable(const char * ResRef);
+	int LoadTable(const ieResRef ResRef, bool silent=false);
 	/** Gets the index of a loaded table, returns -1 on error */
 	int GetTableIndex(const char * ResRef) const;
 	/** Gets a Loaded Table by its index, returns NULL on error */
@@ -83,7 +85,7 @@ public:
 	Palette* GetPalette(const ieResRef resname);
 	void FreePalette(Palette *&pal, const ieResRef name=NULL);
 	
-	Item* GetItem(const ieResRef resname);
+	Item* GetItem(const ieResRef resname, bool silent=false);
 	void FreeItem(Item const *itm, const ieResRef name, bool free=false);
 	Spell* GetSpell(const ieResRef resname, bool silent=false);
 	void FreeSpell(Spell *spl, const ieResRef name, bool free=false);
@@ -94,7 +96,7 @@ public:
 	ScriptedAnimation* GetScriptedAnimation( const char *ResRef, bool doublehint);
 
 	/** returns a single sprite (not cached) from a BAM resource */
-	Sprite2D* GetBAMSprite(const ieResRef ResRef, int cycle, int frame);
+	Sprite2D* GetBAMSprite(const ieResRef ResRef, int cycle, int frame, bool silent=false);
 
 	/** returns factory resource, currently works only with animations */
 	void* GetFactoryResource(const char* resname, SClass_ID type,
@@ -125,8 +127,8 @@ public:
 	ResourceHolder()
 	{
 	}
-	ResourceHolder(const char* resname)
-		: Holder<T>(static_cast<T*>(gamedata->GetResource(resname,&T::ID)))
+	ResourceHolder(const char* resname, bool silent = false)
+		: Holder<T>(static_cast<T*>(gamedata->GetResource(resname, &T::ID, silent)))
 	{
 	}
 	ResourceHolder(const char* resname, const ResourceManager& manager, bool silent = false)
@@ -135,4 +137,6 @@ public:
 	}
 };
 
+}
+
 #endif
