@@ -30,6 +30,9 @@
 #include "Interface.h"
 #include "PluginMgr.h"
 #include "TableMgr.h"
+#include "System/StringBuffer.h"
+
+namespace GemRB {
 
 //debug flags
 // 1 - cache
@@ -101,6 +104,7 @@ static const TriggerLink triggernames[] = {
 	{"dead", GameScript::Dead, 0},
 	{"delay", GameScript::Delay, 0},
 	{"detect", GameScript::Detect, 0}, //so far i see no difference
+	{"detected", GameScript::Detected, 0}, //trap or secret door detected
 	{"die", GameScript::Die, 0},
 	{"died", GameScript::Died, 0},
 	{"difficulty", GameScript::Difficulty, 0},
@@ -149,9 +153,9 @@ static const TriggerLink triggernames[] = {
 	{"happiness", GameScript::Happiness, 0},
 	{"happinessgt", GameScript::HappinessGT, 0},
 	{"happinesslt", GameScript::HappinessLT, 0},
-	{"harmlessclosed", GameScript::Closed, 0}, //pst, not sure
-	{"harmlessentered", GameScript::HarmlessEntered, 0}, //???
-	{"harmlessopened", GameScript::Opened, 0}, //pst, not sure
+	{"harmlessclosed", GameScript::HarmlessClosed, 0}, //pst
+	{"harmlessentered", GameScript::HarmlessEntered, 0}, //pst
+	{"harmlessopened", GameScript::HarmlessOpened, 0}, //pst
 	{"hasbounceeffects", GameScript::HasBounceEffects, 0},
 	{"hasimmunityeffects", GameScript::HasImmunityEffects, 0},
 	{"hasinnateability", GameScript::HaveSpell, 0}, //these must be the same
@@ -351,6 +355,7 @@ static const TriggerLink triggernames[] = {
 	{"sequence", GameScript::Sequence, 0},
 	{"setlastmarkedobject", GameScript::SetLastMarkedObject, 0},
 	{"setmarkedspell", GameScript::SetMarkedSpell_Trigger, 0},
+	{"setspelltarget", GameScript::SetSpellTarget, 0},
 	{"specifics", GameScript::Specifics, 0},
 	{"spellcast", GameScript::SpellCast, 0},
 	{"spellcastinnate", GameScript::SpellCastInnate, 0},
@@ -361,6 +366,9 @@ static const TriggerLink triggernames[] = {
 	{"storehasitem", GameScript::StoreHasItem, 0},
 	{"stuffglobalrandom", GameScript::StuffGlobalRandom, 0},//hm, this is a trigger
 	{"subrace", GameScript::SubRace, 0},
+	{"summoninglimit", GameScript::SummoningLimit, 0},
+	{"summoninglimitgt", GameScript::SummoningLimitGT, 0},
+	{"summoninglimitlt", GameScript::SummoningLimitLT, 0},
 	{"systemvariable", GameScript::SystemVariable_Trigger, 0}, //gemrb
 	{"targetunreachable", GameScript::TargetUnreachable, 0},
 	{"team", GameScript::Team, 0},
@@ -388,6 +396,7 @@ static const TriggerLink triggernames[] = {
 	{"unselectablevariablegt", GameScript::UnselectableVariableGT, 0},
 	{"unselectablevariablelt", GameScript::UnselectableVariableLT, 0},
 	{"unusable",GameScript::Unusable, 0},
+	{"usedexit",GameScript::UsedExit, 0}, //pst unhardcoded trigger for protagonist teleport
 	{"vacant",GameScript::Vacant, 0},
 	{"walkedtotrigger", GameScript::WalkedToTrigger, 0},
 	{"wasindialog", GameScript::WasInDialog, 0},
@@ -471,7 +480,7 @@ static const ActionLink actionnames[] = {
 	{"changestoremarkup", GameScript::ChangeStoreMarkup, 0},//iwd2
 	{"changeteam", GameScript::SetTeam, 0}, //pst
 	{"changetilestate", GameScript::ChangeTileState, 0}, //bg2
-	{"chunkcreature", GameScript::Kill, 0}, //should be more graphical
+	{"chunkcreature", GameScript::ChunkCreature, 0}, //should be more graphical
 	{"clearactions", GameScript::ClearActions, 0},
 	{"clearallactions", GameScript::ClearAllActions, 0},
 	{"clearpartyeffects", GameScript::ClearPartyEffects, 0},
@@ -516,7 +525,7 @@ static const ActionLink actionnames[] = {
 	{"destroyalldestructableequipment", GameScript::DestroyAllDestructableEquipment, 0},
 	{"destroyallequipment", GameScript::DestroyAllEquipment, 0},
 	{"destroygold", GameScript::DestroyGold, 0},
-	{"destroyitem", GameScript::DestroyItem, 0},
+	{"destroyitem", GameScript::DestroyItem, AF_INSTANT}, //Cespenar won't work without this hack So, do we really need instant.ids?
 	{"destroypartygold", GameScript::DestroyPartyGold, 0},
 	{"destroypartyitem", GameScript::DestroyPartyItem, 0},
 	{"destroyself", GameScript::DestroySelf, 0},
@@ -774,6 +783,7 @@ static const ActionLink actionnames[] = {
 	{"returntosavedlocation", GameScript::ReturnToSavedLocation, AF_BLOCKING|AF_ALIVE},
 	{"returntosavedlocationdelete", GameScript::ReturnToSavedLocationDelete, AF_BLOCKING|AF_ALIVE},
 	{"returntosavedplace", GameScript::ReturnToSavedLocation, AF_BLOCKING|AF_ALIVE},
+	{"returntostartloc", GameScript::ReturnToStartLocation, AF_BLOCKING|AF_ALIVE}, // iwd2
 	{"revealareaonmap", GameScript::RevealAreaOnMap, 0},
 	{"runawayfrom", GameScript::RunAwayFrom,AF_BLOCKING|AF_ALIVE},
 	{"runawayfromnointerrupt", GameScript::RunAwayFromNoInterrupt,AF_BLOCKING|AF_ALIVE},
@@ -820,7 +830,7 @@ static const ActionLink actionnames[] = {
 	{"setglobaltimeronce", GameScript::SetGlobalTimerOnce,AF_MERGESTRINGS},
 	{"setglobaltimerrandom", GameScript::SetGlobalTimerRandom,AF_MERGESTRINGS},
 	{"setglobaltint", GameScript::SetGlobalTint, 0},
-	{"sethomelocation", GameScript::SetSavedLocation, 0}, //bg2
+	{"sethomelocation", GameScript::SetHomeLocation, 0}, //bg2
 	{"sethp", GameScript::SetHP, 0},
 	{"sethppercent", GameScript::SetHPPercent, 0},
 	{"setinternal", GameScript::SetInternal, 0},
@@ -878,6 +888,7 @@ static const ActionLink actionnames[] = {
 	{"spellnodec", GameScript::SpellNoDec, AF_BLOCKING|AF_ALIVE},
 	{"spellpoint", GameScript::SpellPoint, AF_BLOCKING|AF_ALIVE},
 	{"spellpointnodec", GameScript::SpellPointNoDec, AF_BLOCKING|AF_ALIVE},
+	{"spellwait", GameScript::Spell, AF_BLOCKING|AF_ALIVE},
 	{"startcombatcounter", GameScript::StartCombatCounter, 0},
 	{"startcutscene", GameScript::StartCutScene, 0},
 	{"startcutsceneex", GameScript::StartCutScene, 0}, //pst (unknown)
@@ -936,6 +947,7 @@ static const ActionLink actionnames[] = {
 	{"turn", GameScript::Turn, 0},
 	{"turnamt", GameScript::TurnAMT, AF_BLOCKING}, //relative Face()
 	{"undoexplore", GameScript::UndoExplore, 0},
+	{"unhide", GameScript::Unhide, 0},
 	{"unhidegui", GameScript::UnhideGUI, 0},
 	{"unloadarea", GameScript::UnloadArea, 0},
 	{"unlock", GameScript::Unlock, 0},
@@ -1048,6 +1060,7 @@ static const ObjectLink objectnames[] = {
 	{"sixthnearestenemyof", GameScript::SixthNearestEnemyOf},
 	{"sixthnearestenemyoftype", GameScript::SixthNearestEnemyOfType},
 	{"sixthnearestmygroupoftype", GameScript::SixthNearestMyGroupOfType},
+	{"spelltarget", GameScript::SpellTarget},
 	{"strongestof", GameScript::StrongestOf},
 	{"strongestofmale", GameScript::StrongestOfMale},
 	{"tenthnearest", GameScript::TenthNearest},
@@ -1143,7 +1156,7 @@ static const IDSLink* FindIdentifier(const char* idsname)
 		}
 	}
 	
-	printMessage("GameScript", "Couldn't assign ids target: %.*s\n", YELLOW,
+	Log(WARNING, "GameScript", "Couldn't assign ids target: %.*s",
 		len, idsname );
 	return NULL;
 }
@@ -1269,21 +1282,55 @@ static void CleanupIEScript()
 	actionsTable.release();
 	objectsTable.release();
 	overrideActionsTable.release();
+	overrideTriggersTable.release();
 	if (ObjectIDSTableNames)
 		free(ObjectIDSTableNames);
 	ObjectIDSTableNames = NULL;
 }
 
-void printFunction(Holder<SymbolMgr> table, int index)
+void printFunction(StringBuffer& buffer, Holder<SymbolMgr> table, int index)
 {
 	const char *str = table->GetStringIndex(index);
 	int value = table->GetValueIndex(index);
 
 	int len = strchr(str,'(')-str;
 	if (len<0) {
-		print("%d %s\n", value, str);
+		buffer.appendFormatted("%d %s", value, str);
 	} else {
-		print("%d %.*s\n", value, len, str);
+		buffer.appendFormatted("%d %.*s", value, len, str);
+	}
+}
+
+void LoadActionFlags(const char *tableName, int flag, bool critical)
+{
+	int i, j;
+
+	int tableIndex = core->LoadSymbol(tableName);
+	if (tableIndex < 0) {
+		if (critical) {
+			error("GameScript", "Couldn't find %s symbols!\n",tableName);
+		} else {
+			return;
+		}
+	}
+	Holder<SymbolMgr> table = core->GetSymbol(tableIndex);
+	if (!table) {
+		error("GameScript", "Couldn't load %s symbols!\n",tableName);
+	}
+	j = table->GetSize();
+	while (j--) {
+		i = table->GetValueIndex( j );
+		if (i >= MAX_ACTIONS) {
+			Log(ERROR, "GameScript", "%s action %d (%s) is too high, ignoring",
+				tableName, i, table->GetStringIndex( j ) );
+			continue;
+		}
+		if (!actions[i]) {
+			Log(WARNING, "GameScript", "%s action %d (%s) doesn't exist, ignoring",
+				tableName, i, table->GetStringIndex( j ) );
+			continue;
+		}
+		actionflags[i] |= flag;
 	}
 }
 
@@ -1304,6 +1351,7 @@ void InitializeIEScript()
 	int aT = core->LoadSymbol( "action" );
 	int oT = core->LoadSymbol( "object" );
 	int gaT = core->LoadSymbol( "gemact" );
+	int gtT = core->LoadSymbol( "gemtrig" );
 	AutoTable objNameTable("script");
 	if (tT < 0 || aT < 0 || oT < 0 || !objNameTable) {
 		error("GameScript", "A critical scripting file is missing!\n");
@@ -1312,6 +1360,7 @@ void InitializeIEScript()
 	actionsTable = core->GetSymbol( aT );
 	objectsTable = core->GetSymbol( oT );
 	overrideActionsTable = core->GetSymbol( gaT );
+	overrideTriggersTable = core->GetSymbol( gtT );
 	if (!triggersTable || !actionsTable || !objectsTable || !objNameTable) {
 		error("GameScript", "A critical scripting file is damaged!\n");
 	}
@@ -1355,31 +1404,35 @@ void InitializeIEScript()
 	memset( actionflags, 0, sizeof( actionflags ) );
 	memset( objects, 0, sizeof( objects ) );
 
-	int j;
+	int j, max;
 
-	j = triggersTable->GetSize();
-	while (j--) {
+	max = triggersTable->GetSize();
+	for (j=0;j<max;j++) {
 		i = triggersTable->GetValueIndex( j );
 		const TriggerLink* poi = FindTrigger(triggersTable->GetStringIndex( j ));
 
 		bool was_condition = (i & 0x4000);
 		i &= 0x3fff;
 		if (i >= MAX_TRIGGERS) {
-			printMessage("GameScript", "trigger %d (%s) is too high, ignoring\n", RED,
+			Log(ERROR, "GameScript", "trigger %d (%s) is too high, ignoring",
 				i, triggersTable->GetStringIndex( j ) );
 			continue;
 		}
 
 		if (triggers[i]) {
 			if (poi && triggers[i]!=poi->Function) {
-				printMessage("GameScript", "%s is in collision with ", YELLOW,
+				StringBuffer buffer;
+				buffer.appendFormatted("%s is in collision with ",
 					triggersTable->GetStringIndex( j ) );
-				printFunction(triggersTable,triggersTable->FindValue(triggersTable->GetValueIndex( j )));
+				printFunction(buffer, triggersTable, triggersTable->FindValue(triggersTable->GetValueIndex(j)));
+				Log(WARNING, "GameScript", buffer);
 			} else {
 				if (InDebug&ID_TRIGGERS) {
-					printMessage("GameScript", "%s is a synonym of ", WHITE,
+					StringBuffer buffer;
+					buffer.appendFormatted("%s is a synonym of ",
 						triggersTable->GetStringIndex( j ) );
-					printFunction(triggersTable,triggersTable->FindValue(triggersTable->GetValueIndex( j )));
+					printFunction(buffer, triggersTable, triggersTable->FindValue(triggersTable->GetValueIndex(j)));
+					Log(DEBUG, "GameScript", buffer);
 				}
 			}
 			continue; //we already found an alternative
@@ -1411,7 +1464,7 @@ void InitializeIEScript()
 			for (i = 0; triggernames[i].Name; i++) {
 				if (f == triggernames[i].Function) {
 					if (InDebug&ID_TRIGGERS) {
-						printMessage("GameScript", "%s is a synonym of %s\n", WHITE,
+						Log(MESSAGE, "GameScript", "%s is a synonym of %s",
 							triggersTable->GetStringIndex( j ), triggernames[i].Name );
 						break;
 					}
@@ -1420,31 +1473,37 @@ void InitializeIEScript()
 			continue;
 		}
 
-		printMessage("GameScript","Couldn't assign function to trigger: ", YELLOW);
-		printFunction(triggersTable,j);
+		StringBuffer buffer;
+		buffer.append("Couldn't assign function to trigger: ");
+		printFunction(buffer, triggersTable, j);
+		Log(WARNING, "GameScript", buffer);
 	}
 
-	j = actionsTable->GetSize();
-	while (j--) {
+	max = actionsTable->GetSize();
+	for (j=0;j<max;j++) {
 		i = actionsTable->GetValueIndex( j );
 		if (i >= MAX_ACTIONS) {
-			printMessage("GameScript", "action %d (%s) is too high, ignoring\n", RED,
+			Log(ERROR, "GameScript", "action %d (%s) is too high, ignoring",
 				i, actionsTable->GetStringIndex( j ) );
 			continue;
 		}
 		const ActionLink* poi = FindAction( actionsTable->GetStringIndex( j ));
 		if (actions[i]) {
 			if (poi && actions[i]!=poi->Function) {
-				printMessage("GameScript", "%s is in collision with ", YELLOW,
+				StringBuffer buffer;
+
+				buffer.appendFormatted("%s is in collision with ",
 					actionsTable->GetStringIndex( j ) );
-				printFunction(actionsTable, actionsTable->FindValue(actionsTable->GetValueIndex(j)));
-//->GetStringIndex(actionsTable->FindValue(actionsTable->GetValueIndex( j )) ) );
+				printFunction(buffer, actionsTable, actionsTable->FindValue(actionsTable->GetValueIndex(j)));
+				Log(WARNING, "GameScript", buffer);
 			} else {
 				if (InDebug&ID_ACTIONS) {
-					printMessage("GameScript", "%s is a synonym of ", WHITE,
+					StringBuffer buffer;
+
+					buffer.appendFormatted("%s is a synonym of ",
 						actionsTable->GetStringIndex( j ) );
-					printFunction(actionsTable, actionsTable->FindValue(actionsTable->GetValueIndex( j )));
-//actionsTable->GetStringIndex(actionsTable->FindValue(actionsTable->GetValueIndex( j )) ) );
+					printFunction(buffer, actionsTable, actionsTable->FindValue(actionsTable->GetValueIndex(j)));
+					Log(DEBUG, "GameScript", buffer);
 				}
 			}
 			continue; //we already found an alternative
@@ -1464,29 +1523,85 @@ void InitializeIEScript()
 		 * we add/replace some actions from gemact.ids
 		 * right now you can't print or generate these actions!
 		 */
-		j = overrideActionsTable->GetSize();
-		while (j--) {
+		max = overrideActionsTable->GetSize();
+		for (j=0;j<max;j++) {
 			i = overrideActionsTable->GetValueIndex( j );
 			if (i >= MAX_ACTIONS) {
-				printMessage("GameScript", "action %d (%s) is too high, ignoring\n", RED,
+				Log(ERROR, "GameScript", "action %d (%s) is too high, ignoring",
 					i, overrideActionsTable->GetStringIndex( j ) );
 				continue;
 			}
 			const ActionLink *poi = FindAction( overrideActionsTable->GetStringIndex( j ));
 			if (!poi) {
+				StringBuffer buffer;
+
+				buffer.append("Couldn't assign function to override action: ");
+				printFunction(buffer, overrideActionsTable, j);
 				continue;
 			}
-			if (actions[i]) {
-				printMessage("GameScript", "%s overrides existing action ", WHITE,
+			if (actions[i] && (actions[i]!=poi->Function || actionflags[i]!=poi->Flags) ) {
+				StringBuffer buffer;
+
+				buffer.appendFormatted("%s overrides existing action ",
 					overrideActionsTable->GetStringIndex( j ) );
-				printFunction( actionsTable, actionsTable->FindValue(overrideActionsTable->GetValueIndex( j )));
-				//printFunction( actionsTable->GetStringIndex(actionsTable->FindValue(overrideActionsTable->GetValueIndex( j )) ) );
+				int x = actionsTable->FindValue(i);
+				if (x>=0) {
+					printFunction(buffer, actionsTable, actionsTable->FindValue(i));
+				} else {
+					printFunction(buffer, overrideActionsTable, overrideActionsTable->FindValue(overrideActionsTable->GetValueIndex(j)));
+				}
+				Log(MESSAGE, "GameScript", buffer);
 			}
 			actions[i] = poi->Function;
 			actionflags[i] = poi->Flags;
 		}
 	}
 
+	if (overrideTriggersTable) {
+		/*
+		 * we add/replace some actions from gemtrig.ids
+		 * right now you can't print or generate these actions!
+		 */
+		max = overrideTriggersTable->GetSize();
+		for (j=0;j<max;j++) {
+			i = overrideTriggersTable->GetValueIndex( j );
+			bool was_condition = (i & 0x4000);
+			i &= 0x3fff;
+			if (i >= MAX_TRIGGERS) {
+				Log(ERROR, "GameScript", "trigger %d (%s) is too high, ignoring",
+					i, overrideTriggersTable->GetStringIndex( j ) );
+				continue;
+			}
+			const TriggerLink *poi = FindTrigger( overrideTriggersTable->GetStringIndex( j ));
+			if (!poi) {
+				StringBuffer buffer;
+
+				buffer.append("Couldn't assign function to override trigger: ");
+				printFunction(buffer, overrideTriggersTable, j);
+				continue;
+			}
+			int tf = poi->Flags | (was_condition?TF_CONDITION:0);
+			if (triggers[i] && ( (triggers[i]!=poi->Function) || (triggerflags[i]!=tf) ) ) {
+				StringBuffer buffer;
+
+				buffer.appendFormatted("%s overrides existing trigger ",
+					overrideTriggersTable->GetStringIndex( j ) );
+				int x = triggersTable->FindValue(i);
+				if (x<0) x = triggersTable->FindValue(i|0x4000);
+				if (x>=0) {
+					printFunction(buffer, triggersTable, x);
+				} else {
+					x = overrideTriggersTable->FindValue(i);
+					if (x<0 || x>=j) x = overrideTriggersTable->FindValue(i|0x4000);
+					printFunction(buffer, overrideTriggersTable, x);
+				}
+				Log(MESSAGE, "GameScript", buffer);
+			}
+			triggers[i] = poi->Function;
+			triggerflags[i] = tf;
+		}
+	}
+
 	for (l = missing_actions.begin(); l!=missing_actions.end();l++) {
 		j = *l;
 		// found later as a different name
@@ -1500,7 +1615,7 @@ void InitializeIEScript()
 			for (i = 0; actionnames[i].Name; i++) {
 				if (f == actionnames[i].Function) {
 					if (InDebug&ID_ACTIONS) {
-						printMessage("GameScript", "%s is a synonym of %s\n", WHITE,
+						Log(MESSAGE, "GameScript", "%s is a synonym of %s",
 							actionsTable->GetStringIndex( j ), actionnames[i].Name );
 						break;
 					}
@@ -1508,31 +1623,34 @@ void InitializeIEScript()
 			}
 			continue;
 		}
-		printMessage("GameScript","Couldn't assign function to action: ", YELLOW);
-		printFunction(actionsTable,j);
-		//printFunction(actionsTable->GetStringIndex(j) );
+		StringBuffer buffer;
+		buffer.append("Couldn't assign function to action: ");
+		printFunction(buffer, actionsTable, j);
+		Log(WARNING, "GameScript", buffer);
 	}
 
 	j = objectsTable->GetSize();
 	while (j--) {
 		i = objectsTable->GetValueIndex( j );
 		if (i >= MAX_OBJECTS) {
-			printMessage("GameScript", "object %d (%s) is too high, ignoring\n", RED,
+			Log(ERROR, "GameScript", "object %d (%s) is too high, ignoring",
 				i, objectsTable->GetStringIndex( j ) );
 			continue;
 		}
 		const ObjectLink* poi = FindObject( objectsTable->GetStringIndex( j ));
 		if (objects[i]) {
 			if (poi && objects[i]!=poi->Function) {
-				printMessage("GameScript", "%s is in collision with ", YELLOW,
+				StringBuffer buffer;
+				buffer.appendFormatted("%s is in collision with ",
 					objectsTable->GetStringIndex( j ) );
-				printFunction(objectsTable,objectsTable->FindValue(objectsTable->GetValueIndex( j )));
-				//printFunction(objectsTable->GetStringIndex(objectsTable->FindValue(objectsTable->GetValueIndex( j )) ) );
+				printFunction(buffer, objectsTable, objectsTable->FindValue(objectsTable->GetValueIndex(j)));
+				Log(WARNING, "GameScript", buffer);
 			} else {
-				printMessage("GameScript", "%s is a synonym of ", WHITE,
+				StringBuffer buffer;
+				buffer.appendFormatted("%s is a synonym of ",
 					objectsTable->GetStringIndex( j ) );
-				printFunction(objectsTable, objectsTable->FindValue(objectsTable->GetValueIndex( j )));
-				//printFunction(objectsTable->GetStringIndex(objectsTable->FindValue(objectsTable->GetValueIndex( j )) ) );
+				printFunction(buffer, objectsTable, objectsTable->FindValue(objectsTable->GetValueIndex(j)));
+				Log(DEBUG, "GameScript", buffer);
 			}
 			continue;
 		}
@@ -1556,46 +1674,27 @@ void InitializeIEScript()
 		if (f) {
 			for (i = 0; objectnames[i].Name; i++) {
 				if (f == objectnames[i].Function) {
-					printMessage("GameScript", "%s is a synonym of %s\n", WHITE,
+					Log(MESSAGE, "GameScript", "%s is a synonym of %s",
 						objectsTable->GetStringIndex( j ), objectnames[i].Name );
 					break;
 				}
 			}
 			continue;
 		}
-		printMessage("GameScript","Couldn't assign function to object: ", YELLOW);
-		printFunction(objectsTable,j);
-		//printFunction(objectsTable->GetStringIndex(j) );
+		StringBuffer buffer;
+		buffer.append("Couldn't assign function to object: ");
+		printFunction(buffer, objectsTable, j);
+		Log(WARNING, "GameScript", buffer);
 	}
 
-	int instantTableIndex = core->LoadSymbol("instant");
-	if (instantTableIndex < 0) {
-		error("GameScript", "Couldn't find instant symbols!\n");
-	}
-	Holder<SymbolMgr> instantTable = core->GetSymbol(instantTableIndex);
-	if (!instantTable) {
-		error("GameScript", "Couldn't load instant symbols!\n");
-	}
-	j = instantTable->GetSize();
-	while (j--) {
-		i = instantTable->GetValueIndex( j );
-		if (i >= MAX_ACTIONS) {
-			printMessage("GameScript", "instant action %d (%s) is too high, ignoring\n", RED,
-				i, instantTable->GetStringIndex( j ) );
-			continue;
-		}
-		if (!actions[i]) {
-			printMessage("GameScript", "instant action %d (%s) doesn't exist, ignoring\n", YELLOW,
-				i, instantTable->GetStringIndex( j ) );
-			continue;
-		}
-		actionflags[i] |= AF_INSTANT;
-	}
+	LoadActionFlags("instant", AF_INSTANT, true);
+	LoadActionFlags("actsleep", AF_SLEEP, false);
+	LoadActionFlags("chase", AF_CHASE, false);
 
 	int savedTriggersIndex = core->LoadSymbol("svtriobj");
 	if (savedTriggersIndex < 0) {
 		// leaving this as not strictly necessary, for now
-		printMessage("GameScript", "Couldn't find saved trigger symbols!\n", YELLOW);
+		Log(WARNING, "GameScript", "Couldn't find saved trigger symbols!");
 	} else {
 		Holder<SymbolMgr> savedTriggersTable = core->GetSymbol(savedTriggersIndex);
 		if (!savedTriggersTable) {
@@ -1606,12 +1705,12 @@ void InitializeIEScript()
 			i = savedTriggersTable->GetValueIndex( j );
 			i &= 0x3fff;
 			if (i >= MAX_ACTIONS) {
-				printMessage("GameScript", "saved trigger %d (%s) is too high, ignoring\n", RED,
+				Log(ERROR, "GameScript", "saved trigger %d (%s) is too high, ignoring",
 					i, savedTriggersTable->GetStringIndex( j ) );
 				continue;
 			}
 			if (!triggers[i]) {
-				printMessage("GameScript", "saved trigger %d (%s) doesn't exist, ignoring\n", YELLOW,
+				Log(WARNING, "GameScript", "saved trigger %d (%s) doesn't exist, ignoring",
 					i, savedTriggersTable->GetStringIndex( j ) );
 				continue;
 			}
@@ -1639,7 +1738,7 @@ GameScript::~GameScript(void)
 		//set 3. parameter to true if you want instant free
 		//and possible death
 		if (InDebug&ID_REFERENCE) {
-			print("One instance of %s is dropped from %d.\n", Name, BcsCache.RefCount(Name) );
+			Log(DEBUG, "GameScript", "One instance of %s is dropped from %d.", Name, BcsCache.RefCount(Name) );
 		}
 		int res = BcsCache.DecRef(script, Name, true);
 
@@ -1647,7 +1746,7 @@ GameScript::~GameScript(void)
 			error("GameScript", "Corrupted Script cache encountered (reference count went below zero), Script name is: %.8s\n", Name);
 		}
 		if (!res) {
-			//print("Freeing script %s because its refcount has reached 0.\n", Name);
+			//print("Freeing script %s because its refcount has reached 0.", Name);
 			script->Release();
 		}
 		script = NULL;
@@ -1663,7 +1762,7 @@ Script* GameScript::CacheScript(ieResRef ResRef, bool AIScript)
 	Script *newScript = (Script *) BcsCache.GetResource(ResRef);
 	if ( newScript ) {
 		if (InDebug&ID_REFERENCE) {
-			print("Caching %s for the %d. time\n", ResRef, BcsCache.RefCount(ResRef) );
+			Log(DEBUG, "GameScript", "Caching %s for the %d. time\n", ResRef, BcsCache.RefCount(ResRef) );
 		}
 		return newScript;
 	}
@@ -1674,14 +1773,14 @@ Script* GameScript::CacheScript(ieResRef ResRef, bool AIScript)
 	}
 	stream->ReadLine( line, 10 );
 	if (strncmp( line, "SC", 2 ) != 0) {
-		printMessage( "GameScript","Not a Compiled Script file\n", YELLOW );
+		Log(WARNING, "GameScript", "Not a Compiled Script file");
 		delete( stream );
 		return NULL;
 	}
 	newScript = new Script( );
 	BcsCache.SetAt( ResRef, (void *) newScript );
 	if (InDebug&ID_REFERENCE) {
-		print("Caching %s for the %d. time\n", ResRef, BcsCache.RefCount(ResRef) );
+		Log(DEBUG, "GameScript", "Caching %s for the %d. time", ResRef, BcsCache.RefCount(ResRef) );
 	}
 
 	while (true) {
@@ -1757,7 +1856,7 @@ static Object* DecodeObject(const char* line)
 		oB->objectFields[i + ObjectFieldsCount] = ParseInt( line );
 	}
 	if (*line != 'O' || *(line + 1) != 'B') {
-		printMessage("GameScript", "Got confused parsing object line: %s\n", YELLOW, origline);
+		Log(WARNING, "GameScript", "Got confused parsing object line: %s", origline);
 	}
 	//let the object realize it has no future (in case of null objects)
 	if (oB->isNull()) {
@@ -1796,6 +1895,11 @@ static Trigger* ReadTrigger(DataStream* stream)
 	tR->objectParameter = DecodeObject( line );
 	stream->ReadLine( line, 1024 );
 	free( line );
+	//discard invalid triggers, so they won't cause a crash
+	if (tR->triggerID>=MAX_TRIGGERS) {
+		delete tR;
+		return NULL;
+	}
 	return tR;
 }
 
@@ -1857,7 +1961,12 @@ bool GameScript::Update(bool *continuing, bool *done)
 						// this one is a bit more complicated, due to possible
 						// interactions with Continue() (lastAction here is always
 						// the first block encountered), needs more testing
-						//if (done) *done = true;
+						//HoW ar9708, djinni would become visible only if this is not commented out
+						//See: SEDJINNI.BCS
+						//BG2 on the other hand ... (eg. spirit trolls trollsp01 in ar1506)
+						if (core->HasFeature(GF_SKIPUPDATE_HACK)) {
+							if (done) *done = true;
+						}
 						return false;
 					}
 
@@ -1926,9 +2035,9 @@ void GameScript::EvaluateAllBlocks()
 					// TODO: this will break blocking instants, if there are any
 					target->ReleaseCurrentAction();
 				} else if ((InDebug&ID_CUTSCENE) || !action->objects[1]) {
-					printMessage("GameScript","Failed to find CutSceneID target!\n",YELLOW);
+					Log(WARNING, "GameScript", "Failed to find CutSceneID target!");
 					if (action->objects[1]) {
-						action->objects[1]->Dump();
+						action->objects[1]->dump();
 					}
 				}
 			}
@@ -2006,7 +2115,7 @@ Response* GameScript::ReadResponse(DataStream* stream)
 		strlwr(aC->string1Parameter);
 		if (aC->actionID>=MAX_ACTIONS) {
 			aC->actionID=0;
-			printMessage("GameScript","Invalid script action ID!",LIGHT_RED);
+			Log(ERROR, "GameScript", "Invalid script action ID!");
 		} else {
 			if (actionflags[aC->actionID] & AF_SCRIPTLEVEL) {
 				//can't set this here, because the same script may be loaded
@@ -2067,7 +2176,7 @@ bool Condition::Evaluate(Scriptable* Sender)
 		if (result > 1) {
 			//we started an Or() block
 			if (ORcount) {
-				printMessage( "GameScript","Unfinished OR block encountered!\n",YELLOW );
+				Log(WARNING, "GameScript", "Unfinished OR block encountered!");
 			}
 			ORcount = result;
 			subresult = false;
@@ -2085,7 +2194,7 @@ bool Condition::Evaluate(Scriptable* Sender)
 		}
 	}
 	if (ORcount) {
-		printMessage( "GameScript","Unfinished OR block encountered!\n",YELLOW );
+		Log(WARNING, "GameScript", "Unfinished OR block encountered!");
 	}
 	return 1;
 }
@@ -2094,7 +2203,7 @@ bool Condition::Evaluate(Scriptable* Sender)
 int Trigger::Evaluate(Scriptable* Sender)
 {
 	if (!this) {
-		printMessage( "GameScript","Trigger evaluation fails due to NULL trigger.\n",LIGHT_RED );
+		Log(ERROR, "GameScript", "Trigger evaluation fails due to NULL trigger.");
 		return 0;
 	}
 	TriggerFunction func = triggers[triggerID];
@@ -2104,14 +2213,12 @@ int Trigger::Evaluate(Scriptable* Sender)
 	}
 	if (!func) {
 		triggers[triggerID] = GameScript::False;
-		printMessage("GameScript"," ",YELLOW);
-		print("Unhandled trigger code: 0x%04x %s\n",
+		Log(WARNING, "GameScript", "Unhandled trigger code: 0x%04x %s",
 			triggerID, tmpstr );
 		return 0;
 	}
 	if (InDebug&ID_TRIGGERS) {
-		printMessage("GameScript"," ",YELLOW);
-		print( "Executing trigger code: 0x%04x %s\n",
+		Log(WARNING, "GameScript", "Executing trigger code: 0x%04x %s",
 				triggerID, tmpstr );
 	}
 	int ret = func( Sender, this );
@@ -2181,9 +2288,9 @@ int Response::Execute(Scriptable* Sender)
 	return ret;
 }
 
-void PrintAction(int actionID)
+void PrintAction(StringBuffer& buffer, int actionID)
 {
-	print("Action: %d %s\n", actionID , actionsTable->GetValue(actionID) );
+	buffer.appendFormatted("Action: %d %s\n", actionID, actionsTable->GetValue(actionID));
 }
 
 void GameScript::ExecuteAction(Scriptable* Sender, Action* aC)
@@ -2198,7 +2305,7 @@ void GameScript::ExecuteAction(Scriptable* Sender, Action* aC)
 
 		if (scr) {
 			if (InDebug&ID_ACTIONS) {
-				printMessage("GameScript", "Sender: %s-->override: %s\n", YELLOW,
+				Log(WARNING, "GameScript", "Sender: %s-->override: %s",
 					Sender->GetScriptName(), scr->GetScriptName() );
 			}
 			scr->ReleaseCurrentAction();
@@ -2213,17 +2320,18 @@ void GameScript::ExecuteAction(Scriptable* Sender, Action* aC)
 				scr->CurrentActionInterruptable = false;
 			}
 		} else {
-			printMessage("GameScript","Actionoverride failed for object: \n",LIGHT_RED);
-			aC->objects[0]->Dump();
+			Log(ERROR, "GameScript", "Actionoverride failed for object: ");
+			aC->objects[0]->dump();
 		}
 
 		aC->Release();
 		return;
 	}
 	if (InDebug&ID_ACTIONS) {
-		printMessage("GameScript"," ",YELLOW);
-		PrintAction(actionID);
-		print("Sender: %s\n",Sender->GetScriptName() );
+		StringBuffer buffer;
+		PrintAction(buffer, actionID);
+		buffer.appendFormatted("Sender: %s\n", Sender->GetScriptName());
+		Log(WARNING, "GameScript", buffer);
 	}
 	ActionFunction func = actions[actionID];
 	if (func) {
@@ -2233,7 +2341,7 @@ void GameScript::ExecuteAction(Scriptable* Sender, Action* aC)
 			Sender->Activate();
 			if (actionflags[actionID]&AF_ALIVE) {
 				if (Sender->GetInternalFlag()&IF_STOPATTACK) {
-					printMessage("GameScript", "Aborted action due to death\n", YELLOW);
+					Log(WARNING, "GameScript", "Aborted action due to death");
 					Sender->ReleaseCurrentAction();
 					return;
 				}
@@ -2242,11 +2350,11 @@ void GameScript::ExecuteAction(Scriptable* Sender, Action* aC)
 		func( Sender, aC );
 	} else {
 		actions[actionID] = NoActionAtAll;
-		printMessage("GameScript", "Unknown ", YELLOW);
-		textcolor(YELLOW);
-		PrintAction(actionID);
+		StringBuffer buffer;
+		buffer.append("Unknown ");
+		PrintAction(buffer, actionID);
+		Log(WARNING, "GameScript", buffer);
 		Sender->ReleaseCurrentAction();
-		textcolor(WHITE);
 		return;
 	}
 
@@ -2254,9 +2362,11 @@ void GameScript::ExecuteAction(Scriptable* Sender, Action* aC)
 	if (actionflags[actionID] & AF_IMMEDIATE) {
 		//this action never entered the action queue, therefore shouldn't be freed
 		if (aC->GetRef()!=1) {
-			print("Immediate action got queued!\n");
-			PrintAction(actionID);
-			abort();
+			StringBuffer buffer;
+			buffer.append("Immediate action got queued!\n");
+			PrintAction(buffer, actionID);
+			Log(ERROR, "GameScript", buffer);
+			error("GameScript", "aborting...\n");
 		}
 		return;
 	}
@@ -2273,7 +2383,7 @@ Trigger* GenerateTrigger(char* String)
 {
 	strlwr( String );
 	if (InDebug&ID_TRIGGERS) {
-		printMessage("GameScript", "Compiling:%s\n", YELLOW, String);
+		Log(WARNING, "GameScript", "Compiling:%s", String);
 	}
 	int negate = 0;
 	if (*String == '!') {
@@ -2283,14 +2393,14 @@ Trigger* GenerateTrigger(char* String)
 	int len = strlench(String,'(')+1; //including (
 	int i = triggersTable->FindString(String, len);
 	if (i<0) {
-		printMessage("GameScript", "Invalid scripting trigger: %s\n", LIGHT_RED, String);
+		Log(ERROR, "GameScript", "Invalid scripting trigger: %s", String);
 		return NULL;
 	}
 	char *src = String+len;
 	char *str = triggersTable->GetStringIndex( i )+len;
 	Trigger *trigger = GenerateTriggerCore(src, str, i, negate);
 	if (!trigger) {
-		printMessage("GameScript", "Malformed scripting trigger: %s\n", LIGHT_RED, String);
+		Log(ERROR, "GameScript", "Malformed scripting trigger: %s", String);
 		return NULL;
 	}
 	return trigger;
@@ -2300,7 +2410,7 @@ Action* GenerateAction(char* String)
 {
 	strlwr( String );
 	if (InDebug&ID_ACTIONS) {
-		printMessage("GameScript", "Compiling:%s\n", YELLOW, String);
+		Log(WARNING, "GameScript", "Compiling:%s", String);
 	}
 	int len = strlench(String,'(')+1; //including (
 	char *src = String+len;
@@ -2317,7 +2427,7 @@ Action* GenerateAction(char* String)
 	if (i<0) {
 		i = actionsTable->FindString(String, len);
 		if (i < 0) {
-			printMessage("GameScript", "Invalid scripting action: %s\n", LIGHT_RED, String);
+			Log(ERROR, "GameScript", "Invalid scripting action: %s", String);
 			return NULL;
 		}
 		str = actionsTable->GetStringIndex( i )+len;
@@ -2325,7 +2435,7 @@ Action* GenerateAction(char* String)
 	}
 	Action *action = GenerateActionCore( src, str, actionID);
 	if (!action) {
-		printMessage("GameScript", "Malformed scripting action: %s\n", LIGHT_RED, String);
+		Log(ERROR, "GameScript", "Malformed scripting action: %s", String);
 		return NULL;
 	}
 	return action;
@@ -2343,6 +2453,34 @@ Action* GenerateActionDirect(char *String, Scriptable *object)
 	return action;
 }
 
+void Object::dump() const
+{
+	StringBuffer buffer;
+	dump(buffer);
+	Log(DEBUG, "GameScript", buffer);
+}
+
+void Object::dump(StringBuffer& buffer) const
+{
+	int i;
+
+	GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
+	if(objectName[0]) {
+		buffer.appendFormatted("Object: %s\n",objectName);
+		return;
+	}
+	buffer.appendFormatted("IDS Targeting: ");
+	for(i=0;i<MAX_OBJECT_FIELDS;i++) {
+		buffer.appendFormatted("%d ",objectFields[i]);
+	}
+	buffer.append("\n");
+	buffer.append("Filters: ");
+	for(i=0;i<MAX_NESTING;i++) {
+		buffer.appendFormatted("%d ",objectFilters[i]);
+	}
+	buffer.append("\n");
+}
+
 /** Return true if object is null */
 bool Object::isNull()
 {
@@ -2360,3 +2498,53 @@ bool Object::isNull()
 	return true;
 }
 
+void Trigger::dump() const
+{
+	StringBuffer buffer;
+	dump(buffer);
+	Log(DEBUG, "GameScript", buffer);
+}
+
+void Trigger::dump(StringBuffer& buffer) const
+{
+	GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
+	buffer.appendFormatted("Trigger: %d\n", triggerID);
+	buffer.appendFormatted("Int parameters: %d %d %d\n", int0Parameter, int1Parameter, int2Parameter);
+	buffer.appendFormatted("Point: [%d.%d]\n", pointParameter.x, pointParameter.y);
+	buffer.appendFormatted("String0: %s\n", string0Parameter);
+	buffer.appendFormatted("String1: %s\n", string1Parameter);
+	if (objectParameter) {
+		objectParameter->dump(buffer);
+	} else {
+		buffer.appendFormatted("No object\n");
+	}
+	buffer.appendFormatted("\n");
+}
+
+void Action::dump() const
+{
+	StringBuffer buffer;
+	dump(buffer);
+	Log(DEBUG, "GameScript", buffer);
+}
+
+void Action::dump(StringBuffer& buffer) const
+{
+	int i;
+
+	GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
+	buffer.appendFormatted("Int0: %d, Int1: %d, Int2: %d\n",int0Parameter, int1Parameter, int2Parameter);
+	buffer.appendFormatted("String0: %s, String1: %s\n", string0Parameter?string0Parameter:"<NULL>", string1Parameter?string1Parameter:"<NULL>");
+	for (i=0;i<3;i++) {
+		if (objects[i]) {
+			buffer.appendFormatted( "%d. ",i+1);
+			objects[i]->dump(buffer);
+		} else {
+			buffer.appendFormatted( "%d. Object - NULL\n",i+1);
+		}
+	}
+
+	buffer.appendFormatted("RefCount: %d\n", RefCount);
+}
+
+}
