@@ -31,9 +31,13 @@
 #include <cstdio>
 #include <vector>
 
+namespace GemRB {
+
 class Action;
 class GameScript;
 
+class StringBuffer;
+
 //escapearea flags
 #define EA_DESTROY 1        //destroy actor at the exit (otherwise move to new place)
 #define EA_NOSEE   2        //no need to see the exit
@@ -49,6 +53,7 @@ class GameScript;
 #define DS_AREA    128
 
 //verbal constant (bg2), we have a lookup table (vcremap) for other games
+#define VB_INITIALMEET 0
 #define VB_PANIC     1
 #define VB_HAPPY     2
 #define VB_UNHAPPY   3
@@ -120,8 +125,7 @@ class GameScript;
 #define GSASSERT(f,c) \
 	if(!(f))  \
 	{  \
-		print("Assertion failed: %s [0x%08lX] Line %d",#f, c, __LINE__); \
-		abort(); \
+		error("GSASSERT", "Assertion failed: %s [0x%08lX] Line %d\n",#f, c, __LINE__); \
 	}
 
 typedef std::vector<ieDword> SrcVector;
@@ -179,27 +183,8 @@ public:
 private:
 	volatile unsigned long canary;
 public:
-	void Dump()
-	{
-		int i;
-
-		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
-		if(objectName[0]) {
-			print("Object: %s\n",objectName);
-			return;
-		}
-		print("IDS Targeting: ");
-		for(i=0;i<MAX_OBJECT_FIELDS;i++) {
-			print("%d ",objectFields[i]);
-		}
-		print("\n");
-		print("Filters: ");
-		for(i=0;i<MAX_NESTING;i++) {
-			print("%d ",objectFilters[i]);
-		}
-		print("\n");
-	}
-
+	void dump() const;
+	void dump(StringBuffer&) const;
 	void Release()
 	{
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
@@ -245,21 +230,8 @@ public:
 private:
 	volatile unsigned long canary;
 public:
-	void Dump()
-	{
-		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
-		print ("Trigger: %d\n", triggerID);
-		print ("Int parameters: %d %d %d\n", int0Parameter, int1Parameter, int2Parameter);
-		print ("Point: [%d.%d]\n", pointParameter.x, pointParameter.y);
-		print ("String0: %s\n", string0Parameter);
-		print ("String1: %s\n", string1Parameter);
-		if (objectParameter) {
-			objectParameter->Dump();
-		} else {
-			print("No object\n");
-		}
-		print("\n");
-	}
+	void dump() const;
+	void dump(StringBuffer&) const;
 
 	void Release()
 	{
@@ -345,32 +317,16 @@ public:
 	int GetRef() {
 		return RefCount;
 	}
-	void Dump()
-	{
-		int i;
 
-		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
-		print("Int0: %d, Int1: %d, Int2: %d\n",int0Parameter, int1Parameter, int2Parameter);
-		print("String0: %s, String1: %s\n", string0Parameter?string0Parameter:"<NULL>", string1Parameter?string1Parameter:"<NULL>");
-		for (i=0;i<3;i++) {
-			if (objects[i]) {
-				print( "%d. ",i+1);
-				objects[i]->Dump();
-			} else {
-				print( "%d. Object - NULL\n",i+1);
-			}
-		}
-
-		print("RefCount: %d\n", RefCount);
-	}
+	void dump() const;
+	void dump(StringBuffer&) const;
 
 	void Release()
 	{
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
 		if (!RefCount) {
-			print( "WARNING!!! Double Freeing in %s: Line %d\n", __FILE__,
-				__LINE__ );
-			abort();
+			error("GameScript", "WARNING!!! Double Freeing in %s: Line %d\n", __FILE__,
+				__LINE__);
 		}
 		RefCount--;
 		if (!RefCount) {
@@ -383,9 +339,8 @@ public:
 		GSASSERT( canary == (unsigned long) 0xdeadbeef, canary );
 		RefCount++;
 		if (RefCount >= 65536) {
-			print( "Refcount increased to: %d in action %d\n", RefCount,
-				actionID );
-			abort();
+			error("GameScript", "Refcount increased to: %d in action %d\n", RefCount,
+				actionID);
 		}
 	}
 };
@@ -402,7 +357,7 @@ public:
 		for (size_t c = 0; c < actions.size(); c++) {
 			if (actions[c]) {
 				if (actions[c]->GetRef()>2) {
-					print("Residue action %d with refcount %d\n", actions[c]->actionID, actions[c]->GetRef());
+					print("Residue action %d with refcount %d", actions[c]->actionID, actions[c]->GetRef());
 				}
 				actions[c]->Release();
 				actions[c] = NULL;
@@ -570,7 +525,9 @@ struct TriggerLink {
 #define AF_INVALID      128
 #define AF_DIRECT       256 //this hack will transfer target from gamecontrol to object1 at compile time
 #define AF_ALIVE        512 //only alive actors can do this
-#define AF_INSTANT      1024
+#define AF_INSTANT      1024 //instant actions
+#define AF_CHASE        2048 // ??? actions involving movement ???
+#define AF_SLEEP        4096 //only awake actors can do this
 
 struct ActionLink {
 	const char* Name;
@@ -588,9 +545,9 @@ struct IDSLink {
 	IDSFunction Function;
 };
 
-#define MAX_TRIGGERS			0xFF
+#define MAX_TRIGGERS			0x100
 #define MAX_ACTIONS			400
-#define MAX_OBJECTS			128
+#define MAX_OBJECTS			256
 #define AI_SCRIPT_LEVEL 4             //the script level of special ai scripts
 
 extern void SetScriptDebugMode(int arg);
@@ -694,6 +651,7 @@ public: //Script Functions
 	static int Dead(Scriptable* Sender, Trigger* parameters);
 	static int Delay(Scriptable* Sender, Trigger* parameters);
 	static int Detect(Scriptable* Sender, Trigger* parameters);
+	static int Detected(Scriptable* Sender, Trigger* parameters);
 	static int Die(Scriptable* Sender, Trigger* parameters);
 	static int Died(Scriptable* Sender, Trigger* parameters);
 	static int Difficulty(Scriptable* Sender, Trigger* parameters);
@@ -739,7 +697,9 @@ public: //Script Functions
 	static int Happiness(Scriptable* Sender, Trigger* parameters);
 	static int HappinessGT(Scriptable* Sender, Trigger* parameters);
 	static int HappinessLT(Scriptable* Sender, Trigger* parameters);
+	static int HarmlessClosed(Scriptable* Sender, Trigger* parameters);
 	static int HarmlessEntered(Scriptable* Sender, Trigger* parameters);
+	static int HarmlessOpened(Scriptable* Sender, Trigger* parameters);
 	static int HasBounceEffects(Scriptable* Sender, Trigger* parameters);
 	static int HasImmunityEffects(Scriptable* Sender, Trigger* parameters);
 	static int HasInnateAbility(Scriptable* Sender, Trigger* parameters);
@@ -912,6 +872,7 @@ public: //Script Functions
 	static int Sequence(Scriptable* Sender, Trigger* parameters);
 	static int SetLastMarkedObject(Scriptable* Sender, Trigger* parameters);
 	static int SetMarkedSpell_Trigger(Scriptable* Sender, Trigger* parameters);
+	static int SetSpellTarget(Scriptable* Sender, Trigger* parameters);
 	static int Specifics(Scriptable* Sender, Trigger* parameters);
 	static int SpellCast(Scriptable* Sender, Trigger* parameters);
 	static int SpellCastInnate(Scriptable* Sender, Trigger* parameters);
@@ -922,6 +883,9 @@ public: //Script Functions
 	static int StoreHasItem(Scriptable* Sender, Trigger* parameters);
 	static int StuffGlobalRandom(Scriptable* Sender, Trigger* parameters);
 	static int SubRace(Scriptable* Sender, Trigger* parameters);
+	static int SummoningLimit(Scriptable* Sender, Trigger* parameters);
+	static int SummoningLimitGT(Scriptable* Sender, Trigger* parameters);
+	static int SummoningLimitLT(Scriptable* Sender, Trigger* parameters);
 	static int SystemVariable_Trigger(Scriptable* Sender, Trigger* parameters);
 	static int TargetUnreachable(Scriptable* Sender, Trigger* parameters);
 	static int Team(Scriptable* Sender, Trigger* parameters);
@@ -948,6 +912,7 @@ public: //Script Functions
 	static int UnselectableVariableGT(Scriptable* Sender, Trigger* parameters);
 	static int UnselectableVariableLT(Scriptable* Sender, Trigger* parameters);
 	static int Unusable(Scriptable* Sender, Trigger* parameters);
+	static int UsedExit(Scriptable* Sender, Trigger* parameters);
 	static int Vacant(Scriptable* Sender, Trigger* parameters);
 	static int WalkedToTrigger(Scriptable* Sender, Trigger* parameters);
 	static int WasInDialog(Scriptable* Sender, Trigger* parameters);
@@ -1017,6 +982,7 @@ public:
 	static void ChangeStatGlobal(Scriptable* Sender, Action* parameters);
 	static void ChangeStoreMarkup(Scriptable* Sender, Action* parameters);
 	static void ChangeTileState(Scriptable* Sender, Action* parameters);
+	static void ChunkCreature(Scriptable *Sender, Action* parameters);
 	static void ClearActions(Scriptable* Sender, Action* parameters);
 	static void ClearAllActions(Scriptable* Sender, Action* parameters);
 	static void ClearPartyEffects(Scriptable* Sender, Action* parameters);
@@ -1282,6 +1248,7 @@ public:
 	static void RestUntilHealed(Scriptable *Sender, Action* parameters);
 	static void ReturnToSavedLocation(Scriptable* Sender, Action* parameters);
 	static void ReturnToSavedLocationDelete(Scriptable* Sender, Action* parameters);
+	static void ReturnToStartLocation(Scriptable* Sender, Action* parameters);
 	static void RevealAreaOnMap(Scriptable* Sender, Action* parameters);
 	static void RunAwayFrom(Scriptable* Sender, Action* parameters);
 	static void RunAwayFromNoInterrupt(Scriptable* Sender, Action* parameters);
@@ -1323,6 +1290,7 @@ public:
 	static void SetGlobalTimerOnce(Scriptable* Sender, Action* parameters);
 	static void SetGlobalTimerRandom(Scriptable* Sender, Action* parameters);
 	static void SetGlobalTint(Scriptable* Sender, Action* parameters);
+	static void SetHomeLocation(Scriptable* Sender, Action* parameters);
 	static void SetHP(Scriptable* Sender, Action* parameters);
 	static void SetHPPercent(Scriptable* Sender, Action* parameters);
 	static void SetInternal(Scriptable* Sender, Action* parameters);
@@ -1425,6 +1393,7 @@ public:
 	static void Turn(Scriptable* Sender, Action* parameters);
 	static void TurnAMT(Scriptable* Sender, Action* parameters);
 	static void UndoExplore(Scriptable *Sender, Action *parameters);
+	static void Unhide(Scriptable* Sender, Action* parameters);
 	static void UnhideGUI(Scriptable* Sender, Action* parameters);
 	static void Unlock(Scriptable* Sender, Action* parameters);
 	static void UnlockScroll(Scriptable* Sender, Action* parameters);
@@ -1521,6 +1490,7 @@ public:
 	static Targets *SixthNearestEnemyOf(Scriptable *Sender, Targets *parameters, int ga_flags);
 	static Targets *SixthNearestEnemyOfType(Scriptable *Sender, Targets *parameters, int ga_flags);
 	static Targets *SixthNearestMyGroupOfType(Scriptable *Sender, Targets *parameters, int ga_flags);
+	static Targets *SpellTarget(Scriptable *Sender, Targets *parameters, int ga_flags);
 	static Targets *StrongestOf(Scriptable *Sender, Targets *parameters, int ga_flags);
 	static Targets *StrongestOfMale(Scriptable *Sender, Targets *parameters, int ga_flags);
 	static Targets *TenthNearest(Scriptable *Sender, Targets *parameters, int ga_flags);
@@ -1555,4 +1525,6 @@ GEM_EXPORT Trigger* GenerateTrigger(char* String);
 
 void InitializeIEScript();
 
+}
+
 #endif
