@@ -30,6 +30,8 @@
 #include "Scriptable/Door.h"
 #include "Scriptable/InfoPoint.h"
 
+namespace GemRB {
+
 /* return a Targets object with a single scriptable inside */
 static inline Targets* ReturnScriptableAsTarget(Scriptable *sc)
 {
@@ -48,7 +50,7 @@ static inline bool DoObjectIDSCheck(Object *oC, Actor *ac, bool *filtered) {
 		*filtered = true;
 		IDSFunction func = idtargets[j];
 		if (!func) {
-			printMessage("GameScript", "Unimplemented IDS targeting opcode: %d\n", YELLOW, j);
+			Log(WARNING, "GameScript", "Unimplemented IDS targeting opcode: %d", j);
 			continue;
 		}
 		if (!func( ac, oC->objectFields[j] ) ) {
@@ -60,13 +62,25 @@ static inline bool DoObjectIDSCheck(Object *oC, Actor *ac, bool *filtered) {
 
 /* do object filtering: Myself, LastAttackerOf(Player1), etc */
 static inline Targets *DoObjectFiltering(Scriptable *Sender, Targets *tgts, Object *oC, int ga_flags) {
+	targetlist::iterator m;
+	const targettype *tt = tgts->GetFirstTarget(m, ST_ACTOR);
+	while (tt) {
+		Actor *target = (Actor *) tt->actor;
+		if (target->ValidTarget(GA_NO_DEAD)) {
+			tt = tgts->GetNextTarget(m, ST_ACTOR);
+		} else {
+			tt = tgts->RemoveTargetAt(m);
+		}
+	}
+
 	for (int i = 0; i < MaxObjectNesting; i++) {
 		int filterid = oC->objectFilters[i];
 		if (!filterid) break;
+		if (filterid < 0) continue;
 
 		ObjectFunction func = objects[filterid];
 		if (!func) {
-			printMessage("GameScript", "Unknown object filter: %d %s\n", YELLOW,
+			Log(WARNING, "GameScript", "Unknown object filter: %d %s",
 				filterid, objectsTable->GetValue(filterid));
 			continue;
 		}
@@ -93,21 +107,8 @@ static inline bool DoObjectChecks(Map *map, Scriptable *Sender, Actor *target, i
 		Actor *source = (Actor *)Sender;
 
 		// Detect() ignores invisibility completely
-		if (!ignoreinvis) {
-			// TODO: move this stuff into a shared function so it can be used elsewhere?
-
-			// SEEINVISIBLE skips these checks :-)
-			if (source->Modified[IE_SEEINVISIBLE] == 0) {
-				ieDword state = target->Modified[IE_STATE_ID];
-				// check for invisibility
-				if ((state & STATE_INVISIBLE) != 0) return false;
-				// check for improved invisibility? probably not
-				//if ((state & STATE_INVIS2) != 0) return false;
-			}
-
-			// maybe this should be setting an invis flag?
-			// TODO: should SEEINVISIBLE ignore this? Detect()?
-			if (target->Modified[IE_AVATARREMOVAL]) return false;
+		if (!ignoreinvis && target->IsInvisibleTo(source)) {
+			return false;
 		}
 
 		// visual range check
@@ -115,7 +116,7 @@ static inline bool DoObjectChecks(Map *map, Scriptable *Sender, Actor *target, i
 		if (dist > visualrange*visualrange) return false;
 
 		// LOS check
-		if (!map->IsVisible(Sender->Pos, target->Pos)) return false;
+		if (!map->IsVisibleLOS(Sender->Pos, target->Pos)) return false;
 
 		// protection against creature
 		if (target->fxqueue.HasEffect(fx_protection_creature_ref)) {
@@ -142,13 +143,25 @@ static Targets* EvaluateObject(Map *map, Scriptable* Sender, Object* oC, int ga_
 	}
 
 	if (oC->objectName[0]) {
-		//We want the object by its name... (doors/triggers don't play here!)
-		Actor* aC = map->GetActor( oC->objectName, ga_flags );
+		//We want the object by its name...
+		Scriptable* aC = map->GetActor( oC->objectName, ga_flags );
 
 		/*if (!aC && (ga_flags&GA_GLOBAL) ) {
 			aC = FindActorNearby(oC->objectName, map, ga_flags );
 		}*/
 
+		//This order is the same as in GetActorObject
+		//TODO:merge them
+		if (!aC) {
+			aC = map->GetTileMap()->GetDoor(oC->objectName);
+		}
+		if (!aC) {
+			aC = map->GetTileMap()->GetContainer(oC->objectName);
+		}
+		if (!aC) {
+			aC = map->GetTileMap()->GetInfoPoint(oC->objectName);
+		}
+
 		//return here because object name/IDS targeting are mutually exclusive
 		return ReturnScriptableAsTarget(aC);
 	}
@@ -192,6 +205,8 @@ static Targets* EvaluateObject(Map *map, Scriptable* Sender, Object* oC, int ga_
 		if (ac == Sender) continue;
 		bool filtered = false;
 		if (DoObjectIDSCheck(oC, ac, &filtered)) {
+			// this is needed so eg. Range trigger gets a good object
+			// HACK: our parsing of Attack([0]) is broken
 			if (!filtered) {
 				// if no filters were applied..
 				assert(!tgts);
@@ -236,10 +251,15 @@ Targets *GetAllActors(Scriptable *Sender, int ga_flags)
 
 	int i = map->GetActorCount(true);
 	Targets *tgts = new Targets();
+	//make sure that Sender is always first in the list, even if there
+	//are other (e.g. dead) targets at the same location
+	tgts->AddTarget(Sender, 0, ga_flags);
 	while (i--) {
 		Actor *ac = map->GetActor(i,true);
-		int dist = Distance(Sender->Pos, ac->Pos);
-		tgts->AddTarget((Scriptable *) ac, dist, ga_flags);
+		if (ac != Sender) {
+			int dist = Distance(Sender->Pos, ac->Pos);
+			tgts->AddTarget(ac, dist, ga_flags);
+		}
 	}
 	return tgts;
 }
@@ -290,7 +310,7 @@ Scriptable* GetStoredActorFromObject(Scriptable* Sender, Object* oC, int ga_flag
 	// maybe store the target if it's an actor..
 	if (tar && tar->Type == ST_ACTOR) {
 		// .. but we only want objects created via objectFilters
-		if (oC->objectFilters[0]) {
+		if (oC && oC->objectFilters[0]) {
 			Sender->CurrentActionTarget = tar->GetGlobalID();
 		}
 	}
@@ -301,16 +321,13 @@ Scriptable* GetActorFromObject(Scriptable* Sender, Object* oC, int ga_flags)
 {
 	Scriptable *aC = NULL;
 
-	if (!oC) {
-		return NULL;
-	}
 	Game *game = core->GetGame();
 	Targets *tgts = GetAllObjects(Sender->GetCurrentArea(), Sender, oC, ga_flags);
 	if (tgts) {
 		//now this could return other than actor objects
 		aC = tgts->GetTarget(0,-1);
 		delete tgts;
-		if (aC || oC->objectFields[0]!=-1) {
+		if (aC || !oC || oC->objectFields[0]!=-1) {
 			return aC;
 		}
 
@@ -318,6 +335,10 @@ Scriptable* GetActorFromObject(Scriptable* Sender, Object* oC, int ga_flags)
 		return game->GetGlobalActorByGlobalID(oC->objectFields[1]);
 	}
 
+	if (!oC) {
+		return NULL;
+	}
+
 	if (oC->objectName[0]) {
 		// if you ActionOverride a global actor, they might not have a map :(
 		// TODO: don't allow this to happen?
@@ -417,8 +438,11 @@ int GetObjectCount(Scriptable* Sender, Object* oC)
 	// GetAllObjects will also return Myself (evaluates object filters)
 	// i believe we need the latter here
 	Targets* tgts = GetAllObjects(Sender->GetCurrentArea(), Sender, oC, 0);
-	int count = tgts->Count();
-	delete tgts;
+	int count = 0; // silly fallback to avoid potential crashes
+	if (tgts) {
+		count = tgts->Count();
+		delete tgts;
+	}
 	return count;
 }
 
@@ -573,14 +597,17 @@ Targets *ClosestEnemySummoned(Scriptable *origin, Targets *parameters, int ga_fl
 	while ( t ) {
 		Actor *tmp = (Actor *) (t->actor);
 		if (tmp->GetStat(IE_SEX) != SEX_SUMMON) {
+			t = parameters->GetNextTarget(m, ST_ACTOR);
 			continue;
 		}
 		if (type) { //origin is PC
 			if (tmp->GetStat(IE_EA) <= EA_GOODCUTOFF) {
+				t = parameters->GetNextTarget(m, ST_ACTOR);
 				continue;
 			}
 		} else {
 			if (tmp->GetStat(IE_EA) >= EA_EVILCUTOFF) {
+				t = parameters->GetNextTarget(m, ST_ACTOR);
 				continue;
 			}
 		}
@@ -621,12 +648,12 @@ Targets *XthNearestEnemyOfType(Scriptable *origin, Targets *parameters, unsigned
 		Actor *actor = (Actor *) (t->actor);
 		// IDS targeting already did object checks (unless we need to override Detect?)
 		if (type) { //origin is PC
-			if (actor->GetStat(IE_EA) <= EA_GOODCUTOFF) {
+			if (actor->GetStat(IE_EA) <= EA_EVILCUTOFF) {
 				t=parameters->RemoveTargetAt(m);
 				continue;
 			}
 		} else {
-			if (actor->GetStat(IE_EA) >= EA_EVILCUTOFF) {
+			if (actor->GetStat(IE_EA) >= EA_GOODCUTOFF) {
 				t=parameters->RemoveTargetAt(m);
 				continue;
 			}
@@ -654,6 +681,7 @@ Targets *XthNearestEnemyOf(Targets *parameters, int count, int ga_flags)
 	Actor *ac;
 	while (i--) {
 		ac=map->GetActor(i,true);
+		if (ac == origin) continue;
 		int distance;
 		//int distance = Distance(ac, origin);
 		// TODO: if it turns out you need to check Sender here, beware you take the right distance!
@@ -673,3 +701,5 @@ Targets *XthNearestEnemyOf(Targets *parameters, int count, int ga_flags)
 	return XthNearestOf(parameters,count, ga_flags);
 }
 
+
+}
