@@ -26,6 +26,8 @@
 #include "Video.h"
 #include "GUI/GameControl.h"
 
+namespace GemRB {
+
 GlobalTimer::GlobalTimer(void)
 {
 	//AI_UPDATE_TIME: how many AI updates in a second
@@ -59,7 +61,9 @@ void GlobalTimer::Freeze()
 	unsigned long thisTime;
 	unsigned long advance;
 
-	GetTime( thisTime );
+	UpdateAnimations(true);
+
+	thisTime = GetTickCount();
 	advance = thisTime - startTime;
 	if ( advance < interval) {
 		return;
@@ -69,7 +73,7 @@ void GlobalTimer::Freeze()
 	if (!game) {
 		return;
 	}
-	game->RealTime+=advance;
+	game->RealTime++;
 
 	ieDword count = advance/interval;
 	// pst/bg2 do this, if you fix it for another game, wrap it in a check
@@ -107,17 +111,17 @@ void GlobalTimer::DoStep(int count)
 	if ( (x != goal.x) || (y != goal.y)) {
 		if (speed) {
 			if (x<goal.x) {
-				x+=speed;
+				x+=speed*count;
 				if (x>goal.x) x=goal.x;
 			} else {
-				x-=speed;
+				x-=speed*count;
 				if (x<goal.x) x=goal.x;
 			}
 			if (y<goal.y) {
-				y+=speed;
+				y+=speed*count;
 				if (y>goal.y) y=goal.y;
 			} else {
-				y-=speed;
+				y-=speed*count;
 				if (y<goal.y) y=goal.y;
 			}
 		} else {
@@ -134,14 +138,18 @@ void GlobalTimer::DoStep(int count)
 			shakeCounter=0;
 		}
 		if (shakeCounter) {
-			x += (rand()%shakeX) - (shakeX>>1);
-			y += (rand()%shakeY) - (shakeY>>1);
+			if (shakeX) {
+				x += rand()%shakeX;
+			}
+			if (shakeY) {
+				y += rand()%shakeY;
+			}
 		}
 	}
 	video->MoveViewportTo(x,y);
 }
 
-void GlobalTimer::Update()
+bool GlobalTimer::Update()
 {
 	Map *map;
 	Game *game;
@@ -153,18 +161,18 @@ void GlobalTimer::Update()
 	if (gc)
 		gc->UpdateScrolling();
 
-	UpdateAnimations();
+	UpdateAnimations(false);
 
-	GetTime( thisTime );
+	thisTime = GetTickCount();
 
 	if (!startTime) {
 		startTime = thisTime;
-		return;
+		return false;
 	}
 
 	advance = thisTime - startTime;
 	if ( advance < interval) {
-		return;
+		return false;
 	}
 	ieDword count = advance/interval;
 	DoStep(count);
@@ -187,15 +195,16 @@ void GlobalTimer::Update()
 		map->UpdateEffects();
 		if (thisTime) {
 			//this measures in-world time (affected by effects, actions, etc)
-			game->AdvanceTime(count);
+			game->AdvanceTime(1);
 		}
 	}
 	//this measures time spent in the game (including pauses)
 	if (thisTime) {
-		game->RealTime+=advance;
+		game->RealTime++;
 	}
 end:
 	startTime = thisTime;
+	return true;
 }
 
 
@@ -264,7 +273,7 @@ void GlobalTimer::AddAnimation(ControlAnimation* ctlanim, unsigned long time)
 	AnimationRef* anim;
 	unsigned long thisTime;
 
-	GetTime( thisTime );
+	thisTime = GetTickCount();
 	time += thisTime;
 
 	// if there are no free animation reference objects,
@@ -305,10 +314,10 @@ void GlobalTimer::RemoveAnimation(ControlAnimation* ctlanim)
 	}
 }
 
-void GlobalTimer::UpdateAnimations()
+void GlobalTimer::UpdateAnimations(bool paused)
 {
 	unsigned long thisTime;
-	GetTime( thisTime );
+	thisTime = GetTickCount();
 	while (animations.begin() + first_animation != animations.end()) {
 		AnimationRef* anim = animations[first_animation];
 		if (anim->ctlanim == NULL) {
@@ -317,7 +326,7 @@ void GlobalTimer::UpdateAnimations()
 		}
 
 		if (anim->time <= thisTime) {
-			anim->ctlanim->UpdateAnimation();
+			anim->ctlanim->UpdateAnimation(paused);
 			first_animation++;
 			continue;
 		}
@@ -330,10 +339,12 @@ void GlobalTimer::ClearAnimations()
 	first_animation = (unsigned int) animations.size();
 }
 
-void GlobalTimer::SetScreenShake(unsigned long shakeX, unsigned long shakeY,
+void GlobalTimer::SetScreenShake(int shakeX, int shakeY,
 	unsigned long Count)
 {
 	this->shakeX = shakeX;
 	this->shakeY = shakeY;
 	shakeCounter = Count+1;
 }
+
+}
