@@ -29,16 +29,17 @@
 #include "SClassID.h"
 #include "exports.h"
 
+#include "Audio.h" // needed for _MSC_VER and SoundHandle (everywhere)
 #include "Cache.h"
 #include "Callback.h"
 #include "Holder.h"
+#include "InterfaceConfig.h"
 
 #include <map>
 #include <string>
 #include <vector>
 
 #ifdef _MSC_VER // No SFINAE
-#include "Audio.h"
 #include "DataFileMgr.h"
 #include "MusicMgr.h"
 #include "SaveGame.h"
@@ -49,6 +50,8 @@
 #include "WindowMgr.h"
 #endif
 
+namespace GemRB {
+
 class Actor;
 class Audio;
 class CREItem;
@@ -115,6 +118,7 @@ struct DamageInfoStruct {
 	unsigned int resist_stat;
 	unsigned int value;
 	int iwd_mod_type;
+	int reduction;
 	// maybe also add the ac bonus and/or the DL_ constants
 };
 
@@ -141,6 +145,7 @@ struct SpellDescType {
 };
 #define SP_IDENTIFY  1      //any spell that cannot be cast from the menu
 #define SP_SILENCE   2      //any spell that can be cast in silence
+#define SP_SURGE     4      //any spell that cannot be cast during a wild surge
 
 struct SurgeSpell {
 	ieResRef spell;
@@ -189,6 +194,7 @@ public:
 #define QF_CHANGESCRIPT  4
 #define QF_LOADGAME      8
 #define QF_ENTERGAME     16
+#define QF_KILL			32
 
 //events that are called out of drawwindow
 //they wait until the condition is right
@@ -220,6 +226,10 @@ public:
 #define AP_SPELLCAST     9
 #define AP_GENERIC       10  //needed for Android stuff
 
+//pause flags
+#define PF_QUIET  1        //no feedback
+#define PF_FORCED 2        //pause even in cutscene/dialog
+
 /** ea relations (derivated from 2 actor's EA value) */
 #define EAR_FRIEND  0
 #define EAR_NEUTRAL 1
@@ -271,6 +281,11 @@ enum PluginFlagsType {
 	PLF_DELAY
 };
 
+enum PauseSetting {
+	PAUSE_OFF = 0,
+	PAUSE_ON = 1
+};
+
 /**
  * @class Interface
  * Central interconnect for all GemRB parts, driving functions and utility functions possibly belonging to a better place
@@ -310,8 +325,6 @@ private:
 	Calendar * calendar;
 	WorldMapArray* worldmap;
 	ieDword GameFeatures[(GF_COUNT+31)/32];
-	//ieDword GameFeatures; //the first 32 bits
-	//ieDword GameFeatures2;//the second 32 bits
 	ieResRef ButtonFont;
 	ieResRef CursorBam;
 	ieResRef ScrollCursorBam;
@@ -329,11 +342,13 @@ private:
 	ieResRef Palette32;
 	ieResRef Palette256;
 	ieDword* slotmatrix; //itemtype vs slottype
+	std::vector<std::vector<int> > itemtypedata; //armor failure, critical multiplier, critical range
 	SlotType* slottypes;
 	int ItemTypes;
 	int tooltip_x;
 	int tooltip_y;
 	int tooltip_currtextw;
+	Holder<SoundHandle> tooltip_sound;
 	// the control owning the tooltip
 	Control* tooltip_ctrl;
 	// Currently dragged item or NULL
@@ -354,7 +369,9 @@ private:
 	int SpecialSpellsCount;
 	SpellDescType *SpecialSpells;
 	KeyMap *keymap;
+	std::string Encoding;
 public:
+	std::string TLKEncoding;
 	Holder<StringMgr> strings;
 	GlobalTimer * timer;
 	Palette *InfoTextPalette;
@@ -383,9 +400,9 @@ public:
 	TimeStruct Time;
 	std::vector<SurgeSpell> SurgeSpells;
 public:
-	Interface(int iargc, char *iargv[]);
+	Interface();
 	~Interface(void);
-	int Init(void);
+	int Init(InterfaceConfig* config);
 	//TODO: Core Methods in Interface Class
 	void SetFeature(int value, int position);
 	/* don't rely on the exact return value of this function */
@@ -423,6 +440,7 @@ public:
 	/** Loads a Window in the Window Manager */
 	int LoadWindow(unsigned short WindowID);
 	/** Creates a Window in the Window Manager */
+	bool IsPresentingModalWindow() {return (bool)(ModalWindow);};
 #ifdef WIN32
 #ifdef CreateWindow
 #undef CreateWindow
@@ -548,6 +566,8 @@ public:
 	/** load saved game by index (-1 is default), ver_override is an optional parameter
 	to override the saved game's version */
 	void LoadGame(SaveGame *save, int ver_override);
+	/** reloads the world map from a resource file */
+	void UpdateWorldMap(ieResRef wmResRef);
 	/** fix changes in global script/worldmap*/
 	void UpdateMasterScript();
 	/*reads the filenames of the portraits folder into a list */
@@ -565,6 +585,12 @@ public:
 	ieDword QuerySlotFlags(unsigned int idx) const;
 	ieDword QuerySlotEffects(unsigned int idx) const;
 	const char * QuerySlotResRef(unsigned int idx) const;
+	int GetArmorFailure(unsigned int itemtype) const;
+	int GetShieldFailure(unsigned int itemtype) const;
+	int GetArmorPenalty(unsigned int itemtype) const;
+	int GetShieldPenalty(unsigned int itemtype) const;
+	int GetCriticalMultiplier(unsigned int itemtype) const;
+	int GetCriticalRange(unsigned int itemtype) const;
 	/*returns true if an itemtype is acceptable for a slottype, also checks the usability flags */
 	int CanUseItemType(int slottype, Item *item, Actor *actor=NULL, bool feedback=false, bool equipped=false) const;
 	/*removes single file from cache*/
@@ -653,8 +679,12 @@ public:
 	int WriteWorldMap(const char *folder);
 	/** saves the .are and .sto files to the destination folder */
 	int CompressSave(const char *folder);
-	/** receives an autopause reason, returns 1 if pause was triggered by this call, -1 if it was already triggered */
-	int Autopause(ieDword reason);
+	/** toggles the pause. returns either PAUSE_ON or PAUSE_OFF to reflect the script state after toggling. */
+	PauseSetting TogglePause();
+	/** returns true the passed pause setting was applied. false otherwise. */
+	bool SetPause(PauseSetting pause, int flags = 0);
+	/** receives an autopause reason, returns true if autopause was accepted and successful */
+	bool Autopause(ieDword flag, Scriptable *target);
 	/** registers engine opcodes */
 	void RegisterOpcodes(int count, const EffectDesc *opcodes);
 	/** reads a list of resrefs into an array, returns array size */
@@ -689,16 +719,18 @@ public:
 	void StripLine(char * string, size_t size);
 	/** Returns the DeathVarFormat of the day */
 	static const char *GetDeathVarFormat();
-	int CheckSpecialSpell(ieResRef resref, Actor *actor);
-	int GetSpecialSpell(ieResRef resref);
+	int CheckSpecialSpell(const ieResRef resref, Actor *actor);
+	int GetSpecialSpell(const ieResRef resref);
 	int GetSpecialSpellsCount() { return SpecialSpellsCount; }
 	SpellDescType *GetSpecialSpells() { return SpecialSpells; }
+	/** Saves config variables to a file */
+	bool SaveConfig();
 private:
 	int LoadSprites();
-	bool LoadConfig(void);
-	bool LoadConfig(const char *filename);
 	bool LoadGemRBINI();
-	bool LoadINI(const char * filename);
+	/** Load the encoding table selected in gemrb.cfg */
+	bool LoadEncoding();
+	bool InitializeVarsWithINI(const char * iniFileName);
 	bool InitItemTypes();
 	bool ReadRandomItems();
 	bool ReadItemTable(const ieResRef item, const char *Prefix);
@@ -727,10 +759,6 @@ private:
 	/** the internal (without cache) part of GetListFrom2DA */
 	ieDword *GetListFrom2DAInternal(const ieResRef resref);
 public:
-	//N900 specific config
-        char RMBK;
-        int ProximitySensorEnabled;
-        //End N900 variables
 	char GameDataPath[_MAX_PATH];
 	char GameOverridePath[_MAX_PATH];
 	char GameSoundsPath[_MAX_PATH];
@@ -738,12 +766,11 @@ public:
 	char GamePortraitsPath[_MAX_PATH];
 	char GameCharactersPath[_MAX_PATH];
 	char GemRBOverridePath[_MAX_PATH];
+	char GemRBUnhardcodedPath[_MAX_PATH];
 	ieResRef GameNameResRef;
 	ieResRef GoldResRef; //MISC07.itm
 	Variables *RtRows;
-	char UserDir[_MAX_PATH];
-	int argc;
-	char **argv;
+	char CustomFontPath[_MAX_PATH];
 	char GameName[_MAX_PATH];
 	char GameType[_MAX_PATH];
 	char GemRBPath[_MAX_PATH];
@@ -759,10 +786,11 @@ public:
 	unsigned int TooltipDelay;
 	int IgnoreOriginalINI;
 	unsigned int FogOfWar;
-	bool CaseSensitive, SlowBIFs, SkipIntroVideos, DrawFPS;
+	bool CaseSensitive, SkipIntroVideos, DrawFPS;
 	bool TouchScrollAreas, UseSoftKeyboard;
 	unsigned short NumFingScroll, NumFingKboard, NumFingInfo;
-	bool GUIEnhancements;
+	int MouseFeedback;
+	int GUIEnhancements;
 	bool KeepCache;
 	bool MultipleQuickSaves;
 	Variables *plugin_flags;
@@ -772,8 +800,8 @@ public:
 	bool IsFreezed();
 	/** Draws the Visible windows in the Windows Array */
 	void DrawWindows(bool allow_delete = false);
-	/** Sends a termination signal to the Video Driver */
-	bool Quit(void);
+	void AskAndExit();
+	void ExitGemRB(void);
 	/** CheatKey support */
 	inline void EnableCheatKeys(int Flag)
 	{
@@ -808,12 +836,10 @@ public:
 	Audio* GetAudioDrv(void) const;
 
 	void SetTickHook(EventHandler);
-
-#ifdef _DEBUG
-	int FileStreamPtrCount;
-#endif
 };
 
 extern GEM_EXPORT Interface * core;
 
+}
+
 #endif
