@@ -35,9 +35,12 @@
 #include "Map.h"
 #include "ScriptEngine.h"
 #include "Scriptable/Actor.h"
+#include "System/StringBuffer.h"
 
 #include <cstdio>
 
+namespace GemRB {
+
 static int SLOT_HEAD = -1;
 static int SLOT_MAGIC = -1;
 static int SLOT_FIST = -1;
@@ -50,6 +53,7 @@ static int LAST_QUICK = -1;
 static int SLOT_INV = -1;
 static int LAST_INV = -1;
 static int SLOT_LEFT = -1;
+static int SLOT_ARMOR = -1;
 
 //IWD2 style slots
 static bool IWD2 = false;
@@ -81,6 +85,7 @@ void Inventory::Init(int mb)
 	SLOT_QUICK=-1;
 	LAST_QUICK=-1;
 	SLOT_LEFT=-1;
+	SLOT_ARMOR=-1;
 	//TODO: set this correctly
 	IWD2 = false;
 	MagicBit = mb;
@@ -152,12 +157,16 @@ CREItem *Inventory::GetItem(unsigned int slot)
 	return item;
 }
 
-//This hack sets the charge counters for non-rechargeable items,
-//if their charge is zero
-static inline void HackCharges(CREItem *item)
+//Make sure the item attributes are valid
+//we don't update all flags here because some need to be set later (like
+//unmovable items in containers (e.g. the bg2 protal key) so that they
+//can actually be picked up)
+void Inventory::SanitizeItem(CREItem *item)
 {
-	Item *itm = gamedata->GetItem( item->ItemResRef );
+	Item *itm = gamedata->GetItem(item->ItemResRef, true);
 	if (itm) {
+		//This hack sets the charge counters for non-rechargeable items,
+		//if their charge is zero
 		for (int i=0;i<3;i++) {
 			if (item->Usages[i]) {
 				continue;
@@ -172,6 +181,19 @@ static inline void HackCharges(CREItem *item)
 				}
 			}
 		}
+
+		//auto identify basic items
+		if (!itm->LoreToID) {
+			item->Flags |= IE_INV_ITEM_IDENTIFIED;
+		}
+
+		//if item is stacked mark it as so
+		if (itm->MaxStackAmount) {
+			item->Flags |= IE_INV_ITEM_STACKED;
+		}
+
+		item->MaxStackAmount = itm->MaxStackAmount;
+
 		gamedata->FreeItem( itm, item->ItemResRef, false );
 	}
 }
@@ -180,13 +202,10 @@ void Inventory::AddItem(CREItem *item)
 {
 	if (!item) return; //invalid items get no slot
 	Slots.push_back(item);
-	HackCharges(item);
-	//this will update the flags (needed for unmovable items in containers)
-	//but those *can* be picked up (like the bg2 portal key), so we skip it
-	//Changed=true;
+	SanitizeItem(item);
 }
 
-void Inventory::CalculateWeight()
+void Inventory::CalculateWeight() const
 {
 	if (!Changed) {
 		return;
@@ -198,7 +217,7 @@ void Inventory::CalculateWeight()
 			continue;
 		}
 		if (slot->Weight == -1) {
-			Item *itm = gamedata->GetItem( slot->ItemResRef );
+			Item *itm = gamedata->GetItem(slot->ItemResRef, true);
 			if (itm) {
 				//simply adding the item flags to the slot
 				slot->Flags |= (itm->Flags<<8);
@@ -222,22 +241,12 @@ void Inventory::CalculateWeight()
 					slot->Flags |= IE_INV_ITEM_STOLEN;
 				}
 
-				//auto identify basic items
-				if (!itm->LoreToID) {
-					slot->Flags |= IE_INV_ITEM_IDENTIFIED;
-				}
-
-				//if item is stacked mark it as so
-				if (itm->MaxStackAmount) {
-					slot->Flags |= IE_INV_ITEM_STACKED;
-				}
-
 				slot->Weight = itm->Weight;
-				slot->MaxStackAmount = itm->MaxStackAmount;
+
 				gamedata->FreeItem( itm, slot->ItemResRef, false );
 			}
 			else {
-				printMessage("Inventory", "Invalid item: %s!\n", LIGHT_RED,
+				Log(ERROR, "Inventory", "Invalid item: %s!",
 					slot->ItemResRef);
 				slot->Weight = 0;
 			}
@@ -257,7 +266,7 @@ void Inventory::AddSlotEffects(ieDword index)
 
 	const Item *itm = GetItemPointer(index, slot);
 	if (!itm) {
-		printMessage("Inventory","Invalid item equipped...\n",LIGHT_RED);
+		Log(ERROR, "Inventory", "Invalid item equipped...");
 		return;
 	}
 	ItemExcl|=itm->ItemExcl;
@@ -303,10 +312,9 @@ void Inventory::SetInventoryType(int arg)
 void Inventory::SetSlotCount(unsigned int size)
 {
 	if (Slots.size()) {
-		print("Inventory size changed???\n");
+		error("Core", "Inventory size changed???\n");
 		//we don't allow reassignment,
 		//if you want this, delete the previous Slots here
-		abort();
 	}
 	Slots.assign((size_t) size, NULL);
 }
@@ -355,6 +363,7 @@ int Inventory::CountItems(const char *resref, bool stacks) const
 		}
 		if (stacks && (item->Flags&IE_INV_ITEM_STACKED) ) {
 			count+=item->Usages[0];
+			assert(count!=0);
 		}
 		else {
 			count++;
@@ -407,12 +416,14 @@ void Inventory::KillSlot(unsigned int index)
 		return;
 	}
 	RemoveSlotEffects( index );
-	Item *itm = gamedata->GetItem(item->ItemResRef);
+	Item *itm = gamedata->GetItem(item->ItemResRef, true);
 	//this cannot happen, but stuff happens!
 	if (!itm) {
+		error("Inventory", "Invalid item: %s!", item->ItemResRef);
 		return;
 	}
 	ItemExcl &= ~itm->ItemExcl;
+	int eqslot = GetEquippedSlot();
 
 	switch (effect) {
 		case SLOT_EFFECT_LEFT:
@@ -420,23 +431,24 @@ void Inventory::KillSlot(unsigned int index)
 			break;
 		case SLOT_EFFECT_MISSILE:
 			//getting a new projectile of the same type
-			if (Equipped + SLOT_MELEE == (int) index) {
+			if (eqslot == (int) index) {
 				if (Equipped < 0) {
 					//always get the projectile weapon header (this quiver was equipped)
 					ITMExtHeader *header = itm->GetWeaponHeader(true);
 					Equipped = FindRangedProjectile(header->ProjectileQualifier);
 					if (Equipped!=IW_NO_EQUIPPED) {
-						EquipItem(Equipped+SLOT_MELEE);
+						EquipItem(GetEquippedSlot());
 					} else {
-						EquipItem(SLOT_FIST);
+						EquipBestWeapon(EQUIP_MELEE);
 					}
 				}
 			}
 			UpdateWeaponAnimation();
 			break;
+		case SLOT_EFFECT_MAGIC:
 		case SLOT_EFFECT_MELEE:
 			// reset Equipped if it was the removed item
-			if (Equipped+SLOT_MELEE == (int)index)
+			if (eqslot == (int)index)
 				Equipped = IW_NO_EQUIPPED;
 			else if (Equipped < 0) {
 				//always get the projectile weapon header (this is a bow, because Equipped is negative)
@@ -447,14 +459,14 @@ void Inventory::KillSlot(unsigned int index)
 					int weaponslot = FindTypedRangedWeapon(type);
 					CREItem *item2 = Slots[weaponslot];
 					if (item2) {
-						Item *itm2 = gamedata->GetItem(item2->ItemResRef);
+						Item *itm2 = gamedata->GetItem(item2->ItemResRef, true);
 						if (itm2) {
 							if (type == header->ProjectileQualifier) {
 								Equipped = FindRangedProjectile(header->ProjectileQualifier);
 								if (Equipped!=IW_NO_EQUIPPED) {
-									EquipItem(Equipped+SLOT_MELEE);
+									EquipItem(GetEquippedSlot());
 								} else {
-									EquipItem(SLOT_FIST);
+									EquipBestWeapon(EQUIP_MELEE);
 								}
 							}
 							gamedata->FreeItem(itm2, item2->ItemResRef, false);
@@ -606,7 +618,7 @@ void Inventory::SetSlotItem(CREItem* item, unsigned int slot)
 		delete Slots[slot];
 	}
 
-	HackCharges(item);
+	SanitizeItem(item);
 
 	Slots[slot] = item;
 
@@ -731,7 +743,7 @@ bool Inventory::ItemsAreCompatible(CREItem* target, CREItem* source) const
 {
 	if (!target) {
 		//this isn't always ok, please check!
-		printMessage("Inventory","Null item encountered by ItemsAreCompatible()",YELLOW);
+		Log(WARNING, "Inventory", "Null item encountered by ItemsAreCompatible()");
 		return true;
 	}
 
@@ -764,9 +776,11 @@ int Inventory::DepleteItem(ieDword flags)
 
 		//if flags = 0 then weapons are not depleted
 		if (!flags) {
-			Item *itm = gamedata->GetItem( item->ItemResRef );
-			if (!itm)
+			Item *itm = gamedata->GetItem(item->ItemResRef, true);
+			if (!itm) {
+				Log(WARNING, "Inventory", "Invalid item to deplete: %s!", item->ItemResRef);
 				continue;
+			}
 			//if the item is usable in weapon slot, then it is weapon
 			int weapon = core->CanUseItemType( SLOT_WEAPON, itm );
 			gamedata->FreeItem( itm, item->ItemResRef, false );
@@ -784,7 +798,7 @@ int Inventory::DepleteItem(ieDword flags)
 // if flags is 0, skips undroppable items
 // if flags is IE_INV_ITEM_UNDROPPABLE, doesn't skip undroppable items
 // TODO: once all callers have been checked, this can be reversed to make more sense
-int Inventory::FindItem(const char *resref, unsigned int flags) const
+int Inventory::FindItem(const char *resref, unsigned int flags, unsigned int skip) const
 {
 	unsigned int mask = (flags^IE_INV_ITEM_UNDROPPABLE);
 	if (core->HasFeature(GF_NO_DROP_CAN_MOVE) ) {
@@ -801,7 +815,11 @@ int Inventory::FindItem(const char *resref, unsigned int flags) const
 		if (resref[0] && strnicmp(item->ItemResRef, resref, 8) ) {
 			continue;
 		}
-		return (int) i;
+		if (skip) {
+			skip--;
+		} else {
+			return (int) i;
+		}
 	}
 	return -1;
 }
@@ -893,7 +911,7 @@ bool Inventory::DropItemAtLocation(const char *resref, unsigned int flags, Map *
 	return dropped;
 }
 
-CREItem *Inventory::GetSlotItem(unsigned int slot) const
+CREItem *Inventory::GetSlotItem(ieDword slot) const
 {
 	if (slot >= Slots.size() ) {
 		InvalidSlot(slot);
@@ -911,7 +929,7 @@ ieDword Inventory::GetItemFlag(unsigned int slot) const
 	return item->Flags;
 }
 
-bool Inventory::ChangeItemFlag(unsigned int slot, ieDword arg, int op)
+bool Inventory::ChangeItemFlag(ieDword slot, ieDword arg, int op)
 {
 	CREItem *item = GetSlotItem(slot);
 	if (!item) {
@@ -929,7 +947,7 @@ bool Inventory::ChangeItemFlag(unsigned int slot, ieDword arg, int op)
 
 //this is the low level equipping
 //all checks have been made previously
-bool Inventory::EquipItem(unsigned int slot)
+bool Inventory::EquipItem(ieDword slot)
 {
 	ITMExtHeader *header;
 
@@ -946,9 +964,9 @@ bool Inventory::EquipItem(unsigned int slot)
 
 	// add effects of an item just being equipped to actor's effect queue
 	int effect = core->QuerySlotEffects( slot );
-	Item *itm = gamedata->GetItem(item->ItemResRef);
+	Item *itm = gamedata->GetItem(item->ItemResRef, true);
 	if (!itm) {
-		print("Invalid item Equipped: %s Slot: %d\n", item->ItemResRef, slot);
+		print("Invalid item Equipped: %s Slot: %d", item->ItemResRef, slot);
 		return false;
 	}
 	switch (effect) {
@@ -958,27 +976,32 @@ bool Inventory::EquipItem(unsigned int slot)
 		UpdateShieldAnimation(itm);
 		break;
 	case SLOT_EFFECT_MELEE:
-		//if weapon is ranged, then find quarrel for it and equip that
-		slot -= SLOT_MELEE;
-		weaponslot = slot;
+		//if weapon is bow, then find quarrel for it and equip that
+		weaponslot = GetWeaponQuickSlot(slot);
 		EquippedHeader = 0;
 		header = itm->GetExtHeader(EquippedHeader);
-		if (header && header->AttackType == ITEM_AT_BOW) {
-			//find the ranged projectile associated with it.
-			slot = FindRangedProjectile(header->ProjectileQualifier);
-			EquippedHeader = itm->GetWeaponHeaderNumber(true);
-		} else if (header && header->AttackType == ITEM_AT_PROJECTILE) {
-			EquippedHeader = itm->GetWeaponHeaderNumber(true);
-		} else {
-			EquippedHeader = itm->GetWeaponHeaderNumber(false);
-		}
-		header = itm->GetExtHeader(EquippedHeader);
 		if (header) {
-			SetEquippedSlot(slot, EquippedHeader);
-			if (slot != IW_NO_EQUIPPED) {
-				Owner->SetupQuickSlot(ACT_WEAPON1+weaponslot, slot+SLOT_MELEE, EquippedHeader);
+			if (header->AttackType == ITEM_AT_BOW) {
+				//find the ranged projectile associated with it, this returns equipped code
+				Equipped = FindRangedProjectile(header->ProjectileQualifier);
+				//this is the real item slot of the quarrel
+				slot = Equipped + SLOT_MELEE;
+			} else {
+				//this is always 0-3
+				Equipped = weaponslot;
+				slot = GetWeaponSlot(weaponslot);
+			}
+			if (Equipped != IW_NO_EQUIPPED) {
+				Owner->SetupQuickSlot(ACT_WEAPON1+weaponslot, slot, EquippedHeader);
+			}
+			SetEquippedSlot(Equipped, EquippedHeader);
+			//don't clear effect in case of a launcher, we need to find it and add its effects too
+			//slot is 'negative' for launchers
+			if ((int) Equipped>=0) {
+				effect = 0; // SetEquippedSlot will already call AddSlotEffects
+			} else {
+				effect = SLOT_EFFECT_MISSILE;
 			}
-			effect = 0; // SetEquippedSlot will already call AddSlotEffects
 			UpdateWeaponAnimation();
 		}
 		break;
@@ -1028,7 +1051,7 @@ bool Inventory::EquipItem(unsigned int slot)
 
 //the removecurse flag will check if it is possible to move the item to the inventory
 //after a remove curse spell
-bool Inventory::UnEquipItem(unsigned int slot, bool removecurse)
+bool Inventory::UnEquipItem(ieDword slot, bool removecurse)
 {
 	CREItem *item = GetSlotItem(slot);
 	if (!item) {
@@ -1081,10 +1104,10 @@ int Inventory::FindRangedProjectile(unsigned int type) const
 int Inventory::FindRangedWeapon() const
 {
 	if (Equipped>=0) return SLOT_FIST;
-	return FindSlotRangedWeapon(Equipped+SLOT_MELEE);
+	return FindSlotRangedWeapon(GetEquippedSlot());
 }
 
-int Inventory::FindSlotRangedWeapon(unsigned int slot) const
+int Inventory::FindSlotRangedWeapon(ieDword slot) const
 {
 	if ((int)slot >= SLOT_MELEE) return SLOT_FIST;
 	CREItem *Slot;
@@ -1164,6 +1187,13 @@ void Inventory::SetInventorySlot(int arg)
 	LAST_INV=arg;
 }
 
+void Inventory::SetArmorSlot(int arg)
+{
+	if (SLOT_ARMOR==-1) {
+		SLOT_ARMOR=arg;
+	}
+}
+
 //multiple shield slots are allowed
 //but in this case they should be interspersed with melee slots
 void Inventory::SetShieldSlot(int arg)
@@ -1196,6 +1226,19 @@ int Inventory::GetWeaponSlot()
 	return SLOT_MELEE;
 }
 
+int Inventory::GetWeaponQuickSlot(int weaponslot)
+{
+	int slot = weaponslot-SLOT_MELEE;
+	if (IWD2 && (slot>=0 && slot<=7) ) slot/=2;
+	return slot;
+}
+
+int Inventory::GetWeaponSlot(int quickslot)
+{
+	if (IWD2 && (quickslot>=0 && quickslot<=3) ) quickslot*=2;
+	return quickslot+SLOT_MELEE;
+}
+
 int Inventory::GetQuickSlot()
 {
 	return SLOT_QUICK;
@@ -1206,13 +1249,23 @@ int Inventory::GetInventorySlot()
 	return SLOT_INV;
 }
 
+int Inventory::GetArmorSlot()
+{
+	return SLOT_ARMOR;
+}
+
 //if shield slot is empty, call again for fist slot!
 int Inventory::GetShieldSlot() const
 {
 	if (IWD2) {
+		//actually, in IWD2, the equipped slot never becomes IW_NO_EQUIPPED, it is always 0-3
+		//this is just a hack to prevent invalid shots from happening
+		if (Equipped == IW_NO_EQUIPPED) return SLOT_MELEE+1;
+
 		if (Equipped>=0 && Equipped<=3) {
 			return Equipped*2+SLOT_MELEE+1;
 		}
+		//still, what about magic weapons...
 		return -1;
 	}
 	return SLOT_LEFT;
@@ -1224,8 +1277,8 @@ int Inventory::GetEquippedSlot() const
 		return SLOT_FIST;
 	}
 	if (IWD2 && Equipped>=0) {
-		//i've absolutely NO idea what is this 4 (Avenger)
-		//Equipped should be 0-3 in iWD2, no???
+		//Equipped should never become IW_NO_EQUIPPED, this is just a hack to cover the bug
+		//about it still becoming invalid
 		if (Equipped >= 4) {
 			return SLOT_MELEE;
 		}
@@ -1246,14 +1299,17 @@ bool Inventory::SetEquippedSlot(ieWordSigned slotcode, ieWord header)
 	}
 
 	//if it is an illegal code, make it fist
-	if ((size_t) (slotcode+SLOT_MELEE)>Slots.size()) {
+	if ((size_t) (GetWeaponSlot(slotcode))>Slots.size()) {
 		slotcode=IW_NO_EQUIPPED;
 	}
 
+	int oldslot = GetEquippedSlot();
+	int newslot = GetWeaponSlot(slotcode);
+
 	//unequipping (fist slot will be used now)
-	if (slotcode == IW_NO_EQUIPPED || !HasItemInSlot("",slotcode+SLOT_MELEE)) {
+	if (slotcode == IW_NO_EQUIPPED || !HasItemInSlot("", newslot)) {
 		if (Equipped != IW_NO_EQUIPPED) {
-			RemoveSlotEffects( SLOT_MELEE+Equipped);
+			RemoveSlotEffects( oldslot);
 		}
 		Equipped = IW_NO_EQUIPPED;
 		//fist slot equipping effects
@@ -1264,18 +1320,18 @@ bool Inventory::SetEquippedSlot(ieWordSigned slotcode, ieWord header)
 
 	//equipping a weapon, but remove its effects first
 	if (Equipped != IW_NO_EQUIPPED) {
-		RemoveSlotEffects( SLOT_MELEE+Equipped);
+		RemoveSlotEffects( oldslot);
 	}
 
 	Equipped = slotcode;
-	int effects = core->QuerySlotEffects( SLOT_MELEE+Equipped );
+	int effects = core->QuerySlotEffects( newslot);
 	if (effects) {
-		CREItem* item = GetSlotItem(SLOT_MELEE+Equipped);
+		CREItem* item = GetSlotItem(newslot);
 		item->Flags|=IE_INV_ITEM_EQUIPPED;
 		if (item->Flags & IE_INV_ITEM_CURSED) {
 			item->Flags|=IE_INV_ITEM_UNDROPPABLE;
 		}
-		AddSlotEffects( SLOT_MELEE+Equipped);
+		AddSlotEffects(newslot);
 	}
 	UpdateWeaponAnimation();
 	return true;
@@ -1315,6 +1371,9 @@ CREItem *Inventory::GetUsedWeapon(bool leftorright, int &slot) const
 				//we don't want to return fist for shield slot
 				return NULL;
 			}
+		} else {
+			// nothing in the shield slot, so nothing in the right hand, so just quit
+			return NULL;
 		}
 	}
 	slot = GetEquippedSlot();
@@ -1385,7 +1444,7 @@ void Inventory::AddSlotItemRes(const ieResRef ItemResRef, int SlotID, int Charge
 				// create or reuse the existing pile
 				area->AddItemToLocation(Owner->Pos, TmpItem);
 			} else {
-				printMessage("Inventory", "AddSlotItemRes: argh, no area and the inventory is full, bailing out!\n", LIGHT_RED);
+				Log(ERROR, "Inventory", "AddSlotItemRes: argh, no area and the inventory is full, bailing out!");
 				delete TmpItem;
 			}
 		}
@@ -1417,6 +1476,38 @@ void Inventory::SetSlotItemRes(const ieResRef ItemResRef, int SlotID, int Charge
 	CalculateWeight();
 }
 
+ieWord Inventory::GetShieldItemType() const
+{
+	ieWord ret;
+	CREItem *Slot;
+	int slotNum = GetShieldSlot();
+
+	if (slotNum < 0) {
+		return 0xffff;
+	}
+	const Item *itm = GetItemPointer(slotNum, Slot);
+	if (!itm) return 0xffff;
+	ret = itm->ItemType;
+	gamedata->FreeItem(itm, Slot->ItemResRef);
+	return ret;
+}
+
+ieWord Inventory::GetArmorItemType() const
+{
+	ieWord ret;
+	CREItem *Slot;
+	int slotNum = GetArmorSlot();
+
+	if (slotNum < 0) {
+		return 0xffff;
+	}
+	const Item *itm = GetItemPointer(slotNum, Slot);
+	if (!itm) return 0xffff;
+	ret = itm->ItemType;
+	gamedata->FreeItem(itm, Slot->ItemResRef);
+	return ret;
+}
+
 void Inventory::BreakItemSlot(ieDword slot)
 {
 	ieResRef newItem;
@@ -1435,9 +1526,16 @@ void Inventory::BreakItemSlot(ieDword slot)
 	SetSlotItemRes(newItem, slot, 0,0,0);
 }
 
-void Inventory::dump()
+void Inventory::dump() const
 {
-	print( "INVENTORY:\n" );
+	StringBuffer buffer;
+	dump(buffer);
+	Log(DEBUG, "Inventory", buffer);
+}
+
+void Inventory::dump(StringBuffer& buffer) const
+{
+	buffer.append( "INVENTORY:\n" );
 	for (unsigned int i = 0; i < Slots.size(); i++) {
 		CREItem* itm = Slots[i];
 
@@ -1445,13 +1543,13 @@ void Inventory::dump()
 			continue;
 		}
 
-		print ( "%2u: %8.8s - (%d %d %d) Fl:0x%x Wt: %d x %dLb\n", i, itm->ItemResRef, itm->Usages[0], itm->Usages[1], itm->Usages[2], itm->Flags, itm->MaxStackAmount, itm->Weight );
+		buffer.appendFormatted( "%2u: %8.8s - (%d %d %d) Fl:0x%x Wt: %d x %dLb\n", i, itm->ItemResRef, itm->Usages[0], itm->Usages[1], itm->Usages[2], itm->Flags, itm->MaxStackAmount, itm->Weight );
 	}
 
-	print( "Equipped: %d\n", Equipped );
+	buffer.appendFormatted( "Equipped: %d\n", Equipped );
 	Changed = true;
 	CalculateWeight();
-	print( "Total weight: %d\n", Weight );
+	buffer.appendFormatted( "Total weight: %d\n", Weight );
 }
 
 void Inventory::EquipBestWeapon(int flags)
@@ -1473,6 +1571,10 @@ void Inventory::EquipBestWeapon(int flags)
 		for(i=SLOT_RANGED;i<LAST_RANGED;i++) {
 			const Item *itm = GetItemPointer(i, Slot);
 			if (!itm) continue;
+			//cannot change equipment when holding a cursed weapon
+			if (Slot->Flags & IE_INV_ITEM_CURSED) {
+				return;
+			}
 			//best ranged
 			int tmp = itm->GetDamagePotential(true, header);
 			if (tmp>damage) {
@@ -1488,6 +1590,10 @@ void Inventory::EquipBestWeapon(int flags)
 		for(i=SLOT_MELEE;i<=LAST_MELEE;i++) {
 			const Item *itm = GetItemPointer(i, Slot);
 			if (!itm) continue;
+			//cannot change equipment when holding a cursed weapon
+			if (Slot->Flags & IE_INV_ITEM_CURSED) {
+				return;
+			}
 			//best ranged
 			int tmp = itm->GetDamagePotential(true, header);
 			if (tmp>damage) {
@@ -1504,6 +1610,10 @@ void Inventory::EquipBestWeapon(int flags)
 		for(i=SLOT_MELEE;i<=LAST_MELEE;i++) {
 			const Item *itm = GetItemPointer(i, Slot);
 			if (!itm) continue;
+			//cannot change equipment when holding a cursed weapon
+			if (Slot->Flags & IE_INV_ITEM_CURSED) {
+				return;
+			}
 			//the Slot flag is enough for this
 			//though we need animation type/damagepotential anyway
 			if (Slot->Flags&IE_INV_ITEM_BOW) continue;
@@ -1662,7 +1772,7 @@ void Inventory::UpdateWeaponAnimation()
 				si = GetSlotItem( (ieDword) slot );
 			}
 			if (si) {
-				Item* it = gamedata->GetItem(si->ItemResRef);
+				Item* it = gamedata->GetItem(si->ItemResRef, true);
 				if (core->CanUseItemType(SLOT_WEAPON, it))
 					twoweapon = true;
 				gamedata->FreeItem(it, si->ItemResRef, false);
@@ -1726,7 +1836,7 @@ int Inventory::WhyCantEquip(int slot, int twohanded) const
 		//see GetShieldSlot
 		int otherslot;
 		if (IWD2) {
-			otherslot = ++i;
+			otherslot = i+1;
 		} else {
 			otherslot = SLOT_LEFT;
 		}
@@ -1739,7 +1849,7 @@ int Inventory::WhyCantEquip(int slot, int twohanded) const
 
 	if (twohanded) {
 		if (IWD2) {
-			if (slot>=SLOT_MELEE&&slot<=LAST_MELEE && (slot&1) ) {
+			if (slot>=SLOT_MELEE&&slot<=LAST_MELEE && (slot-SLOT_MELEE)&1) {
 				return STR_NOT_IN_OFFHAND;
 			}
 		} else {
@@ -1767,7 +1877,7 @@ void Inventory::ChargeAllItems(int hours)
 			continue;
 		}
 
-		Item *itm = gamedata->GetItem( item->ItemResRef );
+		Item *itm = gamedata->GetItem(item->ItemResRef, true);
 		if (!itm)
 			continue;
 		for(int h=0;h<CHARGE_COUNTERS;h++) {
@@ -1793,7 +1903,7 @@ unsigned int Inventory::FindStealableItem()
 	slot = core->Roll(1, Slots.size(),-1);
 	inc = slot&1?1:-1;
 
-	print("Start Slot: %d, increment: %d\n", slot, inc);
+	print("Start Slot: %d, increment: %d", slot, inc);
 	//as the unsigned value underflows, it will be greater than Slots.size()
 	for(;slot<Slots.size(); slot+=inc) {
 		CREItem *item = Slots[slot];
@@ -1802,7 +1912,9 @@ unsigned int Inventory::FindStealableItem()
 		//bit 1 is stealable slot
 		if (!(core->QuerySlotFlags(slot)&1) ) continue;
 		//can't steal equipped weapon
-		if ((unsigned int) (Equipped+SLOT_MELEE) == core->QuerySlot(slot)) continue;
+		int realslot = core->QuerySlot(slot);
+		if (GetEquippedSlot() == realslot) continue;
+		if (GetShieldSlot() == realslot) continue;
 		//can't steal flagged items
 		if ((item->Flags & ITM_STEALING) != IE_INV_ITEM_MOVABLE) continue;
 		return slot;
@@ -1815,31 +1927,29 @@ unsigned int Inventory::FindStealableItem()
 // If an item with bit 25 set is equipped in a helmet slot, aversion is disabled
 bool Inventory::ProvidesCriticalAversion()
 {
-	for (size_t i = 0; i < Slots.size(); i++) {
+	int maxSlot = (int) Slots.size();
+	for (int i = 0; i < maxSlot; i++) {
 		CREItem *item = Slots[i];
-		if (!item || ! (item->Flags & IE_INV_ITEM_EQUIPPED)) {
+		if (!item || ((i>=SLOT_INV) && (i<=LAST_INV))) { // ignore items in the backpack
+			continue;
+		}
+		// weapon, but not equipped
+		if (!((i == SLOT_ARMOR) || (i == SLOT_HEAD)) && !(item->Flags & IE_INV_ITEM_EQUIPPED)) {
 			continue;
 		}
 
-		Item *itm = gamedata->GetItem(item->ItemResRef);
+		Item *itm = gamedata->GetItem(item->ItemResRef, true);
 		if (!itm) {
 			continue;
 		}
-
-		for (int h = 0; h < itm->ExtHeaderCount; h++) {
-			ITMExtHeader *header = itm->GetExtHeader(h);
-			if ((int)i == SLOT_HEAD) {
-				if (header && (header->RechargeFlags & IE_ITEM_TOGGLE_CRITS)) {
-					return false;
-				} else {
-					return true;
-				}
-			} else {
-				if (header && (header->RechargeFlags & IE_ITEM_TOGGLE_CRITS)) {
-					return true;
-				}
-			}
-		}
+		//if the item is worn on head, toggle crits must be 0, otherwise it must be 1
+		//this flag is only stored in the item header, so we need to make some efforts
+		//to get to it (TODO convince ToBEx to move this bit into the accessible range?) - low 24 bits
+		ieDword flag = itm->Flags;
+		gamedata->FreeItem( itm, item->ItemResRef, false );
+		bool togglesCrits = (flag&IE_ITEM_TOGGLE_CRITS);
+		bool isHelmet = (i == SLOT_HEAD);
+		if (togglesCrits ^ isHelmet) return true;
 	}
 	return false;
 }
@@ -1870,3 +1980,5 @@ int Inventory::MergeItems(int slot, CREItem *item)
 	}
 	return ASI_FAILED;
 }
+
+}
