@@ -8,7 +8,7 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
@@ -25,15 +25,15 @@
 #include "ScriptEngine.h"
 #include "System/FileStream.h"
 
+namespace GemRB {
+
 #define KEYLENGTH 64
 
 Function::Function(const char *m, const char *f, int g)
 {
 	//make sure the module an function names are no longer than 32 characters, or they will be truncated
-	strncpy(module, m, sizeof(module)-1);
-	strncpy(function, f, sizeof(function)-1);
-	module[32]=0;
-	function[32]=0;
+	strlcpy(module, m, sizeof(module));
+	strlcpy(function, f, sizeof(function));
 	group = g;
 }
 
@@ -60,12 +60,12 @@ bool KeyMap::InitializeKeyMap(const char *inifile, const char *tablefile)
 		return false;
 	}
 
-        char tINIkeymap[_MAX_PATH];
-        PathJoin( tINIkeymap, core->GamePath, inifile, NULL );
-        FileStream* config = FileStream::OpenFile( tINIkeymap );
+	char tINIkeymap[_MAX_PATH];
+	PathJoin( tINIkeymap, core->GamePath, inifile, NULL );
+	FileStream* config = FileStream::OpenFile( tINIkeymap );
 
 	if (config == NULL) {
-		printMessage("KeyMap","There is no '%s' file...\n", YELLOW, inifile);
+		Log(WARNING, "KeyMap", "There is no '%s' file...", inifile);
 		return false;
 	}
 	char name[KEYLENGTH+1], value[_MAX_PATH + 3];
@@ -93,10 +93,10 @@ bool KeyMap::InitializeKeyMap(const char *inifile, const char *tablefile)
 
 		strnlwrcpy(name,name,KEYLENGTH);
 		//remove trailing spaces (bg1 ini file contains them)
-                char *nameend = name + strlen( name ) - 1;
-                while (nameend >= name && strchr( " \t\r\n", *nameend )) {
-                        *nameend-- = '\0';
-                }
+		char *nameend = name + strlen( name ) - 1;
+		while (nameend >= name && strchr( " \t\r\n", *nameend )) {
+			*nameend-- = '\0';
+		}
 
 		//change internal spaces to underscore
 		for(int c=0;c<KEYLENGTH;c++) if (name[c]==' ') name[c]='_';
@@ -106,7 +106,7 @@ bool KeyMap::InitializeKeyMap(const char *inifile, const char *tablefile)
 		void *tmp;
 
 		if (l<0 || l>1 || keymap.Lookup(value, tmp) ) {
-			print("Ignoring key %s\n", value);
+			print("Ignoring key %s", value);
 			continue;
 		}
 
@@ -122,7 +122,7 @@ bool KeyMap::InitializeKeyMap(const char *inifile, const char *tablefile)
 			module = kmtable->QueryField("Default","MODULE");
 			function = kmtable->QueryField("Default","FUNCTION");
 			group = kmtable->QueryField("Default","GROUP");
-			print("Adding key %s with function %s::%s\n", value, module, function);
+			print("Adding key %s with function %s::%s", value, module, function);
 		}
 		fun = new Function(module, function, atoi(group));
 		keymap.SetAt(value, fun);
@@ -133,7 +133,7 @@ bool KeyMap::InitializeKeyMap(const char *inifile, const char *tablefile)
 
 //group can be:
 //main gamecontrol
-void KeyMap::ResolveKey(int key, int group)
+bool KeyMap::ResolveKey(int key, int group)
 {
 	Function *fun;
 	void *tmp;
@@ -142,19 +142,21 @@ void KeyMap::ResolveKey(int key, int group)
 	keystr[0]=(char) key;
 	keystr[1]=0;
 
-	print("Looking up key: %c (%s) \n", key, keystr);
+	Log(MESSAGE, "KeyMap", "Looking up key: %c(%s) ", key, keystr);
 
 	if (!keymap.Lookup(keystr, tmp) ) {
-		return;
+		return false;
 	}
 	fun = (Function *) tmp;
 	
 	if (fun->group!=group) {
-		return;
+		return false;
 	}
 
-	printMessage("KeyMap", " ", WHITE);
-	print("RunFunction(%s::%s)\n",fun->module, fun->function);
+	Log(MESSAGE, "KeyMap", "RunFunction(%s::%s)", fun->module, fun->function);
 	core->GetGUIScriptEngine()->RunFunction(fun->module, fun->function);
+	return true;
 }
 
+
+}
