@@ -18,8 +18,6 @@
  *
  */
 
-class Map;
-
 #ifndef MAP_H
 #define MAP_H
 
@@ -30,6 +28,8 @@ class Map;
 
 #include <queue>
 
+namespace GemRB {
+
 class Actor;
 class Ambient;
 class Animation;
@@ -51,8 +51,13 @@ class Wall_Polygon;
 //distance of actors from spawn point
 #define SPAWN_RANGE       400
 
+//spawn flags
+#define SPF_NOSPAWN		0x0001	//if set don't span if WAIT is set
+#define SPF_ONCE		0x0002	//only spawn a single time
+#define SPF_WAIT		0x0004	//spawn temporarily disabled
+
 //area flags
-#define AF_SAVE           1
+#define AF_NOSAVE         1
 #define AF_TUTORIAL       2
 #define AF_DEADMAGIC      4
 #define AF_DREAM          8
@@ -73,7 +78,7 @@ class Wall_Polygon;
 #define A_ANI_NO_SHADOW       4        //lightmap doesn't affect it
 #define A_ANI_PLAYONCE        8        //stop after endframe
 #define A_ANI_SYNC            16       //synchronised draw (skip frames if needed)
-#define A_ANI_32              32
+#define A_ANI_RANDOM_START    32       //starts with a random frame in the start range
 #define A_ANI_NO_WALL         64       //draw after walls (walls don't cover it)
 #define A_ANI_NOT_IN_FOG      0x80     //not visible in fog of war
 #define A_ANI_BACKGROUND      0x100    //draw before actors (actors cover it)
@@ -85,6 +90,17 @@ class Wall_Polygon;
 //creature area flags
 #define AF_CRE_NOT_LOADED 1
 #define AF_NAME_OVERRIDE  8
+//same flags in IWD2
+#define AF_SEEN_PARTY     2
+#define AF_INVULNERABLE   4
+#define AF_ENABLED        8
+
+//direction flags (used in AreaLinks)
+#define ADIRF_NORTH       0x01
+#define ADIRF_EAST        0x02
+#define ADIRF_SOUTH       0x04
+#define ADIRF_WEST        0x08
+#define ADIRF_CENTER      0x10 //not in the original engine
 
 //getline flags
 #define GL_NORMAL         0
@@ -128,7 +144,7 @@ public:
 	Point Pos;
 	ieWord color;
 	char *text;
-	MapNote() { text=NULL; }
+	MapNote() { text=NULL; strref=0; Pos.empty(); color=0; }
 	~MapNote() { if(text) free(text); }
 };
 
@@ -148,7 +164,8 @@ public:
 	ieDword appearance;
 	ieWord DayChance;
 	ieWord NightChance;
-	Spawn() { Creatures=NULL;  }
+	ieDword NextSpawn;
+	Spawn() { Creatures=NULL; NextSpawn = 0; }
 	~Spawn() { if(Creatures) free(Creatures); }
 	unsigned int GetCreatureCount() { return Count; }
 };
@@ -191,7 +208,7 @@ public:
 	ieWord transparency;
 	ieWordSigned height;
 	//these are on one dword
-	ieWord unknown3c;
+	ieWord startFrameRange;
 	ieByte skipcycle;
 	ieByte startchance;
 	ieDword unknown48;
@@ -279,7 +296,7 @@ public:
 	static void ReleaseMemory();
 
 	/** prints useful information on console */
-	void DebugDump(bool show_actors=0) const;
+	void dump(bool show_actors=0) const;
 	TileMap *GetTileMap() { return TMap; }
 	/* gets the signal of daylight changes */
 	bool ChangeMap(bool day_or_night);
@@ -327,7 +344,11 @@ public:
 	void ActivateWallgroups(unsigned int baseindex, unsigned int count, int flg);
 	void Shout(Actor* actor, int shoutID, unsigned int radius);
 	void ActorSpottedByPlayer(Actor *actor);
-	void AddActor(Actor* actor);
+	void InitActors();
+	void InitActor(Actor *actor);
+	void AddActor(Actor* actor, bool init);
+	//counts the summons already in the area
+	int CountSummons(ieDword flag, ieDword sex);
 	//returns true if an enemy is near P (used in resting/saving)
 	bool AnyEnemyNearPoint(const Point &p);
 	bool GetBlocked(unsigned int x, unsigned int y, unsigned int size);
@@ -339,7 +360,7 @@ public:
 	Actor* GetActorByGlobalID(ieDword objectID);
 	Actor* GetActor(const Point &p, int flags);
 	Actor* GetActorInRadius(const Point &p, int flags, unsigned int radius);
-	Actor **GetAllActorsInRadius(const Point &p, int flags, unsigned int radius);
+	Actor **GetAllActorsInRadius(const Point &p, int flags, unsigned int radius, Scriptable *see=NULL);
 	Actor* GetActor(const char* Name, int flags);
 	Actor* GetActor(int i, bool any);
 	Actor* GetActorByDialog(const char* resref);
@@ -415,7 +436,7 @@ public:
 	void UpdateFog();
 	//PathFinder
 	/* Finds the nearest passable point */
-	void AdjustPosition(Point &goal, unsigned int radius=0);
+	void AdjustPosition(Point &goal, unsigned int radiusx=0, unsigned int radiusy=0);
 	/* Finds the path which leads the farthest from d */
 	PathNode* RunAway(const Point &s, const Point &d, unsigned int size, unsigned int PathLen, int flags);
 	/* Returns true if there is no path to d */
@@ -433,7 +454,7 @@ public:
 	/* returns false if point isn't visible on visibility/explored map */
 	bool IsVisible(const Point &s, int explored);
 	/* returns false if point d cannot be seen from point d due to searchmap */
-	bool IsVisible(const Point &s, const Point &d);
+	bool IsVisibleLOS(const Point &s, const Point &d);
 	/* returns edge direction of map boundary, only worldmap regions */
 	int WhichEdge(const Point &s);
 
@@ -450,10 +471,10 @@ public:
 	MapNote *GetMapNote(const Point &point);
 	unsigned int GetMapNoteCount() { return (unsigned int) mapnotes.size(); }
 	//restheader
-	/* May spawn creature(s), returns true in case of an interrupted rest */
-	bool Rest(const Point &pos, int hours, int day);
+	/* May spawn creature(s), returns the remaining number of (unrested) hours for interrupted rest */
+	int CheckRestInterruptsAndPassTime(const Point &pos, int hours, int day);
 	/* Spawns creature(s) in radius of position */
-	void SpawnCreature(const Point &pos, const char *CreName, int radius = 0);
+	bool SpawnCreature(const Point &pos, const char *creResRef, int radiusx = 0, int radiusy = 0, int *difficulty = NULL, unsigned int *creCount = NULL);
 
 	//spawns
 	void LoadIniSpawn();
@@ -498,9 +519,12 @@ private:
 	//actor uses travel region
 	void UseExit(Actor *pc, InfoPoint *ip);
 	//separated position adjustment, so their order could be randomised */
-	bool AdjustPositionX(Point &goal, unsigned int radius);
-	bool AdjustPositionY(Point &goal, unsigned int radius);
+	bool AdjustPositionX(Point &goal, unsigned int radiusx,  unsigned int radiusy);
+	bool AdjustPositionY(Point &goal, unsigned int radiusx,  unsigned int radiusy);
 	void DrawPortal(InfoPoint *ip, int enable);
+	void UpdateSpawns();
 };
 
+}
+
 #endif
