@@ -26,6 +26,8 @@
 
 #include <cassert>
 
+namespace GemRB {
+
 enum PaletteType {
 	PAL_MAIN,
 	PAL_WEAPON,
@@ -85,6 +87,7 @@ public:
 	}
 
 	void CreateShadedAlphaChannel();
+  void Brighten();
 
 	void SetupPaperdollColours(const ieDword* Colors, unsigned int type);
 	void SetupRGBModification(const Palette* src, const RGBModifier* mods,
@@ -99,4 +102,6 @@ private:
 
 };
 
+}
+
 #endif
