@@ -23,7 +23,6 @@
 #include "win32def.h"
 
 #include "AnimationFactory.h"
-#include "Audio.h"
 #include "DisplayMessage.h"
 #include "Game.h"
 #include "GameData.h"
@@ -38,6 +37,8 @@
 #include <cmath>
 #include <cstdlib>
 
+namespace GemRB {
+
 //to get gradient color
 //apparently pst doesn't have the small palette correctly
 #define PALSIZE 32
@@ -458,6 +459,8 @@ Actor *Projectile::GetTarget()
 		}
 		effects->SetOwner(original);
 		return target;
+	} else {
+		Log(DEBUG, "Projectile", "GetTarget: Target not set or dummy, using caster!");
 	}
 	target = area->GetActorByGlobalID(Caster);
 	if (target) {
@@ -475,6 +478,7 @@ void Projectile::SetDelay(int delay)
 //copied from Actor.cpp
 #define ATTACKROLL    20
 #define WEAPON_FIST        0
+#define WEAPON_BYPASS      0x10000
 
 bool Projectile::FailedIDS(Actor *target) const
 {
@@ -519,9 +523,10 @@ bool Projectile::FailedIDS(Actor *target) const
 				}
 
 				//handle attack type here, weapon depends on it too?
-				int tohit = caster->GetToHit(0, WEAPON_FIST, target);
+				int tohit = caster->GetToHit(WEAPON_FIST, target);
 				//damage type, should be generic?
-				int defense = target->GetDefense(0, caster);
+				// ignore the armor bonus
+				int defense = target->GetDefense(0, WEAPON_BYPASS, caster);
 				if(target->IsReverseToHit()) {
 					fail = roll + defense < tohit;
 				} else {
@@ -536,7 +541,8 @@ bool Projectile::FailedIDS(Actor *target) const
 
 void Projectile::Payload()
 {
-	Actor *target, *Owner;
+	Actor *target;
+	Scriptable *Owner;
 
 	if(Shake) {
 		core->timer->SetScreenShake( Shake, Shake, Shake);
@@ -567,9 +573,19 @@ void Projectile::Payload()
 		}
 	}
 	Actor *source = area->GetActorByGlobalID(Caster);
+	InfoPoint *source2 = area->GetInfoPointByGlobalID(Caster);
+	Container *source3 = area->GetContainerByGlobalID(Caster);
+	Door *source4 = area->GetDoorByGlobalID(Caster);
 	if (source) {
 		Owner = source;
+	} else if (source2) {
+		Owner = (Scriptable *) source2;
+	} else if (source3) {
+		Owner = (Scriptable *) source3;
+	} else if (source4) {
+		Owner = (Scriptable *) source4;
 	} else {
+		Log(WARNING, "Projectile", "Payload: Caster not found, using target!");
 		Owner = target;
 	}
 
@@ -1284,9 +1300,9 @@ void Projectile::SpawnFragment(Point &dest)
 {
 	Projectile *pro = server->GetProjectileByIndex(Extension->FragProjIdx);
 	if (pro) {
-		if (Extension->AFlags&PAF_SECONDARY) {
-				pro->SetEffectsCopy(effects);
-		}
+//		if (Extension->AFlags&PAF_SECONDARY) {
+//				pro->SetEffectsCopy(effects);
+//		}
 		pro->SetCaster(Caster, Level);
 		if (pro->ExtFlags&PEF_RANDOM) {
 			dest.x+=core->Roll(1,Extension->TileX, -Extension->TileX/2);
@@ -1365,7 +1381,8 @@ void Projectile::DrawExplosion(const Region &screen)
 	//warning: this projectile doesn't inherit any effects, so its payload function
 	//won't be doing anything (any effect of PAF_SECONDARY?)
 
-	if (Extension->FragProjIdx) {
+  //remove PAF_SECONDARY if it is buggy, but that will break the 'HOLD' projectile
+	if ((Extension->AFlags&PAF_SECONDARY) && Extension->FragProjIdx) {
 		if (apflags&APF_TILED) {
 			int i,j;
 			int radius = Extension->ExplosionRadius;
@@ -1619,6 +1636,7 @@ void Projectile::DrawTravel(const Region &screen)
 	//Area tint
 	if (TFlags&PTF_TINT) {
 		tint = area->LightMap->GetPixel( Pos.x / 16, Pos.y / 12);
+		tint.a = 255;
 		flag |= BLIT_TINTED;
 	}
 
@@ -1774,3 +1792,5 @@ void Projectile::Cleanup()
 	phase=P_EXPIRED;
 }
 
+
+}
