@@ -32,11 +32,14 @@
 #include "ie_types.h"
 
 #include "CharAnimations.h" //contains MAX_ORIENT
+#include "EffectQueue.h"
 #include "Map.h"
 #include "Palette.h"
 #include "PathFinder.h"
 #include "Audio.h"
 
+namespace GemRB {
+
 //this is the height of the projectile when Spark Flag Fly = 1
 #define FLY_HEIGHT 50
 //this is supposed to move the projectile to the background
@@ -67,7 +70,8 @@
 #define PTF_TINT    8       //tint projectile
 #define PTF_SHADOW  32      //has shadow bam
 #define PTF_LIGHT   64      //has light shadow
-#define PTF_BLEND   128     //blend colours
+#define PTF_BLEND   128     //blend colours (use alpha)
+#define PTF_BRIGHTEN 256    //brighten alpha
 
 //projectile extended travel flags (gemrb specific)
 #define PEF_BOUNCE     1       //bounce from walls (lightning bolt)
@@ -359,7 +363,7 @@ private:
 	void CreateOrientedAnimations(Animation **anims, AnimationFactory *af, int Seq);
 	void GetPaletteCopy(Animation *anim[], Palette *&pal);
 	void GetSmokeAnim();
-	void SetBlend();
+	void SetBlend(int brighten);
 	//apply spells and effects on the target, only in single travel mode
 	//area effect projectiles call a separate single travel projectile for each affected target
 	void Payload();
@@ -402,5 +406,6 @@ private:
 	void SetupPalette(Animation *anim[], Palette *&pal, const ieByte *gradients);
 };
 
-#endif // PROJECTILE_H
+}
 
+#endif // PROJECTILE_H
