@@ -29,25 +29,35 @@
 #include "ie_types.h"
 
 #include "Audio.h"
+#include "CombatInfo.h"
 #include "EffectQueue.h"
 #include "Palette.h"
 
 #include <cstring>
 #include <vector>
 
+namespace GemRB {
+
 class Animation;
+class ArmorClass;
 class CharAnimations;
 class DataFileMgr;
 class Map;
 class ScriptedAnimation;
+class StringBuffer;
+class ToHitStats;
 struct PolymorphCache;
 
+}
+
 /** USING DEFINITIONS AS DESCRIBED IN STATS.IDS */
 #include "ie_stats.h"
 
 #include "Inventory.h"
 #include "Spellbook.h"
 
+namespace GemRB {
+
 #define MAX_STATS 256
 #define MAX_LEVEL 128
 #define MAX_FEATS 96 //3*sizeof(ieDword)
@@ -90,6 +100,14 @@ struct PolymorphCache;
 #define PANIC_RUNAWAY    2
 #define PANIC_RANDOMWALK 3
 
+//Game Difficulty
+#define DIFF_DEFAULT       0
+#define DIFF_EASY          1
+#define DIFF_NORMAL        2
+#define DIFF_CORE          3
+#define DIFF_HARD          4
+#define DIFF_NIGHTMARE     5
+
 /** flags for GetActor */
 //default action
 #define GA_DEFAULT  0
@@ -149,6 +167,7 @@ struct PolymorphCache;
 //flags for UseItem
 #define UI_SILENT    1       //no sound when used up
 #define UI_MISS      2       //ranged miss (projectile has no effects)
+#define UI_CRITICAL  4       //a critical hit happened
 
 //used to mask off current profs
 #define PROFS_MASK  0x07
@@ -165,7 +184,10 @@ struct PolymorphCache;
 #define ISPALADIN   8
 #define ISRANGER    9
 #define ISSORCERER  10
-#define ISCLASSES   11
+#define ISCLASS12   11
+#define ISCLASS13   12
+
+#define ISCLASSES   13
 
 //appearance flags
 
@@ -213,19 +235,30 @@ struct ActionButtonRow2 {
 };
 
 typedef std::vector< ScriptedAnimation*> vvcVector;
-typedef std::list<ieResRef*> resourceList;
 
 struct WeaponInfo {
 	int slot;
-	int enchantment;
+	ieDword enchantment;
 	unsigned int range;
 	ieDword itemflags;
 	ieDword prof;
 	bool backstabbing;
 	ieDword wflags;
+	int critmulti; //critical hit multiplier (usually 2)
+	int critrange; // the lower value of the critical range (eg. 19 in 19-20/x3)
+	int profdmgbon;
+	int launcherdmgbon;
+	WeaponInfo(): slot(0), enchantment(0), range(0), itemflags(0), prof(0), backstabbing(false), wflags(0), critmulti(0), critrange(0), profdmgbon(0), launcherdmgbon(0) {};
+};
+
+struct BABTable {
+	ieDword level;
+	int bab; // basic attack bonus
+	int apr; // attacks per round
 };
 
 extern void ReleaseMemoryActor();
+GEM_EXPORT void UpdateActorConfig(); //call this from guiscripts when some variable has changed
 
 class GEM_EXPORT Actor : public Movable {
 public:
@@ -248,6 +281,7 @@ public:
 	ieResRef ModalSpell;             //apply this spell once per round
 	ieResRef LingeringModalSpell;    //apply this spell once per round if the effects are lingering
 	ieResRef BardSong;               //custom bard song (updated by fx)
+	ieResRef BackstabResRef;         //apply on successful backstab
 
 	PCStatsStruct*  PCStats;
 	ieResRef SmallPortrait;
@@ -280,8 +314,12 @@ public:
 	ieDword appearance;
 	ieDword ModalState;
 	int PathTries; //the # of previous tries to pick up a new walkpath
+	ArmorClass AC;
+	ToHitStats ToHit;
 public:
 	ieDword LastExit;    //the global ID of the exit to be used
+	ieDword UsedExit;
+	ieResRef LastArea;
 	char ShieldRef[2];
 	char HelmetRef[2];
 	char WeaponRef[2];
@@ -311,6 +349,8 @@ public:
 	ieDword lastInit;
 	bool no_more_steps;
 	int speed;
+	//how many attacks left in this round, must be public for cleave opcode
+	int attackcount;
 
 	PolymorphCache *polymorphCache; // fx_polymorph etc
 	WildSurgeSpellMods wildSurgeMods;
@@ -319,8 +359,8 @@ private:
 	CharAnimations* anims;
 	SpriteCover* extraCovers[EXTRA_ACTORCOVERS];
 	ieByte SavingThrow[5];
-	//how many attacks in this round
-	int attackcount;
+	// true when command has been played after select
+	bool playedCommandSound;
 	//true every second round of attack
 	bool secondround;
 	int attacksperround;
@@ -328,10 +368,13 @@ private:
 	ieDword nextattack;
 	ieDword nextWalk;
 	ieDword lastattack;
+	//trap we're trying to disarm
+	ieDword disarmTrap;
 	ieDword InTrap;
 	char AttackStance;
 	/*The projectile bringing the current attack*/
 	Projectile* attackProjectile ;
+	int TicksLastRested;
 	/** paint the actor itself. Called internally by Draw() */
 	void DrawActorSprite(const Region &screen, int cx, int cy, const Region& bbox,
 				SpriteCover*& sc, Animation** anims,
@@ -341,8 +384,6 @@ private:
 	void SetupColors();
 	/** debugging function, gets the scripting name of an actor referenced by a global ID */
 	const char* GetActorNameByID(ieDword ID) const;
-	/* if Lasttarget is gone, call this */
-	void StopAttack();
 	/* checks a weapon quick slot and resets it to fist if it is empty */
 	void CheckWeaponQuickSlot(unsigned int which);
 	/* helper for usability checks */
@@ -352,7 +393,7 @@ private:
 	/* Set up all the missing stats on load time, or after level up */
 	void CreateDerivedStatsIWD2();
 	/* Gets the given ISCLASS level */
-	ieDword GetClassLevel (const ieDword id) const;
+	ieDword GetClassLevel (const ieDword isclass) const;
 	/* Returns true if the dual class is backwards */
 	bool IsDualSwap() const;
 	/* returns the weapon proficiency stat of the actor */
@@ -362,8 +403,10 @@ private:
 	void RefreshHP();
 	bool ShouldHibernate();
 	void ApplyClassClab(int cls, bool remove);
-	bool ShouldDrawCircle();
+	bool ShouldDrawCircle() const;
+	bool HasBodyHeat() const;
 	void SetupFistData();
+	void UpdateFatigue();
 public:
 	Actor(void);
 	~Actor(void);
@@ -374,18 +417,23 @@ public:
 	/** sets game specific default data about action buttons */
 	static void SetDefaultActions(int qslot, ieByte slot1, ieByte slot2, ieByte slot3);
 	/** prints useful information on console */
-	void DumpMaxValues();
-	void DebugDump();
+	void dump() const;
+	/** prints useful information to given buffer */
+	void dump(StringBuffer&) const;
 	/** fixes the feet circle */
 	void SetCircleSize();
 	/** places the actor on the map */
 	void SetMap(Map *map);
 	/** sets the actor's position, calculating with the nojump flag*/
-	void SetPosition(const Point &position, int jump, int radius=0);
+	void SetPosition(const Point &position, int jump, int radiusx=0, int radiusy=0);
 	/** you better use SetStat, this stuff is only for special cases*/
 	void SetAnimationID(unsigned int AnimID);
 	/** returns the animations */
 	CharAnimations* GetAnims() const;
+	/** returns the gender of actor for cg sound - illusions are tricky */
+	ieDword GetCGGender();
+	/** some hardcoded effects in puppetmaster based on puppet type */
+	void CheckPuppet(Actor *puppet, ieDword type);
 	/** Re/Inits the Modified vector */
 	void RefreshEffects(EffectQueue *eqfx);
 	/** gets saving throws */
@@ -393,7 +441,9 @@ public:
 	/** returns a saving throw */
 	bool GetSavingThrow(ieDword type, int modifier);
 	/** Returns true if the actor is targetable */
-	bool ValidTarget(int ga_flags) const;
+	bool ValidTarget(int ga_flags, Scriptable *checker = NULL) const;
+	/** Clamps a stat value to the valid range for the respective stat */
+	ieDword ClampStat(unsigned int StatIndex, ieDword Value) const;
 	/** Returns a Stat value */
 	ieDword GetStat(unsigned int StatIndex) const;
 	/** Returns a safe Stat value, one, that is not partially computed */
@@ -439,7 +489,8 @@ public:
 	ieDword GetXPLevel(int modified) const;
 	/** Guesses the (base) casting level */
 	ieDword GetCasterLevel(int spelltype);
-	ieDword GetBaseCasterLevel(int spelltype) const;
+	ieDword GetBaseCasterLevel(int spelltype, int flags=0) const;
+	ieDword GetAnyActiveCasterLevel() const;
 	/** Returns the wild mage casting level modifier */
 	int GetWildMod(int level);
 	/** Returns any casting level modifier */
@@ -460,16 +511,16 @@ public:
 	}
 	void SetName(const char* ptr, unsigned char type);
 	void SetName(int strref, unsigned char type);
-	/* returns carried weight atm, could calculate with strength*/
-	int GetEncumbrance();
+	/* calculates speed, encumbrance etc */
+	int CalculateSpeed(bool feedback);
 	/* checks on death of actor, returns true if it should be removed*/
 	bool CheckOnDeath();
 	/* receives undead turning message */
 	void Turn(Scriptable *cleric, ieDword turnlevel);
 	/* call this on gui selects */
-	void SelectActor() const;
+	void PlaySelectionSound();
 	/* call this when adding actions via gui */
-	void CommandActor() const;
+	void CommandActor();
 	/** handle panic and other involuntary actions that mess with scripting */
 	bool OverrideActions();
 	/** handle idle actions, that shouldn't mess with scripting */
@@ -484,20 +535,30 @@ public:
 	ieStrRef GetVerbalConstant(int index) const;
 	/* displaying a random verbal constant */
 	void VerbalConstant(int start, int count) const;
+	/* display string or verbal constant depending on what is available */
+	void DisplayStringOrVerbalConstant(int str, int vcstat, int vccount) const;
 	/* inlined dialogue response */
 	void Response(int type) const;
 	/* called when someone died in the party */
 	void ReactToDeath(const char *deadname);
+	/* sends trigger_died to everyone in visual range */
+	void SendDiedTrigger();
 	/* called when someone talks to Actor */
 	void DialogInterrupt();
 	/* called when actor was hit */
-	void GetHit();
+	void GetHit(int damage=0, int spellLevel=0);
+	/* checks whether concentration cancels damage taken disrupting casting */
+	bool CheckCastingInterrupt(int damage, int spellLevel);
 	/* called when actor starts to cast a spell*/
-	bool HandleCastingStance(const ieResRef SpellResRef, bool deplete);
+	bool HandleCastingStance(const ieResRef SpellResRef, bool deplete, bool instant);
 	/* check if the actor should be just knocked out by a lethal hit */
 	bool AttackIsStunning(int damagetype) const;
+	/* check if the actor is silenced - for casting purposes */
+	bool CheckSilenced();
+	/* check and perform a cleave movement */
+	void CheckCleave();
 	/* deals damage to this actor */
-	int Damage(int damage, int damagetype, Scriptable *hitter, int modtype=MOD_ADDITIVE);
+	int Damage(int damage, int damagetype, Scriptable *hitter, int modtype=MOD_ADDITIVE, int critical=0, int saveflags=0);
 	/* displays the damage taken and other details (depends on the game type) */
 	void DisplayCombatFeedback (unsigned int damage, int resisted, int damagetype, Scriptable *hitter);
 	/* play a random footstep sound */
@@ -548,10 +609,10 @@ public:
 	ITMExtHeader* GetWeapon(WeaponInfo &wi, bool leftorright=false) const;
 	/* Creates player statistics */
 	void CreateStats();
-	/* Heals actor by days */
-	void Heal(int days);
+	/* Heals actor */
+	void Heal(int hp);
 	/* Receive experience (handle dual/multi class) */
-	void AddExperience(int exp);
+	void AddExperience(int exp, int combat);
 	/* Calculate experience bonus */
 	int CalculateExperience(int type, int level);
 	/* Sets the modal state after checks */
@@ -562,29 +623,40 @@ public:
 	int GetAttackStyle() const;
 	/* adds the combatants to the attackers list */
 	void AttackedBy(Actor *actor);
-	/* sets target for immediate attack */
-	void SetTarget( Scriptable *actor);
+	/* reorients to face target (for immediate attack) */
+	void FaceTarget(Scriptable *actor);
+	/* returns the number of attacks (handles monk barehanded bonus) */
+	ieDword GetNumberOfAttacks();
 	/* starts combat round*/
 	void InitRound(ieDword gameTime);
 	/* returns melee penalty */
 	int MeleePenalty() const;
 	/* gets the to hit value */
-	int GetToHit(int bonus, ieDword Flags, Actor *target) const;
+	int GetToHit(ieDword Flags, Actor* target);
+	void GetTHAbilityBonus(ieDword Flags);
 	/* gets the defense against an attack */
-	int GetDefense(int DamageType, Actor *attacker) const;
+	int GetDefense(int DamageType, ieDword wflags, Actor *attacker);
+	/* checks if something is wrong with the weapon we are using for the attack */
+	bool WeaponIsUsable(bool leftorright, ITMExtHeader *header=NULL) const;
 	/* get the current hit bonus */
 	bool GetCombatDetails(int &tohit, bool leftorright, WeaponInfo &wi, ITMExtHeader *&header, ITMExtHeader *&hittingheader,\
-		int &DamageBonus, int &speed, int &CriticalBonus, int &style, Actor *target) const;
+		int &DamageBonus, int &speed, int &CriticalBonus, int &style, Actor *target);
 	/* performs attack against target */
 	void PerformAttack(ieDword gameTime);
+	/* returns the adjusted weapon range, since items have odd values stored */
+	int GetWeaponRange(const WeaponInfo &wi) const;
+	/* filter out any damage reduction that is cancelled by high weapon enchantment and return the resulting resistance */
+	int GetDamageReduction(int resist_stat, ieDword weaponEnchantment) const;
 	/* calculates strength (dexterity) based damage adjustment */
-	int WeaponDamageBonus(WeaponInfo *wi);
-	/* adjusts damage dealt to this actor  */
-	void ModifyDamage(Scriptable *hitter, int &damage, int &resisted, int damagetype, WeaponInfo *wi, bool critical);
+	int WeaponDamageBonus(const WeaponInfo &wi) const;
+	/* handles critical, backstab, etc. damage modifications */
+	void ModifyWeaponDamage(WeaponInfo &wi, Actor *target, int &damage, bool &critical);
+	/* adjusts damage dealt to this actor, handles mirror images  */
+	void ModifyDamage(Scriptable *hitter, int &damage, int &resisted, int damagetype);
 	/* applies modal spell etc, if needed */
 	void UpdateActorState(ieDword gameTime);
 	/* returns the hp adjustment based on constitution */
-	int GetHpAdjustment(int multiplier);
+	int GetHpAdjustment(int multiplier) const;
 	/* does all the housekeeping after loading the actor from file */
 	void InitStatsOnLoad();
 	/* sets a colour gradient stat, handles location */
@@ -593,7 +665,7 @@ public:
 	void SetColorMod( ieDword location, RGBModifier::Type type, int speed,
 		unsigned char r, unsigned char g, unsigned char b,
 		int phase=-1 );
-	bool Schedule(ieDword gametime, bool checkhide);
+	bool Schedule(ieDword gametime, bool checkhide) const;
 	/* call this when path needs to be changed */
 	void NewPath();
 	/* overridden method, won't walk if dead */
@@ -612,6 +684,8 @@ public:
 	void SetActionButtonRow(ActionButtonRow &ar);
 	/* updates the quick slots */
 	void GetActionButtonRow(ActionButtonRow &qs);
+	/* converts the iwd2 qslot index to our internal representation */
+	int IWD2GemrbQslot (int slotindex);
 
 	/* Handling automatic stance changes */
 	bool HandleActorStance();
@@ -634,6 +708,7 @@ public:
 	/* returns the vvc pointer to a hardcoded overlay */
 	/* if it exists (faster than hasvvccell) */
 	ScriptedAnimation *FindOverlay(int index) const;
+	int WantDither() const;
 	/* draw videocells */
 	void DrawVideocells(const Region &screen, vvcVector &vvcCells, const Color &tint);
 
@@ -642,6 +717,15 @@ public:
 	void AddAnimation(const ieResRef resource, int gradient, int height, int flags);
 	/* plays damage animation, if hit is not set, then plays only the splash part */
 	void PlayDamageAnimation(int x, bool hit=true);
+	void PlayCritDamageAnimation(int x);
+	/* returns mage or cleric spell casting failure, iwd2 compatible */
+	ieDword GetSpellFailure(bool arcana) const;
+	/* returns the dexterity AC adjusted by armor, iwd2 compatible */
+	int GetDexterityAC() const;
+	/* returns the monk wisdom AC adjusted by armor (iwd2) */
+	int GetWisdomAC() const;
+	/* PST specific criticals */
+	int GetCriticalType() const;
 	/* restores a spell of maximum maxlevel level, type is a mask of disabled spells */
 	int RestoreSpellLevel(ieDword maxlevel, ieDword typemask);
 	/* rememorizes spells, cures fatigue, etc */
@@ -666,10 +750,13 @@ public:
 	/* initialize the action buttons based on class. If forced, it will override
 		previously customized or set buttons. */
 	void InitButtons(ieDword cls, bool forced);
-	int GetAbilityBonus(unsigned int ability) const;
+	int GetMaxEncumbrance() const;
+	int GetAbilityBonus(unsigned int ability, int value = -1) const;
+	int GetSkillStat(unsigned int skill) const;
 	int GetSkill(unsigned int skill) const;
 	int GetFeat(unsigned int feat) const;
 	void SetFeat(unsigned int feat, int mode);
+	void SetFeatValue(unsigned int feat, int value);
 	void SetUsedWeapon(const char *AnimationType, ieWord *MeleeAnimation,
 		int WeaponType=-1);
 	void SetUsedShield(const char *AnimationType, int WeaponType=-1);
@@ -695,6 +782,10 @@ public:
 	ieDword ImmuneToProjectile(ieDword projectile) const;
 	/* Sets projectile immunity */
 	void AddProjectileImmunity(ieDword projectile);
+	/* Apply feats */
+	void ApplyFeats();
+	/* reapply modal feat spells */
+	void ApplyExtraSettings();
 	/* Set up all the missing stats on load time, chargen, or after level up */
 	void CreateDerivedStats();
 	/* Checks if the actor is multiclassed (excluding dualclassed actors)) */
@@ -737,25 +828,57 @@ public:
 	/* checks whether the actor is behind the target */
 	bool IsBehind(Actor* target) const;
 	/* checks whether the target is the actor's racial enemy */
-	bool IsRacialEnemy(Actor* target) const;
+	int GetRacialEnemyBonus(Actor* target) const;
 	/* checks whether the actor can stay in the current modal state */
 	bool ModalSpellSkillCheck();
+	/* check if this actor is seen by or seeing anyone */
+	bool SeeAnyOne(bool enemy, bool seen);
 	/* does all the game logic checks to see if the actor can hide */
 	bool TryToHide();
+	bool TryToHideIWD2();
 	/* checks if the alignment matches one of the masking constants */
 	//bool MatchesAlignmentMask(ieDword mask);
-	/* returns true if this actor is untargetable */
+	/** untargetable by spells/attack due to invisibility or sanctuary */
+	bool Untargetable();
+	/* returns true if this it is futile to try to harm actor (dead/sanctuaried) */
 	bool InvalidSpellTarget() const;
 	/* returns true if the spell is useless to cast on target
 	or the spell's range is smaller than range */
 	bool InvalidSpellTarget(int spellnum, Actor *caster, int range) const;
 	/* returns true if the lightmap under the actor is dark */
 	bool PCInDark() const;
+	/* function to get a class level used by scripting */
+	ieDword GetLevelInClass (const ieDword classid) const;
 	/* computes the actor's classmask (iwd2) */
 	int GetClassMask() const;
+	/* computes the actor's usable books (iwd2) */
+	int GetBookMask() const;
 	/* computes the thieving skill bonus from race and dexterity */
 	int GetSkillBonus(unsigned int col) const;
 	/* returns true for party members (and familiars) */
 	bool IsPartyMember() const;
+	/* resets the bored and area comment timers */
+	void ResetCommentTime();
+	/* returns the armor check penalty */
+	int GetArmorSkillPenalty(int profcheck=1) const;
+	int GetArmorSkillPenalty(int profcheck, int &armor, int &shield) const;
+	int GetTotalArmorFailure() const;
+	int GetArmorFailure(int &armor, int &shield) const;
+	bool IsDead() const;
+	bool IsInvisibleTo(Scriptable *checker) const;
+	int UpdateAnimationID(bool derived);
+	void MovementCommand(char *command);
+	/* shows hp/maxhp as overhead text */
+	void DisplayHeadHPRatio();
+	/* if Lasttarget is gone, call this */
+	void StopAttack();
+	int SetBaseAPRandAB(bool CheckRapidShot);
+	int BAB2APR(int pBAB, int pBABDecrement, int CheckRapidShot) const;
+	/* set to trap id if current action is disarm; unset after */
+	void SetDisarmingTrap(ieDword trapId) { disarmTrap = trapId; }
+	ieDword GetDisarmingTrap() const { return disarmTrap; }
+	void ReleaseCurrentAction();
 };
+}
+
 #endif
