@@ -19,8 +19,9 @@
 
 #include "Scriptable/InfoPoint.h"
 
-#include "strrefs.h"
 #include "win32def.h"
+#include "strrefs.h"
+#include "ie_cursors.h"
 
 #include "Audio.h"
 #include "DisplayMessage.h"
@@ -36,13 +37,19 @@
 #include "Video.h"
 #include "GameScript/GSUtils.h"
 #include "GUI/GameControl.h"
-#include "GUI/Window.h"
+#include "System/StringBuffer.h"
 
 #include <cassert>
 #include <cmath>
 
+namespace GemRB {
+
 #define YESNO(x) ( (x)?"Yes":"No")
 
+//crazy programmers couldn't decide which bit marks the alternative point
+static ieDword TRAP_USEPOINT = _TRAP_USEPOINT;
+static bool inited = false;
+
 InfoPoint::InfoPoint(void)
 	: Highlightable( ST_TRIGGER )
 {
@@ -53,6 +60,19 @@ InfoPoint::InfoPoint(void)
 	TrapRemovalDiff = 0;
 	TrapDetected = 0;
 	TrapLaunch.empty();
+	if (!inited) {
+		inited = true;
+		//TRAP_USEPOINT may have three values
+		//0     - PST - no such flag
+		//0x200 - IWD2 - it has no TRAP_NONPC flag, the usepoint flag takes it over
+		//0x400 - all other engines (some don't use it)
+		if (core->HasFeature(GF_USEPOINT_400)) TRAP_USEPOINT = _TRAP_USEPOINT;
+		else if (core->HasFeature(GF_USEPOINT_200)) TRAP_USEPOINT = _TRAVEL_NONPC;
+		else TRAP_USEPOINT = 0;
+	}
+	StrRef = 0;
+	UsePoint.empty();
+	TalkPos.empty();
 }
 
 InfoPoint::~InfoPoint(void)
@@ -66,21 +86,31 @@ void InfoPoint::SetEnter(const char *resref)
 	}
 }
 
+ieDword InfoPoint::GetUsePoint() const {
+	return Flags&TRAP_USEPOINT;
+}
+
 //checks if the actor may use this travel trigger
 //bit 1 : can use
 //bit 2 : whole team
 int InfoPoint::CheckTravel(Actor *actor)
 {
 	if (Flags&TRAP_DEACTIVATED) return CT_CANTMOVE;
-	if (!actor->InParty && (Flags&TRAVEL_NONPC) ) return CT_CANTMOVE;
-	if (actor->InParty && (Flags&TRAVEL_PARTY) ) {
+	bool pm = actor->IsPartyMember();
+
+	//this flag does not exist in IWD2, we cannot use it if the infopoint flag took it over
+	if (TRAP_USEPOINT!=_TRAVEL_NONPC) {
+		if (!pm && (Flags&_TRAVEL_NONPC) ) return CT_CANTMOVE;
+	}
+
+	if (pm && (Flags&TRAVEL_PARTY) ) {
 		if (core->HasFeature(GF_TEAM_MOVEMENT) || core->GetGame()->EveryoneNearPoint(actor->GetCurrentArea(), actor->Pos, ENP_CANMOVE) ) {
 			return CT_WHOLE;
 		}
 		return CT_GO_CLOSER;
 	}
-	if(actor->IsSelected() ) {
-		if(core->GetGame()->EveryoneNearPoint(actor->GetCurrentArea(), actor->Pos, ENP_CANMOVE|ENP_ONLYSELECT) ) {
+	if (actor->IsSelected() ) {
+		if (core->GetGame()->EveryoneNearPoint(actor->GetCurrentArea(), actor->Pos, ENP_CANMOVE|ENP_ONLYSELECT) ) {
 			return CT_MOVE_SELECTED;
 		}
 		return CT_SELECTED;
@@ -149,6 +179,10 @@ bool InfoPoint::Entered(Actor *actor)
 	if (outline->PointIn( actor->Pos ) ) {
 		goto check;
 	}
+	// be more lenient for travel regions, fixed iwd2 ar1100 to1101 region
+	if (Type == ST_TRAVEL && outline->BBox.PointInside(actor->Pos)) {
+		goto check;
+	}
 	// why is this here? actors which aren't *in* a trap get IF_INTRAP
 	// repeatedly unset, so this triggers again and again and again.
 	// i disabled it for ST_PROXIMITY for now..
@@ -179,6 +213,11 @@ check:
 		return false;
 	}
 
+	// allow entering trap when trying to disarm
+	if (Type == ST_PROXIMITY && actor->GetDisarmingTrap() == GetGlobalID()) {
+		return false;
+	}
+
 	if (actor->InParty || (Flags&TRAP_NPC) ) {
 		//no need to avoid a travel trigger
 
@@ -190,35 +229,36 @@ check:
 	return false;
 }
 
-void InfoPoint::DebugDump() const
+void InfoPoint::dump() const
 {
+	StringBuffer buffer;
 	switch (Type) {
 		case ST_TRIGGER:
-			print( "Debugdump of InfoPoint Region %s:\n", GetScriptName() );
+			buffer.appendFormatted( "Debugdump of InfoPoint Region %s:\n", GetScriptName() );
 			break;
 		case ST_PROXIMITY:
-			print( "Debugdump of Trap Region %s:\n", GetScriptName() );
+			buffer.appendFormatted( "Debugdump of Trap Region %s:\n", GetScriptName() );
 			break;
 		case ST_TRAVEL:
-			print( "Debugdump of Travel Region %s:\n", GetScriptName() );
+			buffer.appendFormatted( "Debugdump of Travel Region %s:\n", GetScriptName() );
 			break;
 		default:
-			print( "Debugdump of Unsupported Region %s:\n", GetScriptName() );
+			buffer.appendFormatted( "Debugdump of Unsupported Region %s:\n", GetScriptName() );
 			break;
 	}
-	print( "Region Global ID: %d\n", GetGlobalID());
-	print( "Position: %d.%d\n", Pos.x, Pos.y);
+	buffer.appendFormatted( "Region Global ID: %d\n", GetGlobalID());
+	buffer.appendFormatted( "Position: %d.%d\n", Pos.x, Pos.y);
 	switch(Type) {
 	case ST_TRAVEL:
-		print( "Destination Area: %s Entrance: %s\n", Destination, EntranceName);
+		buffer.appendFormatted( "Destination Area: %s Entrance: %s\n", Destination, EntranceName);
 		break;
 	case ST_PROXIMITY:
-		print( "TrapDetected: %d, Trapped: %s\n", TrapDetected, YESNO(Trapped));
-		print( "Trap detection: %d%%, Trap removal: %d%%\n", TrapDetectionDiff,
+		buffer.appendFormatted( "TrapDetected: %d, Trapped: %s\n", TrapDetected, YESNO(Trapped));
+		buffer.appendFormatted( "Trap detection: %d%%, Trap removal: %d%%\n", TrapDetectionDiff,
 			TrapRemovalDiff );
 		break;
 	case ST_TRIGGER:
-		print ( "InfoString: %s\n", overHeadText );
+		buffer.appendFormatted ( "InfoString: %s\n", overHeadText );
 		break;
 	default:;
 	}
@@ -226,8 +266,11 @@ void InfoPoint::DebugDump() const
 	if (Scripts[0]) {
 		name = Scripts[0]->GetName();
 	}
-	print( "Script: %s, Key: %s, Dialog: %s\n", name, KeyResRef, Dialog );
-	print( "Deactivated: %s\n", YESNO(Flags&TRAP_DEACTIVATED));
-	print( "Active: %s\n", YESNO(InternalFlags&IF_ACTIVE));
+	buffer.appendFormatted( "Script: %s, Key: %s, Dialog: %s\n", name, KeyResRef, Dialog );
+	buffer.appendFormatted( "Deactivated: %s\n", YESNO(Flags&TRAP_DEACTIVATED));
+	buffer.appendFormatted( "Active: %s\n", YESNO(InternalFlags&IF_ACTIVE));
+	Log(DEBUG, "InfoPoint", buffer);
 }
 
+
+}
