@@ -27,6 +27,8 @@
 
 #include <list>
 
+namespace GemRB {
+
 class Action;
 class Actor;
 class Container;
@@ -72,12 +74,12 @@ class SpriteCover;
 #define TRAVEL_PARTY    4
 #define TRAP_DETECTABLE 8
 //#define TRAP_16	 16
-#define TRAP_LOWMEM	 32 //special treatment when low on memory ?
+#define TRAP_TUTORIAL	 32 //active only when in tutorial mode
 #define TRAP_NPC	64
 //#define TRAP_128	128
 #define TRAP_DEACTIVATED  256
-#define TRAVEL_NONPC      512
-#define TRAP_USEPOINT       1024 //override usage point of travel regions (used for sound in PST traps)
+#define _TRAVEL_NONPC      512
+#define _TRAP_USEPOINT       1024 //override usage point of travel regions (used for sound in PST traps)
 #define INFO_DOOR	 2048 //info trigger blocked by door
 
 //internal actor flags
@@ -94,7 +96,7 @@ class SpriteCover;
 //#define IF_BECAMEVISIBLE 0x100//actor just became visible (trigger event)
 #define IF_INITIALIZED   0x200
 #define IF_USEDSAVE      0x400  //actor needed saving throws
-//#define IF_TARGETGONE    0x800  //actor's target is gone (trigger event)
+#define IF_GOTAREA    0x800     //actor already moved to an area
 #define IF_USEEXIT       0x1000 //
 #define IF_INTRAP        0x2000 //actor is currently in a trap (intrap trigger event)
 //#define IF_WASINDIALOG   0x4000 //actor just left dialog
@@ -118,14 +120,6 @@ class SpriteCover;
 #define CT_SELECTED       4 //not all selected actors are there
 #define CT_MOVE_SELECTED  5 //all selected can move
 
-//bits for binary trigger bitfield
-#define BT_DIE            1
-#define BT_ONCREATION     2
-#define BT_BECAMEVISIBLE  4
-#define BT_WASINDIALOG    8
-#define BT_PARTYRESTED    16
-#define BT_VACANT         32
-
 //xp bonus types (for xpbonus.2da)
 #define XP_LOCKPICK   0
 #define XP_DISARM     1
@@ -312,7 +306,7 @@ public:
 	void Activate();
 	void Deactivate();
 	void PartyRested();
-	ieDword GetInternalFlag();
+	ieDword GetInternalFlag() const;
 	const char* GetScriptName() const;
 	Map* GetCurrentArea() const;
 	void SetMap(Map *map);
@@ -332,29 +326,30 @@ public:
 	Action* GetNextAction() const;
 	Action* PopNextAction();
 	void ClearActions();
-	void ReleaseCurrentAction();
+	virtual void ReleaseCurrentAction();
 	bool InMove() const;
 	void ProcessActions();
 	//these functions handle clearing of triggers that resulted a
 	//true condition (whole triggerblock returned true)
 	void InitTriggers();
-	void ClearTriggers();
 	void AddTrigger(TriggerEntry trigger);
 	bool MatchTrigger(unsigned short id, ieDword param = 0);
 	bool MatchTriggerWithObject(unsigned short id, class Object *obj, ieDword param = 0);
 	const TriggerEntry *GetMatchingTrigger(unsigned short id, unsigned int notflags = 0);
+	void SendTriggerToAll(TriggerEntry entry);
 	/* re/draws overhead text on the map screen */
 	void DrawOverheadText(const Region &screen);
 	/* check if casting is allowed at all */
-	int CanCast(const ieResRef SpellResRef);
+	int CanCast(const ieResRef SpellResRef, bool verbose=true);
 	/* check for and trigger a wild surge */
 	int CheckWildSurge();
+	void SpellcraftCheck(const Actor *caster, const ieResRef SpellResRef);
 	/* actor/scriptable casts spell */
 	int CastSpellPoint( const Point &Target, bool deplete, bool instant = false, bool nointerrupt = false );
 	int CastSpell( Scriptable* Target, bool deplete, bool instant = false, bool nointerrupt = false );
 	/* spellcasting finished */
-	void CastSpellPointEnd(int level);
-	void CastSpellEnd(int level);
+	void CastSpellPointEnd(int level, int no_stance);
+	void CastSpellEnd(int level, int no_stance);
 	ieDword GetGlobalID() const { return globalID; }
 	/** timer functions (numeric ID, not saved) */
 	bool TimerActive(ieDword ID);
@@ -369,6 +364,8 @@ private:
 	void CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int level, bool fake);
 	/* do some magic for the wierd/awesome wild surges */
 	bool HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor *caster);
+	void ResetCastingState(Actor* caster);
+	void DisplaySpellCastMessage(ieDword tgt, Spell *spl);
 };
 
 class GEM_EXPORT Selectable : public Scriptable {
@@ -400,7 +397,7 @@ public:
 	/* get stored SpriteCover */
 	SpriteCover* GetSpriteCover() const { return cover; }
 	/* want dithered SpriteCover */
-	int WantDither();
+	int WantDither() const;
 };
 
 class GEM_EXPORT Highlightable : public Scriptable {
@@ -435,7 +432,7 @@ public:
 	void SetTrapDetected(int x);
 	void TryDisarm(Actor *actor);
 	//detect trap, set skill to 256 if you want sure fire
-	void DetectTrap(int skill);
+	void DetectTrap(int skill, ieDword actorID);
 	//returns true if trap is visible, only_detected must be true
 	//if you want to see discovered traps, false is for cheats
 	bool VisibleTrap(int only_detected) const;
@@ -470,23 +467,12 @@ public:
 		return step;
 	}
 
+	unsigned char GetNextFace();
+
 	inline unsigned char GetOrientation() const {
 		return Orientation;
 	}
 
-	inline unsigned char GetNextFace() {
-		//slow turning
-		if (Orientation != NewOrientation) {
-			if ( ( (NewOrientation-Orientation) & (MAX_ORIENT-1) ) <= MAX_ORIENT/2) {
-				Orientation++;
-			} else {
-				Orientation--;
-			}
-			Orientation = Orientation&(MAX_ORIENT-1);
-		}
-
-		return Orientation;
-	}
 	inline unsigned char GetStance() const {
 		return StanceID;
 	}
@@ -536,4 +522,6 @@ public:
 	ieDword closedcount;
 };
 
+}
+
 #endif
