@@ -35,6 +35,8 @@
 #include "Sprite2D.h"
 #include "Video.h"
 
+namespace GemRB {
+
 #define ILLEGAL 0         //
 #define ONE 1             //hold
 #define TWO 2             //onset + hold
@@ -243,7 +245,7 @@ ScriptedAnimation::ScriptedAnimation(DataStream* stream)
 
 	stream->Read( Signature, 8);
 	if (strncmp( Signature, "VVC V1.0", 8 ) != 0) {
-		print( "Not a valid VVC File\n" );
+		print("Not a valid VVC File");
 		delete stream;
 		return;
 	}
@@ -429,7 +431,7 @@ void ScriptedAnimation::SetPhase(int arg)
 void ScriptedAnimation::SetSound(int arg, const ieResRef sound)
 {
 	if (arg>=P_ONSET && arg<=P_RELEASE) {
-		memcpy(sounds[arg],sound,sizeof(sound));
+		memcpy(sounds[arg],sound,sizeof(ieResRef));
 	}
 	//no need to call the twin
 }
@@ -548,7 +550,7 @@ bool ScriptedAnimation::HandlePhase(Sprite2D *&frame)
 
 	if (justCreated) {
 		if (Phase == P_NOTINITED) {
-			printMessage("ScriptedAnimation", "Not fully initialised VVC!\n", LIGHT_RED);
+			Log(ERROR, "ScriptedAnimation", "Not fully initialised VVC!");
 			return true;
 		}
 		unsigned long time;
@@ -630,7 +632,7 @@ retry:
 }
 
 //it is not sure if we need tint at all
-bool ScriptedAnimation::Draw(const Region &screen, const Point &Pos, const Color &p_tint, Map *area, int dither, int orientation)
+bool ScriptedAnimation::Draw(const Region &screen, const Point &Pos, const Color &p_tint, Map *area, int dither, int orientation, int height)
 {
 	if (FaceTarget) {
 		SetOrientation(orientation);
@@ -638,7 +640,7 @@ bool ScriptedAnimation::Draw(const Region &screen, const Point &Pos, const Color
 
 	// not sure
 	if (twin) {
-		twin->Draw(screen, Pos, p_tint, area, dither, -1);
+		twin->Draw(screen, Pos, p_tint, area, dither, -1, height);
 	}
 
 	Video *video = core->GetVideoDriver();
@@ -673,7 +675,7 @@ bool ScriptedAnimation::Draw(const Region &screen, const Point &Pos, const Color
 	}
 
 	if (Transparency & IE_VVC_SEPIA) {
-		flag |= BLIT_RED;
+		flag |= BLIT_SEPIA;
 	}
 
 	if (Transparency & BLIT_TINTED) {
@@ -686,6 +688,7 @@ bool ScriptedAnimation::Draw(const Region &screen, const Point &Pos, const Color
 
 	int cx = Pos.x + XPos;
 	int cy = Pos.y - ZPos + YPos;
+  if (SequenceFlags&IE_VVC_HEIGHT) cy-=height;
 
 	if( SequenceFlags&IE_VVC_NOCOVER) {
 		if (cover) SetSpriteCover(NULL);
@@ -786,3 +789,5 @@ ScriptedAnimation *ScriptedAnimation::DetachTwin()
 	twin=NULL;
 	return ret;
 }
+
+}
