@@ -31,6 +31,45 @@
 #include "ProjectileServer.h"
 #include "Scriptable/Actor.h"
 
+namespace GemRB {
+
+struct SpellFocus {
+	ieDword stat;
+	ieDword val1;
+	ieDword val2;
+};
+
+static int pstflags = false;
+static int inited = false;
+SpellFocus *spellfocus = NULL;
+int schoolcount = 0;
+
+static void InitSpellTables()
+{
+	pstflags = !!core->HasFeature(GF_PST_STATE_FLAGS);
+	AutoTable tm("splfocus", true);
+	if (tm) {
+		schoolcount = tm->GetRowCount();
+
+		spellfocus = new SpellFocus [schoolcount];
+		for(int i = 0; i<schoolcount; i++) {
+			ieDword stat = core->TranslateStat(tm->QueryField(i, 0));
+			ieDword val1 = atoi(tm->QueryField(i, 1));
+			ieDword val2 = atoi(tm->QueryField(i, 2));
+			spellfocus[i].stat = stat;
+			spellfocus[i].val1 = val1;
+			spellfocus[i].val2 = val2;
+		}
+	}
+}
+
+
+void ReleaseMemorySpell()
+{
+	inited = false;
+	delete [] spellfocus;
+}
+
 SPLExtHeader::SPLExtHeader(void)
 {
 	features = NULL;
@@ -45,6 +84,10 @@ Spell::Spell(void)
 {
 	ext_headers = NULL;
 	casting_features = NULL;
+	if (!inited) {
+		inited = true;
+		InitSpellTables();
+	}
 }
 
 Spell::~Spell(void)
@@ -80,23 +123,30 @@ void Spell::AddCastingGlow(EffectQueue *fxqueue, ieDword duration, int gender)
 	char g, t;
 	Effect *fx;
 	ieResRef Resource;
-	
+
 	int cgsound = CastingSound;
 	if (cgsound>=0 && duration > 1) {
 		//bg2 style
-		if(cgsound&0x100) {    
-			switch(gender) {
-			default: g = 'm'; break;
-			case SEX_FEMALE: g = 'f'; break;
-			case SEX_OTHER: case SEX_NEITHER: g = 's'; break;
+		if(cgsound&0x100) {
+			//if duration is less than 3, use only the background sound
+			g = 's';
+			if (duration>3) {
+				switch(gender) {
+				//selection of these sounds is almost purely on whim
+				//though the sounds of devas/demons are probably better this way
+				//all other cases (mostly elementals/animals) don't have sound
+				//externalise if you don't mind another 2da
+				case SEX_MALE: case SEX_SUMMON_DEMON: g = 'm'; break;
+				case SEX_FEMALE: case SEX_BOTH: g = 'f'; break;
+				}
 			}
 		} else {
-			//how style
-			
+			//how style, no pure background sound
+
 			switch(gender) {
 			default: g = 'm'; break;
 			case SEX_FEMALE: g = 'f'; break;
-			}         
+			}
 		}
 		if (SpellType==IE_SPL_PRIEST) {
 			t = 'p';
@@ -132,7 +182,7 @@ EffectQueue *Spell::GetEffectBlock(Scriptable *self, const Point &pos, int block
 		} else {
 			features = ext_headers[block_index].features;
 			count = ext_headers[block_index].FeatureCount;
-			if (!(ext_headers[block_index].Hostile&4)) {
+			if (pstflags && !(ext_headers[block_index].Hostile&4)) {
 				pst_hostile = true;
 			}
 		}
@@ -174,6 +224,19 @@ EffectQueue *Spell::GetEffectBlock(Scriptable *self, const Point &pos, int block
 			} else if (caster->Modified[IE_SPELLDURATIONMODPRIEST] && SpellType == IE_SPL_PRIEST) {
 				fx->Duration = (fx->Duration * caster->Modified[IE_SPELLDURATIONMODPRIEST]) / 100;
 			}
+
+			//evaluate spell focus feats
+			//TODO: the usual problem: which saving throw is better? Easy fix in the data file.
+			if (fx->PrimaryType<(ieDword) schoolcount) {
+				ieDword stat = spellfocus[fx->PrimaryType].stat;
+				if (stat>0) {
+					switch (caster->Modified[stat]) {
+						case 0: break;
+						case 1: fx->SavingThrowBonus += spellfocus[fx->PrimaryType].val1; break;
+						default: fx->SavingThrowBonus += spellfocus[fx->PrimaryType].val2; break;
+					}
+				}
+			}
 		}
 
 		if (fx->Target != FX_TARGET_SELF) {
@@ -204,7 +267,7 @@ Projectile *Spell::GetProjectile(Scriptable *self, int header, int level, const
 {
 	SPLExtHeader *seh = GetExtHeader(header);
 	if (!seh) {
-		printMessage("Spell", "Cannot retrieve spell header!!! required header: %d, maximum: %d\n", RED,
+		Log(ERROR, "Spell", "Cannot retrieve spell header!!! required header: %d, maximum: %d",
 			header, (int) ExtHeaderCount);
 		return NULL;
 	}
@@ -233,7 +296,7 @@ unsigned int Spell::GetCastingDistance(Scriptable *Sender) const
 	int idx = GetHeaderIndexFromLevel(level);
 	SPLExtHeader *seh = GetExtHeader(idx);
 	if (!seh) {
-		printMessage("Spell", "Cannot retrieve spell header!!! required header: %d, maximum: %d\n", RED,
+		Log(ERROR, "Spell", "Cannot retrieve spell header!!! required header: %d, maximum: %d",
 			idx, (int) ExtHeaderCount);
 		return 0;
 	}
@@ -243,3 +306,24 @@ unsigned int Spell::GetCastingDistance(Scriptable *Sender) const
 	}
 	return (unsigned int) seh->Range;
 }
+
+static EffectRef fx_damage_ref = { "Damage", -1 };
+// checks if any of the extended headers contains fx_damage
+bool Spell::ContainsDamageOpcode() const
+{
+	ieDword damage_opcode = EffectQueue::ResolveEffect(fx_damage_ref);
+	for (int h=0; h< ExtHeaderCount; h++) {
+		for (int i=0; i< ext_headers[h].FeatureCount; i++) {
+			Effect *fx = ext_headers[h].features+i;
+			if (fx->Opcode == damage_opcode) {
+				return true;
+			}
+		}
+		if (Flags & SF_SIMPLIFIED_DURATION) { // iwd2 has only one header
+			break;
+		}
+	}
+	return false;
+}
+
+}
