@@ -33,6 +33,8 @@
 
 #include "EffectQueue.h"
 
+namespace GemRB {
+
 class Projectile;
 
 //values for Spell usability Flags
@@ -60,6 +62,9 @@ class Projectile;
 #define SPEC_IDENTIFY  1  //spells that don't appear in the casting bar
 #define SPEC_SILENCE   2  //spells that can be cast when silenced
 #define SPEC_DEAD      4  //spells that can target dead actors despite their target type is 1 (pst hack)
+
+extern void ReleaseMemorySpell();
+
 /**
  * @class SPLExtHeader
  * Header for Spell special effects
@@ -169,6 +174,9 @@ public:
 	//returns a projectile created from an extended header
 	Projectile *GetProjectile(Scriptable *self, int headerindex, int level, const Point &pos) const;
 	unsigned int GetCastingDistance(Scriptable *Sender) const;
+	bool ContainsDamageOpcode() const;
 };
 
+}
+
 #endif  // ! SPELL_H
