@@ -31,6 +31,8 @@
 #include "Item.h"
 #include "GameScript/GameScript.h"
 
+namespace GemRB {
+
 Store::Store(void)
 {
 	HasTriggers = false;
@@ -102,6 +104,13 @@ int Store::AcceptableItemType(ieDword type, ieDword invflags, bool pc) const
 	if (!(invflags&IE_INV_ITEM_IDENTIFIED) ) {
 		ret |= IE_STORE_ID;
 	}
+
+	// can't buy/sell if store doesn't allow it at all
+	if (!(Flags&IE_STORE_SELL))
+		ret &= ~IE_STORE_SELL;
+	if (!(Flags&IE_STORE_BUY))
+		ret &= ~IE_STORE_BUY;
+
 	if (pc && (Type<STT_BG2CONT) ) {
 		//can't sell critical items
 		if (!(invflags&IE_INV_ITEM_DESTRUCTIBLE )) {
@@ -114,7 +123,7 @@ int Store::AcceptableItemType(ieDword type, ieDword invflags, bool pc) const
 		}
 
 		//check if store buys stolen items
-		if ((invflags&IE_INV_ITEM_STOLEN) && !(Type&IE_STORE_FENCE) ) {
+		if ((invflags&IE_INV_ITEM_STOLEN) && !(Flags&IE_STORE_FENCE) ) {
 			ret &= ~IE_STORE_SELL;
 		}
 	}
@@ -150,9 +159,9 @@ STODrink *Store::GetDrink(unsigned int idx) const
 }
 
 //We need this weirdness for PST item lookup
-STOItem *Store::GetItem(unsigned int idx)
+STOItem *Store::GetItem(unsigned int idx, bool usetrigger)
 {
-	if (!HasTriggers) {
+	if (!HasTriggers || !usetrigger) {
 		if (idx>=items.size()) {
 			return NULL;
 		}
@@ -212,19 +221,21 @@ STOItem *Store::FindItem(CREItem *item, bool exact)
 	return NULL;
 }
 
-//some stores can recharge items
+//some stores can recharge items - in original engine apparently all stores
+//did this. In gemrb there is a flag.
 void Store::RechargeItem(CREItem *item)
 {
-//is there any flag which store can recharge?
 	Item *itm = gamedata->GetItem(item->ItemResRef);
 	if (!itm) {
 		return;
 	}
-	if (!itm->LoreToID) {
-		item->Flags |= IE_INV_ITEM_IDENTIFIED;
-	}
 	//gemrb extension, some shops won't recharge items
-	if (!(Flags&IE_STORE_RECHARGE)) {
+	//containers' behaviour is inverted
+	//bag      0   1   0   1
+	//flag     0   0   1   1
+	//recharge 1   0   0   1
+	bool bag = (Type==STT_BG2CONT || Type==STT_IWD2CONT);
+	if (bag != !(Flags&IE_STORE_RECHARGE)) {
 		for (int i=0;i<CHARGE_COUNTERS;i++) {
 			ITMExtHeader *h = itm->GetExtHeader(i);
 			if (!h) {
@@ -239,8 +250,25 @@ void Store::RechargeItem(CREItem *item)
 	gamedata->FreeItem(itm, item->ItemResRef, 0);
 }
 
+void Store::IdentifyItem(CREItem *item) const
+{
+	if (item->Flags & IE_INV_ITEM_IDENTIFIED) {
+		return;
+	}
+
+	Item *itm = gamedata->GetItem(item->ItemResRef);
+	if (!itm) {
+		return;
+	}
+
+	if (itm->LoreToID <= Lore) {
+		item->Flags |= IE_INV_ITEM_IDENTIFIED;
+	}
+}
+
 void Store::AddItem(CREItem *item)
 {
+	IdentifyItem(item);
 	RechargeItem(item);
 	STOItem *temp = FindItem(item, true);
 
@@ -249,6 +277,7 @@ void Store::AddItem(CREItem *item)
 			if (item->MaxStackAmount) {
 				// Stacked, so increase usages.
 				ieDword newAmount = 1;
+				if (!temp->Usages[0]) temp->Usages[0] = 1;
 				if (item->Usages[0] != temp->Usages[0] && item->Usages[0] > 0) {
 					// Stack sizes differ!
 					// For now, we do what bg2 does and just round up.
@@ -292,6 +321,18 @@ void Store::RemoveItem( unsigned int idx )
 	ItemsCount--;
 }
 
+void Store::RemoveItem( STOItem *itm )
+{
+	size_t i = items.size();
+	while(i--) {
+		if (items[i]==itm) {
+			items.erase(items.begin()+i);
+			ItemsCount--;
+			break;
+		}
+	}
+}
+
 ieDword Store::GetOwnerID() const
 {
 	return StoreOwnerID;
@@ -301,3 +342,5 @@ void Store::SetOwnerID(ieDword owner)
 {
 	StoreOwnerID = owner;
 }
+
+}
