@@ -34,6 +34,8 @@
 
 #include <vector>
 
+namespace GemRB {
+
 //bah!
 class CREItem;
 class Trigger;
@@ -54,7 +56,7 @@ STA_CURE=3, STA_DONATE=4, STA_DRINK=5, STA_ROOMRENT=6, STA_OPTIONAL=0x80} StoreA
 #define IE_STORE_SELECT   0x40   //valid when these flags used as store action
 #define IE_STORE_RENT     128    //gemrb extension
 #define IE_STORE_QUALITY  0x600  //2 bits
-#define IE_STORE_FENCE    0x2000 //
+#define IE_STORE_FENCE    0x1000 //
 #define IE_STORE_RECHARGE 0x4000 //gemrb extension, if set, store won't recharge
 
 
@@ -69,7 +71,7 @@ struct STOItem {
 	ieDword Flags;
 	// 2 cached values from associated item. LEAVE IT SIGNED!
 	int Weight;
-	int StackAmount;
+	int MaxStackAmount;
 	ieDword AmountInStock;
 	ieDwordSigned InfiniteSupply;
 	// V1.1
@@ -156,7 +158,7 @@ public: //queries
 	int AcceptableItemType(ieDword type, ieDword invflags, bool pc) const;
 	STOCure *GetCure(unsigned int idx) const;
 	STODrink *GetDrink(unsigned int idx) const;
-	STOItem *GetItem(unsigned int idx);
+	STOItem *GetItem(unsigned int idx, bool usetrigger);
 	/** Evaluates item availability triggers */
 	int GetRealStockSize();
 	/** Recharges item */
@@ -164,15 +166,20 @@ public: //queries
 	/** Adds a new item to the store (selling) */
 	void AddItem(CREItem* item);
 	void RemoveItem(unsigned int idx);
+	void RemoveItem( STOItem *itm);
 	/** Returns index of item */
 	unsigned int FindItem(const ieResRef item, bool usetrigger) const;
 	const char *GetOwner() const;
 	ieDword GetOwnerID() const;
 	void SetOwnerID(ieDword owner);
 private:
+	/** Identifies item according to store lore */
+	void IdentifyItem(CREItem *item) const;
 	/** Finds a mergeable item in the stock, if exact is set, it checks for usage counts too */
 	STOItem *FindItem(CREItem *item, bool exact);
 	bool IsItemAvailable(unsigned int slot) const;
 };
 
+}
+
 #endif // ! STORE_H
