@@ -34,7 +34,7 @@
 
 #include "ie_types.h"
 
-#define VERSION_GEMRB "0.6.6"
+#define VERSION_GEMRB "0.8.0"
 
 #define GEMRB_STRING "GemRB v" VERSION_GEMRB
 
@@ -58,9 +58,14 @@
 
 #include <cstdio>
 #include <cstdlib>
+#ifndef WIN32
+# include <sys/time.h>
+#endif
 
 #endif //GLOBALS_ONLY_DEFS
 
+namespace GemRB {
+
 //Global Variables
 
 #define IE_NORMAL 0
@@ -144,8 +149,22 @@
 #define  GF_FORCE_AREA_SCRIPT           59 //how and iwd2 (maybe iwd1)
 #define  GF_AREA_OVERRIDE               60 //pst maze and other hardcode
 #define  GF_NO_NEW_VARIABLES            61 //pst
+#define  GF_SOUNDS_INI                  62 //iwd/how/iwd2
+#define  GF_USEPOINT_400                63 //all except pst and iwd2
+#define  GF_USEPOINT_200                64 //iwd2
+#define  GF_HAS_FLOAT_MENU              65 //pst
+#define  GF_RARE_ACTION_VB              66 //pst
+#define  GF_NO_UNDROPPABLE              67 //iwd,how
+#define  GF_START_ACTIVE                68 //bg1
+#define  GF_CLEAR_UNUSED_AREA           69 //bg1
+#define  GF_IMPLICIT_AREAANIM_BACKGROUND    70 //idw,how,iwd2
+#define  GF_HEAL_ON_100PLUS             71 //bg1, bg2, pst
+#define  GF_IN_PARTY_ALLOWS_DEAD	72 //all except bg2
+#define  GF_ZERO_TIMER_IS_VALID         73 // how, not bg2, other unknown
+#define  GF_SKIPUPDATE_HACK             74 // how, not bg2
+
 //update this or bad things can happen
-#define GF_COUNT 62
+#define GF_COUNT 75
 
 //the number of item usage fields (used in CREItem and STOItem)
 #define CHARGE_COUNTERS  3
@@ -173,9 +192,10 @@ GEM_EXPORT unsigned int PersonalDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT unsigned int SquaredPersonalDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT unsigned int SquaredMapDistance(Scriptable *a, Scriptable *b);
 GEM_EXPORT int EARelation(Scriptable *a, Actor *b);
+GEM_EXPORT void CopyResRef(ieResRef d, const ieResRef s);
+
 
 #ifndef WIN32
-#include <sys/time.h>
 inline unsigned long GetTickCount()
 {
 	struct timeval tv;
@@ -184,6 +204,8 @@ inline unsigned long GetTickCount()
 }
 #endif
 
+#ifndef __APPLE__
+// MIN and MAX are already similarly defined by Cocoa
 inline int MIN(int a, int b)
 {
 	return (a > b ? b : a);
@@ -193,6 +215,7 @@ inline int MAX(int a, int b)
 {
 	return (a < b ? b : a);
 }
+#endif
 
 inline bool valid_number(const char* string, long& val)
 {
@@ -209,5 +232,8 @@ inline bool valid_number(const char* string, long& val)
 //the maximum supported game CD count
 #define MAX_CD               6
 
+}
+
 #endif //! GLOBALS_H
 
+
