@@ -25,35 +25,27 @@
 #include "Interface.h"
 #include "System/FileStream.h"
 
+using namespace GemRB;
+
 #define MAXLENGTH 4096      //if a 2da has longer lines, change this
 #define SIGNLENGTH 256      //if a 2da has longer default value, change this
 
 p2DAImporter::p2DAImporter(void)
 {
-	str = NULL;
-	autoFree = false;
 }
 
 p2DAImporter::~p2DAImporter(void)
 {
-	if (str && autoFree) {
-		delete( str );
-	}
 	for (unsigned int i = 0; i < ptrs.size(); i++) {
 		free( ptrs[i] );
 	}
 }
 
-bool p2DAImporter::Open(DataStream* stream, bool autoFree)
+bool p2DAImporter::Open(DataStream* str)
 {
-	if (stream == NULL) {
+	if (str == NULL) {
 		return false;
 	}
-	if (str && this->autoFree) {
-		delete( str );
-	}
-	str = stream;
-	this->autoFree = autoFree;
 	char Signature[SIGNLENGTH];
 	str->CheckEncrypted();
 
@@ -62,9 +54,10 @@ bool p2DAImporter::Open(DataStream* stream, bool autoFree)
 	while (*strp == ' ')
 		strp++;
 	if (strncmp( strp, "2DA V1.0", 8 ) != 0) {
-		printMessage("2DAImporter", "Bad signature!\n",YELLOW );
-// we don't care about this, so exptable.2da of iwd2 won't cause a bigger problem
-//		return false;
+		Log(WARNING, "2DAImporter", "Bad signature (%s)! Ignoring...", str->filename );
+		// we don't care about this, so exptable.2da of iwd2 won't cause a bigger problem
+		// also, certain creatures are described by 2da's without signature.
+		// return false;
 	}
 	Signature[0] = 0;
 	str->ReadLine( Signature, sizeof(Signature) );
@@ -110,6 +103,7 @@ bool p2DAImporter::Open(DataStream* stream, bool autoFree)
 			row++;
 		}
 	}
+	delete str;
 	return true;
 }
 
