@@ -22,6 +22,7 @@
 
 #include "win32def.h"
 #include "strrefs.h"
+#include "ie_cursors.h"
 
 #include "ActorMgr.h"
 #include "Ambient.h"
@@ -38,13 +39,14 @@
 #include "TileMapMgr.h"
 #include "Video.h"
 #include "GameScript/GameScript.h"
-#include "GUI/Window.h"
 #include "Scriptable/Container.h"
 #include "Scriptable/Door.h"
 #include "Scriptable/InfoPoint.h"
 #include "System/FileStream.h"
 #include "System/SlicedStream.h"
 
+using namespace GemRB;
+
 #define DEF_OPEN   0
 #define DEF_CLOSE  1
 #define DEF_HOPEN  2
@@ -52,8 +54,6 @@
 
 #define DEF_COUNT 4
 
-#define DOOR_HIDDEN 128
-
 //something non signed, non ascii
 #define UNINITIALIZED_BYTE  0x11
 
@@ -95,7 +95,7 @@ static int GetTrackString(const ieResRef areaName)
 	bool trackflag = displaymsg->HasStringReference(STR_TRACKING);
 
 	if (!tracks) {
-		AutoTable tm("tracking");
+		AutoTable tm("tracking", true);
 		if (!tm.ok())
 			return -1;
 		trackcount = tm->GetRowCount();
@@ -248,13 +248,18 @@ bool AREImporter::ChangeMap(Map *map, bool day_or_night)
 	}
 	tm = tmm->GetTileMap(tm);
 	if (!tm) {
-		print( "[AREImporter]: No Tile Map Available.\n" );
+		print("[AREImporter]: No Tile Map Available.");
 		return false;
 	}
 
 	// Small map for MapControl
 	ResourceHolder<ImageMgr> sm(TmpResRef);
-	// small map is *optional*!
+
+	// night small map is *optional*!
+	if (!sm) {
+		//fall back to day minimap
+		sm = ResourceHolder<ImageMgr> (map->WEDResRef);
+	}
 
 	//the map state was altered, no need to hold this off for any later
 	map->DayNight = day_or_night;
@@ -268,7 +273,7 @@ bool AREImporter::ChangeMap(Map *map, bool day_or_night)
 
 	ResourceHolder<ImageMgr> lm(TmpResRef);
 	if (!lm) {
-		print( "[AREImporter]: No lightmap available.\n" );
+		print("[AREImporter]: No lightmap available.");
 		return false;
 	}
 
@@ -277,6 +282,34 @@ bool AREImporter::ChangeMap(Map *map, bool day_or_night)
 	return true;
 }
 
+// everything is the same up to DOOR_FOUND, but then it gets messy (see Door.h)
+static const ieDword gemrbDoorFlags[6] = { DOOR_TRANSPARENT, DOOR_KEY, DOOR_SLIDE, DOOR_USEUPKEY, DOOR_LOCKEDINFOTEXT, DOOR_WARNINGINFOTEXT };
+// the last two are 0, since they are outside the original bit range, so all the constants can coexist
+static const ieDword iwd2DoorFlags[6] = { DOOR_LOCKEDINFOTEXT, DOOR_TRANSPARENT, DOOR_WARNINGINFOTEXT, DOOR_KEY, 0, 0 };
+inline ieDword FixIWD2DoorFlags(ieDword Flags, bool reverse)
+{
+	if (!core->HasFeature(GF_IWD2_SCRIPTNAME)) {
+		return Flags;
+	}
+
+	ieDword bit, otherbit, maskOff = 0, maskOn= 0;
+	for (int i=0; i < 6; i++) {
+		if (!reverse) {
+			bit = gemrbDoorFlags[i];
+			otherbit = iwd2DoorFlags[i];
+		} else {
+			bit = iwd2DoorFlags[i];
+			otherbit = gemrbDoorFlags[i];
+		}
+		if (Flags & bit) {
+			maskOff |= bit;
+			maskOn |= otherbit;
+		}
+	}
+	// delayed bad bit removal due to chain overlapping
+	return Flags = (Flags & ~maskOff) | maskOn;
+}
+
 Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 {
 	unsigned int i,x;
@@ -315,7 +348,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	}
 
 	if (!core->IsAvailable( IE_WED_CLASS_ID )) {
-		print( "[AREImporter]: No Tile Map Manager Available.\n" );
+		print("[AREImporter]: No Tile Map Manager Available.");
 		return NULL;
 	}
 	ieResRef TmpResRef;
@@ -333,13 +366,16 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	//there was no tilemap set yet, so lets just send a NULL
 	TileMap* tm = tmm->GetTileMap(NULL);
 	if (!tm) {
-		print( "[AREImporter]: No Tile Map Available.\n" );
+		print("[AREImporter]: No Tile Map Available.");
 		return NULL;
 	}
 
 	// Small map for MapControl
-	// small map is *optional*!
 	ResourceHolder<ImageMgr> sm(TmpResRef);
+	if (!sm) {
+		//fall back to day minimap
+		sm = ResourceHolder<ImageMgr> (map->WEDResRef);
+	}
 
 	//if the Script field is empty, the area name will be copied into it on first load
 	//this works only in the iwd branch of the games
@@ -362,7 +398,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 
 	ResourceHolder<ImageMgr> lm(TmpResRef);
 	if (!lm) {
-		print( "[AREImporter]: No lightmap available.\n" );
+		print("[AREImporter]: No lightmap available.");
 		return NULL;
 	}
 
@@ -370,7 +406,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 
 	ResourceHolder<ImageMgr> sr(TmpResRef);
 	if (!sr) {
-		print( "[AREImporter]: No searchmap available.\n" );
+		print("[AREImporter]: No searchmap available.");
 		return NULL;
 	}
 
@@ -378,7 +414,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 
 	ResourceHolder<ImageMgr> hm(TmpResRef);
 	if (!hm) {
-		print( "[AREImporter]: No heightmap available.\n" );
+		print("[AREImporter]: No heightmap available.");
 		return NULL;
 	}
 
@@ -409,7 +445,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	str->ReadWord( &map->RestHeader.DayChance );
 	str->ReadWord( &map->RestHeader.NightChance );
 
-	print( "Loading regions\n" );
+	print("Loading regions");
 	core->LoadProgress(70);
 	//Loading InfoPoints
 	for (i = 0; i < InfoPointsCount; i++) {
@@ -440,7 +476,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadWord( &VertexCount );
 		str->ReadDword( &FirstVertex );
 		ieDword tmp2;
-		str->ReadDword( &tmp2 );
+		str->ReadDword( &tmp2 ); //named triggerValue in the IE source
 		str->ReadDword( &Cursor );
 		str->ReadResRef( Destination );
 		str->Read( Entrance, 32 );
@@ -460,11 +496,21 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadWord( &PosY);
 		//maybe we have to store this
 		str->Seek( 36, GEM_CURRENT_POS );
-		str->ReadResRef( WavResRef );
-		str->ReadWord( &TalkX);
-		str->ReadWord( &TalkY);
-		str->ReadDword( &DialogName );
-		str->ReadResRef( DialogResRef );
+
+		if (core->HasFeature(GF_CLEAR_UNUSED_AREA)) {
+			memset( WavResRef, 0, sizeof(WavResRef));
+			TalkX = 0;
+			TalkY = 0;
+			DialogName = -1;
+			memset( DialogResRef, 0, sizeof(DialogResRef));
+		}	else {
+			str->ReadResRef( WavResRef );
+			str->ReadWord( &TalkX);
+			str->ReadWord( &TalkY);
+			str->ReadDword( &DialogName );
+			str->ReadResRef( DialogResRef );
+		}
+
 		char* string = core->GetString( StrRef );
 		str->Seek( VerticesOffset + ( FirstVertex * 4 ), GEM_STREAM_START );
 		Point* points = ( Point* ) malloc( VertexCount*sizeof( Point ) );
@@ -493,7 +539,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		ip->UsePoint.x = PosX;
 		ip->UsePoint.y = PosY;
 		//FIXME: PST doesn't use this field
-		if (ip->Flags&TRAP_USEPOINT) {
+		if (ip->GetUsePoint()) {
 			ip->Pos = ip->UsePoint;
 		} else {
 			ip->Pos.x = bbox.x + ( bbox.w / 2 );
@@ -517,7 +563,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		}
 	}
 
-	print( "Loading containers\n" );
+	print("Loading containers");
 	//Loading Containers
 	for (i = 0; i < ContainersCount; i++) {
 		str->Seek( ContainersOffset + ( i * 0xC0 ), GEM_STREAM_START );
@@ -563,11 +609,11 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		//the vertex count is only 16 bits, there is a weird flag
 		//after it, which is usually 0, but sometimes set to 1
 		str->ReadWord( &vertCount );
-		str->ReadWord( &unknown );
-		//str->Read( Name, 32 );
+		str->ReadWord( &unknown );   //trigger range
+		//str->Read( Name, 32 );     //owner's scriptname
 		str->Seek( 32, GEM_CURRENT_POS);
 		str->ReadResRef( KeyResRef);
-		str->Seek( 4, GEM_CURRENT_POS);
+		str->Seek( 4, GEM_CURRENT_POS); //break difficulty
 		str->ReadDword( &OpenFail );
 
 		str->Seek( VerticesOffset + ( firstIndex * 4 ), GEM_STREAM_START );
@@ -624,7 +670,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		c->OpenFail = OpenFail;
 	}
 
-	print( "Loading doors\n" );
+	print("Loading doors");
 	//Loading Doors
 	for (i = 0; i < DoorsCount; i++) {
 		str->Seek( DoorsOffset + ( i * 0xc8 ), GEM_STREAM_START );
@@ -647,12 +693,13 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		ieStrRef OpenStrRef;
 		ieStrRef NameStrRef;
 		ieResRef Dialog;
-		ieDword Unknown54;
+		ieWord hp, ac;
 
 		str->Read( LongName, 32 );
 		LongName[32] = 0;
 		str->ReadResRef( ShortName );
 		str->ReadDword( &Flags );
+		Flags = FixIWD2DoorFlags(Flags, false);
 		str->ReadDword( &OpenFirstVertex );
 		str->ReadWord( &OpenVerticesCount );
 		str->ReadWord( &ClosedVerticesCount );
@@ -677,7 +724,8 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadWord( &OpenImpededCount );
 		str->ReadWord( &ClosedImpededCount );
 		str->ReadDword( &ClosedFirstImpeded );
-		str->ReadDword( &Unknown54);
+		str->ReadWord( &hp); //TODO: store these
+		str->ReadWord( &ac); //hitpoints AND armorclass, according to IE dev info
 		ieResRef OpenResRef, CloseResRef;
 		str->ReadResRef( OpenResRef );
 		str->ReadResRef( CloseResRef );
@@ -782,7 +830,8 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		door->cibcount = ClosedImpededCount;
 		door->SetMap(map);
 
-		door->Unknown54 = Unknown54;
+		door->hp = hp;
+		door->ac = ac;
 		door->TrapDetectionDiff = TrapDetect;
 		door->TrapRemovalDiff = TrapRemoval;
 		door->Trapped = Trapped;
@@ -804,7 +853,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		if (OpenResRef[0])
 			memcpy( door->OpenSound, OpenResRef, sizeof(OpenResRef) );
 		else {
-			if (Flags & DOOR_HIDDEN)
+			if (Flags & DOOR_SECRET)
 				memcpy( door->OpenSound, Sounds[DEF_HOPEN], 9 );
 			else
 				memcpy( door->OpenSound, Sounds[DEF_OPEN], 9 );
@@ -812,7 +861,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		if (CloseResRef[0])
 			memcpy( door->CloseSound, CloseResRef, sizeof(CloseResRef) );
 		else {
-			if (Flags & DOOR_HIDDEN)
+			if (Flags & DOOR_SECRET)
 				memcpy( door->CloseSound, Sounds[DEF_HCLOSE], 9 );
 			else
 				memcpy( door->CloseSound, Sounds[DEF_CLOSE], 9 );
@@ -827,7 +876,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		door->SetDialog(Dialog);
 	}
 
-	print( "Loading spawnpoints\n" );
+	print("Loading spawnpoints");
 	//Loading SpawnPoints
 	for (i = 0; i < SpawnCount; i++) {
 		str->Seek( SpawnOffset + (i*0xc8), GEM_STREAM_START );
@@ -882,11 +931,11 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	}
 
 	core->LoadProgress(75);
-	print( "Loading actors\n" );
+	print("Loading actors");
 	//Loading Actors
 	str->Seek( ActorOffset, GEM_STREAM_START );
 	if (!core->IsAvailable( IE_CRE_CLASS_ID )) {
-		print( "[AREImporter]: No Actor Manager Available, skipping actors\n" );
+		print("[AREImporter]: No Actor Manager Available, skipping actors");
 	} else {
 		PluginHolder<ActorMgr> actmgr(IE_CRE_CLASS_ID);
 		for (i = 0; i < ActorCount; i++) {
@@ -916,9 +965,9 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			memset(Scripts,0,sizeof(Scripts));
 
 			str->ReadResRef( Scripts[SCR_OVERRIDE] );
+			str->ReadResRef( Scripts[SCR_GENERAL] );
 			str->ReadResRef( Scripts[SCR_CLASS] );
 			str->ReadResRef( Scripts[SCR_RACE] );
-			str->ReadResRef( Scripts[SCR_GENERAL] );
 			str->ReadResRef( Scripts[SCR_DEFAULT] );
 			str->ReadResRef( Scripts[SCR_SPECIFICS] );
 			str->ReadResRef( CreResRef );
@@ -942,22 +991,37 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 				crefile = gamedata->GetResource( CreResRef, IE_CRE_CLASS_ID );
 			}
 			if(!actmgr->Open(crefile)) {
-				print("Couldn't read actor: %s!\n", CreResRef);
+				print("Couldn't read actor: %s!", CreResRef);
 				continue;
 			}
 			ab = actmgr->GetActor(0);
 			if(!ab)
 				continue;
-			map->AddActor(ab);
+			map->AddActor(ab, false);
 			ab->Pos.x = XPos;
 			ab->Pos.y = YPos;
-			ab->Destination.x = XDes;
-			ab->Destination.y = YDes;
+			ab->Destination.x = XPos;
+			ab->Destination.y = YPos;
+			ab->HomeLocation.x = XDes;
+			ab->HomeLocation.y = YDes;
 			//copying the scripting name into the actor
 			//if the CreatureAreaFlag was set to 8
 			if ((Flags&AF_NAME_OVERRIDE) || (core->HasFeature(GF_IWD2_SCRIPTNAME)) ) {
 				ab->SetScriptName(DefaultName);
 			}
+			//IWD2 specific hacks
+			if (core->HasFeature(GF_IWD2_SCRIPTNAME)) {
+				//This flag is used for something else in IWD2
+				if (Flags&AF_NAME_OVERRIDE) {
+					ab->BaseStats[IE_EA]=EA_EVILCUTOFF;
+				}
+				if (Flags&AF_SEEN_PARTY) {
+					ab->SetMCFlag(MC_SEENPARTY,BM_OR);
+				}
+				if (Flags&AF_INVULNERABLE) {
+					ab->SetMCFlag(MC_INVULNERABLE,BM_OR);
+				}
+			}
 
 			if (Dialog[0]) {
 				ab->SetDialog(Dialog);
@@ -977,16 +1041,16 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	}
 
 	core->LoadProgress(90);
-	print( "Loading animations\n" );
+	print("Loading animations");
 	//Loading Animations
 	str->Seek( AnimOffset, GEM_STREAM_START );
 	if (!core->IsAvailable( IE_BAM_CLASS_ID )) {
-		print( "[AREImporter]: No Animation Manager Available, skipping animations\n" );
+		print("[AREImporter]: No Animation Manager Available, skipping animations");
 	} else {
 		for (i = 0; i < AnimCount; i++) {
 			AreaAnimation* anim = new AreaAnimation();
 			str->Read(anim->Name, 32);
-			ieWord animX, animY;
+			ieWord animX, animY, startFrameRange;
 			str->ReadWord( &animX );
 			str->ReadWord( &animY );
 			anim->Pos.x=animX;
@@ -998,11 +1062,15 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 			str->ReadDword( &anim->Flags );
 			str->ReadWordSigned( &anim->height );
 			str->ReadWord( &anim->transparency );
-			str->ReadWord( &anim->unknown3c ); //not completely understood, if not 0, sequence is started
+			str->ReadWord( &startFrameRange );
 			str->Read( &anim->startchance,1 );
 			if (anim->startchance<=0) {
 				anim->startchance=100; //percentage of starting a cycle
 			}
+			if (startFrameRange && (anim->Flags&A_ANI_RANDOM_START) ) {
+				anim->frame = rand() % startFrameRange;
+			}
+			anim->startFrameRange = 0; //this will never get resaved (iirc)
 			str->Read( &anim->skipcycle,1 ); //how many cycles are skipped	(100% skippage)
 			str->ReadResRef( anim->PaletteRef );
 			str->ReadDword( &anim->unknown48 );
@@ -1016,7 +1084,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		}
 	}
 
-	print( "Loading entrances\n" );
+	print("Loading entrances");
 	//Loading Entrances
 	str->Seek( EntrancesOffset, GEM_STREAM_START );
 	for (i = 0; i < EntrancesCount; i++) {
@@ -1031,7 +1099,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		map->AddEntrance( Name, XPos, YPos, Face );
 	}
 
-	print( "Loading variables\n" );
+	print("Loading variables");
 	map->locals->LoadInitialValues(ResRef);
 	//Loading Variables
 	str->Seek( VariablesOffset, GEM_STREAM_START );
@@ -1046,7 +1114,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		map->locals->SetAt( Name, Value );
 	}
 
-	print( "Loading ambients\n" );
+	print("Loading ambients");
 	str->Seek( AmbiOffset, GEM_STREAM_START );
 	for (i = 0; i < AmbiCount; i++) {
 		int j;
@@ -1086,14 +1154,9 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		map->AddAmbient(ambi);
 	}
 
-	print( "Loading automap notes\n" );
+	print("Loading automap notes");
 	str->Seek( NoteOffset, GEM_STREAM_START );
 
-	//this feature exists in all blackisle games but not in bioware games
-	if (core->HasFeature(GF_SPAWN_INI)) {
-		map->LoadIniSpawn();
-	}
-
 	Point point;
 	ieDword color;
 	char *text;
@@ -1161,14 +1224,13 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	}
 
 	//this is a ToB feature (saves the unexploded projectiles)
-	print( "Loading traps\n" );
+	print("Loading traps");
 	for (i = 0; i < TrapCount; i++) {
 		ieResRef TrapResRef;
 		ieDword TrapEffOffset;
 		ieWord TrapSize, ProID;
-		ieWord X,Y;
+		ieWord X,Y,Z;
 		ieDword Ticks;
-		ieWord Unknown;
 		ieByte PartyID;
 		ieByte Owner;
 
@@ -1178,15 +1240,15 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		str->ReadDword( &TrapEffOffset );
 		str->ReadWord( &TrapSize );
 		str->ReadWord( &ProID );
-		str->ReadDword( &Ticks );
+		str->ReadDword( &Ticks );  //actually, delaycount/repetitioncount
 		str->ReadWord( &X );
 		str->ReadWord( &Y );
-		str->ReadWord( &Unknown );
-		str->Read( &PartyID,1 );
+		str->ReadWord( &Z );
+		str->Read( &PartyID,1 );  //according to dev info, this is 'targettype'
 		str->Read( &Owner,1 );
 		int TrapEffectCount = TrapSize/0x108;
 		if(TrapEffectCount*0x108!=TrapSize) {
-			printMessage("AREImporter", "TrapEffectSize in game: %d != %d. Clearing it\n", LIGHT_RED,
+			Log(ERROR, "AREImporter", "TrapEffectSize in game: %d != %d. Clearing it",
 				TrapSize, TrapEffectCount*0x108);
 				continue;
 		}
@@ -1211,30 +1273,43 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 		map->AddProjectile( pro, pos, pos);
 	}
 
-	print( "Loading tiles\n" );
+	print("Loading tiles");
 	//Loading Tiled objects (if any)
 	str->Seek( TileOffset, GEM_STREAM_START );
 	for (i = 0; i < TileCount; i++) {
 		ieVariable Name;
 		ieResRef ID;
 		ieDword Flags;
-		ieDword OpenIndex, OpenCount;
-		ieDword ClosedIndex, ClosedCount;
+		//these fields could be different size
+		//ieDword ClosedCount, OpenCount;
+		ieWord ClosedCount, OpenCount;
+		ieDword ClosedIndex, OpenIndex;
 		str->Read( Name, 32 );
 		Name[32] = 0;
 		str->ReadResRef( ID );
 		str->ReadDword( &Flags );
+		//TODO check this structure again
+/*
 		str->ReadDword( &OpenIndex );
 		str->ReadDword( &OpenCount );
 		str->ReadDword( &ClosedIndex );
 		str->ReadDword( &ClosedCount );
+*/
+		//IE dev info says this:
+		str->ReadDword( &OpenIndex );
+		str->ReadWord( &OpenCount );
+		str->ReadWord( &ClosedCount );
+		str->ReadDword( &ClosedIndex );
+		//end of disputed section
+
+
 		str->Seek( 48, GEM_CURRENT_POS );
 		//absolutely no idea where these 'tile indices' are stored
 		//are they tileset tiles or impeded block tiles
 		map->TMap->AddTile( ID, Name, Flags, NULL,0, NULL, 0 );
 	}
 
-	print( "Loading explored bitmap\n" );
+	print("Loading explored bitmap");
 	i = map->GetExploredMapSize();
 	if (ExploredBitmapSize==i) {
 		map->ExploredBitmap = (ieByte *) malloc(i);
@@ -1243,7 +1318,7 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	}
 	else {
 		if( ExploredBitmapSize ) {
-			printMessage("AREImporter", "ExploredBitmapSize in game: %d != %d. Clearing it\n", LIGHT_RED,
+			Log(ERROR, "AREImporter", "ExploredBitmapSize in game: %d != %d. Clearing it",
 				ExploredBitmapSize, i);
 		}
 		ExploredBitmapSize = i;
@@ -1251,13 +1326,14 @@ Map* AREImporter::GetMap(const char *ResRef, bool day_or_night)
 	}
 	map->VisibleBitmap = (ieByte *) calloc(i, 1);
 
-	print( "Loading wallgroups\n");
+	print("Loading wallgroups");
 	map->SetWallGroups( tmm->GetPolygonsCount(),tmm->GetWallGroups() );
 	//setting up doors
 	for (i=0;i<DoorsCount;i++) {
 		Door *door = tm->GetDoor(i);
 		door->SetDoorOpen(door->IsOpen(), false, 0);
 	}
+
 	return map;
 }
 
@@ -1492,6 +1568,7 @@ int AREImporter::PutDoors( DataStream *stream, Map *map, ieDword &VertIndex)
 
 		stream->Write( d->GetScriptName(), 32);
 		stream->WriteResRef( d->ID);
+		d->Flags = FixIWD2DoorFlags(d->Flags, true);
 		stream->WriteDword( &d->Flags);
 		stream->WriteDword( &VertIndex);
 		tmpWord = (ieWord) d->open->count;
@@ -1528,7 +1605,8 @@ int AREImporter::PutDoors( DataStream *stream, Map *map, ieDword &VertIndex)
 		stream->WriteWord( &tmpWord);
 		stream->WriteDword( &VertIndex);
 		VertIndex += tmpWord;
-		stream->WriteDword( &d->Unknown54);
+		stream->WriteWord( &d->hp);
+		stream->WriteWord( &d->ac);
 		stream->WriteResRef( d->OpenSound);
 		stream->WriteResRef( d->CloseSound);
 		stream->WriteDword( &d->Cursor);
@@ -1834,9 +1912,9 @@ int AREImporter::PutActors( DataStream *stream, Map *map)
 		stream->WriteWord( &tmpWord);
 		tmpWord = (ieWord) ac->Pos.y;
 		stream->WriteWord( &tmpWord);
-		tmpWord = (ieWord) ac->Destination.x;
+		tmpWord = (ieWord) ac->HomeLocation.x;
 		stream->WriteWord( &tmpWord);
-		tmpWord = (ieWord) ac->Destination.y;
+		tmpWord = (ieWord) ac->HomeLocation.y;
 		stream->WriteWord( &tmpWord);
 
 		stream->WriteDword( &tmpDword); //used fields flag always 0 for saved areas
@@ -1852,9 +1930,9 @@ int AREImporter::PutActors( DataStream *stream, Map *map)
 		stream->WriteDword( &ac->TalkCount);
 		stream->WriteResRef( ac->GetDialog());
 		PutScript(stream, ac, SCR_OVERRIDE);
+		PutScript(stream, ac, SCR_GENERAL);
 		PutScript(stream, ac, SCR_CLASS);
 		PutScript(stream, ac, SCR_RACE);
-		PutScript(stream, ac, SCR_GENERAL);
 		PutScript(stream, ac, SCR_DEFAULT);
 		PutScript(stream, ac, SCR_SPECIFICS);
 		//creature reference is empty because we are embedding it
@@ -1900,7 +1978,7 @@ int AREImporter::PutAnimations( DataStream *stream, Map *map)
 		stream->WriteDword( &an->Flags);
 		stream->WriteWord( (ieWord *) &an->height);
 		stream->WriteWord( &an->transparency);
-		stream->WriteWord( &an->unknown3c); //animation already played?
+		stream->WriteWord( &an->startFrameRange); //used by A_ANI_RANDOM_START
 		stream->Write( &an->startchance,1);
 		stream->Write( &an->skipcycle,1);
 		stream->WriteResRef( an->PaletteRef);
@@ -2003,7 +2081,7 @@ int AREImporter::PutMapnotes( DataStream *stream, Map *map)
 		int x;
 
 		if (pst) {
-			tmpDword = (ieWord) mn->Pos.x;
+			tmpDword = (ieDword) mn->Pos.x;
 			stream->WriteDword( &tmpDword );
 			tmpDword = (ieDword) mn->Pos.y;
 			stream->WriteDword( &tmpDword );
@@ -2018,8 +2096,8 @@ int AREImporter::PutMapnotes( DataStream *stream, Map *map)
 			if (x) {
 				stream->Write( filling, x);
 			}
-			stream->WriteWord( &mn->color);
-			stream->WriteWord( &tmpWord);
+			tmpDword = (ieDword) mn->color;
+			stream->WriteDword(&tmpDword);
 			for (x=0;x<5;x++) { //5 empty dwords
 				stream->Write( filling, 4);
 			}
