@@ -24,6 +24,9 @@
 #include "MapMgr.h"
 
 #include "Map.h" // proIterator
+
+namespace GemRB {
+
 class Animation;
 class AnimationFactory;
 class EffectQueue;
@@ -90,4 +93,6 @@ private:
 	int PutSongHeader(DataStream *stream, Map *map);
 };
 
+}
+
 #endif
