@@ -34,35 +34,32 @@
 #include "GUI/Slider.h"
 #include "GUI/TextArea.h"
 #include "GUI/TextEdit.h"
+#include "GUI/Window.h"
+
+using namespace GemRB;
 
 CHUImporter::CHUImporter()
 {
 	str = NULL;
-	autoFree = false;
 }
 
 CHUImporter::~CHUImporter()
 {
-	if (autoFree) {
-		delete str;
-	}
+	delete str;
 }
 
 /** This function loads all available windows from the 'stream' parameter. */
-bool CHUImporter::Open(DataStream* stream, bool autoFree)
+bool CHUImporter::Open(DataStream* stream)
 {
 	if (stream == NULL) {
 		return false;
 	}
-	if (this->autoFree) {
-		delete str;
-	}
+	delete str;
 	str = stream;
-	this->autoFree = autoFree;
 	char Signature[8];
 	str->Read( Signature, 8 );
 	if (strncmp( Signature, "CHUIV1  ", 8 ) != 0) {
-		printMessage( "CHUImporter","Not a Valid CHU File\n",LIGHT_RED );
+		Log(ERROR, "CHUImporter", "Not a Valid CHU File");
 		return false;
 	}
 	str->ReadDword( &WindowCount );
@@ -79,6 +76,11 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 	ieResRef MosFile;
 	unsigned int i;
 
+	if (!str) {
+		Log(ERROR, "CHUImporter", "No data stream to read from, skipping controls");
+		return NULL;
+	}
+
 	bool found = false;
 	for (unsigned int c = 0; c < WindowCount; c++) {
 		str->Seek( WEOffset + ( 0x1c * c ), GEM_STREAM_START );
@@ -106,11 +108,10 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 		ResourceHolder<ImageMgr> mos(MosFile);
 		if (mos != NULL) {
 			win->SetBackGround( mos->GetSprite2D(), true );
-		} else
-			printMessage( "CHUImporter","Cannot Load BackGround, skipping\n",YELLOW );
+		}
 	}
 	if (!core->IsAvailable( IE_BAM_CLASS_ID )) {
-		printMessage( "CHUImporter","No BAM Importer Available, skipping controls\n",LIGHT_RED );
+		Log(ERROR, "CHUImporter", "No BAM Importer Available, skipping controls");
 		return win;
 	}
 	for (i = 0; i < ControlsCount; i++) {
@@ -183,7 +184,7 @@ Window* CHUImporter::GetWindow(unsigned int wid)
 					gamedata->GetFactoryResource( BAMFile,
 							IE_BAM_CLASS_ID, IE_NORMAL );
 				if (!bam ) {
-					printMessage( "CHUImporter","Cannot Load Button Images, skipping control\n",LIGHT_RED );
+					Log(ERROR, "CHUImporter", "Cannot Load Button Images, skipping control");
 					/* IceWind Dale 2 has fake BAM ResRefs for some Buttons,
 					this will handle bad ResRefs */
 					win->AddControl( btn );
@@ -513,7 +514,7 @@ endalign:
 			break;
 
 			default:
-				printMessage( "CHUImporter","Control Not Supported\n",LIGHT_RED );
+				Log(ERROR, "CHUImporter", "Control Not Supported");
 		}
 	}
 	return win;
