@@ -24,6 +24,8 @@
 
 #include "Interface.h"
 
+using namespace GemRB;
+
 EFFImporter::EFFImporter(void)
 {
 	str = NULL;
@@ -63,10 +65,11 @@ bool EFFImporter::Open(DataStream* stream, bool autoFree)
 //if level>than maximum affected or level<than minimum affected, then the
 //effect is resisted
 // copy the info into the EFFV2 fields (separate), so it is clearer
-inline static void fixAffectedLevels(Effect *fx) {
+static inline void fixAffectedLevels(Effect *fx) {
 	if (fx->DiceSides > 0 || fx->DiceThrown > 0) {
-		fx->MinAffectedLevel = fx->DiceThrown;
-		fx->MaxAffectedLevel = fx->DiceSides;
+		//cloudkill needs these in this order
+		fx->MinAffectedLevel = fx->DiceSides;
+		fx->MaxAffectedLevel = fx->DiceThrown;
 	}
 }
 
@@ -103,9 +106,9 @@ Effect* EFFImporter::GetEffectV1(Effect *fx)
 	fx->Resistance = tmpByte;
 	str->ReadDword( &fx->Duration );
 	str->Read( &tmpByte, 1 );
-	fx->Probability1 = tmpByte;
+	fx->ProbabilityRangeMax = tmpByte;
 	str->Read( &tmpByte, 1 );
-	fx->Probability2 = tmpByte;
+	fx->ProbabilityRangeMin = tmpByte;
 	str->ReadResRef( fx->Resource );
 	str->ReadDword( &fx->DiceThrown );
 	str->ReadDword( &fx->DiceSides );
@@ -134,8 +137,8 @@ Effect* EFFImporter::GetEffectV20(Effect *fx)
 	str->ReadWord( &fx->TimingMode );
 	str->ReadWord( &fx->unknown2 );
 	str->ReadDword( &fx->Duration );
-	str->ReadWord( &fx->Probability1 );
-	str->ReadWord( &fx->Probability2 );
+	str->ReadWord( &fx->ProbabilityRangeMax );
+	str->ReadWord( &fx->ProbabilityRangeMin );
 	str->ReadResRef( fx->Resource );
 	str->ReadDword( &fx->DiceThrown );
 	str->ReadDword( &fx->DiceSides );
@@ -150,7 +153,8 @@ Effect* EFFImporter::GetEffectV20(Effect *fx)
 	str->ReadDword( &fx->Resistance );
 	str->ReadDword( &fx->Parameter3 );
 	str->ReadDword( &fx->Parameter4 );
-	str->Seek( 8, GEM_CURRENT_POS );
+	str->ReadDword( &fx->Parameter5 );
+	str->ReadDword( &fx->Parameter6 );
 	str->ReadResRef( fx->Resource2 );
 	str->ReadResRef( fx->Resource3 );	
 	str->ReadDword( &fx->PosX);
@@ -195,25 +199,28 @@ void EFFImporter::PutEffectV2(DataStream *stream, const Effect *fx) {
 	stream->WriteWord( &fx->TimingMode);
 	stream->WriteWord( &fx->unknown2);
 	stream->WriteDword( &fx->Duration);
-	stream->WriteWord( &fx->Probability1);
-	stream->WriteWord( &fx->Probability2);
-	stream->WriteResRef(fx->Resource);
+	stream->WriteWord( &fx->ProbabilityRangeMax);
+	stream->WriteWord( &fx->ProbabilityRangeMin);
+	if (fx->IsVariable) {
+		stream->Write( filling,8 );
+	} else {
+		stream->WriteResRef(fx->Resource);
+	}
 	stream->WriteDword( &fx->DiceThrown );
 	stream->WriteDword( &fx->DiceSides );
 	stream->WriteDword( &fx->SavingThrowType );
 	stream->WriteDword( &fx->SavingThrowBonus );
-	//isvariable
-	stream->Write( filling,4 );
+	stream->WriteWord( &fx->IsVariable );
+	stream->Write( filling,2 ); // SaveForHalfDamage
 	stream->WriteDword( &fx->PrimaryType );
-	stream->Write( filling,12 );
+	stream->Write( filling,12 ); // MinAffectedLevel, MaxAffectedLevel, Resistance
 	stream->WriteDword( &fx->Resistance );
 	stream->WriteDword( &fx->Parameter3 );
 	stream->WriteDword( &fx->Parameter4 );
-	stream->Write( filling,8 );
+	stream->WriteDword( &fx->Parameter5 );
+	stream->WriteDword( &fx->Parameter6 );
 	if (fx->IsVariable) {
-		stream->Write(fx->Resource+8, 8);
-		//resource1-4 are used as a continuous memory
-		stream->Write(((ieByte *) fx->Resource)+16, 8);
+		stream->Write( filling,16 );
 	} else {
 		stream->WriteResRef(fx->Resource2);
 		stream->WriteResRef(fx->Resource3);
@@ -231,7 +238,14 @@ void EFFImporter::PutEffectV2(DataStream *stream, const Effect *fx) {
 	stream->WriteDword( &fx->Projectile );
 	tmpDword1 = (ieDword) fx->InventorySlot;
 	stream->WriteDword( &tmpDword1 );
-	stream->Write( filling,40 ); //12+32+8
+	if (fx->IsVariable) {
+		//resource1-4 are used as a continuous memory
+		stream->Write(fx->Resource, 32);
+	} else {
+		stream->Write( filling,32 );
+	}
+	stream->WriteDword( &fx->CasterLevel);
+	stream->Write( filling,4);
 	stream->WriteDword( &fx->SecondaryType );
 	stream->Write( filling,60 );
 }
