@@ -32,6 +32,8 @@
 #include "Video.h" //for tints
 #include "Scriptable/Actor.h"
 
+using namespace GemRB;
+
 int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx);//6e
 int fx_set_status (Scriptable* Owner, Actor* target, Effect* fx);//ba
 int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx);//bb
@@ -97,7 +99,7 @@ void RegisterTormentOpcodes()
 //retreat_from (works only in PST) - forces target to run away/walk away from Owner
 int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_retreat_from (%2d): Mod: %d, Type: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if(0) print("fx_retreat_from(%2d): Mod: %d, Type: %d", fx->Opcode, fx->Parameter1, fx->Parameter2);
 
 	if (!Owner) {
 		return FX_NOT_APPLIED;
@@ -128,7 +130,7 @@ int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx)
 //0xba fx_set_status
 int fx_set_status (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_set_status (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_set_status(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (fx->Parameter1) {
 		if (fx->TimingMode==FX_DURATION_INSTANT_PERMANENT) {
 			BASE_STATE_SET (fx->Parameter2);
@@ -152,7 +154,7 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 {
 	bool playonce;
 
-	if (0) print( "fx_play_bam_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_play_bam_blended(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (!Owner)
 		Owner = target;
 	if (!Owner)
@@ -182,17 +184,17 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 		rgb.type=RGBModifier::TINT;
 		sca->AlterPalette(rgb);
 	}
-	if (fx->TimingMode==FX_DURATION_INSTANT_PERMANENT) {
-		playonce=true;
-	} else {
+	if ((fx->TimingMode==FX_DURATION_INSTANT_LIMITED) && (fx->Parameter2&1) ) {
 		playonce=false;
+	} else {
+		playonce=true;
 	}
 	if (playonce) {
-			sca->PlayOnce();
+		sca->PlayOnce();
 	} else {
 		if (fx->Parameter2&1) {
 			//four cycles, duration is in millisecond
-			sca->SetDefaultDuration(sca->GetSequenceDuration(AI_UPDATE_TIME));		
+			sca->SetDefaultDuration(sca->GetSequenceDuration(AI_UPDATE_TIME));
 		} else {
 			sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
 		}
@@ -229,7 +231,7 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	bool playonce;
 	bool doublehint;
 
-	if (0) print( "fx_play_bam_not_blended (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_play_bam_not_blended(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (!Owner)
 		Owner = target;
 	if (!Owner)
@@ -335,7 +337,7 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 //0xc0 fx_transfer_hp
 int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_transfer_hp (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_transfer_hp(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (Owner->Type!=ST_ACTOR) {
 		return FX_NOT_APPLIED;
 	}
@@ -375,7 +377,7 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 		damage=(signed) fx->Parameter1;
 	}
 	if (damage) {
-		damage = donor->Damage(damage, fx->Parameter2, owner);
+		damage = donor->Damage(damage, fx->Parameter2, owner, fx->IsVariable, fx->SavingThrowType);
 		receiver->NewBase( IE_HITPOINTS, damage, MOD_ADDITIVE );
 	}
 	return FX_NOT_APPLIED;
@@ -386,7 +388,7 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 //0xc2 fx_flash_screen
 int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_flash_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_flash_screen(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	core->GetVideoDriver()->SetFadeColor(((unsigned char *) &fx->Parameter1)[0],((unsigned char *) &fx->Parameter1)[1],((unsigned char *) &fx->Parameter1)[2]);
 	//this needs to be at least 2 for any effect
 	core->timer->SetFadeFromColor(2);
@@ -397,7 +399,7 @@ int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //FIXME: implement bit4 which would mean duration
 int fx_tint_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_tint_screen (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_tint_screen(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	int fromTime = fx->DiceSides;
 	int toTime = fx->DiceSides;
 	switch(fx->Parameter2&6) {
@@ -413,7 +415,7 @@ int fx_tint_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //it is a mystery, why they needed to make this effect
 int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_special_effect (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_special_effect(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	//param2 determines the effect's behaviour
 	//0 - adder's kiss projectile (0xcd)
 	//  adds play bam and damage opcodes to the projectile
@@ -433,8 +435,19 @@ int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 			strnuprcpy(fx->Resource,"rdead",8);
 			break;
 	}
-	Owner->CastSpell(fx->Resource, target, false);
-	Owner->CastSpellEnd(fx->CasterLevel);
+
+	ieResRef OldSpellResRef;
+
+	memcpy(OldSpellResRef, Owner->SpellResRef, sizeof(OldSpellResRef));
+	Owner->SetSpellResRef(fx->Resource);
+	//cast spell on target
+	//flags: deplete, instant, no interrupt
+	Owner->CastSpell(target, false, true, true);
+	//actually finish casting (if this is not good enough, use an action???)
+	//flag: instant - no casting animation
+	Owner->CastSpellEnd(fx->CasterLevel, 1);
+	Owner->SetSpellResRef(OldSpellResRef);
+
 	return FX_NOT_APPLIED;
 }
 //0xc5 fx_multiple_vvc
@@ -442,7 +455,7 @@ int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 //it plays multiple vvc's with a given delay and duration
 int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_multiple_vvc (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_multiple_vvc(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 
 	Map *area = Owner->GetCurrentArea();
 	if (!area)
@@ -461,7 +474,7 @@ int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
 		offset.y=atoi(tab->QueryField(rows,1));
 		delay = atoi(tab->QueryField(rows,3));
 		duration = atoi(tab->QueryField(rows,4));
-	 	ScriptedAnimation *sca = gamedata->GetScriptedAnimation(tab->QueryField(rows,2), true);
+		ScriptedAnimation *sca = gamedata->GetScriptedAnimation(tab->QueryField(rows,2), true);
 		if (!sca) continue;
 		sca->SetBlend();
 		sca->SetDelay(AI_UPDATE_TIME*delay);
@@ -477,7 +490,7 @@ int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
 //GemRB specific, to support BMP area background changes (desert hell projectile)
 int fx_change_background (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_change_background (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_change_background(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	Map *map = core->GetGame()->GetCurrentArea();
 	if (map) {
 		map->SetBackground(fx->Resource, fx->Duration);
@@ -487,12 +500,336 @@ int fx_change_background (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 
 //0xc7-c8 fx_unknown
 //0xc9 fx_overlay
-int fx_overlay (Scriptable* /*Owner*/, Actor* target, Effect* fx)
+static EffectRef fx_armor_ref = { "ACVsDamageTypeModifier", -1 };
+static EffectRef fx_breath_ref = { "SaveVsBreathModifier", -1 };
+static EffectRef fx_death_ref = { "SaveVsDeathModifier", -1 };
+static EffectRef fx_poly_ref = { "SaveVsPolyModifier", -1 };
+static EffectRef fx_spell_ref = { "SaveVsSpellsModifier", -1 };
+static EffectRef fx_wands_ref = { "SaveVsWandsModifier", -1 };
+static EffectRef fx_damage_opcode_ref = { "Damage", -1 };
+static EffectRef fx_colorchange_ref = { "Color:SetRGBGlobal", -1 };
+static EffectRef fx_colorpulse_ref = { "Color:PulseRGBGlobal", -1 };
+static EffectRef fx_single_color_pulse_ref = { "Color:BriefRGB", -1 };
+static EffectRef fx_resistfire_ref = { "FireResistanceModifier", -1 };
+static EffectRef fx_resistmfire_ref = { "MagicalFireResistanceModifier", -1 };
+static EffectRef fx_protection_ref = { "Protection:SpellLevel", -1 };
+static EffectRef fx_magicdamage_ref = { "MagicDamageResistanceModifier", -1 };
+static EffectRef fx_dispel_ref = { "DispelEffects", -1 };
+static EffectRef fx_miscast_ref = { "MiscastMagicModifier", -1 };
+static EffectRef fx_set_state_ref = { "SetStatus", -1 };
+
+static inline int DamageLastHitter(Effect *fx, Actor *target, int param1, int param2)
 {
-	if (0) print( "fx_overlay (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
-	target->AddAnimation(fx->Resource,-1,0,true);
+	if (fx->Parameter3) {
+		Map *map = target->GetCurrentArea();
+		Actor *actor = map->GetActorByGlobalID(target->LastHitter);
+		if (actor && PersonalDistance(target, actor)<30 ) {
+			const TriggerEntry *entry = target->GetMatchingTrigger(trigger_hitby, TEF_PROCESSED_EFFECTS);
+			if (entry) {
+				Effect *newfx = EffectQueue::CreateEffect( fx_damage_opcode_ref, param1, param2<<16, FX_DURATION_INSTANT_PERMANENT);
+				newfx->Target = FX_TARGET_PRESET;
+				newfx->Power = fx->Power;
+				memcpy(newfx->Source, fx->Source, sizeof(newfx->Source) );
+				core->ApplyEffect(newfx, actor, target);
+				if (fx->Parameter3!=0xffffffff) {
+					fx->Parameter3--;
+				}
+			}
+		}
+	}
+
+	if (!fx->Parameter3) {
+		return FX_NOT_APPLIED;
+	}
+	return FX_APPLIED;
+}
+
+static inline void ConvertTiming(Effect *fx, int Duration)
+{
+	fx->Duration = Duration;
+	fx->TimingMode = FX_DURATION_INSTANT_LIMITED;
+	ieDword GameTime = core->GetGame()->GameTime;
+	PrepareDuration(fx);
+}
+
+int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
+{
+	if(0) print("fx_overlay(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
+	if (!target) {
+		return FX_NOT_APPLIED;
+	}
+	int terminate = FX_APPLIED;
+	bool playonce = false;
+	ieDword tint = 0;
+	Effect *newfx;
+
 	//special effects based on fx_param2
-	return FX_NOT_APPLIED;
+	if (fx->FirstApply) {
+		switch(fx->Parameter2) {
+		case 0: //cloak of warding
+			ConvertTiming (fx, 5 * fx->CasterLevel);
+			fx->Parameter3 = core->Roll(3,4,fx->CasterLevel);
+			break;
+		case 1: //shield
+			ConvertTiming(fx, 25 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_armor_ref, 3, 16);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_breath_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_death_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_poly_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_spell_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_wands_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+			break;
+		case 2: //black barbed shield
+			ConvertTiming (fx, core->Roll(10,3,0));
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_armor_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+			fx->Parameter3=0xffffffff;
+			break;
+		case 3: //pain mirror
+			fx->Parameter3 = 1;
+			ConvertTiming (fx, 5 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_colorpulse_ref, 0xFAFF7000, 0x30000C);
+			core->ApplyEffect(newfx, target, Owner);
+			break;
+		case 4: //guardian mantle
+			ConvertTiming (fx, 50 + 5 * fx->CasterLevel);
+			break;
+		case 5: //shroud of shadows
+
+			break;
+		case 6: //duplication
+			core->GetAudioDrv()->Play("magic02", target->Pos.x, target->Pos.y);
+			break;
+		case 7: //armor
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_colorchange_ref, 0x825A2800, -1);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_armor_ref, 6, 16);
+			core->ApplyEffect(newfx, target, Owner);
+			break;
+		case 8: //antimagic shell
+			{
+				int i;
+
+				newfx = EffectQueue::CreateEffectCopy(fx, fx_dispel_ref, 100, 0);
+				newfx->Power = 10;
+				core->ApplyEffect(newfx, target, Owner);
+
+				for (i=0;i<2;i++) {
+					newfx = EffectQueue::CreateEffectCopy(fx, fx_miscast_ref, 100, i);
+					core->ApplyEffect(newfx, target, Owner);
+				}
+
+				for (i=1;i<10;i++) {
+					newfx = EffectQueue::CreateEffectCopy(fx, fx_protection_ref, i, 0);
+					core->ApplyEffect(newfx, target, Owner);
+				}
+				newfx = EffectQueue::CreateEffectCopy(fx, fx_magicdamage_ref, 100, 0);
+				core->ApplyEffect(newfx, target, Owner);
+
+				newfx = EffectQueue::CreateEffectCopy(fx, fx_set_state_ref, 1, STATE_ANTIMAGIC);
+				core->ApplyEffect(newfx, target, Owner);
+			}
+			break;
+		case 9: case 10: //unused
+		default:
+			break;
+		case 11: //flame walk
+			ConvertTiming (fx, 10 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_single_color_pulse_ref, 0xFF00, 0x400040);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_colorchange_ref, 0x64FA00, 0x50005);
+			//wtf is this
+			newfx->IsVariable = 0x23;
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_resistfire_ref, 50, 1);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_resistmfire_ref, 50, 1);
+			core->ApplyEffect(newfx, target, Owner);
+			break;
+		case 12: //protection from evil
+			ConvertTiming (fx, 10 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_armor_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_breath_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_death_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_poly_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_spell_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_wands_ref, 2, 0);
+			core->ApplyEffect(newfx, target, Owner);
+			//terminate = FX_NOT_APPLIED;
+			break;
+		case 13: //conflagration
+			ConvertTiming (fx, 50);
+			playonce = true;
+			break;
+		case 14: //infernal shield
+			tint = 0x5EC2FE;
+			ConvertTiming (fx, 5 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_resistfire_ref, 150, 1);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_resistmfire_ref, 150, 1);
+			core->ApplyEffect(newfx, target, Owner);
+			break;
+		case 15: //submerge the will
+			tint = 0x538D90;
+			ConvertTiming (fx, 12 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_armor_ref, 2, 16);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_breath_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_death_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_poly_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_spell_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_wands_ref, 1, 0);
+			core->ApplyEffect(newfx, target, Owner);
+			break;
+		case 16: //balance in all things
+			tint = 0x615AB4;
+			fx->Parameter3 = fx->CasterLevel/4;
+			ConvertTiming (fx, 5 * fx->CasterLevel);
+
+			newfx = EffectQueue::CreateEffectCopy(fx, fx_colorpulse_ref, 0x615AB400, 0x30000C);
+			core->ApplyEffect(newfx, target, Owner);
+			playonce = true;
+			break;
+		}
+
+		if (!target->HasVVCCell(fx->Resource)) {
+			ScriptedAnimation *sca = gamedata->GetScriptedAnimation(fx->Resource, true);
+			if (sca) {
+				if (tint) {
+					RGBModifier rgb;
+
+					rgb.speed=-1;
+					rgb.phase=0;
+					rgb.rgb.r=tint;
+					rgb.rgb.g=tint >> 8;
+					rgb.rgb.b=tint >> 16;
+					rgb.rgb.a=tint >> 24;
+					rgb.type=RGBModifier::TINT;
+
+					sca->AlterPalette(rgb);
+				}
+				sca->SetBlend();
+				if (playonce) {
+					sca->PlayOnce();
+				} else {
+					sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
+				}
+				sca->SetBlend();
+				sca->SetEffectOwned(true);
+				ScriptedAnimation *twin = sca->DetachTwin();
+				if (twin) {
+					target->AddVVCell(twin);
+				}
+				target->AddVVCell(sca);
+			}
+		}
+	}
+
+	ScriptedAnimation *vvc = target->GetVVCCell(&target->vvcOverlays, fx->Resource);
+	if (vvc) {
+		vvc->active = true;
+		vvc = target->GetVVCCell(&target->vvcShields, fx->Resource);
+		if (vvc) {
+			vvc->active = true;
+		}
+	} else {
+		return FX_NOT_APPLIED;
+	}
+
+	switch(fx->Parameter2) {
+	case 0: //cloak of warding
+		if (fx->Parameter3<=0) {
+			return FX_NOT_APPLIED;
+		}
+		//flag for removal trigger
+		target->Modified[IE_STONESKINS]=1;
+		break;
+	case 2: //black barbed shield (damage opponents)
+		if (target->LastHitter) {
+			terminate = DamageLastHitter(fx, target, core->Roll(2, 6, 0),0x100000 );
+		}
+		break;
+	case 3: case 16: //pain mirror or balance in all things
+		if (target->LastHitter) {
+			//well, someone took LastDamage out
+			//terminate = DamageLastHitter(fx, target, target->LastDamage, target->LastDamageType);
+		}
+		break;
+	case 4:
+		//this is not the original position, but the immunity stat was already used for things like this
+		//as an added benefit, HasImmunityEffects also works with the pst spell
+		STAT_BIT_OR( IE_IMMUNITY, IMM_GUARDIAN);
+		break;
+	case 5:
+		break;
+	case 6:
+		STATE_SET(STATE_EE_DUPL);
+		break;
+	case 7:
+		break;
+	case 8: //antimagic shell
+		if (!target->HasVVCCell("S061GLWB") ) {
+			ScriptedAnimation *sca = gamedata->GetScriptedAnimation("S061GLWB", false);
+			if (sca) {
+				sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
+				target->AddVVCell(sca);
+			}
+		}
+		break;
+	case 11:
+	case 12:
+	case 13:
+	case 14:
+	case 15:
+		break;
+	default:;
+	}
+	//PST doesn't keep these effects applied, but we do, because we don't have
+	//separate stats for these effects.
+	//We just use Parameter3 for the stats where needed
+	return terminate;
 }
 //0xca fx_unknown
 
@@ -502,7 +839,7 @@ int fx_overlay (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //but i would rather use the IWD2 method
 int fx_bless (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_curse (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
+	if(0) print("fx_curse(%2d): Par1: %d", fx->Opcode, fx->Parameter1);
 	//this bit is the same as the invisibility bit in other games
 	//it should be considered what if we replace the pst invis bit
 	//with this one (losing binary compatibility, gaining easier
@@ -513,7 +850,7 @@ int fx_bless (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	target->SetColorMod(255, RGBModifier::ADD, 0x18, 0xc8, 0xc8, 0xc8);
 
 	STATE_SET( STATE_BLESS );
-	STAT_SUB( IE_TOHIT, fx->Parameter1);
+	target->ToHit.HandleFxBonus(-fx->Parameter1, fx->TimingMode==FX_DURATION_INSTANT_PERMANENT);
 	STAT_SUB( IE_SAVEVSDEATH, fx->Parameter1);
 	STAT_SUB( IE_SAVEVSWANDS, fx->Parameter1);
 	STAT_SUB( IE_SAVEVSPOLY, fx->Parameter1);
@@ -525,7 +862,7 @@ int fx_bless (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xcb fx_curse
 int fx_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_curse (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
+	if(0) print("fx_curse(%2d): Par1: %d", fx->Opcode, fx->Parameter1);
 	//this bit is the same as the invisibility bit in other games
 	//it should be considered what if we replace the pst invis bit
 	//with this one (losing binary compatibility, gaining easier
@@ -533,7 +870,7 @@ int fx_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	if (STATE_GET (STATE_PST_CURSE) ) //curse is non cumulative
 		return FX_NOT_APPLIED;
 	STATE_SET( STATE_PST_CURSE );
-	STAT_SUB( IE_TOHIT, fx->Parameter1);
+	target->ToHit.HandleFxBonus(-fx->Parameter1, fx->TimingMode==FX_DURATION_INSTANT_PERMANENT);
 	STAT_SUB( IE_SAVEVSDEATH, fx->Parameter1);
 	STAT_SUB( IE_SAVEVSWANDS, fx->Parameter1);
 	STAT_SUB( IE_SAVEVSPOLY, fx->Parameter1);
@@ -548,7 +885,7 @@ static EffectRef fx_bless_ref = { "Bless", -1 };
 
 int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_prayer (%2d): Par1: %d\n", fx->Opcode, fx->Parameter1 );
+	if(0) print("fx_prayer(%2d): Par1: %d", fx->Opcode, fx->Parameter1);
 	int ea = target->GetStat(IE_EA);
 	int type;
 	if (ea>EA_EVILCUTOFF) type = 1;
@@ -559,7 +896,7 @@ int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 	int i = map->GetActorCount(true);
 	Effect *newfx = EffectQueue::CreateEffect(type?fx_curse_ref:fx_bless_ref, fx->Parameter1, fx->Parameter2, FX_DURATION_INSTANT_LIMITED);
 	memcpy(newfx, fx->Source,sizeof(ieResRef));
-	newfx->Duration=60;
+	newfx->Duration = 60;
 	while(i--) {
 		Actor *tar=map->GetActor(i,true);
 		ea = tar->GetStat(IE_EA);
@@ -579,7 +916,7 @@ int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 //0xcd fx_move_view
 int fx_move_view (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_move_view (%2d): Speed: %d\n", fx->Opcode, fx->Parameter1 );
+	if(0) print("fx_move_view(%2d): Speed: %d", fx->Opcode, fx->Parameter1);
 	Map *map = core->GetGame()->GetCurrentArea();
 	if (map) {
 		core->timer->SetMoveViewPort( fx->PosX, fx->PosY, fx->Parameter1, true);
@@ -590,7 +927,7 @@ int fx_move_view (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //0xce fx_embalm
 int fx_embalm (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_embalm (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_embalm(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (STATE_GET (STATE_EMBALM) ) //embalm is non cumulative
 		return FX_NOT_APPLIED;
 	STATE_SET( STATE_EMBALM );
@@ -604,16 +941,16 @@ int fx_embalm (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	}
 	STAT_ADD( IE_MAXHITPOINTS, fx->Parameter1);
 	if (fx->Parameter2) {
-		STAT_ADD( IE_ARMORCLASS,2 );
+		target->AC.HandleFxBonus(2, fx->TimingMode==FX_DURATION_INSTANT_PERMANENT);
 	} else {
-		STAT_ADD( IE_ARMORCLASS,1 );
+		target->AC.HandleFxBonus(1, fx->TimingMode==FX_DURATION_INSTANT_PERMANENT);
 	}
 	return FX_APPLIED;
 }
 //0xcf fx_stop_all_action
 int fx_stop_all_action (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_stop_all_action (%2d): Par2: %d\n", fx->Opcode, fx->Parameter2 );
+	if(0) print("fx_stop_all_action(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (fx->Parameter2) {
 		core->GetGame()->TimeStop(NULL, 0xffffffff);
 	} else {
@@ -628,7 +965,7 @@ int fx_iron_fist (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
 	ieDword p1,p2;
 
-	if (0) print( "fx_iron_fist (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if(0) print("fx_iron_fist(%2d): Par1: %d Par2: %d", fx->Opcode, fx->Parameter1, fx->Parameter2);
 	switch (fx->Parameter2)
 	{
 	case 0: p1 = 3; p2 = 6; break;
@@ -644,16 +981,14 @@ int fx_iron_fist (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xd1 fx_hostile_image
 int fx_hostile_image (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (0) print( "fx_hostile_image (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if(0) print("fx_hostile_image(%2d): Par1: %d Par2: %d", fx->Opcode, fx->Parameter1, fx->Parameter2);
 	return FX_NOT_APPLIED;
 }
 
 //0xd2 fx_detect_evil
-static EffectRef fx_single_color_pulse_ref = { "Color:BriefRGB", -1 };
-
 int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_detect_evil (%2d): Par1: %d Par2: %d\n", fx->Opcode, fx->Parameter1, fx->Parameter2 );
+	if(0) print("fx_detect_evil(%2d): Par1: %d Par2: %d", fx->Opcode, fx->Parameter1, fx->Parameter2);
 	ieDword type = fx->Parameter2;
 	//default is alignment/evil/speed 30/range 10
 	if (!type) type = 0x08031e0a;
@@ -680,7 +1015,7 @@ int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 //0xd3 fx_jumble_curse
 int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) print( "fx_jumble_curse (%2d)\n", fx->Opcode );
+	if(0) print("fx_jumble_curse(%2d)", fx->Opcode);
 
 	if (STATE_GET( STATE_DEAD) ) {
 		return FX_NOT_APPLIED;
@@ -712,7 +1047,7 @@ int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //In GemRB it is used in a custom spell
 int fx_speak_with_dead (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	if (0) print("fx_speak_with_dead (%2d)\n", fx->Opcode );
+	if(0) print("fx_speak_with_dead(%2d)", fx->Opcode);
 	if (!STATE_GET( STATE_DEAD) ) {
 		return FX_NOT_APPLIED;
 	}
