@@ -27,7 +27,11 @@
 #include "DialogHandler.h"
 #include "Game.h"
 #include "Interface.h"
+#include "TableMgr.h"
 #include "GUI/GameControl.h"
+#include "Scriptable/Actor.h"
+
+using namespace GemRB;
 
 //set this to -1 if charname is gabber (iwd2)
 static int charname=0;
@@ -53,7 +57,6 @@ TLKImporter::TLKImporter(void)
 	}
 	str = NULL;
 	override = NULL;
-	autoFree = false;
 
 	AutoTable tm("gender");
 	if (tm) {
@@ -73,17 +76,14 @@ TLKImporter::TLKImporter(void)
 	}
 }
 
-void ReleaseGtEntry(void *poi)
+static void ReleaseGtEntry(void *poi)
 {
 	delete (gt_type *) poi;
 }
 
 TLKImporter::~TLKImporter(void)
 {
-	if (str && autoFree) {
-		delete( str );
-	}
-
+	delete str;
 	
 	gtmap.RemoveAll(ReleaseGtEntry);
 
@@ -105,25 +105,22 @@ void TLKImporter::OpenAux()
 	if (override) {
 		if (!override->Init()) {
 			CloseAux();
-			printMessage("TlkImporter","Cannot open tlk override!\n", LIGHT_RED);
+			Log(ERROR, "TlkImporter", "Cannot open tlk override!");
 		}
 	}
 }
 
-bool TLKImporter::Open(DataStream* stream, bool autoFree)
+bool TLKImporter::Open(DataStream* stream)
 {
 	if (stream == NULL) {
 		return false;
 	}
-	if (str && this->autoFree) {
-		delete( str );
-	}
+	delete str;
 	str = stream;
-	this->autoFree = autoFree;
 	char Signature[8];
 	str->Read( Signature, 8 );
 	if (strncmp( Signature, "TLK\x20V1\x20\x20", 8 ) != 0) {
-		printMessage( "TLKImporter","Not a valid TLK File.\n", LIGHT_RED );
+		Log(ERROR, "TLKImporter", "Not a valid TLK File.");
 		return false;
 	}
 	str->Seek( 2, GEM_CURRENT_POS );
@@ -132,21 +129,24 @@ bool TLKImporter::Open(DataStream* stream, bool autoFree)
 	return true;
 }
 
-inline char* mystrncpy(char* dest, const char* source, int maxlength,
+//when copying the token, skip spaces
+inline const char* mystrncpy(char* dest, const char* source, int maxlength,
 	char delim)
 {
 	while (*source && ( *source != delim ) && maxlength--) {
-		*dest++ = *source++;
+		char chr = *source++;
+		if (chr!=' ') *dest++ = chr;
+		//*dest++ = *source++;
 	}
 	*dest = 0;
-	return dest;
+	return source;
 }
 
 /* -1	 - GABBER
 		0	 - PROTAGONIST
 		1-9 - PLAYERx
 */
-inline Actor *GetActorFromSlot(int slot)
+static inline Actor *GetActorFromSlot(int slot)
 {
 	if (slot==-1) {
 		GameControl *gc = core->GetGameControl();
@@ -187,6 +187,26 @@ char *TLKImporter::CharName(int slot)
 	return override->CS("?");
 }
 
+int TLKImporter::ClassStrRef(int slot)
+{
+	Actor *act;
+	int clss;
+
+	act=GetActorFromSlot(slot);
+	if (act) {
+		clss=act->GetStat(IE_CLASS);
+	} else {
+		clss=0;
+	}
+
+	AutoTable tab("classes");
+	if (!tab) {
+		return -1;
+	}
+	int row = tab->FindTableValue("ID", clss, 0);
+	return atoi(tab->QueryField(row,0) );
+}
+
 int TLKImporter::RaceStrRef(int slot)
 {
 	Actor *act;
@@ -223,7 +243,7 @@ int TLKImporter::BuiltinToken(char* Token, char* dest)
 {
 	char* Decoded = NULL;
 	int TokenLength;	 //decoded token length
-	gt_type *entry;
+	gt_type *entry = NULL;
 
 	//these are gender specific tokens, they are customisable by gender.2da
 	if (gtmap.Lookup(Token, (void *&) entry) ) {
@@ -245,6 +265,14 @@ int TLKImporter::BuiltinToken(char* Token, char* dest)
 		Decoded = GetString( 10174, 0 );
 		goto exit_function;
 	}
+	if (!strcmp( Token, "CLASS" )) {
+		//allow this to be used by direct setting of the token
+		int strref = ClassStrRef(-1);
+		if (strref<=0) return -1;
+		Decoded = GetString( strref, 0);
+		goto exit_function;
+	}
+
 	if (!strcmp( Token, "RACE" )) {
 		Decoded = GetString( RaceStrRef(-1), 0);
 		goto exit_function;
@@ -262,6 +290,10 @@ int TLKImporter::BuiltinToken(char* Token, char* dest)
 		Decoded = CharName(charname);
 		goto exit_function;
 	}
+	if (!strcmp( Token, "PRO_CLASS" )) {
+		Decoded = GetString( ClassStrRef(0), 0);
+		goto exit_function;
+	}
 	if (!strcmp( Token, "PRO_RACE" )) {
 		Decoded = GetString( RaceStrRef(0), 0);
 		goto exit_function;
@@ -301,7 +333,7 @@ bool TLKImporter::ResolveTags(char* dest, char* source, int Length)
 	NewLength = 0;
 	for (int i = 0; source[i]; i++) {
 		if (source[i] == '<') {
-			i += (int) (mystrncpy( Token, source + i + 1, MAX_VARIABLE_LENGTH, '>' ) - Token) + 1;
+			i = (int) (mystrncpy( Token, source + i + 1, MAX_VARIABLE_LENGTH, '>' ) - source);
 			int TokenLength = BuiltinToken( Token, dest + NewLength );
 			if (TokenLength == -1) {
 				TokenLength = core->GetTokenDictionary()->GetValueLength( Token );
@@ -341,7 +373,7 @@ bool TLKImporter::GetNewStringLength(char* string, int& Length)
 		if (string[i] == '<') {
 			// token
 			lChange = true;
-			i += (int) (mystrncpy( Token, string + i + 1, MAX_VARIABLE_LENGTH, '>' ) - Token) + 1;
+			i = (int) (mystrncpy( Token, string + i + 1, MAX_VARIABLE_LENGTH, '>' ) - string);
 			int TokenLength = BuiltinToken( Token, NULL );
 			if (TokenLength == -1) {
 				NewLength += core->GetTokenDictionary()->GetValueLength( Token );
@@ -369,7 +401,7 @@ bool TLKImporter::GetNewStringLength(char* string, int& Length)
 ieStrRef TLKImporter::UpdateString(ieStrRef strref, const char *newvalue)
 {
 	if (!override) {
-		printMessage("TLKImporter", "Custom string is not supported by this game format.\n", LIGHT_RED);
+		Log(ERROR, "TLKImporter", "Custom string is not supported by this game format.");
 		return 0xffffffff;
 	}
 
@@ -377,7 +409,7 @@ ieStrRef TLKImporter::UpdateString(ieStrRef strref, const char *newvalue)
 		return override->UpdateString(strref, newvalue);
 	}
 
-	printMessage("TLKImporter", "Cannot set custom string.\n", LIGHT_RED);
+	Log(ERROR, "TLKImporter", "Cannot set custom string.");
 	return 0xffffffff;
 }
 
@@ -443,7 +475,7 @@ empty:
 		if (SoundResRef[0] != 0) {
 			int xpos = 0;
 			int ypos = 0;
-			unsigned int flag = GEM_SND_RELATIVE | (flags&GEM_SND_SPEECH);
+			unsigned int flag = GEM_SND_RELATIVE | (flags&(GEM_SND_SPEECH|GEM_SND_QUEUE));
 			//IE_STR_SPEECH will stop the previous sound source
 			core->GetAudioDrv()->Play( SoundResRef, xpos, ypos, flag);
 		}
