@@ -18,10 +18,6 @@
  *
  */
 
-#if defined(__HAIKU__)
-#include <unistd.h>
-#endif
-
 #ifdef ANDROID
 #include "swab.h"
 #endif
@@ -32,8 +28,15 @@
 
 #include "GameData.h"
 #include "Interface.h"
+#include "PluginMgr.h"
 #include "TileSetMgr.h"
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+using namespace GemRB;
+
 struct wed_polygon {
 	ieDword FirstVertex;
 	ieDword CountVertex;
@@ -47,30 +50,24 @@ struct wed_polygon {
 WEDImporter::WEDImporter(void)
 {
 	str = NULL;
-	autoFree = false;
 }
 
 WEDImporter::~WEDImporter(void)
 {
-	if (str && autoFree) {
-		delete( str );
-	}
+	delete str;
 }
 
-bool WEDImporter::Open(DataStream* stream, bool autoFree)
+bool WEDImporter::Open(DataStream* stream)
 {
 	if (stream == NULL) {
 		return false;
 	}
-	if (str && this->autoFree) {
-		delete( str );
-	}
+	delete str;
 	str = stream;
-	this->autoFree = autoFree;
 	char Signature[8];
 	str->Read( Signature, 8 );
 	if (strncmp( Signature, "WED V1.3", 8 ) != 0) {
-		printf( "[WEDImporter]: This file is not a valid WED File\n" );
+		Log(ERROR, "WEDImporter", "This file is not a valid WED File");
 		return false;
 	}
 	str->ReadDword( &OverlaysCount );
@@ -214,7 +211,7 @@ void WEDImporter::GetDoorPolygonCount(ieWord count, ieDword offset)
 	ieDword basecount = offset-PolygonsOffset;
 	if (basecount%WED_POLYGON_SIZE) {
 		basecount+=WED_POLYGON_SIZE;
-		printf("[WEDImporter]: Found broken door polygon header!\n");
+		Log(WARNING, "WEDImporter", "Found broken door polygon header!");
 	}
 	ieDword polycount = basecount/WED_POLYGON_SIZE+count-WallPolygonsCount;
 	if (polycount>DoorPolygonsCount) {
@@ -249,7 +246,7 @@ ieWord* WEDImporter::GetDoorIndices(char* ResRef, int* count, bool& BaseClosed)
 	//The door has no representation in the WED file
 	if (i == DoorsCount) {
 		*count = 0;
-		printf( "[WEDImporter]: Found door without WED entry!\n" );
+		Log(ERROR, "WEDImporter", "Found door without WED entry!");
 		return NULL;
 	}
 
